<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* CITE class.
*
* Citation handling. This is the main file to handle citations.
*
*	@package wikindx4\core\bibcitation
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*/
class CITE
{
/** object */
public $citeStyle;

/**
* CITE
*
* @param string $output 'html', plain', 'rtf'. Default is 'html'
*/
	public function __construct($output = 'html')
	{
		$this->citeStyle = FACTORY_CITESTYLE::getInstance($output);
	}
/**
* parse [cite]...[/cite] and format citations
*
* @author Mark Grimshaw
* @version 1
* @param string $text Input text
* @param string $output 'html', plain', 'rtf'
* @param boolean $citeLink Link external citations in the returned citation. Default is TRUE
* @param boolean $rtfBibExport If TRUE, we're exporting to RTF. Default is FALSE
* @param boolean $suwpExport If TRUE, we're exporting to the word processor plug-in. Default is FALSE
* @return string parsed text
*/
	public function parseCitations($text, $output, $citeLink = TRUE, $rtfBibExport = FALSE, $suwpExport = FALSE)
	{
// If no citations, return doing nothing
		if(strpos(strtolower($text), "[cite]") === FALSE)
			return $text;
		$this->citeStyle->output = $output;
		$this->citeStyle->rtfBibExport = $rtfBibExport;
		if($suwpExport) // Exporting from the SUWP
		{
			$this->citeStyle->citeFormat->suwpExport = TRUE;
			if($this->citeStyle->citeFormat->style['citationStyle'] && ($output == 'rtf'))
			{
				$text = preg_replace_callback("/(\[footnote])(.*)(\[\/footnote\])/Us", 
					array($this, 'footnoteCallback'), $text);
			}
			if($this->citeStyle->citeFormat->style['endnoteStyle'] == 1)
			{
				$this->citeStyle->citeFormat->citeOffsets = preg_split("/\[cite\]/Uis", $text, -1, 			
					PREG_SPLIT_OFFSET_CAPTURE);
				array_shift($this->citeStyle->citeFormat->citeOffsets);
				$this->citeStyle->citeFormat->footnoteOffsets = preg_split("/\[footnote\]/Uis", $text, -1, 			
					PREG_SPLIT_OFFSET_CAPTURE);
				array_shift($this->citeStyle->citeFormat->footnoteOffsets);
			}
		}
		return $this->citeStyle->start($text, $citeLink);
	}
/**
* Callback
*
* @param array $matches
*/
	private function footnoteCallback($matches)
	{
		if(preg_match("/(.*)\[cite\].*\[\/cite\](.*)/Us", $matches[2], $null))
			return $matches[1] . preg_replace_callback("/\s*\[cite].*\[\/cite\](.*)/Us", 
				array($this, 'footnoteCiteCallback'), $matches[2]) . $matches[3];
		else
			return $matches[1] . $matches[2] . $matches[3];
	}
/**
* Callback
*
* @param array $matches
*/
	private function footnoteCiteCallback($matches)
	{
		return $matches[1];
	}
}
?>
