<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/** 
* Format a bibliographic resource for output.
* 
* @author	Andrea Rossato/Mark Grimshaw <sirfragalot@users.sourceforge.net>
* @version	1
*
*	@package wikindx4\core\bibcitation
*/
class EXPORTFILTER
{
/** object */
private $bibformat;
/** string */
private $format;
/** object */
private $html;
/** string */
public $newline;

/**
* EXPORTFILTER
*
* @param object $ref
* @param string $output 'html', plain', 'rtf'. Default is 'html'
*/
	public function __construct(&$ref, $output)
	{
		$this->html = FACTORY_HTML::getInstance();
		$this->bibformat =& $ref;
		$this->format = $output;
// New line (used in CITEFORMAT::endnoteProcess)
// Also, bibliographic/footnote templates may have the special string 'NEWLINE'
		if($this->format == 'rtf')
			$this->newLine = "\\par\\qj ";
		else if($this->format == 'html') // 'html'
			$this->newLine = "<br>";
		else
			$this->newLine = "\n";
	}
/**
* Format for HTML or RTF/plain?
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $data	Input string
* @param boolean $htmlDone Default FALSE (deprecated)
*/
	public function format($data, $htmlDone = FALSE)
	{
		if(!$data)
			return $data;
		if($this->format == 'html')
		{
/**
* Scan for search patterns and highlight accordingly
*/
/**
* Temporarily replace any URL - works for just one URL in the output string.
*/
			if(preg_match("/(<a.*>.*<\/a>)/i", $data, $match))
			{
				$url = preg_quote($match[1], '/');
				$data = preg_replace("/$url/", "OSBIB__URL__OSBIB", $data);
			}
			else
				$url = FALSE;
			$data = str_replace("\"", "&quot;", $data);
			if(!$htmlDone)
			{
//				$data = str_replace("<", "&lt;", $data);
//				$data = str_replace(">", "&gt;", $data);    
//				$data = preg_replace("/&(?![a-zA-Z0-9#]+?;)/", "&amp;", $data);
			}
// This double replace stops stops the search term 'ass' being used to replace the 'ass' in 'class'
//			$data = $this->bibformat->patterns ? 
//				preg_replace($this->bibformat->patterns, 
//				"<span class=\"" . $this->bibformat->patternHighlight . "\">$1</span>", $data) : $data;
			$data = preg_replace("/\[b\](.*?)\[\/b\]/is", "<strong>$1</strong>", $data);
			$data = preg_replace("/\[i\](.*?)\[\/i\]/is", "<em>$1</em>", $data);
			$data = preg_replace("/\[sup\](.*?)\[\/sup\]/is", "<sup>$1</sup>", $data);
			$data = preg_replace("/\[sub\](.*?)\[\/sub\]/is", "<sub>$1</sub>", $data);
			$data = preg_replace("/\[u\](.*?)\[\/u\]/is", 
				"<span style=\"text-decoration: underline;\">$1</span>", $data);
// Recover any URL
			if($url)
				$data = str_replace("OSBIB__URL__OSBIB", $match[1], $data);
			$data = str_replace("WIKINDX_NDASH", "&ndash;", $data);
			$data = str_replace("NEWLINE", $this->newLine, $data);
			if($this->bibformat->patterns)
			{
				$data = preg_replace($this->bibformat->patterns, 
					$this->html->span("$1", "highlight"), $data);
//				$data = preg_replace($this->bibformat->patterns, 
//					"W!I!K!I!N!D!X!$1W!I!K!I!N!D!X!", $data);
//				$data = preg_replace("/W!I!K!I!N!D!X!(.*)W!I!K!I!N!D!X!/U", 
//					"<span class=\"" . $this->bibformat->patternHighlight . "\">$1</span>", $data);
			}
		}
		else if($this->format == 'rtf')
		{
			$data = preg_replace("/&#(.*?);/", "\\u$1", $data);
			$data = preg_replace("/\[b\](.*?)\[\/b\]/is", "{{\\b $1}}", $data);
			$data = preg_replace("/\[i\](.*?)\[\/i\]/is", "{{\\i $1}}", $data);
			$data = preg_replace("/\[u\](.*?)\[\/u\]/is", "{{\\ul $1}}", $data);
        	$data = preg_replace("/\[sup\](.*?)\[\/sup\]/is", "{{\\super $1}}", $data);
        	$data = preg_replace("/\[sub\](.*?)\[\/sub\]/is", "{{\\sub $1}}", $data);
			$data = str_replace("WIKINDX_NDASH", "\\u8212\\'14", $data);
			$data = str_replace("NEWLINE", $this->newLine, $data);
		}
/**
* OpenOffice-1.x.
*/
		else if($this->format == 'sxw')
		{
			$data = $this->bibformat->utf8->decodeUtf8($data);
			$data = str_replace("\"", "&quot;", $data);
			$data = str_replace("<", "&lt;", $data);
			$data = str_replace(">", "&gt;", $data);    
			$data = preg_replace("/&(?![a-zA-Z0-9#]+?;)/", "&amp;", $data);
			$data = preg_replace("/\[b\](.*?)\[\/b\]/is", "<text:span text:style-name=\"textbf\">$1</text:span>", $data);
			$data = preg_replace("/\[i\](.*?)\[\/i\]/is", "<text:span text:style-name=\"emph\">$1</text:span>", $data);
			$data = preg_replace("/\[sup\](.*?)\[\/sup\]/is", "<text:span text:style-name=\"superscript\">$1</text:span>", $data);
			$data = preg_replace("/\[sub\](.*?)\[\/sub\]/is", "<text:span text:style-name=\"subscript\">$1</text:span>", $data);
			$data = preg_replace("/\[u\](.*?)\[\/u\]/is", 
				"<text:span text:style-name=\"underline\">$1</text:span>", $data);
			$data = "<text:p text:style-name=\"Text body\">".$data."</text:p>\n";
			$data = str_replace("WIKINDX_NDASH", "-", $data);
		}
/**
* 'noScan' means do nothing (leave BBCodes intact)
*/
		else if($this->format == 'noScan')
		{
			$data = str_replace("WIKINDX_NDASH", "-", $data);
			$data = str_replace("NEWLINE", $this->newLine, $data);
			return $data;
		}
/**
* StripBBCode for 'plain'.
*/
		else
		{
			$data = preg_replace("/\[.*\]|\[\/.*\]/U", "", $data);
			$data = str_replace("WIKINDX_NDASH", "-", $data);
			$data = str_replace("NEWLINE", $this->newLine, $data);
		}
		return $data;
	}
}
?>