<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/** 
* Load the bibliographic style
* 
* @version	1
*
*	@package wikindx4\core\bibcitation
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class LOADSTYLE
{
/**
* LOADSTYLE
*/
	public function __construct()
	{
	}
/**
* Read $stylesDir directory for XML style files and return an associative array. Each XML file should
* be within its own folder within $stylesDir.  This folder name should match the first part of the XML file name e.g.
* apa/APA.xml or chicago/CHICAGO.xml.
*
* CONSTANTS.php stores OSBIB_VERSION -- the osbibVersion field of the style XML file must equal this.
*
* @author	Mark Grimshaw
* @version	1
*
* @param	string $stylesDir	Path to styles directory
* @param	boolean $compatible  If TRUE (default), return only files compatible with OSBIB_VERSION
* @return	array Sorted assoc. array - keys = filename (less '.xml'), values = Style description.
*
*/
	public function loadDir($stylesDir, $compatible = TRUE)
	{
		if($compatible)
			$parseXML = FACTORY_PARSEXML::getInstance();
		$handle = opendir($stylesDir);
		while(FALSE !== ($dir = readdir($handle)))
		{
			$fileName = strtoupper($dir) . ".xml";
			$filePath = $stylesDir . '/' . $dir . "/" . $fileName;
			if(is_dir($stylesDir . '/' . $dir) && file_exists($filePath))
			{
				if($compatible)
				{
					$parseXML->extractEntries($filePath);
					$info = $parseXML->info;
					if($info['osbibVersion'] != OSBIB_VERSION)
						continue;
				}
				if($fh = fopen($filePath, "r"))
				{
					while(!feof($fh))
					{ 
					  $line = stream_get_line($fh, 1000000, "\n");
					  if(preg_match("/<description>(.*)<\\/description>/Ui", $line, $matches))
					  {
					$array[strtoupper($dir)] = $matches[1];
					break;
					  }
					}
				}
				fclose($fh);
			}
		}
		$parseXML->info = array(); // Empty because parseXML->info acts as a trigger to actually load a style in BIBSTYLE()
		if(!isset($array))
			return $array = array();
/**
* Sort alphabetically on the key.
*/
		ksort($array);
		return $array;
	}
}
?>
