<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/** 
* Parse the bibliographic style's XML
*
* Conversion to use with PHP's simpleXML by Ritesh Agrawal and Mark Grimshaw 2007/2008
* 
* @version	1
*
*	@package wikindx4\core\bibcitation
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class PARSEXML
{
/** object */
private $db;
/** object */
private $session;
/** string */
private $setupStyle;
/** array */
public $info = array();
/** array */
public $citation = array();
/** array */
public $footnote = array();
/** array */
public $common = array();
/** array */
public $types = array();

/**
* PARSEXML
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
	}
/**
* Read the chosen bibliographic style and create arrays based on resource type.
* 
* @author	Mark Grimshaw
* @version	1
*
* @param string $output 'html', plain', 'rtf'. Default is 'html'
* @param	boolean $export		The requested bibliographic output style.
* @param	string $stylePath	The path where the styles are.
* @return	boolean
*/
	public function loadStyle($output, $export, $stylePath)
	{
		$this->getStyle($output, $export);
//05/05/2005 G.GARDEY: add a last "/" to $stylePath if not present.
		$stylePath = trim($stylePath);
		if($stylePath[strlen($stylePath)-1] != "/"){
			$stylePath .= "/";
		}
		$uc = $stylePath . strtolower($this->setupStyle) . "/" . strtolower($this->setupStyle) . ".xml";
		$lc = $stylePath . strtolower($this->setupStyle) . "/" . strtoupper($this->setupStyle) . ".xml";
		$styleFile = file_exists($uc) ? $uc : $lc;
// If the file cannot be opened, it may have just now been disabled by the admin so load the default 
// style instead and reset the user's session
		if(!file_exists($styleFile) || (!$fh = fopen($styleFile, "r")))
		{
// style file is stored uppercase in the DB
			$styleFile = $this->setupStyle = $this->db->fetchOne($this->db->select('config', 'configStyle'));
			if(!$fh = fopen($stylePath . strtolower($this->setupStyle) . "/" . $styleFile . '.xml', "r"))
				return FALSE;
			if(!$this->session->getVar('setup_ReadOnly'))
				$this->session->setVar("setup_Style", $styleFile);
			$styleFile = $stylePath . strtolower($this->setupStyle) . "/" . strtoupper($this->setupStyle) . ".xml";
		}
		if(!$this->loadCache())
		{
			$this->extractEntries($styleFile);
			fclose($fh);
			$this->createCache();
		}
		return TRUE;
	}
/**
* Load style cache file if available
* 
* @return boolean
*/
	private function loadCache()
	{
//return FALSE; // disable cache by uncommenting -- however, reading from the cache is quicker
		$cacheDir = 'styles/CACHE';
		$cacheFile = $cacheDir . '/' . $this->setupStyle;
		if(is_dir($cacheDir) && is_file($cacheFile))
		{
			if(!$fh = fopen($cacheFile, "r"))
				return FALSE;
			$this->info = unserialize(fgets($fh));
			$this->citation = unserialize(fgets($fh));
			$this->footnote = unserialize(fgets($fh));
			$this->common = unserialize(fgets($fh));
			$this->types = unserialize(fgets($fh));
			fclose($fh);
			return TRUE;
		}
		else
			return FALSE;
	}
/**
* Create style cache
*/
	private function createCache()
	{
// print "INFO: "; print_r($this->info); print "<P>";
// print "CITATION: "; print_r($this->citation); print "<P>";
// print "FOOTNOTE: "; print_r($this->footnote); print "<P>";
// print "COMMON: "; print_r($this->common); print "<P>";
// print "TYPES: "; print_r($this->types); print "<P>";
		$cacheDir = 'styles/cache';
		if(!is_dir($cacheDir))
			return;  // do nothing -- i.e. continue to read directly from XML files
		$cacheFile = $cacheDir . '/' . $this->setupStyle;
		if(!$fh = fopen($cacheFile, "w"))
			return; // do nothing -- i.e. continue to read directly from XML files
// Serialize each array and write as one line to cache file
		fputs($fh, serialize($this->info) . "\n");
		fputs($fh, serialize($this->citation) . "\n");
		fputs($fh, serialize($this->footnote) . "\n");
		fputs($fh, serialize($this->common) . "\n");
		fputs($fh, serialize($this->types) . "\n");
		fclose($fh);
	}
/**
* Get the bibliographic style file
*
* @param string $output 'html', plain', 'rtf'. Default is 'html'
* @param	boolean $export		The requested bibliographic output style.
*/
	private function getStyle($output, $export)
	{
		if($export)
		{
			if($output == 'rtf')
				$this->setupStyle = $this->session->getVar("exportPaper_Style");
		}
		else
		{
			if($output == 'rtf')
				$this->setupStyle = $this->session->getVar("wp_ExportStyle");
		}
		if(!$this->setupStyle)
			$this->setupStyle = $this->session->getVar("setup_Style");
	}
/**
* Extract entries from file
*
* @param string $file - Location of StyleFile
*/
	public function extractEntries($file)
	{
		$xmlString = simplexml_load_file($file);
		$this->info = $this->XMLToArray($xmlString->info);
		$this->getStyleTypes($xmlString);
		$this->getFootnotes($xmlString);
		$this->common = $this->XMLToArray($xmlString->bibliography->common);
		$this->citation = $this->XMLToArray($xmlString->citation);
//		$this->footnote = $this->XMLToArray($xmlString->footnote);
	}
/**
* Convert XML to array
*
* code borrowed from http://php.net
* @param string $xml
* @return mixed
*/
	private function XMLToArray($xml)
	{
		if ($xml instanceof SimpleXMLElement)
		{
			$children = $xml->children();
			$return = null;
		}
		foreach ($children as $element => $value)
		{
			if ($value instanceof SimpleXMLElement)
			{
				$values = (array)$value->children();
				if (count($values) > 0)
					$return[$element] = $this->XMLToArray($value);
				else
				{
					if (!isset($return[$element]))
						$return[$element] = (string)$value;
					else
					{
						if (!is_array($return[$element]))
							$return[$element] = array($return[$element], (string)$value);
						else
							$return[$element][] = (string)$value;
					}
				}
			}
		}
		if (is_array($return))
			return $return;
		else
			return FALSE;
	}
/**
* Cycle through XML
*
* @param string $xmlString
*/
	private function getFootnotes($xmlString)
	{
		foreach($xmlString->footnote->resource as $element => $value)
		{
			$name = null;
			foreach($value->attributes() as $id => $v)
			{
				if($id == "name")
					$name = trim($v);
			}	
			$this->footnote[$name] = $this->XMLToArray($value);
		}
	}
/**
* Cycle through XML
*
* @param string $xmlString
*/
	private function getStyleTypes($xmlString)
	{
		foreach($xmlString->bibliography->resource as $element => $value)
		{
			$name = null;
			$creatorRewriteArray = array();
			foreach($value->attributes() as $id => $v)
			{
				if($id == "name")
					$name = trim($v);
				else
					$creatorRewriteArray[$id] = (string)$v;
			}	
			$this->types[$name] = $this->XMLToArray($value) + $creatorRewriteArray;
			if(isset($value->fallbackstyle))
			{
				$this->types['fallback'][$name] = trim((string)($value->fallbackstyle));
			}
		}
	}
}
?>