<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/** 
* Common methods for user bibliographies
*
* 
* @version	1
*
*	@package wikindx4\core\bibliographies
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class BIBLIOGRAPHYCOMMON
{
/** object */
private $db;
/** array */
private $vars;
/** object */
private $session;
/** object */
private $html;
/** object */
private $messages;
/** boolean */
public $bailOut = FALSE;

/**
* BIBLIOGRAPHYCOMMON
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
	}
/**
* Get an array of all available bibliographies inc. the MASTER bibliography
*
* @return array
*/
	public function getBibsArray()
	{
		$otherBibs = $bibsU = $bibsUG = $bibsArray = array();
// grab other users' bibliographies
/*		$subQ = $this->db->subQuery($this->db->selectNoExecute('user_bibliography_resource', 'userbibliographyresourceBibliographyId', TRUE), 
			't');
		$this->db->leftJoin('user_bibliography', 'userbibliographyId', 'userbibliographyresourceBibliographyId');
		$this->db->formatConditions(array('userbibliographyUserId' => $this->session->getVar('setup_UserId')), TRUE);
		$this->db->orderBy('userbibliographyTitle');
		$recordset = $this->db->selectFromSubQuery(FALSE, 
			array('userbibliographyId', 'userbibliographyTitle'), $subQ);
		while($row = $this->db->fetchRow($recordset))
			$otherBibs[$row['userbibliographyId']] = $this->html->dbToFormTidy($row['userbibliographyTitle']);
*/
// Get this user's bibliographies and group bibliographies user belongs to
		$subQ = $this->db->subQuery($this->db->selectNoExecute('user_bibliography_resource', 'userbibliographyresourceBibliographyId', TRUE), 
			't');
		$this->db->leftJoin('user_bibliography', 'userbibliographyId', 'userbibliographyresourceBibliographyId');
		$this->db->formatConditions(array('userbibliographyUserId' => $this->session->getVar('setup_UserId')));
		$this->db->orderBy('userbibliographyTitle');
		$recordset = $this->db->selectFromSubQuery(FALSE, 
			array('userbibliographyId', 'userbibliographyUserGroupId', 'userbibliographyTitle'), $subQ);
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['userbibliographyUserGroupId'])
				$bibsUG[$row['userbibliographyId']] = $this->html->dbToFormTidy($row['userbibliographyTitle']);
			else
				$bibsU[$row['userbibliographyId']] = $this->html->dbToFormTidy($row['userbibliographyTitle']);
		}
// add main wikindx bibliography to array with id of 0
		$bibsArray[0] = $this->messages->text("user", "masterBib");
		if(!empty($bibsU))
		{
			$bibsArray[-1] = $this->messages->text('user', 'userBibs');
			foreach($bibsU as $key => $value)
				$bibsArray[$key] = $value;
		}
		if(!empty($bibsUG))
		{
			$bibsArray[-2] = $this->messages->text('user', 'userGroupBibs');
			foreach($bibsUG as $key => $value)
				$bibsArray[$key] = $value;
		}
		if(!empty($otherBibs))
		{
			$bibsArray[-3] = $this->messages->text('user', 'otherBibs');
			foreach($otherBibs as $key => $value)
				$bibsArray[$key] = $value;
		}
		return $bibsArray;
	}
/**
* Display bibliography being browsed
*
* @param boolean $hint Default is FALSE
* @return string
*/
	public function displayBib($hint = FALSE)
	{
		$userBib = $this->session->getVar("mywikindx_Bibliography_use");
		if($userBib)
		{
			$this->db->formatConditions(array('userbibliographyId' => $userBib));
			$recordset = $this->db->select('user_bibliography', 'userbibliographyTitle');
			$row = $this->db->fetchRow($recordset);
			$bib = $row['userbibliographyTitle'];
		}
		else if($this->session->getVar('setup_MultiUser'))
			$bib = $this->messages->text("user", "masterBib");
		else
			return '';
		if($hint)
			return $this->html->span(' (' . $this->messages->text("user", "bibliography") . ": " . $bib . ')', 'hint');
		return ' (' . $this->messages->text("user", "bibliography") . ": " . $bib . ')'; // else
	}
/**
* Get user bibliographies
*
* @return array
*/
	public function getUserBibs()
	{
		if(!$this->session->getVar('setup_UserId'))
			return array();
// Get this user's bibliographies
		$tempU = array();
		$this->db->formatConditions(array('userbibliographyUserId' => $this->session->getVar('setup_UserId')));
		$this->db->formatConditions($this->db->formatFields('userbibliographyUserGroupId') . ' IS NULL');
		$this->db->orderBy('userbibliographyTitle');
		$recordset = $this->db->select('user_bibliography', array('userbibliographyId', 'userbibliographyTitle'));
		while($row = $this->db->fetchRow($recordset))
			$tempU[$row['userbibliographyId']] = $this->html->dbToFormTidy($row['userbibliographyTitle']);
		return $tempU;
	}
/**
* Get group bibliographies
*
* @return array
*/
	public function getGroupBibs()
	{
		if(!$this->session->getVar('setup_UserId'))
			return array();
		$tempUG = array();
// Get group bibliographies user belongs to
		$this->db->formatConditions(array('usergroupsusersUserId' => $this->session->getVar('setup_UserId')));
		$subQ = $this->db->subQuery($this->db->selectNoExecute('user_groups_users', 'usergroupsusersGroupId', TRUE), 
			't', TRUE, TRUE);
		$this->db->formatConditions($this->db->formatFields('userbibliographyUserGroupId') . $this->db->equal . 
			$this->db->formatFields('usergroupsusersGroupId'));
		$this->db->orderBy('userbibliographyTitle');
		$recordset = $this->db->selectFromSubQuery('user_bibliography', 
			array('userbibliographyId', 'userbibliographyUserGroupId', 'userbibliographyTitle'), $subQ);
		while($row = $this->db->fetchRow($recordset))
			$tempUG[$row['userbibliographyId']] = $this->html->dbToFormTidy($row['userbibliographyTitle']);
		return $tempUG;
	}
/**
* Set a SQL condition clause if we are browsing a user bibliography to ensure that 
* listed, selected or searched resources come only from that user bibliography.
*
* @param int $joinField The resource ID field on which to join the user_bibliography_resource table. Default is FALSE
* @param boolean $list If TRUE, this is a request coming from SEARCH or SELECT operations requiring the condition and join to be 
* stored in the SQLSTATEMENT object for reuse. Default is FALSE
*/
	public function userBibCondition($joinField = FALSE, $list = FALSE)
	{
		if($this->bailOut)
			return;
		if($useBib = $this->session->getVar("mywikindx_Bibliography_use"))
		{
			if($list)
			{
				$stmt = FACTORY_SQLSTATEMENTS::getInstance();
				$stmt->execCond[] = array('userbibliographyresourceBibliographyId' => $useBib);
				if($joinField)
				{
					$stmt->execJoin['user_bibliography_resource']['intField'] = 'userbibliographyresourceResourceId';
					$stmt->execJoin['user_bibliography_resource']['extField'] = $joinField;
				}
			}
			else
			{
				$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $useBib));
				if($joinField)
					$this->db->leftJoin('user_bibliography_resource', 'userbibliographyresourceResourceId', $joinField);
			}
		}
	}
}
?>