<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/** 
* Common methods for categories
*
* 
* @version	1
*
*	@package wikindx4\core\category
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class CATEGORY
{
/** object */
private $db;
/** object */
private $html;
/** object */
private $messages;
/** object */
private $commonBib;

/**
* CATEGORY
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
	}
/**
* Get category names from WKX_category.  Return associative array of id => category.
*
* @param boolean $userBib FALSE (default) or TRUE (return only resource types within current user bibliography)
* @param boolean $extant FALSE (default) or TRUE (return only categories for which resources exist)
* @param mixed $typeArray FALSE (default) or limits categories to those belonging to these types
* @param boolean $withGeneral FALSE (default) or include 'General' category in returned array
* @return mixed associative array or FALSE
*/
	public function grabAll($userBib = FALSE, $extant = FALSE, $typeArray = FALSE, $withGeneral = TRUE)
	{
		if($userBib)
		{
			$this->db->leftJoin('resource_category', 'resourcecategoryCategoryId', 'categoryId');
			$this->commonBib->userBibCondition('resourceCategoryId');
		}
		if($extant)
		{
			if(!$userBib)
			{
				$this->db->leftJoin('resource_category', 'resourcecategoryCategoryId', 'categoryId');
				$userBib = TRUE;
			}
		}
		if(is_array($typeArray) && !empty($typeArray))
		{
			if(!$userBib)
				$this->db->leftJoin('resource_category', 'resourcecategoryCategoryId', 'categoryId');
			$this->db->leftJoin('resource', 'resourceId', 'resourcecategoryResourceId');
			foreach($typeArray as $type)
				$conditions[] = $type;
			$this->db->formatConditionsOneField($conditions, 'resourceType');
		}
		if($withGeneral)
			$categories[1] = 'General';
		$this->db->orderBy('categoryCategory');
		$recordset = $this->db->select('category', array("categoryId", "categoryCategory"), TRUE);
		while($row = $this->db->fetchRow($recordset))
		{
			if($withGeneral && $row['categoryCategory'] == 'General')
				continue;
			$categories[$row['categoryId']] = $this->html->dbToFormTidy($row['categoryCategory']);
		}
		if(isset($categories))
		{
// $categories[1] is hard-wired to 'General' in the database -- for display, change it to the required localization
// 4/March/2013 -- keep 'General' as is in database but ensure it is placed first in list
//			$categories[1] = $this->messages->text('resources', 'general');
			return $categories;
		}
// Shouldn't ever happen since 1 => 'General' should always be set and each resource must belong to at least one category.
		return FALSE;
	}
/**
* Get subcategory names from WKX_subcategory.  Return associative array of id => subcategory.
*
* @param boolean $withCategory FALSE (default) or TRUE (return list of subcategories repended by their relevant category name)
* @param boolean $userBib FALSE (default) or TRUE (return only resource types within current user bibliography)
* @param mixed $matchCategoryIds FALSE (default) or array of category IDs (returned subcategories must belong to those categories)
* @param boolean $extant FALSE (default) or TRUE (return only resource types for which resources exist)
* @param boolean $extant FALSE (default) or TRUE (return only subcategories for which resources exist)
* @return mixed associative array or FALSE
*/
	public function grabSubAll($withCategory = FALSE, $userBib = FALSE, $matchCategoryIds = FALSE, $extant = FALSE)
	{
		if($userBib)
		{
			$this->db->leftJoin('resource_category', 'resourcecategorySubcategoryId', 'subcategoryId');
			$this->commonBib->userBibCondition('resourceCategoryId');
		}
		if($extant)
		{
			if(!$userBib)
				$this->db->leftJoin('resource_category', 'resourcecategorySubcategoryId', 'subcategoryId');
		}
		$this->db->orderBy('subcategorySubcategory');
		$recordset = $this->db->select('subcategory', array("subcategoryId", "subcategorySubcategory", "subcategoryCategoryId"), TRUE);
		while($row = $this->db->fetchRow($recordset))
		{
// Gather only subcategories within specified categories if $matchCategoryIds is given
			if(!$matchCategoryIds || 
				(is_array($matchCategoryIds) && (array_search($row['subcategoryCategoryId'], $matchCategoryIds) !== FALSE)))
			$subcategories[$row['subcategoryId']] = $this->html->dbToFormTidy($row['subcategorySubcategory']);
		}
		if(isset($subcategories))
		{
			if($withCategory)
			{
				$categories = $this->grabAll($userBib);
				foreach($subcategories as $key => $value)
				{
					$this->db->formatConditions(array('subcategoryId' => $key));
					$catKey = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
					$list[$key] = $categories[$catKey] . ' -- ' . $value;
				}
				natcasesort($list);
				return $list;
			}
			else
				return $subcategories;
		}
		return FALSE;
	}
/**
* Make the category select transfer arrows to transfer categories between select boxes with onclick
*
* @return array (toRightImage, toLeftImage)
*/
	public function transferArrowsCat()
	{
		$ajax = FACTORY_AJAX::getInstance();
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'selectCategory', 
			);
		$toRightImage = $ajax->jActionIcon('toRight', 'onclick', $jsonArray);
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'discardCategory', 
			);
		$toLeftImage = $ajax->jActionIcon('toLeft', 'onclick', $jsonArray);
		return array($toRightImage, $toLeftImage);
	}
/**
* Make the subcategory select transfer arrows to transfer subcategories between select boxes with onclick
*
* @return array (toRightImage, toLeftImage)
*/
	public function transferArrowsSubcat()
	{
		$ajax = FACTORY_AJAX::getInstance();
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'selectSubcategory', 
			);
		$toRightImage = $ajax->jActionIcon('toRight', 'onclick', $jsonArray);
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'discardSubcategory', 
			);
		$toLeftImage = $ajax->jActionIcon('toLeft', 'onclick', $jsonArray);
		return array($toRightImage, $toLeftImage);
	}
/**
* Make the category select transfer arrows to transfer categories between select boxes with onclick (Up and Down)
*
* @return array (toRightImage, toLeftImage)
*/
	public function transferArrowsCatUpDown()
	{
		$ajax = FACTORY_AJAX::getInstance();
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'selectCategory', 
			);
		$toBottomImage = $ajax->jActionIcon('toBottom', 'onclick', $jsonArray);
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'discardCategory', 
			);
		$toTopImage = $ajax->jActionIcon('toTop', 'onclick', $jsonArray);
		return array($toBottomImage, $toTopImage);
	}
/**
* Make the subcategory select transfer arrows to transfer subcategories between select boxes with onclick (Up and Down)
*
* @return array (toRightImage, toLeftImage)
*/
	public function transferArrowsSubcatUpDown()
	{
		$ajax = FACTORY_AJAX::getInstance();
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'selectSubcategory', 
			);
		$toBottomImage = $ajax->jActionIcon('toBottom', 'onclick', $jsonArray);
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'discardSubcategory', 
			);
		$toTopImage = $ajax->jActionIcon('toTop', 'onclick', $jsonArray);
		return array($toBottomImage, $toTopImage);
	}
}
?>
