<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/** 
* Close WIKINDX tidily and print footer.
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class CLOSE
{
/** object */
private $message;
/** object */
protected $html;
/** object */
private $loadstyle;
/** object */
protected $session;
/** object */
protected $db;
/** object */
protected $template;
/** object */
protected $config;

/**
* CLOSE
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->template->loadTemplate();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->loadStyle = FACTORY_LOADSTYLE::getInstance();
		include_once("core/navigation/MENU.php");
// Debugging - print session details
//global $_SESSION;
//print_r($_SESSION); print "<P>";
// End Debugging
		$menu = new MENU();
		$menu->menus();
		$this->header();
		$this->template->tpl->assign('heading', GLOBALS::getOutputHeading());
		$this->template->tpl->assign('scripts', GLOBALS::getOutputScript());
		$this->template->tpl->assign('help', GLOBALS::getOutputHelp());
		$this->template->tpl->assign('content', GLOBALS::getOutputString());
		$this->footer();
		$this->template->tpl->display('display.tpl');
		if(ob_get_contents())
			ob_end_flush();
		$this->tidySession();
		die;
	}
/**
* Set the WIKINDX title bar etc.
*/
	protected function header()
	{
		$html = FACTORY_HTML::getInstance();
		$recordset = $this->db->select("config", "configTitle");
		$title = $html->dbToHtmlTidy($this->db->fetchOne($recordset));
        $this->template->tpl->assign('title', $title);
        $this->template->tpl->assign('headTitle', $title);
        $this->template->tpl->assign('tplPath', $this->template->tpl->template_dir);
		if($this->config->WIKINDX_RSS_ALLOW)
		{
			$this->template->tpl->assign('rssTitle', $this->config->WIKINDX_RSS_TITLE);
			$this->template->tpl->assign('displayRss', TRUE);
		}
	}
/**
* set the WIKINDX footer vars
*/
	private function footer()
	{
		$timer = FACTORY_TIMER::getInstance();
		$this->template->tpl->assign('wkx_link', WIKINDX_URL);
		$this->template->tpl->assign('wkx_title', 'wikindx');
		$footer['wikindxVersion'] = WIKINDX_VERSION . "&nbsp;&copy;" . WIKINDX_COPYRIGHT_YEAR;
		$recordset = $this->db->select("database_summary", "databasesummaryTotalResources");
        $footer['numResources'] = $this->messages->text("footer", "resources") . "&nbsp;" . $this->db->fetchOne($recordset);
		$footer['numQueries'] = $this->messages->text("footer", "queries") . "&nbsp;" . GLOBALS::getDbQueries();
		$footer['scriptTime'] = $this->messages->text("footer", "execution") . "&nbsp;" . $timer->elapsedTime() . "&nbsp;secs";
// if this is a logged in user, display the username after the heading
		if(($userId = $this->session->getVar('setup_UserId')) && $this->db)
		{
			$this->db->formatConditions(array('usersId' => $userId));
			$recordset = $this->db->select('users', 'usersUsername');
			$username = $this->html->dbToHtmlTidy($this->db->fetchOne($recordset));
			if($username)
				$footer['username'] = $this->messages->text("user", "username") . ":&nbsp;" . $username;
		}
		$styles = $this->loadStyle->loadDir("styles/bibliography");
		$footer['style'] = $this->messages->text("footer", "style") . "&nbsp;" . $styles[$this->session->getVar("setup_Style")];
		if($useBib = $this->session->getVar("mywikindx_Bibliography_use"))
		{
			$this->db->formatConditions(array('userbibliographyId' => $useBib));
			$bib = $this->html->dbToHtmlTidy($this->db->fetchOne($this->db->select('user_bibliography', 'userbibliographyTitle')));
			$footer['bibliography'] = $this->messages->text("footer", "bib") . "&nbsp;" . $bib;
		}
		else
			$footer['bibliography'] = $this->messages->text("footer", "bib") . "&nbsp;" . $this->messages->text("user", "masterBib");
		$this->template->tpl->assign("footerInfo", $footer);
	}
/**
* tidySession
*
* @author Mark Grimshaw
* @version 1
*
* A convenient place to clear certain session values which we definitely don't want the next time around
*/
	protected function tidySession()
	{
// This is used in BIBSTYLE to remember the previous primary creator name in case it's repeated in the subsequent
// entry in the bibliographic list and the bibliographic style requires that it be replaced by '____' or equivalent.
// Not clearing it may produce the replacement incorrectly for the next listing.
		$this->session->delVar("style_PreviousCreator");
// This is TRUE is the last operation made use of LISTCOMMON::display()
		$this->session->delVar("list_On");
	}
}
/** 
* Close WIKINDX tidily (no menu - used for initial logon screen).
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class CLOSENOMENU extends CLOSE
{
/**
* CLOSENOMENU
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->template->loadTemplate();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->template->tpl->assign('displayMenu', FALSE);
		$this->template->tpl->assign('displayFooter', FALSE);
		$this->header();
		$this->template->tpl->assign('heading', GLOBALS::getOutputHeading());
		$this->template->tpl->assign('scripts', GLOBALS::getOutputScript());
		$this->template->tpl->assign('content',  GLOBALS::getOutputString());
		$this->template->tpl->display('display.tpl');
		if(ob_get_contents())
			ob_end_flush();
		die;
	}
}
/** 
* Close WIKINDX tidily.  Used for javascript pop-ups such as citation that don't require header, images, menus etc.
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class CLOSEPOPUP extends CLOSE
{
/**
* CLOSEPOPUP
*/
	public function __construct()
	{
		$template = FACTORY_TEMPLATE::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$template->loadTemplate();
        $template->tpl->assign('tplPath', $template->tpl->template_dir);
		$template->tpl->assign('displayPopUp', TRUE);
		$template->tpl->assign('displayMenu', FALSE);
		$template->tpl->assign('displayFooter', FALSE);
		$template->tpl->assign('scripts', GLOBALS::getOutputScript());
		$template->tpl->assign('content',  GLOBALS::getOutputString());
		$template->tpl->display('display.tpl');
		if(ob_get_contents())
			ob_end_flush();
		die;
	}
}
/** 
* Close WIKINDX by simply printing GLOBALS::getOutputString() without any more content.  Typically used with AJAX to print 
* strings to a DIV within the WIKINDX page
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class CLOSERAW extends CLOSE
{
/**
* CLOSERAW
*/
	public function __construct()
	{
		print GLOBALS::getOutputString();
		if(ob_get_contents())
			ob_end_flush();
		die;
	}
}
?>
