<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/**
* Miscellaneous HTML elements
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class HTML
{
/** string */
public $insertJscriptAction = FALSE;

/**
* HTML
*/
	public function __construct()
	{
		// Nothing to do
	}
	
/**
* Build a string for insertion of an HTML tag attribute.
*
* Ensures that the attribute value is never empty (incorrect syntax)
* @param string $name
* @param string $value Default is ""
* @return string
*/
	private function inlineHtmlAttribute($name, $value = "")
	{
		$value = trim($value);
		if ($value == "")
			return "";
		else
			return " $name=\"$value\"";
	}
/// HTML BLOCK TAGS (memento)
/// * HTML strict: address, blockquote, dl, div, fieldset, form, h(1-6), hr, noscript, ol, p, pre, table, ul
/// * HTML transitional only: center, dir, menu, noframes, isindex

/**
* <Hx> heading element
*
* @param string $data
* @param string $class Default is ""
* @param int $level Default is 4
* @return string
*/
	public function h($data, $class = "", $level = 4)
	{
		return "<h$level".$this->inlineHtmlAttribute("class", $class).">$data</h$level>\n";
	}
/**
* <DIV> element
*
* If no $data, then this is probably used in conjunction with AJAX to hide or unhide a page element
*
* @param int $id
* @param string $data Default is ""
* @param string $class Default is ""
* @return string
*/
	public function div($id, $data = "", $class = "")
	{
		return "<div"
			.$this->inlineHtmlAttribute("class", $class)
// Name removed to pass HTML validation
//			.$this->inlineHtmlAttribute("name", $id)
			.$this->inlineHtmlAttribute("id", $id)
			.">\n$data\n</div>\n";
	}
/**
* <IFRAME> element
*
* If no $data, then this is probably used in conjunction with AJAX to hide or unhide a page element
*
* @param int $id
* @param string $data Default is ""
* @param string $class Default is ""
* @return string
*/
	public function iframe($id, $data = "", $class = "")
	{
		return "<iframe"
			.$this->inlineHtmlAttribute("class", $class)
// Name removed to pass HTML validation
//			.$this->inlineHtmlAttribute("name", $id)
			.$this->inlineHtmlAttribute("id", $id)
			.$this->inlineHtmlAttribute("name", $id)
			.">\n$data\n</iframe>\n";
	}
/**
* <P> element
*
* @param string $data Default is ""
* @param string $class Default is ""
* @param string $align Default is 'left'
* @return string
*/
	public function p($data = "", $class = "", $align = "left")
	{
		return "<p".$this->inlineHtmlAttribute("class", "$class $align").">\n$data\n</p>\n";
	}
/**
* <P> element
*
* <P> for browsing creators, collections etc. (tag colour) where a background colour needs to be specified
*
* @param string $data Default is ""
* @param string $align Default is 'left'
* @return string
*/
	public function pBrowse($data = "", $align = "left")
	{
		return $this->p($data, "browseParagraph", $align);
	}
/**
* <UL> element
*
* @param string $data
* @param string $class Default is ""
* @return string
*/
	public function ul($data, $class = "")
	{
		return "<ul".$this->inlineHtmlAttribute("class", $class).">\n$data\n</ul>\n";
	}
/**
* <OL> element
*
* @param string $data
* @param string $class Default is ""
* @return string
*/
	public function ol($data, $class = "")
	{
		return "<ol".$this->inlineHtmlAttribute("class", $class).">\n$data\n</ol>\n";
	}
/**
* <LI> element
*
* @param string $data
* @param string $class Default is ""
* @return string
*/
	public function li($data, $class = "")
	{
		return "<li".$this->inlineHtmlAttribute("class", $class).">\n$data\n</li>\n";
	}
/**
* <HR> element
*
* @param string $class Default is ""
* @return string
*/
	public function hr($class = "")
	{
		return "<hr".$this->inlineHtmlAttribute("class", $class).">\n";
	}


/// HTML INLINE TAGS (memento)
/// * HTML strict: a, abbr, acronym, bdo, big, br, button, cite, code, dfn,
///                em, img, input, iframe, kbd, label, map, object, q, samp,
///                script, select, small, span, strong, sub, sup, textarea,
///                tt, var
/// * HTML transitional only: applet, basefont, font, iframe, u, s, strike
/// These elements should not be followed by newline without reason
/// because it is included in the course of the text.


/**
* <SPAN> element
*
* @param string $data
* @param string $class Default is ""
* @return string
*/
	public function span($data, $class = "")
	{
		return "<span".$this->inlineHtmlAttribute("class", $class).">$data</span>";
	}
/**
* <SPAN color> element
*
* @param string $data
* @param string $color Default is #000
* @return string
*/
	public function color($data, $color = "#000")
	{
		return "<span style=\"color:$color\">$data</span>";
	}
/**
* <BR> element
*
* @return string
*/
	public function br()
	{
		return "<br>";
	}
/**
* <STRONG> element
*
* @param string $data
* @param string $class Default is ""
* @return string
*/
	public function strong($data, $class = "")
	{
		return "<strong".$this->inlineHtmlAttribute("class", $class).">$data</strong>";
	}
/**
* <EM> element
*
* @param string $data
* @param string $class Default is ""
* @return string
*/
	public function em($data, $class = "")
	{
		return "<em".$this->inlineHtmlAttribute("class", $class).">$data</em>";
	}
/**
* <B> element
*
* @param string $data
* @param string $class Default is ""
* @return string
* @todo Kill this deprecated HTML syntax -- LkpPo, 20120720
*/
		public function b($data, $class = "")
		{
			return $this->strong($data, $class);
		}
/**
* <I> element
*
* @param string $data
* @param string $class Default is ""
* @return string
* @todo Kill this deprecated HTML syntax -- LkpPo, 20120720
*/
		public function i($data, $class = "")
		{
			return $this->em($data, $class);
		}
/**
* <U> element
*
* @param string $data
* @param string $class Default is ""
* @return string
*/
	public function u($data, $class = "")
	{
		return $this->span($data, "u $class");
	}
/**
* <IMG> element
*
* @param string $src
* @param int $width
* @param int $height
* @param string $title Default is ""
* @param string $alt Default is ""
* @return string
*/
	public function img($src, $width, $height, $title = "", $alt = "")
	{
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$string = "<img"
			.$this->inlineHtmlAttribute("src", $src)
			.$this->inlineHtmlAttribute("width", $width)
			.$this->inlineHtmlAttribute("height", $height)
			.$this->inlineHtmlAttribute("title", $title)
			." alt=\"$alt\""
			." $js>";
		$this->insertJscriptAction = FALSE; // reset immediately !
		return $string;
	}
/**
* <A> element used as internal anchor
*
* @param string $name Default is ""
* @param string $data Default is ""
* @return string
*/
	public function aName($name = "", $data = "")
	{
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$string = "<a"
			.$this->inlineHtmlAttribute("id", $name)
			.$this->inlineHtmlAttribute("name", $name)
			." $js>$data</a>";
		$this->insertJscriptAction = FALSE; // reset immediately !
		return $string;
	}
/**
* <A> element used as hyperlink
*
* @param string $class
* @param string $label
* @param string $link
* @param string $target Default is ""
* @return string
*/
	public function a($class, $label, $link, $target = "")
	{
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$string = "<a"
			.$this->inlineHtmlAttribute("class", $class)
			.$this->inlineHtmlAttribute("href", $link)
			.$this->inlineHtmlAttribute("target", $target)
			." $js>$label</a>";
		$this->insertJscriptAction = FALSE; // reset immediately !
		return $string;
	}
/**
* <A> element used as hyperlink
*
* Hyperlinks for browsing creators, collections etc. (tag clouds) where the text colour and size is provided by the scripts to indicate frequency
*
* @param string $color Default is '#000'
* @param string $size Default is '1em'
* @param string $label
* @param string $link
* @param string $target Default is ""
* @return string
*/
	public function aBrowse($color = "#000", $size = "1em", $label, $link, $target = "")
	{
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$string = "<a class=\"browseLink\" style=\"color: $color; font-size: $size;\""
			.$this->inlineHtmlAttribute("href", $link)
			.$this->inlineHtmlAttribute("target", $target)
			." $js>$label</a>";
		$this->insertJscriptAction = FALSE; // reset immediately !
		return $string;
	}


/**
* Inlining JavaScript code
*
* @param string $function Default is ''
* @return string
*/
	public function jsInline($function = "")
	{
		$string = trim($function);
		if($string != "")
			$string = "<script type=\"text/javascript\">\n<!--\n$string\n//-->\n</script>\n";
		return $string;
	}
/**
* Insert a call to an external javascript
* @param string $src Default is ''
* @return string
*/
	public function jsInlineExternal($src = "")
	{
		$string = trim($src);
		if($string != "")
			$string = "<script type=\"text/javascript\" src=\"$string\"></script>\n";
		return $string;
	}
/** 
* Calling an external javascript and/or inlining some code
*
* @param string $src Default is ''
* @param string $function Default is ''
* @todo Replace all calls to this method by one of the above according to our needs. -- LkpPo, 20120720
* @return string
*/
	public function jsExternal($src = "", $function = "")
	{
		$string  = "";
		$string .= $this->jsInlineExternal($src);
		$string .= $this->jsInline($function);
		return $string;
	}


/// METHODS FOR CLEANING UP HTML CODE

/**
* replace newlines and carriage returns with appropriate HTML code.
* 
* first multiples then singles.
* Used for display back from DB table
* @param string $string
* @param boolean $updatev4 Default is FALSE
* @return string
*/
	public function nlToHtml($string, $updatev4 = FALSE)
	{
		$string = preg_replace("/(\015?\012){2,}/", "<br><br>", $string);
		if($updatev4)
			$string = preg_replace("/(\015?\012){1,}/", "<br>", $string);
		$string = preg_replace("/(\\\\r?\\\\n){2,}/", "<br><br>", $string);
		return preg_replace("/(\\\\r?\\\\n){1,}/", "<br>\n", $string);
	}
/**
* replace HTML newlines and carriage returns with appropriate ANSI code.
*
* first multiples then singles.
* Used for display back from DB table
* @param string $string
* @return string
*/
	public function htmlToNl($string)
	{
		$string = preg_replace("#(.*)<br>#i", "$1\r\n", $string);
		return preg_replace("#<p>(.*)</p>#i", "$1\r\n\r\n", $string);
	}
/**
* remove all newlines.
*
* For cases when user cut 'n' pastes multiple lines into single-line text box
* Used before writing to DB table
* @param string $string
* @return string
*/
	public function removeNl($string)
	{
		$string = preg_replace("/(\015?\012){2,}/", " ", $string);
		return preg_replace("/\r|\n/", " ", $string);
	}
/**
* Format text grabbed from database for printing to browser. Don't use for form elements.
*
* @param string $string
* @return string
*/
	public function dbToHtmlTidy($string)
	{
		$string = stripslashes($string);
		// $bbcode = FACTORY_BBCODE::getInstance();
		// $string = $bbcode->codeToHtml($string);
		return $this->nlToHtml($string);
	}
/**
* Format text grabbed from database for printing to form elements.
* @param string $string
* @param boolean $stripHtml Default is FALSE
* @return string
*/
	public function dbToFormTidy($string, $stripHtml = FALSE)
	{
		if($stripHtml)
			return stripslashes($this->stripHtml($string));
		else
			return htmlspecialchars(stripslashes($string), ENT_QUOTES);
	}
/**
* Strip HTML from string
* @param string $string
* @return string
*/
	public function stripHtml($string)
	{
		$search = array('@<script[^>]*?>.*?</script>@si',  // Strip out javascript
		   '@<style[^>]*?>.*?</style>@siU',    // Strip style tags properly
		   '@<[\/\!]*?[^<>]*?>@si',            // Strip out HTML tags
		   '@<![\s\S]*?--[ \t\n\r]*>@'         // Strip multi-line comments including CDATA
		);
		return preg_replace($search, '', $string);
	}
/**
* Format text grabbed from database for printing to tinyMCE form elements.
* @param string $string
* @return string
*/
	public function dbToTinyMCE($string)
	{
		return stripslashes($string);
	}
}
/**
* Miscellaneous BBCode elements
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class BBCODE
{
/** object */
private $session;
/** int */
private $imgWidthLimit;
/** int */
private $imgHeightLimit;

/**
* BBCODE class
*/
	public function __construct()
	{
		$this->session = FACTORY_SESSION::getInstance();
	}
/**
* Strip all BBCode tags
* @param string $string
* @return string
*/
	public function stripBBCode($string)
	{
		return preg_replace("/\[.*\]|\[\/.*\]/U", "", $string);
	}
/**
* replace [x]...[/x] BBcode with HTML code
*
* Used for display back from DB table
* @param string $string
* @return string
*/
	public function codeToHtml($string)
	{
// First things first: If there aren't any "[*" or hyperlinks strings in the message, we don't
// need to process it at all.
		if (!preg_match("/\[.*|www\.|ftp\./", $string))
			return $string;
// Get BBCode image display limits
		$this->imgWidthLimit = $this->session->getVar("config_imgWidthLimit");
		$this->imgHeightLimit = $this->session->getVar("config_imgHeightLimit");
// code - this must be parsed first so that other BBCode within [code]...[/code] can be safely removed.
		$string = preg_replace_callback("/\[code\](.*)\[\/code\]/is", 
			array($this, 'codeCallback'), $string);
		$pattern = array(
							"/\[nl\]/si",
							"/\[b\](.*?)\[\/b\]/si",
							"/\[u\](.*?)\[\/u\]/si",
							"/\[i\](.*?)\[\/i\]/si",
							"/\[sup\](.*?)\[\/sup\]/si",
							"/\[sub\](.*?)\[\/sub\]/si",
							"/\[size=(.*?)\](.*?)\[\/size\]/is",
							"/\[color=(.*?)\](.*?)\[\/color\]/is",
							"/\[float=(.*?)\](.*?)\[\/float\]/is",
//							"@(www([-\w\.]+)+(:\d+)?(/([\w/_\.]*(\?\S+)?)?)?)@is",
//							"@(ftp([-\w\.]+)+(:\d+)?(/([\w/_\.]*(\?\S+)?)?)?)@is",
//							"@((https?|ftp)://([-\w\.]+)+(:\d+)?(/([\w/_\.]*(\?\S+)?)?)?)@is",
							"/\[url\](.*?)\[\/url\]/is",
							"/\[url=(.*?)\](.*?)\[\/url\]/is",
							"/\[email\](.*?)\[\/email\]/is",
							"/\[img\](.*?)\[\/img\]/is",
							"/\[align=(.*?)\](.*?)\[\/align\]/is",
						);
		$change = array(
							"<br>",
							"<strong>$1</strong>",
							"<span style=\"text-decoration: underline;\">$1</span>",
							"<em>$1</em>",
							"<sup>$1</sup>",
							"<sub>$1</sub>",
							"<span style=\"font-size: $1px;\">$2</span>",
							"<span style=\"color: $1;\">$2</span>",
							"<span style=\"float: $1; margin: 0.25em\" display: inline>$2</span>",
//							"<a class=\"link\" href=\"http://\\1\" target=\"_blank\">\\1</a>",
//							"<a class=\"link\" href=\"ftp://\\1\" target=\"_blank\">\\1</a>",
							"<a class=\"link\" href=\"$1\" target=\"_blank\">$1</a>",
							"<a class=\"link\" href=\"$1\" target=\"_blank\">$2</a>",
							"<a class=\"link\" href=\"mailto:$1\">$1</a>",
							"<img src=\"$1\" border=\"0\" alt=\"\">",
							"<div align=\"$1\">$2</div>",
						);
		$string = preg_replace($pattern, $change, $string);
// image + dimensions
		$string = preg_replace_callback("/\[img=(.*)\*(.*)\](.*?)\[\/img\]/is",
			array($this, 'imgCallback'), $string);
// bbencode_list requires initial ' ' - remove it after processing.
		return substr($this->bbencode_list(' ' . $string), 1);
	}
/**
* Callback function for [code]...[/code] which cannot have other BBCode within it
* @param array $matches
* @return string
*/
	private function codeCallback($matches)
	{
		return "<code>" . preg_replace("#\[.*\]+(.+?)\[/.*\]*#s", "$1", $matches[1]) . "</code>";
	}
/**
* Callback function for [img=width*height]...[/img] to check display limitations
* @param array $matches
* @return string
*/
	private function imgCallback($matches)
	{
		$width = ($matches[1] > $this->imgWidthLimit) ? $this->imgWidthLimit : $matches[1];
		$height = ($matches[2] > $this->imgHeightLimit) ? $this->imgHeightLimit : $matches[2];
		return "<img src=\"$matches[3]\" width=\"$width\" height=\"$height\" alt=\"\">";
	}
/**
* Decode [list]...[/list] and [list=xx]...[/list]
*
* This has been shamelessly pinched from PHP Bulletin Board code with a little debugging....
 * Nathan Codding - Jan. 12, 2001.
 * Performs [list][/list] and [list=?][/list] bbencoding on the given string, and returns the results.
 * Any unmatched "[list]" or "[/list]" token will just be left alone. 
 * This works fine with both having more than one list in a message, and with nested lists.
 * Since that is not a regular language, this is actually a PDA and uses a stack. Great fun.
 *
 * Note: This function assumes the first character of $message is a space, which is added by 
 * bbencode().
* @param string $message
* @return string
*/
	public function bbencode_list($message)
	{		
		$start_length = Array();
		$start_length['ordered'] = 8;
		$start_length['unordered'] = 6;
		
		// First things first: If there aren't any "[list" strings in the message, we don't
		// need to process it at all.
		
		if(strpos(strtolower($message), "[list") === FALSE)
		{
			return $message;	
		}
		$stack = Array();
		$curr_pos = 1;
		while ($curr_pos && ($curr_pos < strlen($message)))
		{	
			$curr_pos = strpos($message, "[", $curr_pos);
		
			// If not found, $curr_pos will be 0, and the loop will end.
			if ($curr_pos)
			{
				// We found a [. It starts at $curr_pos.
				// check if it's a starting or ending list tag.
				$possible_ordered_start = substr($message, $curr_pos, $start_length['ordered']);
				$possible_unordered_start = substr($message, $curr_pos, $start_length['unordered']);
				$possible_end = substr($message, $curr_pos, 7);
				if (strcasecmp("[list]", $possible_unordered_start) == 0)
				{
					// We have a starting unordered list tag.
					// Push its position on to the stack, and then keep going to the right.
					array_push($stack, array($curr_pos, ""));
					++$curr_pos;
				}
				else if (preg_match("/\[list=([a1i])\]/si", $possible_ordered_start, $matches))
				{
					// We have a starting ordered list tag.
					// Push its position on to the stack, and the starting char onto the start
					// char stack, the keep going to the right.
					array_push($stack, array($curr_pos, $matches[1]));
					++$curr_pos;
				}
				else if (strcasecmp("[/list]", $possible_end) == 0)
				{
					// We have an ending list tag.
					// Check if we've already found a matching starting tag.
					if (sizeof($stack) > 0)
					{
						// There exists a starting tag. 
						// We need to do 2 replacements now.
						$start = array_pop($stack);
						$start_index = $start[0];
						$start_char = $start[1];
						$is_ordered = ($start_char != "");
						$start_tag_length = ($is_ordered) ? $start_length['ordered'] : $start_length['unordered'];
						// everything before the [list] tag.
						$before_start_tag = substr($message, 0, $start_index);
						// everything after the [list] tag, but before the [/list] tag.
						$between_tags = substr($message, $start_index + $start_tag_length, 
							$curr_pos - $start_index - $start_tag_length);
						// Need to replace [*] with <LI> inside the list.
						$between_tags = str_replace("[*]", "<li>", $between_tags);					
						// everything after the [/list] tag.
						$after_end_tag = substr($message, $curr_pos + 7);

						if ($is_ordered)
						{
							$message = $before_start_tag . "<ol type=\"" . $start_char . "\">";
							$message .= $between_tags . "</ol>";
						}
						else
						{
							$message = $before_start_tag . "<ul>";
							$message .= $between_tags . "</ul>";
						}					
						$message .= $after_end_tag;					
						// Now.. we've screwed up the indices by changing the length of the string. 
						// So, if there's anything in the stack, we want to resume searching just after it.
						// otherwise, we go back to the start.
						if (sizeof($stack) > 0)
						{
							$a = array_pop($stack);
							$curr_pos = $a[0];
							array_push($stack, $a);
							++$curr_pos;
						}
						else
						{
							$curr_pos = 1;
						}
					}
					else
					{
						// No matching start tag found. Increment pos, keep going.
						++$curr_pos;	
					}
				}
				else
				{
					// No starting tag or ending tag.. Increment pos, keep looping.
					++$curr_pos;	
				}
			}
		} // while	
		return $message;
	} // bbencode_list()
}
/**
* Miscellaneous FORM elements
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class FORM
{
/** object */
private $messages;
/** object */
private $session;
/** boolean */
public $insertJscriptAction = FALSE;
/** string */
public $class = 'formElements'; // see .css file in template

/**
* FORM widgets class
*/
	public function __construct()
	{
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
	}
/**
* Build a string for insertion of an HTML tag attribute.
*
* Ensures that the attribute value is never empty (incorrect syntax)
* @param string $name
* @param string $value Default is ''
* @return string
*/
	private function inlineHtmlAttribute($name, $value = "")
	{
		$value = trim($value);
		if ($value == "")
			return "";
		else
			return " $name=\"$value\"";
	}
/**
* print form header with hidden action field
*
* $js is for javascript functions
* See UTF8::testFormInput for the hidden field 'utf8CharTest'
* @param string $action
* @param string $js Default is FALSE
* @return string
*/
	public function formHeader($action, $js = FALSE)
	{
		if(!$js)
			$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString = "<form action=\"\" method=\"post\" accept-charset=\"utf-8\" $js>"
			."<input type=\"hidden\" name=\"action\" value=\"$action\">"
			."<input type=\"hidden\" name=\"utf8CharTest\" value=\"ä™®\">";
		return $pString . "\n";
	}
/**
* print form header with visible action field -- typically used for tinyMCE popups
* 
* $js is for javascript functions
* See UTF8::testFormInput for the hidden field 'utf8CharTest'
* @param string $action
* @param string $name
* @param string $js Default is FALSE
* @return string
*/
	public function formHeaderVisibleAction($action, $name, $js = FALSE)
	{
		if(!$js)
			$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString = "<form action=\"$action\""
			.$this->inlineHtmlAttribute("name", $name) . " "
			.$this->inlineHtmlAttribute("id", $name)
			." method=\"post\" accept-charset=\"utf-8\" $js>";
		$pString .= "<input type=\"hidden\" name=\"utf8CharTest\" value=\"ä™®\">";
		return $pString . "\n";
	}
/**
* print form header with hidden action field and name and id fields
*
* js is for javascript functions
* @param string $action
* @param string $name
* @param string $js Default is FALSE
* @return string
*/
	public function formHeaderName($action, $name, $js = FALSE)
	{
		if(!$js)
			$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString = "<form action=\"\" method=\"post\" name=\"$name\" id=\"$name\" accept-charset=\"utf-8\" $js>"
			."<input type=\"hidden\" name=\"action\" value=\"$action\">"
			."<input type=\"hidden\" name=\"utf8CharTest\" value=\"ä™®\">";
		return $pString . "\n";
	}
/**
* end a form
*
* @return string
*/
	public function formEnd()
	{
		return "</form>\n";
	}
/**
* print form header with hidden action field for multi-part upload forms
*
* @param string $action
* @param string $js Default is FALSE
* @return string
*/
	public function formMultiHeader($action, $js = FALSE)
	{
		if(!$js)
			$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString = "<form action=\"\" enctype=\"multipart/form-data\" method=\"post\" accept-charset=\"utf-8\" $js>"
			."<input type=\"hidden\" name=\"action\" value=\"$action\">"
			."<input type=\"hidden\" name=\"utf8CharTest\" value=\"ä™®\">";
		return $pString . "\n";
	}
/**
* print form footer with submit field
*
* @param string $value Default is FALSE
* @param string $js Default is FALSE
* @param string $name Default is FALSE
* @return string
*/
	public function formSubmit($value = FALSE, $js = FALSE, $name = FALSE)
	{
		$class = $this->class;
		if(!$value)
			$value = $this->messages->text("submit", "Submit");
		else if(array_key_exists($value, $this->messages))
			$value = $this->messages->text("submit", $value);
		if(!$js)
			$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		if(!$name)
			$name = 'submit';
		if(!$value)
			$value = 'submit';
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString = "<input".$this->inlineHtmlAttribute("class", $class)." type=\"submit\" id=\"$name\" name=\"$name\" value=\"$value\" $js>";
		return $pString . "\n";
	}
/**
* print form footer with cancel/goBack field.  Not recommended to use this.
* See:  https://sourceforge.net/p/wikindx/v4-feature-requests/32/
*
* @return string
*/
	public function goBack()
	{
		$class = $this->class;
		$value = $this->messages->text("misc", "goBack");
		$name = 'cancel';
		$pString = "<input".$this->inlineHtmlAttribute("class", $class)." type=\"button\" id=\"$name\" name=\"$name\" value=\"$value\" 
			onclick=\"coreGoBack()\">";
		return $pString . "\n";
	}
/**
* print form footer with close popup button
*
* @return string
*/
	public function closePopup()
	{
		$class = $this->class;
		$value = $this->messages->text("misc", "closePopup");
		$name = 'cancel';
		$pString = "<input".$this->inlineHtmlAttribute("class", $class)." type=\"button\" id=\"$name\" name=\"$name\" value=\"$value\" 
			onclick=\"coreClosePopup()\">";
		return $pString . "\n";
	}
/**
* print form footer with submit button field
*
* @param string $value Default is FALSE
* @param string $name Default is FALSE
* @return string
*/
	public function formSubmitButton($value = FALSE, $name = FALSE)
	{
		$class = $this->class;
		if(!$value)
			$value = $this->messages->text("submit", "Submit");
		else
			$value = $this->messages->text("submit", $value);
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		if(!$name)
			$name = 'submit';
		if(!$value)
			$value = 'submit';
		$pString = "<input".$this->inlineHtmlAttribute("class", $class)." type=\"button\" id=\"$name\" name=\"$name\" value=\"$value\" $js>";
		return $pString . "\n";
	}
/**
* print form reset button
*
* @return string
*/
	public function formReset()
	{
		$class = $this->class;
		$value = $this->messages->text("submit", "Reset");
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString = "<input".$this->inlineHtmlAttribute("class", $class)." type=\"reset\" value=\"$value\" $js>";
		return $pString . "\n";
	}
/**
* print hidden form input
*
* @param string $name
* @param string $value
* @return string
*/
	public function hidden($name, $value)
	{
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString = "<input type=\"hidden\" name=\"$name\" id=\"$name\" value=\"$value\" $js>";
		return $pString . "\n";
	}
/**
* print radio button
*
* @param string $label
* @param string $name
* @param string $value Default is FALSE
* @param string $checked Default is FALSE
* @return string
*/
	public function radioButton($label, $name, $value = FALSE, $checked = FALSE)
	{
		$class = $this->class;
		$checked ? $checked = " checked" : "";
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<input".$this->inlineHtmlAttribute("class", $class)." type=\"radio\" name=\"$name\" id=\"$name\" value=\"$value\" $checked $js>";
		return $pString . "\n";
	}
/**
* print checkbox
*
* @param string $label
* @param string $name
* @param string $checked Default is FALSE
* @param string $title Default is ''
* @return string
*/
	public function checkbox($label, $name, $checked = FALSE, $title = "")
	{
		$class = $this->class;
		$checked ? $checked = " checked" : "";
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<input".$this->inlineHtmlAttribute("class", $class)." type=\"checkbox\" name=\"$name\" id=\"$name\""
			.$this->inlineHtmlAttribute("title", $title)
			." $checked $js>";
		return $pString . "\n";
	}
/**
* create select boxes for HTML forms
*
* First OPTION is always SELECTED
* optional $override allows the programmer to override the user set preferences for character limiting in select boxes
*
* @param string $label
* @param string $name
* @param array $array
* @param int $size Default  is 3
* @param int $override Default is FALSE
* @return string
*/
	public function selectFBox($label, $name, $array, $size = 3, $override = FALSE)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<select".$this->inlineHtmlAttribute("class", $class)." name=\"$name\" id=\"$name\" size=\"$size\" $js>\n";
		$value = array_shift($array);
		$string = $this->reduceLongText($value, $override);
		if(!empty($array))
			$pString .= "<option value=\"$value\" selected>" . $string . "</option>\n";
		foreach($array as $value)
		{
			$string = $this->reduceLongText($value, $override);
			$pString .= "<option value=\"$value\">$string</option>\n";
		}
		$pString .= "</select>\n";
		return $pString;
	}
/**
* create select boxes for HTML forms
*
* 'selected value' is set SELECTED
* optional $override allows the programmer to override the user set preferences for character limiting in select boxes
* @param string $label
* @param string $name
* @param array $array
* @param string $select
* @param int $size Default  is 3
* @param int $override Default is FALSE
* @return string
*/
	public function selectedBox($label, $name, $array, $select, $size = 3, $override = FALSE)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label:<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<select".$this->inlineHtmlAttribute("class", $class)." name=\"$name\" id=\"$name\" size=\"$size\" $js>\n";
		foreach($array as $value)
		{
			if($value == $select)
			{
				$string = $this->reduceLongText($value, $override);
				$pString .= "<option value=\"$value\" selected>$string</option>\n";
			}
			else
			{
				$value = $this->reduceLongText($value, $override);
				$pString .= "<option>$value</option>\n";
			}
		}
		$pString .= "</select>\n";
		return $pString;
	}
/**
* create select boxes form HTML forms
*
* First entry is default selection.
* OPTION VALUE is set so expects assoc. array where key holds this value
* optional $override allows the programmer to override the user set preferences for character limiting in select boxes
* @param string $label
* @param string $name
* @param array $array
* @param int $size Default  is 3
* @param int $override Default is FALSE
* @return string
*/
	public function selectFBoxValue($label, $name, $array, $size = 3, $override = FALSE)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>\n";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<select".$this->inlineHtmlAttribute("class", $class)." name=\"$name\" id=\"$name\" size=\"$size\" $js>\n";
		if(!empty($array))
		{
			$pString .= "<option value=\"" . key($array) . "\" selected>" .
				$this->reduceLongText(current($array), $override) . "</option>\n";
			$doneFirst = FALSE;
		}
		foreach($array as $key => $value)
		{
			$value = $this->reduceLongText($value, $override);
			if(!$doneFirst)
			{
				$doneFirst = TRUE;
				continue;
			}
			$pString .= "<option value=\"$key\">$value</option>\n";
		}
		$pString .= "</select>\n";
		return $pString;
	}
/**
* create select boxes form HTML forms
*
* $select is default selection.
* OPTION VALUE is set so expects assoc. array where key holds this value
* optional $override allows the programmer to override the user set preferences for character limiting in select boxes
* @param string $label
* @param string $name
* @param array $array
* @param string $select
* @param int $size Default  is 3
* @param int $override Default is FALSE
* @return string
*/
	public function selectedBoxValue($label, $name, $array, $select, $size = 3, $override = FALSE)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>\n";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<select".$this->inlineHtmlAttribute("class", $class)." name=\"$name\" id=\"$name\" size=\"$size\" $js>\n";
		foreach($array as $key => $value)
		{
			$value = $this->reduceLongText($value, $override);
			($key == $select) ?
				$pString .= "<option value=\"$key\" selected>$value</option>\n" :
				$pString .= "<option value=\"$key\">$value</option>\n";
		}
		$pString .= "</select>\n";
		return $pString;
	}
/**
* create select boxes form HTML forms
*
* First entry is default selection.
* OPTION VALUE is set so expects assoc. array where key holds this value.
* MULTIPLE values may be selected
* optional $override allows the programmer to override the user set preferences for character limiting in select boxes
* @param string $label
* @param string $name
* @param array $array
* @param int $size Default  is 3
* @param int $override Default is FALSE
* @return string
*/
	public function selectFBoxValueMultiple($label, $name, $array, $size = 3, $override = FALSE)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>\n";
		$id = $name;
		$name .= '[]';
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<select".$this->inlineHtmlAttribute("class", $class)." name=\"$name\" id=\"$id\" size=\"$size\" multiple $js>\n";
		if(!empty($array))
		{
			$pString .= "<option value=\"" . key($array) . "\" selected>" .
				$this->reduceLongText(current($array), $override) . "</option>\n";
			$doneFirst = FALSE;
		}
		foreach($array as $key => $value)
		{
			$value = $this->reduceLongText($value, $override);
			if(!$doneFirst)
			{
				$doneFirst = TRUE;
				continue;
			}
			$pString .= "<option value=\"$key\">$value</option>\n";
		}
		$pString .= "</select>\n";
		return $pString;
	}
/**
* create select boxes form HTML forms
*
* OPTION VALUE is set so expects assoc. array where key holds this value.
* MULTIPLE values may be selected
* optional $override allows the programmer to override the user set preferences for character limiting in select boxes
* @param string $label
* @param string $name
* @param array $array
* @param array $values
* @param int $size Default  is 3
* @param int $override Default is FALSE
* @return string
*/
	public function selectedBoxValueMultiple($label, $name, $array, $values, $size = 3, $override = FALSE)
	{
		$class = $this->class;
		$this->override = $override;
		$pString = '';
		if($label)
			$pString = "$label<br>\n";
		$id = $name;
		$name .= '[]';
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<select".$this->inlineHtmlAttribute("class", $class)." name=\"$name\" id=\"$id\" size=\"$size\" multiple $js>\n";
		foreach($array as $key => $value)
		{
			$value = $this->reduceLongText($value, $override);
			if((array_search($key, $values) !== FALSE) && $key)
				$pString .= "<option value=\"$key\" selected>" . $value . "</option>\n";
			else
				$pString .= "<option value=\"$key\">$value</option>\n";
		}
//		$pString .= join('', array_values($array));
		$pString .= "</select>\n";
		return $pString;
	}
/**
* password input type
*
* @param string $label
* @param string $name
* @param string $value Default is FALSE
* @param int $size Default is 20
* @param int $maxLength Default is 255
* @return string
*/
	public function passwordInput($label, $name, $value = FALSE, $size = 20, $maxLength = 255)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<input"
			.$this->inlineHtmlAttribute("class", $class)
			." type=\"password\" name=\"$name\" id=\"$name\" value=\"$value\" size=\"$size\" maxlength=\"$maxLength\" $js>";
		return $pString . "\n";
	}
/**
* text input type
*
* @param string $label
* @param string $name
* @param string $value Default is FALSE
* @param int $size Default is 20
* @param int $maxLength Default is 255
* @return string
*/
	public function textInput($label, $name, $value = FALSE, $size = 20, $maxLength = 255)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<input"
			.$this->inlineHtmlAttribute("class", $class)
			." type=\"text\" name=\"$name\" id=\"$name\" value=\"$value\" size=\"$size\" maxlength=\"$maxLength\" $js>";
		return $pString . "\n";
	}
/**
* textarea input type
*
* @param string $label
* @param string $name
* @param string $value Default is FALSE
* @param int $cols Default is 30
* @param int $rows Default is 5
* @return string
*/
	public function textareaInput($label, $name, $value = FALSE, $cols = 30, $rows = 5)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<textarea"
			.$this->inlineHtmlAttribute("class", $class)
			." name=\"$name\" id=\"$name\" cols=\"$cols\" rows=\"$rows\" $js>$value</textarea>";
		return $pString . "\n";
	}
/**
* textarea readonly
*
* @param string $label
* @param string $name
* @param string $value Default is FALSE
* @param int $cols Default is 30
* @param int $rows Default is 5
* @return string
*/
	public function textareaReadonly($label, $name, $value = FALSE, $cols = 30, $rows = 5)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<textarea"
			.$this->inlineHtmlAttribute("class", $class)
			." name=\"$name\" id=\"$name\" cols=\"$cols\" rows=\"$rows\" readonly $js>$value</textarea>";
		return $pString . "\n";
	}
/**
* upload box
*
* @param string $label
* @param string $name
* @param int $size Default is 30
* @return string
*/
	public function fileUpload($label, $name, $size = 20)
	{
		$class = $this->class;
		$pString = '';
		if($label)
			$pString = "$label<br>";
		$js = $this->insertJscriptAction ? $this->insertJscriptAction : FALSE;
		$this->insertJscriptAction = FALSE; // reset immediately
		$pString .= "<input".$this->inlineHtmlAttribute("class", $class)." type=\"file\" name=\"$name\" size=\"$size\" $js>";
		return $pString . "\n";
	}
/**
* reduce the size of long text (in select boxes usually) to keep web browser display tidy
*
* optional $override allows the programmer to override the user set preferences
*
* @param string $text
* @param string $override Default is FALSE
* @return string
*/
	public function reduceLongText($text, $override = FALSE)
	{
		$limit = $override ? $override : $this->session->getVar("setup_StringLimit");
		$text = str_replace("&nbsp;", " ", $text);
		$count = function_exists('mb_strlen') ? mb_strlen($text, 'UTF-8') : strlen($text);
		if(($limit != -1) && ($count > $limit))
		{
			if(function_exists('mb_substr'))
			{
				$start = 0;
				$length = floor(($limit/2) - 2);
				$substr1 = mb_substr($text, $start, $length, 'UTF-8');
				$start = $count - $length;
				$substr2 = mb_substr($text, $start, $length, 'UTF-8');
				$text = $substr1 . ' ... ' . $substr2;
			}
			else
			{
				$start = floor(($limit/2) - 2);
				$length = $count - (2 * $start);
				$text = substr_replace($text, " ... ", $start, $length);
			}
		}
		$text = str_replace(" ", "&nbsp;", $text);
		return $text;
	}
}

/**
* Miscellaneous TABLE elements
* 
* @version	1
*
*	@package wikindx4\core\display
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
* @todo 1. Make a list of all occurrences of $width param used by Wikindx (%, px, ...) with the methods of this class. 
* @todo 2. Write the corresponding specialized CSS class
* @todo 3. Convert $width param in $class param calls. For example: 
*
*	tableStart ("mytable", "", "", "", "", "50%")
*becomes
*
*    tableStart ("mytable 50pc", "", "", "", "")
* @todo 4. Delete $width params in methods.
* @todo 5. Repeat the same work for $valign, $align, $padding, $spacing and $border params.
*LkpPo
*20120720
*/
class TABLE
{
/**
* TABLE widgets class
*/
	public function __construct()
	{
		// Nothing to do
	}
/**
* Build a string for insertion of an HTML tag attribute.
*
* Ensures that the attribute value is never empty (incorrect syntax)
*
* @param string $name
* @param string $value Default is FALSE
* @return string
*/
	private function inlineHtmlAttribute($name, $value = "")
	{
		$value = trim($value);
		if ($value == "")
			return "";
		else
			return " $name=\"$value\"";
	}
/**
* Start a <TABLE> tag
*
* @param string $class Default is ""
* @param string $border Default is "0"
* @param string $cellspacing Default is "0"
* @param string $cellpadding Default is "0"
* @param string $align Default is "center"
* @param string $width Default is "100%"
* @return string
*/
	public function tableStart($class = "", $border = "0", $cellspacing = "0", $cellpadding = "0", $align = "center", $width = "100%")
	{
		return "<table"
			.$this->inlineHtmlAttribute("class", "$class $align")
			.$this->inlineHtmlAttribute("border", $border)
			.$this->inlineHtmlAttribute("cellspacing", $cellspacing)
			.$this->inlineHtmlAttribute("cellpadding", $cellpadding)
			.$this->inlineHtmlAttribute("width", $width)
			.">\n";
	}
/**
* Close a <TABLE> tag
*
* @return string
*/
	public function tableEnd()
	{
		return "</table>\n";
	}
/**
* provide a table <caption>
*
* @param string $caption Default is ""
* @param string $class Default is ""
* @return string
*/
	public function tableCaption($caption = "", $class = "")
	{
		return (($caption == "") ? "" : "<caption ".$this->inlineHtmlAttribute("class", $class).">$caption</caption>\n");
	}
/**
* Provide a <tbody> tag
*
* @param string $class Default is ""
* @return string
*/
	public function tbodyStart($class = "")
	{
		return "<tbody".$this->inlineHtmlAttribute("class", $class).">\n";
	}
/**
* Provide a </tbody> tag
*
* @return string
*/
	public function tbodyEnd()
	{
		return "</tbody>\n";
	}
/**
* Provde a <thead> tag
*
* @param string $class Default is ""
* @return string
*/
	public function theadStart($class = "")
	{
		return "<thead".$this->inlineHtmlAttribute("class", $class).">\n";
	}
/**
* Provide a </thead> tag
*
* @return string
*/
	public function theadEnd()
	{
		return "</thead>\n";
	}
/**
* Provide a <tfoot> tag
*
* @param string $class Default is ""
* @return string
*/
	public function tfootStart($class = "")
	{
		return "<tfoot".$this->inlineHtmlAttribute("class", $class).">\n";
	}
/**
* Provide a </tfoot> tag
*
* @return string
*/
	public function tfootEnd()
	{
		return "</tfoot>\n";
	}
/**
* Provide a <tr> tag
*
* @param string $class Default is ""
* @param string $align Default is "left"
* @param string $valign Default is "top"
* @param string $colspan Default is ""
* @return string
*/
	public function trStart($class = "", $align = "left", $valign = "top", $colspan = "")
	{
		return "<tr"
			.$this->inlineHtmlAttribute("class", "$class $align $valign")
			.$this->inlineHtmlAttribute("colspan", $colspan)
			.">\n";
	}
/**
* Provide a </tr> tag
*
* @return string
*/
	public function trEnd()
	{
		return "</tr>\n";
	}
/**
* Provide a <td> tag without closing it or encapsulating data
*
* @param string $class Default is ""
* @param string $align Default is "left"
* @param string $valign Default is "top"
* @param string $colspan Default is ""
* @param string $width Default is "100%"
* @return string
*/
	public function tdStart($class = "", $align = "left", $valign = "top", $colspan = "", $width = "100%")
	{
		return "<td"
			.$this->inlineHtmlAttribute("class", "$class $align $valign")
			.$this->inlineHtmlAttribute("colspan", $colspan)
			.$this->inlineHtmlAttribute("width", $width)
			.">\n";
	}
/**
* Provide a </td> tag
*
* @return string
*/
	public function tdEnd()
	{
		return "</td>\n";
	}
/**
* Provide a <td>...</td> tag
*
* @param string $data Default is ""
* @param string $class Default is ""
* @param string $align Default is "left"
* @param string $valign Default is "top"
* @param string $colspan Default is ""
* @param string $width Default is ""
* @return string
*/
	public function td($data = "", $class = "", $align = "left", $valign = "top", $colspan = "", $width = "")
	{
		return "<td"
			.$this->inlineHtmlAttribute("class", "$class $align $valign")
			.$this->inlineHtmlAttribute("colspan", $colspan)
			.$this->inlineHtmlAttribute("width", $width)
			.">\n"
				.(($data == "") ? "&nbsp;" : $data)."\n"
			."</td>\n";
	}
/**
* Provide a <th> tag without closing it or encapsulating data
*
* @param string $class Default is ""
* @param string $align Default is "left"
* @param string $valign Default is "top"
* @param string $colspan Default is ""
* @return string
*/
	public function thStart($class = "", $align = "left", $valign = "top", $colspan = "")
	{
		return "<th"
			.$this->inlineHtmlAttribute("class", "$class $align $valign")
			.$this->inlineHtmlAttribute("colspan", $colspan)
			.">\n";
	}
/**
* Provide a </th> tag
*
* @return string
*/
	public function thEnd()
	{
		return "</th>\n";
	}
/**
* Provide a <th>...</th> tag
*
* @param string $data Default is ""
* @param string $class Default is ""
* @param string $align Default is "left"
* @param string $valign Default is "top"
* @param string $colspan Default is ""
* @param string $width Default is ""
* @return string
*/
	public function th($data = "", $class = "", $align = "left", $valign = "top", $colspan = "", $width = "")
	{
		return "<th"
			.$this->inlineHtmlAttribute("class", "$class $align $valign")
			.$this->inlineHtmlAttribute("colspan", $colspan)
			.$this->inlineHtmlAttribute("width", $width)
			.">\n"
				.(($data == "") ? "&nbsp;" : $data)."\n"
			."</th>\n";
	}
}
?>
