<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* TINYMCETEXTEXPORT
*
* @version	1
*
*	@package wikindx4\core\importexport
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
* Format and export text from TinyMCE to a range of external text formats
*
* @author Mark Grimshaw
* 
*/
abstract class TINYMCETEXTEXPORT
{
/** object */
protected $session;
/** object */
protected $cite;
/** string */
protected $lineSpacingIndentQ;
/** string */
protected $paperSize;
/** string */
protected $footnoteText = FALSE;
/** string */
protected $tableStyle = FALSE;
/** string */
protected $justify;
/** string */
protected $nested = 1;
/** array */
protected $fonts = array(
							"andale mono",
							"arial",
							"arial black",
							"book antiqua",
							"comic sans ms",
							"courier new",
							"georgia",
							"helvetica",
							"impact",
							"symbol",
							"tahoma",
							"terminal",
							"times new roman",
							"trebuchet ms",
							"verdana",
							"webdings",
							"wingdings"
						);
/** array */
protected $fontSizes = array(
							'xx-small',
							'x-small',
							'small',
							'medium',
							'large',
							'x-large',
							'xx-large'
						);
/** array */
protected $lists;
/** string */
protected $indentL;
/** string */
protected $indentR;
/** sint */
protected $fontIndex = 1;
/** int */
protected $tableWidth;
/** boolean */
protected $isIE;
/** string */
protected $spanParse;
/** array */
protected $styleArray;

/**
* Initialize the class
*
* @param string $output
*/
	protected function initClass($output)
	{
		$this->session = FACTORY_SESSION::getInstance();
		$this->cite = FACTORY_CITE::getInstance($output);
		$this->styleArray = $this->cite->citeStyle->citeFormat->style;
	}
/** create header of output file */
	abstract protected function header();
/** create footer of output file */
	abstract protected function footer();
/**
* Parse text and convert
*
* @param string $text
*/
	abstract protected function parse($text);

/**
* parse <span>...</span>
*
* @author Mark Grimshaw
* @version 1
* @param string $text
* @param array $callbackStyle
* @return string
*/
	public function parseSpan($text, $callbackStyle)
	{
		$this->spanParse = TRUE;
		$this->isIE = FALSE;
		$text = preg_replace_callback("/<span\s*style\s*=\s*\"(.*?)\">(.*?)<\/span>/si", 
			$callbackStyle, $text);
		$text = preg_replace_callback("/\s*<p\s*style\s*=\s*\"(.*?)\".*?[>]+(.*?)<\/p>\s*/si", 
			array($this, "styleCallback"), $text);
// Sometimes, not all SPANs are caught - this removes them from the RTF until this problem can be fixed.
//		$text = preg_replace("/\s*<span.*?\>(.*?)<\/span>\s*/si", '$1', $text);
		if(preg_match("/<span.*?\>.*?<\/span>/si", $text)) // deal with nested span tags
			$text = $this->parseSpan($text, $callbackStyle);
		return $text;
	}
/**
* parse <div>...</div>
*
* @author Mark Grimshaw
* @version 1
* @param string $text
* @return string
*/
	protected function parseDiv($text)
	{
		$this->spanParse = TRUE;
		$this->isIE = FALSE;
//print "TEXT: " . htmlentities($text) . "<P>";
//		$text = preg_replace_callback("/\s*<div\s*style\s*=\s*\"(.*?)\">(.*?)<\/div>\s*/si", 
		$text = preg_replace_callback("/\s*<div\s*style\s*=\s*\"(.*?)\".*?[>]+(.*?)<\/div>\s*/si", 
			array($this, "styleCallback"), $text);
// IE prefers <P align = xxx>...</P> instead of DIV
		$text = preg_replace_callback("/\s*<p\s*align\s*=\s*\"*(.*?)\"*[>]+(.*?)<\/p>\s*/si", 
			array($this, "paraCallback"), $text);
		$text = preg_replace_callback("/\s*<p\s*style\s*=\s*\"(.*?)\".*?[>]+(.*?)<\/p>\s*/si", 
			array($this, "styleCallback"), $text);
// Sometimes, not all DIVs are caught - this removes them from the RTF until this problem can be fixed.
//		$text = preg_replace("/\s*<div.*?\>(.*?)<\/div>\s*/si", '$1', $text);
		return $text;
	}
/**
* Create lists
*
* @author Mark Grimshaw
* @version 1
* @param string $text
* @param string $callbackUnorderedList
* @param string $callbackOrderedList
* @return string
*/
	public function parseLists($text, $callbackUnorderedList, $callbackOrderedList)
	{
		$text = preg_replace_callback("/\s*<ul.*>\s*(.*)\s*<\/ul>\s*/Usi", $callbackUnorderedList, $text);
		$text = preg_replace_callback("/\s*<ol.*>\s*(.*)\s*<\/ol>\s*/Usi", $callbackOrderedList, $text);
		if(preg_match("/<ul.*?\>.*?<\/ul>/si", $text)) // deal with nested lists
		{
			$this->nested++;
			$text = $this->parseLists($text, $callbackUnorderedList, $callbackOrderedList);
			$this->nested--;
		}
		return $text;
	}
/**
* UTF-8 to unicode
*
* returns a string of unicode character codes 
 * Code adapted from opensource PHP code by Scott Reynen at:
 * http://www.randomchaos.com/document.php?source=php_and_unicode
 *
 * @param string $string UTF-8 encoded string
 * @return string unicode character codes
 */
	public function utf8_2_unicode($string)
	{
		$unicode = array();        
		$values = array();
		$lookingFor = 1;
		for($i = 0; $i < strlen($string); $i++)
		{
			$thisValue = ord($string[$i]);
			if($thisValue < 128)
				$unicode[] = $string[$i];
			else
			{
				if(count($values) == 0)
					$lookingFor = ($thisValue < 224) ? 2 : 3;
				$values[] = $thisValue;
				if(count($values) == $lookingFor)
				{
					$number = ($lookingFor == 3) ?
						(($values[0] % 16) * 4096) + (($values[1] % 64) * 64) + ($values[2] % 64) :
						(($values[0] % 32) * 64) + ($values[1] % 64);
					$decModulus = $number % 256;
					$modulus = dechex($number % 256);
					if($decModulus < 16)
						$modulus = '0' . $modulus;
					$unicode[] = '\u' . $number . "\'$modulus";
					$values = array();
					$lookingFor = 1;
				}
			}
		}
		return join('', $unicode);
	}
}
?>