<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/**
* PARSEPHRASE -- break up a search phrase into component parts such as 'exact phrase', AND, OR and NOT
* 
* @version	1
*
*	@package wikindx4\core\lists
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class PARSEPHRASE
{
/** object */
private $db;
/** object */
private $session;
/** object */
private $utf8;
/** object */
private $config;
/** array */
private $stringArray = array();
/** array */
private $NOTfragments = array();

/**
* PARSEPHRASE
*/
	public function __construct()
	{
		$this->session = FACTORY_SESSION::getInstance();
		$this->db = FACTORY_DB::getInstance();
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
	}
/**
* Parse input and create SQL condition
*
* @param array $input $input['Word'] is the input string from the search form
* @return mixed FALSE|string
*/
	public function parse($input)
	{
// Check for malformed or empty $phrase
		$phrase = $this->utf8->smartUtf8_encode(trim(stripslashes($input['Word'])));
		if(!$phrase || !$this->malformedString($phrase))
			return FALSE;
		if(!$phrase = $this->tidySearch($phrase))
			return FALSE;
// split up search string on single spaces -- NB, multiple spaces will result in empty elements
		$fragments = $this->splitSpaces(trim($phrase));
// loop through $fragments sorting into 'exact phrase', AND fragments, NOT fragments and OR fragments
		$this->sortFragments($fragments);
		$this->removeNot_and_FilterWords();
		return $this->createCondition($input);
	}
/**
* Create the condition clause and store search highlighting in the session
*
* @param array $input
* @return string
*/
	private function createCondition($input)
	{
		$lastType = 'AND';
		$count = 0;
		$first = TRUE;
		$lastType = FALSE;
		$conditions = $searchHighlight = array();
		foreach($this->stringArray as $wordArray)
		{
// temporarily prepend '!WIKINDXFIELDWIKINDX!' which will later be replaced by the actual field name to search on.
// Each type that equals AND means we must bracket until the next AND or the end of the search -- thus including any 
// exact phrases and OR statements in the bracket.  NOT statements are treated as AND statements.
// We always start the search with an AND clause -- parsePhrase() 
// does not allow the search word phrase to start with OR or NOT.
			$type = array_shift($wordArray);
			$highlighSearch = $search = array_shift($wordArray);
			$search = addslashes($search);
			if(array_key_exists('Partial', $input) && 
				(($type == 'AND') || ($type == 'OR') || ($type == 'NOT')))
			{
				if($type == 'NOT')
				{
					$conditions[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->like('%', $search, '%', TRUE);
					$lastType = 'NOT';
				}
				else if($type == 'OR')
				{
					if($lastType == 'NOT')
						$clauseOr[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->like('%', $search, '%', TRUE);
					else
						$clauseOr[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->like('%', $search, '%');
					$lastType = 'OR';
				}
				else if($type == 'AND')
				{
					$conditions[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->like('%', $search, '%');
					$lastType = 'AND';
				}
			}
			else
			{
				if(($type == 'NOT') || ($type == 'exactNOT'))
				{
					$conditions[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->regexp('[[:<:]]', $search, '[[:>:]]', TRUE);
					$lastType = 'NOT';
				}
				else if(($type == 'OR') || ($type == 'exactOR'))
				{
					if($lastType == 'NOT')
						$clauseOr[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->regexp('[[:<:]]', $search, '[[:>:]]', TRUE);
					else
						$clauseOr[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->regexp('[[:<:]]', $search, '[[:>:]]');
					$lastType = 'OR';
				}
				else if(($type == 'AND') || ($type == 'exactAND'))
				{
					$conditions[] = '!WIKINDXFIELDWIKINDX! ' . $this->db->regexp('[[:<:]]', $search, '[[:>:]]');
					$lastType = 'AND';
				}
			}
			if($first) // first clause
				$first = FALSE;
			else if(($type == 'AND') || ($type == 'NOT') || ($type == 'exactAND') || ($type == 'exactNOT'))
			{
				if(!empty($clauseOr))
				{
					$conditions[] = $this->db->delimit(array_pop($conditions) . $this->db->or . 
						join($this->db->or, $clauseOr), 'parentheses');
					$clauseOr = array();
				}
			}
			else if(($type == 'OR') || ($type == 'exactOR')) // phrase ends with '...OR xxx'
			{
				if(!empty($clauseOr))
				{
					$conditions[] = $this->db->delimit(array_pop($conditions) . $this->db->or . 
						join($this->db->or, $clauseOr), 'parentheses');
					$clauseOr = array();
				}
				
			}
			++$count;
			$searchHighlight[] = $highlighSearch;
		}
		$this->session->setVar("search_Highlight", implode(",", $searchHighlight));
		return join($this->db->and, $conditions);
	}
/**
* loop through $fragments sorting into 'exact phrase', AND fragments, NOT fragments and OR fragments
*
* @param array $fragments
*/
	private function sortFragments($fragments)
	{
		$searchArrayIndex = 0;
		while(!empty($fragments))
		{
			$fragment = array_shift($fragments);
// one word in quotes?
			if((strpos($fragment, '"') === 0) && (strpos($fragment, '"', 1) === strlen($fragment) - 1))
			{
				$this->stringArray[$searchArrayIndex]['type'] = 'exactAND'; // treat this word as an AND clause
				$this->stringArray[$searchArrayIndex]['string'] = trim($fragment, '"');
			}
// AND keyword
			else if(($fragment === 'AND') && (sizeOf($fragments)))
			{
				$this->stringArray[$searchArrayIndex]['type'] = 'AND';
				$this->tidyFragment($fragment, $fragments, $searchArrayIndex, 'AND');
			}
// OR keyword
			else if(($fragment === 'OR') && (sizeOf($fragments)))
			{
				$this->stringArray[$searchArrayIndex]['type'] = 'OR';
				$this->tidyFragment($fragment, $fragments, $searchArrayIndex, 'OR');
			}
// NOT keyword
			else if(($fragment === 'NOT') && (sizeOf($fragments)))
			{
				$this->stringArray[$searchArrayIndex]['type'] = 'NOT';
				$this->tidyFragment($fragment, $fragments, $searchArrayIndex, 'NOT');
			}
// start of exact phrase search?
			else if(strpos($fragment, '"') === 0)
			{
				$this->stringArray[$searchArrayIndex]['type'] = 'exactAND';
				$this->extractExactPhrase($fragment, $fragments, $searchArrayIndex);
			}
// everthing else, treat as AND <word>
			else
			{
				$this->stringArray[$searchArrayIndex]['type'] = 'AND';
				$this->tidyFragment($fragment, $fragments, $searchArrayIndex, 'noKeyword');
			}
			if(!array_key_exists('string', $this->stringArray[$searchArrayIndex]) || !$this->stringArray[$searchArrayIndex]['string'])
				unset($this->stringArray[$searchArrayIndex]);
			++$searchArrayIndex;
		}
	}
/**
* For all AND, OR or NOT fragments, tidy the input so we have something usable
*
* @param string $fragment
* @param array $fragments Reference to array
* @param int $searchArrayIndex
* @param string $type Default is FALSE
*/
	private function tidyFragment($fragment, &$fragments, $searchArrayIndex, $type = FALSE)
	{
		if(sizeOf($fragments) && ($type != 'noKeyword'))
		{
			do
				$fragment = array_shift($fragments);
				while(!$fragment);
		}
// one word in quotes?
		if((strpos($fragment, '"') === 0) && (strpos($fragment, '"', 1) === strlen($fragment) - 1))
		{
			if($type == 'noKeyword')
				$type = 'AND';
			$this->stringArray[$searchArrayIndex]['type'] = 'exact' . $type;
			$this->stringArray[$searchArrayIndex]['string'] = trim($fragment, '"');
		}
		else if(strpos($fragment, '"') === 0) // keyword is followed by exact phrase
		{
			if($type == 'noKeyword')
				$type = 'AND';
			$this->stringArray[$searchArrayIndex]['type'] = 'exact' . $type;
			$this->extractExactPhrase($fragment, $fragments, $searchArrayIndex);
		}
		else
		{
			if($fragment = preg_replace("/[^\pL\pN\p{Zs}'-]/u", '', $fragment)) // remove useless characters
				$this->stringArray[$searchArrayIndex]['string'] = $fragment;
			if($type == 'NOT')
				$this->NOTfragments[] = $fragment;
		}
	}
/**
* extract all fragment elements until end of exact phrase is found
*
* @param string $fragment
* @param array $fragments Reference to array
* @param int $searchArrayIndex
*/
	private function extractExactPhrase($fragment, &$fragments, $searchArrayIndex)
	{
		$this->stringArray[$searchArrayIndex]['string'] = ltrim($fragment, '"'); // start of exact phrase
		$extracted = FALSE;
		while(!$extracted)
		{
			$fragment = array_shift($fragments);
			if(!$fragment) // extra space -- must keep for exact phrase
				$this->stringArray[$searchArrayIndex]['string'] .= ' ' . $fragment;
			if(strpos($fragment, '"') === strlen($fragment) - 1) // end of exact phrase
			{
				$this->stringArray[$searchArrayIndex]['string'] .= ' ' . rtrim($fragment, '"');
				$extracted = TRUE;
			}
			else
				$this->stringArray[$searchArrayIndex]['string'] .= ' ' . $fragment;
		}
	}
/**
* split up search string on single spaces
*
* @param string $phrase
*/
	private function splitSpaces($phrase)
	{
		return explode(' ', $phrase);
	}
/**
* check for malformed search strings -- i.e. odd no. of " chars
*
* @param string $phrase
* @return boolean
*/	
	private function malformedString($phrase)
	{
// odd no. of " chars
		if(substr_count($phrase, '"') % 2)
			return FALSE;
// Reserved word
		if(strpos($phrase, '!WIKINDXFIELDWIKINDX!') !== FALSE)
			return FALSE;
// discard empty exact phrases and phrases where the final quote is not followed by a space
		else if(substr_count($phrase, '"'))
		{
			preg_match_all('/"/u', $phrase, $matches, PREG_OFFSET_CAPTURE);
			$count = 0;
			foreach($matches as $array)
			{
				foreach($array as $key => $null)
				{
					$offset = $array[$key][1] + 1;
					if($offset == strlen($phrase)) // reached end of search phrase
						return TRUE;
// 2nd of a pair of quotes not at the end of the phrase and not followed by space
					if(($count % 2) && (substr($phrase, $offset, 1) != ' '))
						return FALSE;
					++$count;
				}
			}
		}
		return TRUE;
	}
/**
* Return words governed by AND, OR or NOT
*
* @param string $phrase
* @return string
*/
	private function tidySearch($phrase)
	{
		if(preg_match('/^AND(.*)|^OR(.*)|(.*)AND$|(.*)OR$|(.*)NOT$/u', $phrase, $matches))
			return array_pop($matches);
		return $phrase;
	}
/**
* Remove any NOT words that are listed as AND or OR and any words to be filtered
*/
	private function removeNot_and_FilterWords()
	{
		$delete = array();
		foreach($this->stringArray as $index => $array)
		{
			if((($array['type'] == 'AND') || ($array['type'] == 'OR') || ($array['type'] == 'NOT'))
				&& 
				(array_search($this->utf8->utf8_strtolower($array['string']), $this->config->WIKINDX_SEARCHFILTER) !== FALSE))
			{
				$delete[$index] = FALSE;
				continue;
			}
			foreach($this->NOTfragments as $not)
			{
				if((($array['type'] == 'AND') || ($array['type'] == 'OR')) && 
					($this->utf8->utf8_strtolower($array['string']) == $this->utf8->utf8_strtolower($not)))
					$delete[$index] = FALSE;
			}
		}
		foreach($delete as $index => $null)
			unset($this->stringArray[$index]);
	}
/**
* Neatly print the elements of the quoteFragments array -- for debugging purposes only
*
* @param array $fragments
*/
	private function printFragments($fragments)
	{
		foreach($fragments as $key => $value)
		{
			if(!$value)
				$value = 'EXTRASPACE';
			print "$key: $value<BR>";
		}
	}
}
?>