<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/**
* PRINTMESSAGES
*
* Handle localizations
* 
* @version	1
*
*	@package wikindx4\core\messages
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class PRINTMESSAGE
{
/** string */
private $messageFile;
/** object */
protected $utf8;
/** object */
private $session;
/** object */
protected $html;
/** array */
protected $languageArray;
/** array */
protected $temp = array();

/**
* PRINTMESSAGES
*
* @param string $file
*/
	public function __construct($file)
	{
		$this->messageFile = $file;
		$this->session = FACTORY_SESSION::getInstance();
		if(!$languageDir = $this->session->getVar("setup_Language"))
			$languageDir = "en";
// language may have been disabled by admin so set to default English
		if(!is_dir('languages/' . $languageDir) || !file_exists('languages/' . $languageDir . '/description.txt'))
		{
			$languageDir = 'en';
			if(!$this->session->getVar('setup_ReadOnly'))
				$this->session->setVar("setup_Language", $languageDir);
		}
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->languageArray = $this->loadArray($languageDir);
	}
/**
* Grab the message
*
* @param string $arrayName
* @param string $indexName
* @return string
*/
	protected function text($arrayName, $indexName)
	{
		if(!array_key_exists($arrayName, $this->languageArray))
			$this->loadEnglish($arrayName);
/**
* For ease of use, copy $languageArray[$arrayname] to $temp
*/
		if(array_key_exists($arrayName, $this->languageArray))
			$this->temp = $this->languageArray[$arrayName];
/**
* if message missing from language pack, use default English
*/
		if(!array_key_exists($indexName, $this->temp) || 
			($this->temp[$indexName] == "__WIKINDX__NOT_YET_TRANSLATED__"))
			$this->loadEnglish($arrayName);
		if(!array_key_exists($indexName, $this->temp))
			print "Message <b>$indexName</b> from <b>$arrayName</b> not found in English language pack.<P>";
	}
/**
* Load English message arrays in case a localized message is missing
*
* @param string $arrayName
*/
	private function loadEnglish($arrayName)
	{
		include_once("languages/en/" . $this->messageFile . ".php");
		$class = $this->messageFile . "_en";
		$messages = new $class();
		$arrayEnglish = $messages->loadArrays();
		$this->temp = $arrayEnglish[$arrayName];
	}
/**
* load language array
*
* @param string $languageDir
* @return array
*/
	private function loadArray($languageDir)
	{
		if(($languageDir == 'en') || ($this->messageFile == 'HELP'))
		{
			include_once("languages/$languageDir/" . $this->messageFile . '.php');
			$className = $this->messageFile . "_" . $languageDir;
			$languageClass = new $className;
			return $languageClass->loadArrays();
		}
		else
		{
			$languageArray = array();
			$file = 'languages/' . $languageDir . "/" . $this->messageFile . '.xml';
			$sxi = new SimpleXmlIterator($file, NULL, TRUE);
			$array = $this->sxiToArray($sxi);
			if($this->messageFile == 'SUCCESS')
			{
				foreach($array as $key => $subArray)
					$languageArray[$key] = stripslashes($subArray[0]);
			}
			else
			{
				foreach($array as $topKey => $topArray)
				{
					if(!is_array($topArray[0]))
						continue;
					foreach($topArray[0] as $subKey => $subArray)
						$languageArray[$topKey][$subKey] = stripslashes($subArray[0]);
				}
			}
		}
		return $languageArray;
	}
/**
* convert XML to array
*
* @param object $sxi
* @return array
*/
	private function sxiToArray($sxi)
	{
		$a = array();
		for($sxi->rewind(); $sxi->valid(); $sxi->next())
		{
			if(!array_key_exists($sxi->key(), $a))
				$a[$sxi->key()] = array();
			if($sxi->hasChildren())
				$a[$sxi->key()][] = $this->sxiToArray($sxi->current());
			else
				$a[$sxi->key()][] = strval($sxi->current());
		}
		return $a;
	}
/**
* read and return languages/ directory for language preferences and check we have a sane environment
*
* @return array
*/
	public function loadDir()
	{
		$handle = opendir('languages');
		while(FALSE !== ($dir = readdir($handle)))
		{
			if($dir == 'en') // English uses .php for the language files
			{
				if(is_dir('languages/' . $dir) 
					&& file_exists('languages/' . $dir . '/description.txt')
					&& file_exists('languages/' . $dir . '/CONSTANTS.php')
					&& file_exists('languages/' . $dir . '/ERRORS.php')
					&& file_exists('languages/' . $dir . '/HELP.php')
					&& file_exists('languages/' . $dir . '/MESSAGES.php')
					&& file_exists('languages/' . $dir . '/SUCCESS.php'))
				{
// read one line
					if($fh = fopen('languages/' . $dir . '/description.txt', "r"))
						$string = fgets($fh);
					if($string)
						$array[$dir] = $string;
				}
			}
			else
			{
				if(is_dir('languages/' . $dir) 
					&& file_exists('languages/' . $dir . '/description.txt')
					&& file_exists('languages/' . $dir . '/CONSTANTS.php')
					&& file_exists('languages/' . $dir . '/HELP.php')
					&& file_exists('languages/' . $dir . '/MESSAGES.xml')
					&& file_exists('languages/' . $dir . '/ERRORS.xml')
					&& file_exists('languages/' . $dir . '/SUCCESS.xml'))
				{
// read one line
					if($fh = fopen('languages/' . $dir . '/description.txt', "r"))
						$string = fgets($fh);
					if($string)
						$array[$dir] = $string;
				}
			}
		}
// sort alphabetically on the key
		ksort($array);
		return $array;
	}
}

/**
* MESSAGES extends PRINTMESSAGES
*
* Handle general messages
* 
* @version	1
*
*	@package wikindx4\core\messages
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class MESSAGES extends PRINTMESSAGE
{
/**
*  MESSAGES
*/
	public function __construct()
	{
		parent::__construct("MESSAGES");
	}
/**
* Grab the message
*
* @param string $arrayName
* @param string $indexName
* @param string $extra Optional string that replaces '###' in the array element value string. Default is FALSE
* @return string
*/
	public function text($arrayName, $indexName, $extra = FALSE)
	{
		parent::text($arrayName, $indexName);
		$pString = $extra !== FALSE ? 
			preg_replace("/###/", $this->utf8->smartUtf8_decode(trim($extra)), $this->temp["$indexName"]) :
			preg_replace("/###/", '', $this->temp["$indexName"]);
		return $this->utf8->encodeUtf8(trim($pString));
	}
}

/**
* ERRORS extends PRINTMESSAGES
*
* Handle error messages
* 
* @version	1
*
*	@package wikindx4\core\messages
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class ERRORS extends PRINTMESSAGE
{
/**
* ERRORS
*/
	public function __construct()
	{
		parent::__construct("ERRORS");
	}
/**
* Grab the message
*
* @param string $arrayName
* @param string $indexName
* @param string $extra Optional string that replaces '###' in the array element value string. Default is FALSE
* @return string
*/
	public function text($arrayName, $indexName, $extra = FALSE)
	{
		parent::text($arrayName, $indexName);
		$pString = $extra !== FALSE ? 
			preg_replace("/###/", $this->utf8->smartUtf8_decode(trim($extra)), $this->temp["$indexName"]) :
			preg_replace("/###/", '', $this->temp["$indexName"]);
		return $this->html->p($this->utf8->encodeUtf8(trim($pString)), "error", "center");
	}
}

/**
* SUCCESS extends PRINTMESSAGES
*
* Handle success messages
* 
* @version	1
*
*	@package wikindx4\core\messages
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class SUCCESS extends PRINTMESSAGE
{
/**
* SUCCESS
*/
	public function __construct()
	{
		parent::__construct("SUCCESS");
	}
/**
* Grab the message
*
* 
* Override class function as /languages/xx/SUCCESS.xml has only one array
* @param string $indexName
* @param string $extra Optional string that replaces '###' in the array element value string. Default is FALSE
* @return string
*/
	public function text($indexName, $extra = FALSE)
	{
// if message missing from language pack, use default English
		if(!array_key_exists($indexName, $this->languageArray) || 
			($this->languageArray[$indexName] == "__WIKINDX__NOT_YET_TRANSLATED__"))
		{
			include_once("languages/en/SUCCESS.php");
			$class = "SUCCESS_en";
			$messages = new $class();
			$arrayEnglish = $messages->loadArrays();
			$temp = $arrayEnglish[$indexName];
		}
		else
			$temp = $this->languageArray[$indexName];
		$pString = $extra !== FALSE ?	
			preg_replace("/###/", $this->utf8->smartUtf8_decode(trim($extra)), $temp) :
			preg_replace("/###/", '', $temp);
		return $this->html->p($this->utf8->encodeUtf8(trim($pString)), "success", "center");
	}
}

/**
* HELP extends PRINTMESSAGES
*
* Handle help messages
* 
* @version	1
*
*	@package wikindx4\core\messages
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class HELP extends PRINTMESSAGE
{
/** object */
private $messages;
/** array */
private $arrayEnglish;
/**
* HELP
*/
	public function __construct()
	{
		parent::__construct("HELP");
		include_once("languages/en/HELP.php");
		$class = "HELP_en";
		$this->messages = new $class();
		$this->arrayEnglish = $this->messages->loadArrays();
	}
/**
* Grab the message
*
* 
* Override class function as /languages/xx/HELP.php has only one array
* @param string $indexName
* @param string $extra Optional string that replaces '###' in the array element value string. Default is FALSE
* @return string
*/
	public function text($indexName, $extra = FALSE)
	{
// if message missing from language pack, use default English
		if(!array_key_exists($indexName, $this->languageArray) || 
			($this->languageArray[$indexName] == "__WIKINDX__NOT_YET_TRANSLATED__"))
			$temp = $this->arrayEnglish[$indexName];
		else
			$temp = $this->languageArray[$indexName];
// replace #xxx# variables
		$temp = preg_replace_callback('/#(.*)#/U', array($this, 'textCallback'), $temp);
		$pString = $extra !== FALSE ?	
			preg_replace("/###/", $this->utf8->smartUtf8_decode(trim($extra)), $temp) :
			preg_replace("/###/", '', $temp);
		return $this->utf8->encodeUtf8(trim($pString));
	}
/**
* Callback function for preg_replace_callback() in text()
*
* @param array $matches
* @return string
*/
	private function textCallback($matches)
	{
		$var = 'public' . $matches[1];
		return $this->messages->{$var};
	}
}
?>
