<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* STATISTICS
*
* WIKINDX statistics
* 
* @version	1
*
*	@package wikindx4\core\miscellaneous
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class STATISTICS
{
/** object */
private $db;
/** array */
private $vars;
/** object */
private $session;
/** object */
private $messages;
/** string */
private $wikindxTitle;
/** int */
private static $maxAccesses = FALSE;
/** int */
private static $maxDownloads = FALSE;
/** int */
private static $AR = FALSE;
/** int */
private static $DR = FALSE;
/** int */
private static $MaxAR = FALSE;
/** int */
private static $MaxDR = FALSE;
/** boolean */
public $list = FALSE;

/**
* STATISTICS
*
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
	}
/**
* Check if statistics need compiling and emailing to registered users.
*
* Compiling is done if the date a user accesses wikindx is at least a month after the last compile time.
*/
	public function compile()
	{
		$field1 = $this->db->fetchOne($this->db->select('config', 'configStatisticsCompiled'));
		if($this->db->monthDiff($field1))
			$this->runCompile();
	}
/**
* Run the statistics compilation and manage any emailing required
*/
	private function runCompile()
	{
		$date = date("y/m") . '/01 00:00:00'; // i.e. 1st of this month
		$this->db->updateTimestamp('config', array('configStatisticsCompiled' => "'" . $date . "'"));
		$config = FACTORY_CONFIG::getInstance();
		$row = $this->db->fetchRow($this->db->select('config', array('configTitle', 'configEmailStatistics')));
		if(($row['configEmailStatistics'] == 'Y') && $config->WIKINDX_MAIL_SERVER)
			$emailStats = TRUE;
		else
			$emailStats = FALSE;
		$this->wikindxTitle = $row['configTitle'];
		$thisMonth = date('Ym');
		$resArray = array();
// Add all resource views and attachment downloads for this period to statistics table and gather any user details
		$resultset = $this->db->select('resource_misc', array('resourcemiscId', 'resourcemiscAccessesPeriod', 'resourcemiscAccesses'));
		while($row = $this->db->fetchRow($resultset))
		{
			if($emailStats)
				$resArray[$row['resourcemiscId']] = array('viewsPeriod' => $row['resourcemiscAccessesPeriod'], 
					'viewsTotal' => $row['resourcemiscAccesses']);
			$this->db->formatConditions(array('statisticsResourceId' => $row['resourcemiscId'], 
				'statisticsAttachmentId' => ' IS NULL'));
			$stats = $this->db->fetchOne($this->db->select('statistics', 'statisticsStatistics'));
			if($stats)
				$stats = unserialize(base64_decode($stats));
			else
				$stats = array();
			$stats[$thisMonth] = $row['resourcemiscAccessesPeriod'];
//print_r($stats); print '<P>';
			$this->db->formatConditions(array('statisticsResourceId' => $row['resourcemiscId'], 
				'statisticsAttachmentId' => ' IS NULL'));
			$this->db->update('statistics', array('statisticsStatistics' => base64_encode(serialize($stats))));
			$this->db->formatConditions(array('resourceattachmentsResourceId' => $row['resourcemiscId']));
			$resultset2 = $this->db->select('resource_attachments', array('resourceattachmentsId', 
				'resourceattachmentsDownloads', 'resourceattachmentsDownloadsPeriod', 'resourceattachmentsFileName'));
			while($row2 = $this->db->fetchRow($resultset2))
			{
				if($emailStats)
					$resArray[$row['resourcemiscId']]['downloads'][$row2['resourceattachmentsId']] = 
						array($row2['resourceattachmentsFileName'], $row2['resourceattachmentsDownloads'], 
						$row2['resourceattachmentsDownloadsPeriod']);
				$this->db->formatConditions(array('statisticsAttachmentId' => $row2['resourceattachmentsId']));
				$stats = $this->db->fetchOne($this->db->select('statistics', 'statisticsStatistics'));
				if($stats)
				{
					$stats = unserialize(base64_decode($stats));
					$stats[$thisMonth] = $row2['resourceattachmentsDownloadsPeriod'];
//print_r($stats); print '<P>';
				$this->db->formatConditions(array('statisticsAttachmentId' => $row2['resourceattachmentsId']));
				$this->db->update('statistics', array('statisticsStatistics' => base64_encode(serialize($stats))));
				}
			}
			$this->db->update('resource_misc', array('resourcemiscAccessesPeriod' => '0'));
			$this->db->update('resource_attachments', array('resourceattachmentsDownloadsPeriod' => '0'));
		}
		if($emailStats)
			$this->emailStats($resArray);
	}
/**
* Email users various stats
*
* @param array $resArray
*/
	private function emailStats($resArray)
	{
		$res = FACTORY_RESOURCECOMMON::getInstance();
		$bibStyle = FACTORY_BIBSTYLE::getInstance();
		$bibstyle->output = 'plain';
		$usersArray = array();
		$this->list = TRUE;
		foreach($resArray as $rId => $rArray)
		{
			$union = array();
			$this->db->formatConditions(array('resourcecreatorResourceId' => $rId));
			$union[] = $this->db->selectNoExecute('resource_creator', 'resourcecreatorCreatorId', FALSE, TRUE, TRUE);
			$this->db->formatConditions(array('resourcecreatorResourceId' => $rId));
			$this->db->leftJoin('resource_creator', 'resourcecreatorCreatorId', 'creatorId');
			$union[] = $this->db->selectNoExecute('creator', 'creatorSameAs', TRUE, TRUE, TRUE);
			$this->db->formatConditions($this->db->formatFields('usersIsCreator') . $this->db->inClause($this->db->union($union)));
			$resultset = $this->db->select('users', array('usersId', 'usersFullname', 'usersEmail'));
			while($row = $this->db->fetchRow($resultset))
			{
				if(!array_key_exists($row['usersId'], $usersArray))
				{
					$usersArray[$row['usersId']]['email'] = $row['usersEmail'];
					$usersArray[$row['usersId']]['name'] = $row['usersFullname'];
				}
				$title = $bibStyle->process($this->db->fetchRow($res->getResource($rId)), TRUE);
				$usersArray[$row['usersId']]['resources'][$rId] = $title;
			}
		}
		if(!empty($usersArray))
		{
			$smtp = FACTORY_SMTP::getInstance();
			$subject = $this->messages->text('statistics', 'emailSubject', $this->wikindxTitle);
			foreach($usersArray as $uArray)
			{
				$message = $uArray['name'] . "\n\n";
				$message .= $this->messages->text('statistics', 'emailIntro', $this->wikindxTitle) . "\n\n";
				foreach($uArray['resources'] as $rId => $title)
				{
					$viewsIndex = $this->accessRatio($rId);
					$downloadsIndex = $this->downloadRatio($rId);
					$popIndex = $this->getPopularityIndex($rId);
					$message .= $title . "\n";
					$message .= "\t" . $this->messages->text('statistics', 'emailViewsMonth', $resArray[$rId]['viewsPeriod']) . ' ' . 
						$this->messages->text('statistics', 'emailViewsTotal', $resArray[$rId]['viewsTotal']) . ', ' . 
						$this->messages->text('viewResource', 'viewIndex', $viewsIndex) .  ', ' . 
						$this->messages->text('viewResource', 'download', $downloadsIndex) .  ', ' . 
						$this->messages->text('viewResource', 'popIndex', $popIndex) . "\n";
					if(array_key_exists('downloads', $resArray[$rId]))
					{
						if(sizeof($resArray[$rId]['downloads']) == 1) // just the one attachment
						{
							$dArray = array_shift($resArray[$rId]['downloads']);
							$message .= "\t" . $this->messages->text('statistics', 'emailDownloadsMonth', $dArray[2]) . ' ' . 
								$this->messages->text('statistics', 'emailDownloadsTotal', $dArray[1]) . "\n";
						}
						else
						{
							foreach($resArray[$rId]['downloads'] as $dArray)
								$message .= "\t" . $dArray[0] . ' -- ' . 
									$this->messages->text('statistics', 'emailDownloadsMonth', $dArray[2]) . ' ' . 
									$this->messages->text('statistics', 'emailDownloadsTotal', $dArray[1]) . "\n";
						}
					}
					$message .= "\n";
				}
				$smtp->sendEmail($uArray['email'], $subject, $message);
			}
		}
	}
/**
* Calculate the maximum download ratio in the database
*
* @author Mark Grimshaw
* @return int
*/
	public function getMaxDownloadRatio()
	{
		if(self::$MaxDR !== FALSE)
			return self::$MaxDR;
		$dateDiffClause = $this->db->dateDiffRatio('resourceattachmentsDownloads', 'resourceattachmentsTimestamp', 'downloadRatio', TRUE);
		$this->db->groupBy('resourceattachmentsResourceId');
		$subQ = $this->db->subQuery($this->db->selectNoExecute('resource_attachments', $dateDiffClause, FALSE, FALSE), 
			't', TRUE, TRUE);
		self::$MaxDR = $this->db->fetchOne($this->db->selectMax(FALSE, 'downloadRatio', 'max', FALSE, $subQ));
		return self::$MaxDR;
	}
/**
* Calculate the maximum access ratio in the database
*
* @author Mark Grimshaw
* @return int
*/
	public function getMaxAccessRatio()
	{
		if(self::$MaxAR !== FALSE)
			return self::$MaxAR;
		$dateDiffClause = $this->db->dateDiffRatio('resourcemiscAccesses', 'resourcetimestampTimestampAdd', 'accessRatio');
		$this->db->leftJoin('resource_timestamp', 'resourcetimestampId', 'resourcemiscId');
		$subQ = $this->db->subQuery($this->db->selectNoExecute('resource_misc', $dateDiffClause, FALSE, FALSE), 
			't', TRUE, TRUE);
		self::$MaxAR = $this->db->fetchOne($this->db->selectMax(FALSE, 'accessRatio', 'max', FALSE, $subQ));
		return self::$MaxAR;
	}
/**
* return the popularity index for a resource
*
* @author Mark Grimshaw
* @param int $id Resource ID
* @return int
*/
	public function getPopularityIndex($id)
	{
		$accessRatio = $this->accessRatio($id);
		$downloadRatio = $this->downloadRatio($id);
		$ratio = (($downloadRatio * 1.5)  + ($accessRatio * 0.5)) / 2; // Give more weight to downloads
		$maxRatio = (($this->getMaxDownloadRatio() * 1.5) + ($this->getMaxAccessRatio() * 0.5)) / 2;
		if($ratio == 0)
			return 0;
		return $ratio;
	}
/**
* Return download ratio of attachments for each resource
*
* @author Mark Grimshaw
* @param int $id Resource ID
* @return int
*/
	public function downloadRatio($id)
	{
		if(!$this->list && (self::$DR !== FALSE))
			return self::$DR * 100;
		$dateDiffClause = $this->db->dateDiffRatio('resourceattachmentsDownloads', 'resourceattachmentsTimestamp', 'downloadRatio', TRUE);
		$this->db->formatConditions(array('resourceattachmentsResourceId' => $id));
		$this->db->groupBy('resourceattachmentsResourceId');
		$thisSum = $this->db->fetchOne($this->db->select('resource_attachments', $dateDiffClause, FALSE, FALSE));
		$setSum = $this->getMaxDownloadRatio();
		if($setSum == 0)
		{
			self::$DR = 0;
			return self::$DR;
		}
		self::$DR = round(($thisSum / $setSum), 2);
		return self::$DR * 100;
	}
/**
* Return views ratio for each resource
*
* @author Mark Grimshaw
* @param int $id Resource ID
* @return int
*/
	public function accessRatio($id)
	{
		if(!$this->list && (self::$AR !== FALSE))
			return self::$AR * 100;
		$dateDiffClause = $this->db->dateDiffRatio('resourcemiscAccesses', 'resourcetimestampTimestampAdd', 'accessRatio');
		$this->db->formatConditions(array('resourcemiscId' => $id));
		$this->db->leftJoin('resource_timestamp', 'resourcetimestampId', 'resourcemiscId');
		$thisSum = $this->db->fetchOne($this->db->select('resource_misc', $dateDiffClause, FALSE, FALSE));
		$setSum = $this->getMaxAccessRatio();
		if($setSum == 0)
		{
			self::$AR = 0;
			return self::$AR;
		}
		self::$AR = round(($thisSum / $setSum), 2);
		return self::$AR * 100;
	}
/**
* Return the no. attachment downloads for the resource with the greatest number of downloads
*
* @return int
*/
	private function getMaxDownloads()
	{
		if(self::$maxDownloads !== FALSE)
			return self::$maxDownloads;
		$this->db->groupBy('resourceattachmentsResourceId');
		$subQ = $this->db->subQuery($this->db->selectNoExecute('resource_attachments', 
			$this->db->sum('resourceattachmentsDownloads', 'sum'), FALSE, FALSE), 't', TRUE, TRUE);
		$resultset = $this->db->selectMax(FALSE, 'sum', 'max', FALSE, $subQ);
		self::$maxDownloads = $this->db->fetchOne($resultset);
		return self::$maxDownloads;
	}
/**
* Return the no. views for the resource with the greatest number of views
*
* @return int
*/
	private function getMaxAccesses()
	{
		if(self::$maxAccesses !== FALSE)
			return self::$maxAccesses;
		$resultset = $this->db->selectMax("resource_misc", 'resourcemiscAccesses');
		self::$maxAccesses = $this->db->fetchOne($resultset);
		return self::$maxAccesses;
	}
}
?>
