<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINCREATOR class.
*
*	Administration of creators
*****/
class ADMINCREATOR
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $creator;
private $html;
private $table;
private $gatekeep;
private $badInput;
private $newCreatorId;
private $newSurname;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->creator = FACTORY_CREATOR::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep->init();
		$this->session->clearArray('edit');
	}
// display options for creator merging
	public function mergeInit($message = FALSE)
	{
		$creators = $this->creator->grabAll();
		GLOBALS::setOutputHeading($this->messages->text("heading", "mergeCreators"));
		$pString = $message;
		if(is_array($creators) && !empty($creators))
		{
			$pString .= $this->html->p($this->messages->text("misc", "creatorMerge"));
			$pString .= $this->form->formHeader('admin_ADMINCREATOR_CORE');
			$pString .= $this->form->hidden("method", "mergeProcess");
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple(FALSE, "creatorIds", $creators, 20) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->tdStart() . $this->html->b($this->messages->text("misc", "creatorMergeTarget"));
			$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
			$pString .= $this->table->trStart();
// add 0 => IGNORE to creators array
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($creators as $key => $value)
				$temp[$key] = $value;
			$creators = $temp;
			unset($temp);
			$pString .= $this->table->td($this->form->selectFBoxValue(FALSE, "creatorIdsOutput", $creators, 20));
			$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "firstname"), 
				"firstname", FALSE, 20, 255));
			$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "initials"), 
				"initials", FALSE, 6, 255) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "initials"), 'hint'));
			$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "prefix"), 
				"prefix", FALSE, 11, 10));
			$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "surname"), 
				"surname", FALSE, 20, 255) . " " . $this->html->span('*', 'required'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->html->p($this->form->formSubmit("Proceed"));
			$pString .= $this->form->formEnd();
			GLOBALS::buildOutputString($pString);
		}
		else
			GLOBALS::buildOutputString($this->messages->text('misc', 'noCreators'));
	}
// start merging process
	public function mergeProcess()
	{
		$this->validateInput('merge');
		$creatorIds = $this->vars['creatorIds'];
		$this->newCreatorId = $this->insertCreator();
		$this->db->formatConditions(array('creatorId' => $this->newCreatorId));
		$this->newSurname = $this->db->fetchOne($this->db->select('creator', 'creatorSurname'));
		foreach($creatorIds as $oldId)
		{
// Remove old creators
			if($oldId != $this->newCreatorId)
			{
				$this->db->formatConditions(array('creatorId' => $oldId));
				$this->db->delete('creator');
			}
			$this->updateTableMerge($oldId);
		}
		$this->db->optimize('creator');
// remove cache files for creators
		$this->db->deleteCache('cacheResourceCreators');
		$this->db->deleteCache('cacheMetadataCreators');
		return $this->mergeInit($this->success->text("creatorMerge"));
	}
// Remove old creator references from resource_creator and add new creator reference.
	private function updateTableMerge($oldId)
	{
// Select all resources referencing this old creator and replace reference with existing creator -- check if main or not.
		$this->db->formatConditions(array('resourcecreatorCreatorId' => $oldId));
		$this->db->update('resource_creator', array('resourcecreatorCreatorId' => $this->newCreatorId));
// Next, update all rows where $oldId = creatorMain
		$this->db->formatConditions(array('resourcecreatorCreatorMain' => $oldId));
		$updateArray['resourcecreatorCreatorMain'] = $this->newCreatorId;
		$updateArray['resourcecreatorCreatorSurname'] = $this->newSurname;
		$this->db->update('resource_creator', $updateArray);
	}
// Insert new creator or return ID if already exists
	function insertCreator()
	{
		if($this->vars['creatorIdsOutput'])
			return $this->vars['creatorIdsOutput'];
		return $this->creator->insert(array('surname' => $this->vars['surname'], 'initials' => $this->vars['initials'], 
			'firstname' => $this->vars['firstname'], 'prefix' => $this->vars['prefix']));
	}
// display options for creator grouping
	public function groupInit($message = FALSE)
	{
		$ajax = FACTORY_AJAX::getInstance();
		$ajax->loadJavascript(array('core/modules/list/searchSelect.js'));
		$potentialMasters = $this->creator->grabGroupAvailableMasters();
		GLOBALS::setOutputHeading($this->messages->text("heading", "groupCreators"));
		$pString = $message;
		if(is_array($potentialMasters) && !empty($potentialMasters))
		{
			$potentialMembers = $this->creator->grabGroupAvailableMembers();
			if(!is_array($potentialMembers))
				$potentialMembers = $this->creator->grabGroupAvailableMembers(TRUE);
			foreach($potentialMasters as $id => $name) // array_shift() breaks ids!
				break;
			reset($potentialMasters);
			$existingMembers = $this->creator->grabGroupMembers($id);
// add 0 => IGNORE to potentialMembers array
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($potentialMembers as $key => $value)
				$temp[$key] = $value;
			$potentialMembers = $temp;
			unset($temp);
			$pString .= $this->html->p($this->messages->text("misc", "creatorGroup"));
			$pString .= $this->form->formHeader('admin_ADMINCREATOR_CORE');
			$pString .= $this->form->hidden("method", "groupProcess");
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
			$jScript = 'index.php?action=admin_ADMINCREATOR_CORE&method=groupDiv';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromMultiSelect', 
				'script' => "$jScript",
				'triggerField' => 'creatorMaster',
				'targetDiv' => 'creatorIds',
				);
			$ajax->jActionForm('onclick', $jsonArray);
			$pString .= $this->table->td($this->form->selectFBoxValue($this->html->b($this->messages->text("misc", "creatorGroupMaster")), 
				"creatorMaster", $potentialMasters, 20));
			if(!is_array($existingMembers))
				$td = $this->html->div('creatorIds', $this->form->selectFBoxValueMultiple('&nbsp;', "creatorIds", $potentialMembers, 20) . 
					$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
					$this->html->p($this->form->formSubmit("Proceed")));
			else
			{
				$existingMembers = array_keys($existingMembers);
				$td = $this->html->div('creatorIds', $this->form->selectedBoxValueMultiple('&nbsp;', "creatorIds", $potentialMembers, 
					$existingMembers, 20) . 
					$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
					$this->html->p($this->form->formSubmit("Proceed")));
			}
			$pString .= $this->table->td($td);
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->form->formEnd();
			GLOBALS::buildOutputString($pString);
		}
		else
			GLOBALS::buildOutputString($this->messages->text('misc', 'noCreators'));
	}
// AJAX driven select box for creator group members
	public function groupDiv()
	{
		$ajax = FACTORY_AJAX::getInstance();
		$potentialMembers = $this->creator->grabGroupAvailableMembers();
		if(!is_array($potentialMembers))
			$potentialMembers = $this->creator->grabGroupAvailableMembers(TRUE);
		$existingMembers = $this->creator->grabGroupMembers($this->vars['ajaxReturn']);
// add 0 => IGNORE to potentialMembers array
		$temp[0] = $this->messages->text("misc", "ignore");
		foreach($potentialMembers as $key => $value)
			$temp[$key] = $value;
		$potentialMembers = $temp;
		unset($temp);
		if(!is_array($existingMembers))
		{
			unset($potentialMembers[$this->vars['ajaxReturn']]);
			$div = $this->html->div('creatorIdsOutput', $this->form->selectFBoxValueMultiple('&nbsp;', 
				"creatorIds", $potentialMembers, 20) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($this->form->formSubmit("Proceed")));
		}
		else
		{
			unset($potentialMembers[$this->vars['ajaxReturn']]);
			$existingMembers = array_keys($existingMembers);
			$div = $this->html->div('creatorIdsOutput', $this->form->selectedBoxValueMultiple('&nbsp;', 
				"creatorIds", $potentialMembers, $existingMembers, 20) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($this->form->formSubmit("Proceed")));
		}
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
// start grouping process
	public function groupProcess()
	{
		$this->validateInput('group');
		$creatorIds = $this->vars['creatorIds'];
		$targetCreatorId = $this->vars['creatorMaster'];
		if(($index = array_search($targetCreatorId, $creatorIds)) !== FALSE)
			unset($creatorIds[$index]);
// First, remove references to this creator as group master
		$this->db->formatConditions(array('creatorSameAs' => $targetCreatorId));
		$this->db->updateNull('creator', 'creatorSameAs');
		$this->db->formatConditionsOneField($creatorIds, 'creatorId');
		$this->db->update('creator', array('creatorSameAs' => $targetCreatorId));
		return $this->groupInit($this->success->text("creatorGroup"));
	}
// display options for creator ungrouping
	public function ungroupInit($message = FALSE)
	{
		$ajax = FACTORY_AJAX::getInstance();
		$ajax->loadJavascript(array('core/modules/list/searchSelect.js'));
		$mastersCopy = $masters = $this->creator->grabGroupMasters();
		GLOBALS::setOutputHeading($this->messages->text("heading", "ungroupCreators"));
		$pString = $message;
		if(is_array($masters) && !empty($masters))
		{
			foreach($mastersCopy as $id => $name)
			{
				$initialMasterId = $id;
				break;
			}
			$creators = $this->creator->grabGroupMembers($initialMasterId);
			$pString .= $this->form->formHeader('admin_ADMINCREATOR_CORE');
			$pString .= $this->form->hidden("method", "ungroupProcess");
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
			$jScript = 'index.php?action=admin_ADMINCREATOR_CORE&method=ungroupDiv';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromMultiSelect', 
				'script' => "$jScript",
				'triggerField' => 'creatorMaster',
				'targetDiv' => 'creatorIds',
				);
			$ajax->jActionForm('onclick', $jsonArray);
			$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text("misc", "creatorGroupMaster"), 
				"creatorMaster", $masters, 20));
			$td = $this->html->div('creatorIds', $this->form->selectFBoxValueMultiple($this->messages->text("misc", "creatorUngroup"), 
				"creatorIds", $creators, 20) . $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->p($this->form->formSubmit("Remove")));
			$pString .= $this->table->td($td);
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->form->formEnd();
			GLOBALS::buildOutputString($pString);
		}
		else
			GLOBALS::buildOutputString($this->messages->text('misc', 'noGroupMasterCreators'));
	}
// AJAX driven select box for creator group members
	public function ungroupDiv()
	{
		$ajax = FACTORY_AJAX::getInstance();
		$creators = $this->creator->grabGroupMembers($this->vars['ajaxReturn']);
		$div = $this->html->div('creatorIds', $this->form->selectFBoxValueMultiple($this->messages->text("misc", "creatorUngroup"), 
			"creatorIds", $creators, 20) . $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
			$this->html->p($this->form->formSubmit("Proceed")));
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
	public function ungroupProcess()
	{
		$this->validateInput('ungroupProcess');
		$creatorIds = $this->vars['creatorIds'];
		foreach($creatorIds as $oldId)
		{
			$this->db->formatConditionsOneField($creatorIds, 'creatorId');
			$this->db->updateNull('creator', 'creatorSameAs');
		}
		return $this->ungroupInit($this->success->text("creatorGroup"));
	}
// validate input
	private function validateInput($process)
	{
		if($process == 'merge')
		{
			if(!array_key_exists("creatorIds", $this->vars) || !$this->vars['creatorIds'])
				$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'mergeInit');
			if(!array_key_exists("creatorIdsOutput", $this->vars) || !$this->vars['creatorIdsOutput'])
			{
				if(!array_key_exists("surname", $this->vars) || !trim($this->vars['surname']))
					$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'mergeInit');
			}
			else if((!array_key_exists("surname", $this->vars) || !trim($this->vars['surname'])) && 
				(sizeof($this->vars['creatorIds']) == 1) && $this->vars['creatorIds'][0] == $this->vars['creatorIdsOutput'])
				$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'mergeInit');
		}
		else if($process == 'group')
		{
			if(!array_key_exists("creatorIds", $this->vars) || !$this->vars['creatorIds'])
				$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'groupInit');
			if(!array_key_exists("creatorMaster", $this->vars))
				$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'groupInit');
			if((sizeof($this->vars['creatorIds']) == 1) && $this->vars['creatorIds'][0] == $this->vars['creatorMaster'])
				$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'groupInit');
		}
		else if($process == 'ungroupSelect')
		{
			if(!array_key_exists("creatorIds", $this->vars) || !$this->vars['creatorIds'])
				$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'ungroupSelect');
		}
	}
}
?>
