<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINCUSTOM class.
*
*	Administration of custom fields
*****/
class ADMINCUSTOM
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $form;
private $html;
private $gatekeep;
private $badInput;
private $ajax;
private $table;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep->init();
		$this->session->clearArray('custom');
	}
// display options
	public function init($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminCustom"));
		$pString = $message ? $this->html->p($message, "error", "center") : FALSE;
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
// Add
		$td = $this->form->formHeader("admin_ADMINCUSTOM_CORE");
		$td .= $this->form->hidden("method", "addCustom");
		$array = array("small" => $this->messages->text("custom", "small"), 
			"large" => $this->messages->text("custom", "large"));
		if(!$size = $this->session->getVar("custom_size"))
			$size = "small";
		$td .= $this->form->selectedBoxValue($this->messages->text("custom", "size"), 
			"custom_size", $array, $size, 2) . " " . $this->html->span('*', 'required');
		$td .= $this->html->p($this->form->textInput($this->messages->text("custom", "addLabel"), 
			"custom_label", FALSE, 50, 255) . " " . $this->html->span('*', 'required'));
		$td .= $this->html->p($this->form->formSubmit('Add'));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		$recordset = $this->db->select('custom', array('customId', 'customLabel'));
		while($row = $this->db->fetchRow($recordset))
			$customs[$row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
		if(isset($customs))
		{
// Edit
// If preferences reduce long custom labels, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($customs as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$fields[$key] = $value;
			}
			$td = $this->form->formHeader("admin_ADMINCUSTOM_CORE");
			$td .= $this->form->hidden("method", "editCustom");
			$td .= $this->form->selectFBoxValue($this->messages->text("custom", "editLabel"), 
				'customId', $fields, 10);
			$td .= $this->html->p($this->transferArrow('transferCustom'));
			$td .= $this->html->p($this->form->textInput(FALSE, "customEdit", FALSE, 30, 255));
			$td .= $this->form->hidden('customEditId', FALSE);
			$td .= $this->html->p($this->form->formSubmit('Edit'));
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
// Delete
			$td = $this->form->formHeader("admin_ADMINCUSTOM_CORE");
			$td .= $this->form->hidden("method", "deleteConfirm");
			$td .= $this->form->selectFBoxValueMultiple($this->messages->text("custom", "deleteLabel"), 
				'customIds', $customs, 10) . $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
			$td .= $this->html->p($this->form->formSubmit('Delete'));
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$this->ajax->loadJavascript(array('core/modules/admin/customEdit.js'));
		GLOBALS::buildOutputString($pString);
	}
	private function transferArrow($function)
	{
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => $function,
			);
		$image = $this->ajax->jActionIcon('toBottom', 'onclick', $jsonArray);
		return $image;
	}
// Create the new field
	public function addCustom()
	{
		if(!$this->validateInput('add'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$fields = array("customLabel", "customSize");
		$values[] = trim($this->label);
		if($this->size == 'large')
			$values[] = 'L';
		else
			$values[] = 'S';
		$this->db->insert('custom', $fields, $values);
		$pString = $this->success->text("field", " " . $this->messages->text("misc", "added") . " ");
		return $this->init($pString);
	}
// Edit field
	public function editCustom()
	{
		if(!$this->validateInput('edit'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$this->db->formatConditions(array('customId' => $this->vars['customEditId']));
		$this->db->update('custom', array("customLabel" => $this->label));
		$pString = $this->success->text("field", " " . $this->messages->text("misc", "edited") . " ");
		return $this->init($pString);
	}
// Ask for confirmation of delete field(s)
	public function deleteConfirm()
	{
		if(!$this->validateInput('delete'))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminCustom"));
		$pString = $this->html->p($this->html->b($this->messages->text("custom", "warning")));
		$pString .= $this->form->formHeader('admin_ADMINCUSTOM_CORE');
		$pString .= $this->form->hidden('method', 'deleteCustom');
		foreach($this->vars['customIds'] as $id)
		{
			$pString .= $this->form->hidden("delete_" . $id, $id);
			$ids[] = $this->db->tidyInput($id);
		}
		$this->db->formatConditions($this->db->formatFields('customId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('customId') . $this->db->equal, $ids));
		$recordset = $this->db->select('custom', 'customLabel');
		while($row = $this->db->fetchRow($recordset))
			$fieldValues[] = "'" . $this->html->dbToHtmlTidy($row['customLabel']) . "'";
		$pString .= $this->html->p($this->messages->text("custom", "deleteConfirm", ": " . join(", ", $fieldValues)));
		$pString .= $this->html->br() . $this->form->formSubmit('Confirm');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete field(s)
	public function deleteCustom()
	{
		if(!$ids = $this->validateInput('deleteConfirm'))
			$this->badInput($this->errors->text("inputError", "invalid"), 'init');
// $ids is an array of field IDs
		$this->db->formatConditions($this->db->formatFields('customId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('customId') . $this->db->equal, $ids));
		$this->db->delete('custom');
		$this->db->formatConditions($this->db->formatFields('resourcecustomId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('resourcecustomId') . $this->db->equal, $ids));
		$this->db->delete('resource_custom');
		$this->db->optimize('custom');
		$this->db->optimize('resource_custom');
		$pString = $this->success->text("field", " " . $this->messages->text("misc", "deleted") . " ");
		return $this->init($pString);
	}
// validate input
	private function validateInput($type)
	{
		if($type == 'add')
		{
// Write to session
			$this->size = isset($this->vars['custom_size']) ? $this->vars['custom_size'] : FALSE;
			$this->label = isset($this->vars['custom_label']) ? trim($this->vars['custom_label']) : FALSE;
			$this->session->setVar("custom_size", $this->size);
			$this->session->setVar("custom_label", $this->label);
			if(!$this->size)
				return FALSE;
			if(!$this->label)
				return FALSE;
		}
		else if($type == 'edit')
		{
			$this->label = isset($this->vars['customEdit']) ? trim($this->vars['customEdit']) : FALSE;
			if(!array_key_exists('customEditId', $this->vars) || !$this->label)
				return FALSE;
		}
		else if($type == 'delete')
		{
			if(!array_key_exists('customIds', $this->vars) || empty($this->vars['customIds']))
				return FALSE;
		}
		else if($type == 'deleteConfirm')
		{
			$ids = array();
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/delete_/", $key))
					continue;
				$ids[] = $value;
			}
			if(empty($ids))
				return FALSE;
			return $ids;
		}
		return TRUE;
	}
}
?>
