<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINLANGUAGES class.
*
*	ADMINLANGUAGES of resource languages
*****/
class ADMINLANGUAGES
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $keyword;
private $html;
private $gatekeep;
private $badInput;
private $languages;
private $admin;
private $table;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		$this->session->clearArray('edit');
		$this->languages = $this->grabAll();
	}
// display options
	public function init($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminLanguage"));
		$pString = $message ? $this->html->p($message, "error", "center") : FALSE;
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
// Add
		$td = $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
		$td .= $this->form->hidden("method", "addLanguage");
		$td .= $this->form->textInput($this->messages->text("misc", "languageAdd"), "languageAdd", FALSE, 30, 255);
		$td .= $this->html->p($this->form->formSubmit('Add'));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		if(!empty($this->languages))
		{
// Edit
// If preferences reduce long language titles, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($this->languages as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$languages[$key] = $value;
			}
			$td = $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
			$td .= $this->form->hidden("method", "editLanguage");
			$td .= $this->form->selectFBoxValue($this->messages->text("misc", "languageEdit"), 
				'languageId', $languages, 10);
			$td .= $this->html->p($this->transferArrow('transferLanguage'));
			$td .= $this->html->p($this->form->textInput(FALSE, "languageEdit", FALSE, 30, 255));
			$td .= $this->form->hidden('languageEditId', FALSE);
			$td .= $this->html->p($this->form->formSubmit('Edit'));
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
// Delete
			$td = $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
			$td .= $this->form->hidden("method", "deleteConfirm");
			$td .= $this->form->selectFBoxValueMultiple($this->messages->text("misc", "languageDelete"), 
				'languageIds', $this->languages, 10) . $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
			$td .= $this->html->p($this->form->formSubmit('Delete'));
			$td .= $this->form->formEnd();
			$pString .= $this->table->td($td);
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$this->ajax->loadJavascript(array('core/modules/admin/languageEdit.js'));
		GLOBALS::buildOutputString($pString);
	}
	private function transferArrow($function)
	{
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => $function,
			);
		$image = $this->ajax->jActionIcon('toBottom', 'onclick', $jsonArray);
		return $image;
	}
// Add a language
	public function addLanguage()
	{
		if(!$input = $this->validateInput('add'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		$utf8 = FACTORY_UTF8::getInstance();
		$pString = $this->success->text("languages", " " . $this->messages->text("misc", "added") . " ");
// If language already exists quietly return without error.
		foreach($this->languages as $language)
		{
			if($utf8->utf8_strtolower($input) == $utf8->utf8_strtolower($language))
				return $this->addInit($pString);
		}
		$this->db->insert('language', array('languageLanguage'), array($input));
		$this->languages = $this->grabAll();
		return $this->init($pString);
	}
// Delete languages display.
	public function deleteInit($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "delete2", " (" . 
			$this->messages->text("resources", "languages") . ")"));
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		if(empty($this->languages))
		{
			GLOBALS::buildOutputString($pString . $this->messages->text("misc", "noLanguages"));
			return;
		}
		$pString .= $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
		$pString .= $this->form->hidden("method", "deleteConfirm");
		foreach($this->languages as $key => $value)
			$pString .= $this->form->checkbox(FALSE, "languageDelete_" . $key) . " $value" . $this->html->br();
		$pString .= $this->html->br() . $this->form->formSubmit('Delete');
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Ask for confirmation of delete languages
	public function deleteConfirm()
	{
		if(!$this->validateInput('delete'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminLanguage"));
		$pString = $this->form->formHeader("admin_ADMINLANGUAGES_CORE");
		$pString .= $this->form->hidden("method", "deleteLanguage");
		foreach($this->vars['languageIds'] as $id)
		{
			$pString .= $this->form->hidden("delete_" . $id, $id);
			$ids[] = $this->db->tidyInput($id);
		}
		$this->db->formatConditions($this->db->formatFields('languageId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('languageId') . $this->db->equal, $ids));
		$recordset = $this->db->select('language', 'languageLanguage');
		while($row = $this->db->fetchRow($recordset))
			$fieldValues[] = "'" . $this->html->dbToHtmlTidy($row['languageLanguage']) . "'";
		$pString .= $this->html->p($this->messages->text("misc", "confirmDeleteLanguage") . ": " . join(", ", $fieldValues));
		$pString .= $this->html->p($this->form->formSubmit('Confirm'));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Delete languages
	public function deleteLanguage()
	{
		$input = $this->validateInput('deleteConfirm');
		if(empty($input))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		foreach($input as $id)
		{
			if(array_key_exists($id, $this->languages))
			{
				$this->db->formatConditions(array('resourcelanguageLanguageId' => $id));
				$this->db->delete('resource_language');
				$this->db->formatConditions(array('languageId' => $id));
				$this->db->delete('language');
			}
		}
		$pString = $this->success->text("languages", " " . $this->messages->text("misc", "deleted") . " ");
		$this->languages = $this->grabAll();
		return $this->init($pString);
	}
// Edit languages
	public function editLanguage()
	{
		if(!$this->validateInput('edit'))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		$this->db->formatConditions(array('languageId' => $this->vars['languageEditId']));
		$this->db->update('language', array('languageLanguage' => trim($this->vars['languageEdit'])));
		$this->languages = $this->grabAll();
		$pString = $this->success->text("languages", " " . $this->messages->text("misc", "edited") . " ");
		return $this->init($pString);
	}
// Grab any languages from config table
	private function grabAll()
	{
		$array = array();
		$this->db->orderBy('languageLanguage');
		$resultset = $this->db->select('language', array('languageId', 'languageLanguage'));
		while($row = $this->db->fetchRow($resultset))
			$array[$row['languageId']] = $row['languageLanguage'];
		return $array;
	}
// validate input
	private function validateInput($type)
	{
		if($type == 'add')
		{
			if(!$input = trim($this->vars['languageAdd']))
				return FALSE;
		}
		else if($type == 'delete')
		{
			if(!array_key_exists('languageIds', $this->vars) || empty($this->vars['languageIds']))
				return FALSE;
			return TRUE;
		}
		else if($type == 'deleteConfirm')
		{
			$input = array();
			foreach($this->vars as $key => $value)
			{
				if(!preg_match("/delete_(.*)/", $key, $match))
					continue;
				$input[] = $match[1];
			}
		}
		else if($type == 'edit')
		{
			if(!array_key_exists('languageEdit', $this->vars) || !trim($this->vars['languageEdit']) || 
				!array_key_exists('languageEditId', $this->vars))
				return FALSE;
			return TRUE;
		}
		return $input;
	}
}
?>