<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	ADMINPLUGINS class.
*
*	Administration of plugin modules including bibliography styles, languages and templates.
*****/
class ADMINPLUGINS
{
private $db;
private $vars;
private $session;
private $errors;
private $messages;
private $success;
private $form;
private $html;
private $table;
private $gatekeep;
private $ajax;
private $possibleMenus;
private $possibleContainers;
private $configFile;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		if(!array_key_exists('method', $this->vars) || 
			(($this->vars['method'] != 'initConfigMenu') && ($this->vars['method'] != 'initConfigInline')))
			$this->ajax->loadJavascript('core/modules/admin/adminPlugins.js');
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		$this->possibleMenus = array(
			'wikindx' => 'Wikindx', 
			'res' => $this->messages->text('menu', 'res'), 
			'search' => $this->messages->text('menu', 'search'), 
			'text' => $this->messages->text('menu', 'text'), 
			'admin' => $this->messages->text('menu', 'admin'), 
			'plugin1' => $this->messages->text('menu', 'plugin1'), 
			'plugin2' => $this->messages->text('menu', 'plugin2'), 
			'plugin3' => $this->messages->text('menu', 'plugin3')
		);
		$this->possibleContainers = array('inline1' => 'inline1', 'inline2' => 'inline2', 
			'inline3' => 'inline3', 'inline4' => 'inline4');
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('plugins'));
		GLOBALS::setOutputHeading($this->messages->text("heading", "adminPlugins"));
	}
// Display options.
	public function init($message = FALSE)
	{
		$pString = '';
		if($message)
			$pString .= $this->html->p($message, "error", "center");
		$pString .= $this->displayEnableDisable();
		$pString .= $this->html->hr();
		$pString .= $this->displayConfig();
		GLOBALS::buildOutputString($pString);
	}
// Display the config settings for the plugin
	private function displayConfig()
	{
		list($enabledMenu, $enabledInline) = $this->configurablePlugins();
		if(empty($enabledMenu) && empty($enabledInline))
			return FALSE;
		$pString = '';
		if(!empty($enabledMenu))
		{
			$pString .= $this->form->formHeader("admin_ADMINPLUGINS_CORE");
			$pString .= $this->form->hidden("method", "writeConfigMenu");
			$jsonArray = array();
			$jScript = 'index.php?action=admin_ADMINPLUGINS_CORE&method=initConfigMenu';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromMultiSelect', 
				'script' => "$jScript",
				'triggerField' => 'configFileMenu',
				'targetDiv' => 'divMenu',
				);
			$this->ajax->jActionForm('onclick', $jsonArray);
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('misc', 
				'pluginsConfigureMenu'), 'configFileMenu', $enabledMenu, 5));
			$pString .= $this->table->td($this->html->div('divMenu', $this->getConfigDetailsMenu($enabledMenu))); 
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->html->p($this->form->formSubmit('Proceed'));
			$pString .= $this->form->formEnd();
		}
		if(!empty($enabledInline))
		{
			if(!empty($enabledMenu))
				$pString .= $this->html->hr();
			$pString .= $this->form->formHeader("admin_ADMINPLUGINS_CORE");
			$pString .= $this->form->hidden("method", "writeConfigInline");
			$jsonArray = array();
			$jScript = 'index.php?action=admin_ADMINPLUGINS_CORE&method=initConfigInline';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromMultiSelect', 
				'script' => "$jScript",
				'triggerField' => 'configFileInline',
				'targetDiv' => 'divInline',
				);
			$this->ajax->jActionForm('onclick', $jsonArray);
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValue($this->messages->text('misc', 
				'pluginsConfigureInline'), 'configFileInline', $enabledInline, 5));
			$pString .= $this->table->td($this->html->div('divInline', $this->getConfigDetailsInline($enabledInline))); 
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->html->p($this->form->formSubmit('Proceed'));
			$pString .= $this->form->formEnd();
		}
		return $pString;
	}
/**
* return list of plugins that are configurable.
*/
	private function configurablePlugins()
	{
		$enabledMenu = $enabledInline = array();
		$handle = opendir('plugins');
		while(FALSE !== ($dir = readdir($handle)))
		{
			if($dir == '..')
				continue;
			if(is_dir('plugins/' . $dir) && file_exists('plugins/' . $dir . '/index.php') && 
				file_exists('plugins/' . $dir . '/config.php') && 
				file_exists('plugins/' . $dir . '/plugintype.txt'))
			{
				$fh = fopen('plugins/' . $dir . '/plugintype.txt', "r");
				$type = fgets($fh);
				fclose($fh);
				if(file_exists('plugins/' . $dir . '/description.txt') && 
					($fh = fopen('plugins/' . $dir . '/description.txt', "r")))
				{
					$string = fgets($fh);
					if($type == 'menu')
						$enabledMenu[$dir] = $string;
					else if($type == 'inline')
						$enabledInline[$dir] = $string;
					fclose($fh);
				}
				else
				{
					if($type == 'menu')
						$enabledMenu[$dir] = $dir;
					else if($type == 'inline')
						$enabledInline[$dir] = $dir;
				}
			}
		}
		return array($enabledMenu, $enabledInline);
	}
/**
* Get config details for menu plugins and put into form elements
*/
	private function getConfigDetailsMenu($enabled)
	{
		if(array_key_exists('ajaxReturn', $this->vars))
			$file = $this->vars['ajaxReturn'];
		else // grab the first of the list
		{
			foreach($enabled as $file => $null)
				break;		
		}
		$pString = $this->table->tableStart('generalTable', 1);
		$pString .= $this->table->trStart();
		if(file_exists('plugins/' . $file . '/README.txt') || file_exists('plugins/' . $file . '/README'))
		{
			$this->html->insertJscriptAction = "onClick=\"coreOpenPopup('index.php?action=admin_ADMINPLUGINS_CORE&method=readMe&file=$file'); 
				return false\"";
			$pString .= $this->table->td($this->html->div("divReadMeMenu", $this->html->aBrowse('green', '1em', 
				$this->messages->text('misc', 'openReadme'), '#')));
		}
		else
			$pString .= $this->table->td($this->html->div("divReadMeMenu", $this->messages->text('misc', 'noReadme')));
		if($fh = fopen('plugins/' . $file . '/config.php', "r"))
		{
			$string = '';
			while(!feof($fh))
				$string .= fgets($fh);
			fclose($fh);
			$pString .= $this->table->td($this->form->textareaInput($this->messages->text('misc', 
				'pluginConfig'), 'configConfig', $string, 100, 8));
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		return $pString;
	}
/**
* Display README file
*/
	public function readMe()
	{
		$fh = FALSE;
		if(file_exists('plugins/' . $this->vars['file'] . '/README.txt'))
			$fh = fopen('plugins/' . $this->vars['file'] . '/README.txt', "r");
		if(!$fh && file_exists('plugins/' . $this->vars['file'] . '/README'))
			$fh = fopen('plugins/' . $this->vars['file'] . '/README', "r");
		if(!$fh)
			$pString = $this->errors->text('file', 'read');
		else
		{
			$pString = '';
			while(!feof($fh))
				$pString .= fgets($fh) . $this->html->br();
			fclose($fh);
		}
		GLOBALS::buildOutputString($pString);
		FACTORY_CLOSEPOPUP::getInstance();
	}
/**
* Get config details for inline plugins and put into form elements
*/
	private function getConfigDetailsInline($enabled)
	{
		$arrayIndex = 0;
		$usedContainers = $conflictContainers = array();
		foreach($enabled as $file => $null)
		{
			include_once("plugins/$file/config.php");
			$configClass = $file . '_CONFIG';
			if(!$arrayIndex)
			{
				$firstConfigClass = $configClass;
				$firstFile = $null;
				$arrayIndex++;
			}
			$config = new $configClass();
			if(($index = array_search($config->container, $usedContainers)) === FALSE)
				$usedContainers[$file] = $config->container;
			else
			{
				$conflictContainers[$index] = $usedContainers[$index];
				$conflictContainers[$file] = $config->container;
			}
		}
		if(array_key_exists('ajaxReturn', $this->vars))
		{
			$file = $this->vars['ajaxReturn'];
			include_once("plugins/$file/config.php");
			$configClass = $file . '_CONFIG';
		}
		else // grab the first of the list
		{
			$configClass = $firstConfigClass;
			$file = $firstFile;
		}
		$config = new $configClass();
		$pString = $this->table->tableStart('generalTable', 1);
		$pString .= $this->table->trStart();
		if(file_exists('plugins/' . $file . '/README.txt') || file_exists('plugins/' . $file . '/README'))
		{
			$this->html->insertJscriptAction = "onClick=\"coreOpenPopup('index.php?action=admin_ADMINPLUGINS_CORE&method=readMe&file=$file'); 
				return false\"";
			$pString .= $this->table->td($this->html->div("divReadMeInline", $this->html->aBrowse('green', '1em', 
				$this->messages->text('misc', 'openReadme'), '#')));
		}
		else
			$pString .= $this->table->td($this->html->div("divReadMeInline", $this->messages->text('misc', 'noReadme')));
		$pString .= $this->table->td($this->form->selectedBoxValue($this->messages->text('misc', 
			'pluginsContainers'), 'configContainers', $this->possibleContainers, $config->container, 4));
		if(!empty($conflictContainers))
		{
			foreach($conflictContainers as $plugin => $container)
				$array[] = $plugin . ':&nbsp;' . $container;
			$pString .= $this->table->td($this->errors->text('warning', 'pluginConflict', join(', ', $array)));
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		return $pString;
	}
/**
* AJAX-based DIV content creator for configuration of plugin menus
*/
	public function initConfigMenu()
	{
		list($enabledMenu, $enabledInline) = $this->configurablePlugins();
		$div = $this->html->div('divMenu', $this->getConfigDetailsMenu($enabledMenu));
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for configuration of inline plugins
*/
	public function initConfigInline()
	{
		list($enabledMenu, $enabledInline) = $this->configurablePlugins();
		$div = $this->html->div('divInline', $this->getConfigDetailsInline($enabledInline));
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
// Display enable and disable select boxes for plugins etc.
	private function displayEnableDisable()
	{
		$pString = $this->form->formHeader("admin_ADMINPLUGINS_CORE", "onsubmit=\"selectAll();return true;\"");
		$pString .= $this->form->hidden("method", "enableDisable");
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left", "100%");
		$pString .= $this->table->trStart();
// plugins
		list($enabled, $disabled) = $this->listPlugins();
		if(empty($enabled) && empty($disabled))
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'pluginsEnabled'), 'pluginsEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->td('&nbsp;');
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'pluginsDisabled'), 'pluginsDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
		else
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'pluginsEnabled'), 'pluginsEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'disablePlugin', 
				);
			$toRightImage = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'enablePlugin', 
				);
			$toLeftImage = $this->ajax->jActionIcon('toLeft', 'onclick', $jsonArray);
			$pString .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage));
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'pluginsDisabled'), 'pluginsDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
// styles
		list($enabled, $disabled) = $this->listStyles();
		if((sizeof($enabled) + sizeof($disabled)) < 2)
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'stylesEnabled'), 'stylesEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->td('&nbsp;');
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'stylesDisabled'), 'stylesDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
		else
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'stylesEnabled'), 'stylesEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'disableStyle', 
				);
			$toRightImage = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'enableStyle', 
				);
			$toLeftImage = $this->ajax->jActionIcon('toLeft', 'onclick', $jsonArray);
			$pString .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage));
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'stylesDisabled'), 'stylesDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
// templates
		list($enabled, $disabled) = $this->listTemplates();
		if((sizeof($enabled) + sizeof($disabled)) < 2)
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'templatesEnabled'), 'templatesEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->td('&nbsp;');
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'templatesDisabled'), 'templatesDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
		else
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'templatesEnabled'), 'templatesEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'disableTemplate', 
				);
			$toRightImage = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'enableTemplate', 
				);
			$toLeftImage = $this->ajax->jActionIcon('toLeft', 'onclick', $jsonArray);
			$pString .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage));
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'templatesDisabled'), 'templatesDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
// languages
		list($enabled, $disabled) = $this->listLanguages();
		if((sizeof($enabled) + sizeof($disabled)) < 2)
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'languagesEnabled'), 'languagesEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->td('&nbsp;');
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'languagesDisabled'), 'languagesDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
		else
		{
			$pString .= $this->table->tdStart();
			$pString .= $this->table->tableStart('generalTable', 1);
			$pString .= $this->table->trStart();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'languagesEnabled'), 'languagesEnabled', $enabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'disableLanguage', 
				);
			$toRightImage = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'enableLanguage', 
				);
			$toLeftImage = $this->ajax->jActionIcon('toLeft', 'onclick', $jsonArray);
			$pString .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage));
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('misc', 
				'languagesDisabled'), 'languagesDisabled', $disabled, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->table->tdEnd();
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit('Proceed'));
		$pString .= $this->form->formEnd();
		return $pString;
	}
// List plugins
	private function listPlugins()
	{
		$enabled = $disabled = array();
		$handle = opendir('plugins');
		while(FALSE !== ($dir = readdir($handle)))
		{
			if($dir == '..')
				continue;
			if(is_dir('plugins/' . $dir) && file_exists('plugins/' . $dir . '/index.php') 
				&& file_exists('plugins/' . $dir . '/description.txt'))
			{
				if($fh = fopen('plugins/' . $dir . '/description.txt', "r"))
				{
					$string = fgets($fh);
					$enabled[$dir] = $string;
					fclose($fh);
				}
			}
			else if(is_dir('plugins/' . $dir) && file_exists('plugins/' . $dir . '/index.php'))
				$enabled[$dir] = $dir;
			else if(is_dir('plugins/' . $dir) && file_exists('plugins/' . $dir . '/index.php~') 
				&& file_exists('plugins/' . $dir . '/description.txt'))
			{
				if($fh = fopen('plugins/' . $dir . '/description.txt', "r"))
				{
					$string = fgets($fh);
					$disabled[$dir] = $string;
					fclose($fh);
				}
			}
			else if(is_dir('plugins/' . $dir) && file_exists('plugins/' . $dir . '/index.php~'))
				$disabled[$dir] = $dir;
		}
		return array($enabled, $disabled);
	}
// List styles
	private function listStyles()
	{
		$enabled = $disabled = array();
		$stylesDir = 'styles/bibliography';
		$handle = opendir($stylesDir);
		while(FALSE !== ($dir = readdir($handle)))
		{
			if($dir == '..')
				continue;
			if(is_dir($stylesDir . '/' . $dir) && file_exists($stylesDir . '/' . $dir . '/' . strtoupper($dir) . ".xml"))
			{
				$filePath = $stylesDir . '/' . $dir . "/" . strtoupper($dir) . ".xml";
				if($fh = fopen($filePath, "r"))
				{
					while(!feof($fh))
					{ 
					  $line = stream_get_line($fh, 1000000, "\n");
					  if(preg_match("/<description>(.*)<\\/description>/Ui", $line, $matches))
					  {
						$array[strtoupper($dir)] = $matches[1];
						$enabled[$dir] = $matches[1];
						break;
					  }
					}
					fclose($fh);
				}
			}
			else if(is_dir($stylesDir . '/' . $dir) && file_exists($stylesDir . '/' . $dir . '/' . strtoupper($dir) . ".xml~"))
			{
				$filePath = $stylesDir . '/' . $dir . "/" . strtoupper($dir) . ".xml~";
				if($fh = fopen($filePath, "r"))
				{
					while(!feof($fh))
					{ 
					  $line = stream_get_line($fh, 1000000, "\n");
					  if(preg_match("/<description>(.*)<\\/description>/Ui", $line, $matches))
					  {
						$array[strtoupper($dir)] = $matches[1];
						$disabled[$dir] = $matches[1];
						break;
					  }
					}
					fclose($fh);
				}
			}
		}
		return array($enabled, $disabled);
	}
// List templates
	private function listTemplates()
	{
		$enabled = $disabled = array();
		$handle = opendir('templates');
		while(FALSE !== ($dir = readdir($handle)))
		{
			if($dir == '..')
				continue;
			if(is_dir('templates/' . $dir) && file_exists('templates/' . $dir . '/description.txt'))
			{
				$filePath = 'templates/' . $dir . '/description.txt';
				if($fh = fopen($filePath, "r"))
				{
					$enabled[$dir] = fgets($fh);
					fclose($fh);
				}
			}
			else if(is_dir('templates/' . $dir) && file_exists('templates/' . $dir . '/description.txt~'))
			{
				$filePath = 'templates/' . $dir . '/description.txt~';
				if($fh = fopen($filePath, "r"))
				{
					$disabled[$dir] = fgets($fh);
					fclose($fh);
				}
			}
		}
		return array($enabled, $disabled);
	}
// List languages
	private function listLanguages()
	{
		$enabled = $disabled = array();
		$handle = opendir('languages');
		while(FALSE !== ($dir = readdir($handle)))
		{
			if($dir == '..')
				continue;
			if(is_dir('languages/' . $dir) && file_exists('languages/' . $dir . '/description.txt'))
			{
				$filePath = 'languages/' . $dir . '/description.txt';
				if($fh = fopen($filePath, "r"))
				{
					$enabled[$dir] = fgets($fh);
					fclose($fh);
				}
			}
			else if(is_dir('languages/' . $dir) && file_exists('languages/' . $dir . '/description.txt~'))
			{
				$filePath = 'languages/' . $dir . '/description.txt~';
				if($fh = fopen($filePath, "r"))
				{
					$disabled[$dir] = fgets($fh);
					fclose($fh);
				}
			}
		}
		return array($enabled, $disabled);
	}
/**
* Actually enable/disable plugins
*/
	public function enableDisable()
	{
		$bool = FALSE;
		$stylesDir = 'styles/bibliography/';
		if(array_key_exists('pluginsEnabled', $this->vars))
		{
			foreach($this->vars['pluginsEnabled'] as $dir)
			{
				if(file_exists('plugins/' . $dir . '/index.php~'))
					$bool = @rename('plugins/' . $dir . '/index.php~', 'plugins/' . $dir . '/index.php');
			}
		}
		if(array_key_exists('pluginsDisabled', $this->vars))
		{
			foreach($this->vars['pluginsDisabled'] as $dir)
			{
				if(file_exists('plugins/' . $dir . '/index.php'))
					$bool = @rename('plugins/' . $dir . '/index.php', 'plugins/' . $dir . '/index.php~');
			}
		}
// styles
		$enabled = array();
		if(array_key_exists('stylesEnabled', $this->vars))
		{
			foreach($this->vars['stylesEnabled'] as $dir)
			{
				if(file_exists($stylesDir . $dir . '/' . strtoupper($dir) . ".xml~"))
					$bool = @rename($stylesDir . $dir . '/' . strtoupper($dir) . ".xml~", $stylesDir . $dir . '/' . strtoupper($dir) . ".xml");
				$enabled[] = strtoupper($dir);
			}
		}
		if(array_key_exists('stylesDisabled', $this->vars))
		{
			$disabled = array();
			foreach($this->vars['stylesDisabled'] as $dir)
			{
				if((sizeof($enabled) > 0) && file_exists($stylesDir . $dir . '/' . strtoupper($dir) . ".xml"))
					$bool = @rename($stylesDir . $dir . '/' . strtoupper($dir) . ".xml", $stylesDir . $dir . '/' . strtoupper($dir) . ".xml~");
			}
			$this->dbStyles();
		}
// templates
		$enabled = array();
		if(array_key_exists('templatesEnabled', $this->vars))
		{
			foreach($this->vars['templatesEnabled'] as $dir)
			{
				if(file_exists('templates/' . $dir . '/description.txt~'))
					$bool = @rename('templates/' . $dir . '/description.txt~', 'templates/' . $dir . '/description.txt');
				$enabled[] = strtoupper($dir);
			}
		}
		if(array_key_exists('templatesDisabled', $this->vars))
		{
			$disabled = array();
			foreach($this->vars['templatesDisabled'] as $dir)
			{
				if((sizeof($enabled) > 0) && file_exists('templates/' . $dir . '/description.txt'))
					$bool = @rename('templates/' . $dir . '/description.txt', 'templates/' . $dir . '/description.txt~');
			}
			$this->dbTemplates();
		}
// languages
		$enabled = array();
		if(array_key_exists('languagesEnabled', $this->vars))
		{
			foreach($this->vars['languagesEnabled'] as $dir)
			{
				if(file_exists('languages/' . $dir . '/description.txt~'))
					$bool = @rename('languages/' . $dir . '/description.txt~', 'languages/' . $dir . '/description.txt');
				$enabled[] = strtoupper($dir);
			}
		}
		if(array_key_exists('languagesDisabled', $this->vars))
		{
			$disabled = array();
			foreach($this->vars['languagesDisabled'] as $dir)
			{
				if((sizeof($enabled) > 0) && file_exists('languages/' . $dir . '/description.txt'))
					$bool = @rename('languages/' . $dir . '/description.txt', 'languages/' . $dir . '/description.txt~');
			}
			$this->dbLanguages();
		}
		if(!$bool)
			return $this->init($this->errors->text('file', 'write'));
		return $this->init($this->success->text("plugins"));
	}
/**
Change menu plugin configuration settings
*/
	public function writeConfigMenu()
	{
		$bool = FALSE;
		if($fh = @fopen('plugins/' . $this->vars['configFileMenu'] . '/config.php', "w"))
		{
			$bool = @fwrite($fh, stripslashes($this->vars['configConfig']));
			fclose($fh);
		}
		if($bool === FALSE)
			return $this->init($this->errors->text('file', 'write'));
		return $this->init($this->success->text("plugins"));
	}
/**
Change inline plugin configuration settings
*/
	public function writeConfigInline()
	{
		$bool = FALSE;
		$configFile = 'plugins/' . $this->vars['configFileInline'] . '/config.php';
		include_once($configFile);
		$class = $this->vars['configFileInline'] . '_CONFIG';
		$config = new $class();
		$vars = get_class_vars($class);
		$varString = '';
		foreach($vars as $var => $value)
		{
			if($var == 'container')
				$varString .= "\n" . 'public $' . $var . " = '" . $this->vars['configContainers'] . "';";
			else
			{
				if(is_string($value))
					$value = "'" . $value . "'";
				$varString .= "\n" . 'public $' . $var . ' = ' . $value . ";";
			}
		}
		if($fh = @fopen($configFile, "w"))
		{
			$string = "<?php\nclass $class {";
			$string .= $varString;
			$string .= "\n}\n?>";
			$bool = @fwrite($fh, $string);
			fclose($fh);
		}
		if($bool === FALSE)
			return $this->init($this->errors->text('file', 'write'));
		return $this->init($this->success->text("plugins"));
	}
/**
* If styles have been disabled, check they no longer exist in user preferences.
* If they do, replace with the first style available
*/
	private function dbStyles()
	{
		$stylesDir = 'styles/bibliography/';
		$disabled = array();
		$defaultStyle = FALSE;
		$handle = opendir($stylesDir);
		while(FALSE !== ($dir = readdir($handle)))
		{
			if(file_exists($stylesDir . $dir . '/' . strtoupper($dir) . ".xml~"))
				$disabled[] = strtoupper($dir);
			if(file_exists($stylesDir . $dir . '/' . strtoupper($dir) . ".xml") && !$defaultStyle)
				$defaultStyle = strtoupper($dir);
		}
		$this->db->formatConditionsOneField($disabled, 'usersStyle');
		$recordset = $this->db->select('users', 'usersId');
		while($row = $this->db->fetchRow($recordset))
		{
			$this->db->formatConditions(array('usersId' => $row['usersId']));
			$this->db->update('users', array('usersStyle' => $defaultStyle));
// reset session for this user
			if($row['usersId'] == $this->session->getVar("setup_UserId"))
				$this->session->setVar("setup_Style", $defaultStyle);
		}
		$style = $this->db->fetchOne($this->db->select('config', 'configStyle'));
		if(array_search($style, $disabled) !== FALSE)
			$this->db->update('config', array('configStyle' => $defaultStyle));
// delete style cache files
		foreach($disabled as $dir)
			@unlink('styles/CACHE/' . $dir);
	}
/**
* If templates have been disabled, check they no longer exist in user preferences.
* If they do, replace with the first template available
*/
	private function dbTemplates()
	{
		$disabled = array();
		$default = FALSE;
		$handle = opendir('templates');
		while(FALSE !== ($dir = readdir($handle)))
		{
			if(file_exists('templates/' . $dir . '/description.txt~'))
				$disabled[] = $dir;
			if(file_exists('templates/' . $dir . '/description.txt') && !$default)
				$default = $dir;
		}
		$this->db->formatConditionsOneField($disabled, 'usersTemplate');
		$recordset = $this->db->select('users', 'usersId');
		while($row = $this->db->fetchRow($recordset))
		{
			$this->db->formatConditions(array('usersId' => $row['usersId']));
			$this->db->update('users', array('usersTemplate' => $default));
// reset session for this user
			if($row['usersId'] == $this->session->getVar("setup_UserId"))
				$this->session->setVar("setup_Template", $default);
		}
		$style = $this->db->fetchOne($this->db->select('config', 'configTemplate'));
		if(array_search($style, $disabled) !== FALSE)
			$this->db->update('config', array('configTemplate' => $default));
	}
/**
* If languages have been disabled, check they no longer exist in user preferences.
* If they do, replace with the first language available
*/
	private function dbLanguages()
	{
		$disabled = array();
		$default = FALSE;
		$handle = opendir('languages');
		while(FALSE !== ($dir = readdir($handle)))
		{
			if(file_exists('languages/' . $dir . '/description.txt~'))
				$disabled[] = $dir;
			if(file_exists('languages/' . $dir . '/description.txt') && !$default)
				$default = $dir;
		}
		$this->db->formatConditionsOneField($disabled, 'usersLanguage');
		$recordset = $this->db->select('users', 'usersId');
		while($row = $this->db->fetchRow($recordset))
		{
			$this->db->formatConditions(array('usersId' => $row['usersId']));
			$this->db->update('users', array('usersLanguage' => $default));
// reset session for this user
			if($row['usersId'] == $this->session->getVar("setup_UserId"))
				$this->session->setVar("setup_Language", $default);
		}
		$style = $this->db->fetchOne($this->db->select('config', 'configLanguage'));
		if(array_search($style, $disabled) !== FALSE)
			$this->db->update('config', array('configLanguage' => $default));
	}
}
?>
