<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************/
/**
*	CONFIGURE WIKINDX class
*
*/
class CONFIGURE
{
private $errors;
private $messages;
private $success;
private $template;
private $session;
private $config;
private $html;
private $user;
private $db;
private $vars;
private $fields;
private $badInput;
private $gatekeep;
private $loadStyle;
private $table;
private $tinymce;
private $utf8;
public $insert = FALSE;

	public function __construct()
	{
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->loadStyle = FACTORY_LOADSTYLE::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->tinymce = FACTORY_LOADTINYMCE::getInstance();
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		$this->fields = $this->db->listFields('config');
	}
// Interface to configure WIKINDX
	public function init($message = FALSE)
	{
		if(!isset($this->config->WIKINDX_SEARCHFILTER))
			$this->config->WIKINDX_SEARCHFILTER = array();
		if(!$this->insert)
			$values = $this->fromDbToSession();
		else // initial install
			$values = $this->session->getArray("config");
		if(!is_array($values))
			$values = array();
		$pString = $this->tinymce->loadMinimalTextarea(array('configDescription'), TRUE);
		$pString .= $message;
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('configure'));
		GLOBALS::setOutputHeading($this->messages->text("heading", "configure"));
		$form = FACTORY_FORM::getInstance();
		$pString .= $form->formHeader("admin_CONFIGURE_CORE");
		$pString .= $form->hidden("method", "write");
// Only for superadmin who is always userid = 1
		if(!$this->session->issetVar('setup_UserId') || ($this->session->getVar('setup_UserId') == 1))
		{
			$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
			$pString .= $this->table->trStart();
			$input = array_key_exists("configUsername", $values) ?
				$values["configUsername"] : FALSE;
			$pString .= $this->table->td($form->textInput($this->messages->text("config", "superUsername"),
				"configUsername", $input, 30) . " " . $this->html->span('*', 'required'));
			$input = array_key_exists("configPassword", $values) ?
				$values["configPassword"] : FALSE;
			$pString .= $this->table->td($form->passwordInput($this->messages->text("config", "superPassword"),
				"configPassword", $input, 30) . " " . $this->html->span('*', 'required'));
			$input = array_key_exists("configPasswordConfirm", $values) ?
				$values["configPasswordConfirm"] : FALSE;
			$pString .= $this->table->td($form->passwordInput($this->messages->text("user", "passwordConfirm"),
				"configPasswordConfirm", $input, 30) . " " . $this->html->span('*', 'required'));
			$input = array_key_exists("configEmail", $values) ?
				$values["configEmail"] : FALSE;
			$pString .= $this->table->td($form->textInput($this->messages->text("user", "email"), 
				"configEmail", $input, 30) . " " . $this->html->span('*', 'required'));
			$input = array_key_exists("configFullname", $values) ?
				$values["configFullname"] : FALSE;
			$pString .= $this->table->td($form->textInput($this->messages->text("user", "fullname"), 
				"configFullname", $input, 30));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
		}
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		array_key_exists("configFileDeleteSeconds", $values) ?
			$input = $values["configFileDeleteSeconds"] : $input = 3600;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "deleteSeconds"),
			"configFileDeleteSeconds", $input, 10, 10) . " " . $this->html->span('*', 'required'));
		array_key_exists("configPaging", $values) ?
			$input = $values["configPaging"] : $input = 20;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "paging"), "configPaging", $input, 10, 10) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "pagingLimit"), 'hint') . 
			" " . $this->html->span('*', 'required'));
		array_key_exists("configPagingMaxLinks", $values) ?
			$input = $values["configPagingMaxLinks"] : $input = 11;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "maxPaging"),
			"configPagingMaxLinks", $input, 10, 10) . " " . $this->html->span('*', 'required'));
		array_key_exists("configPagingTagCloud", $values) ?
			$input = $values["configPagingTagCloud"] : $input = 100;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "pagingTagCloud"),
			"configPagingTagCloud", $input, 10, 10) . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		array_key_exists("configStringLimit", $values) ?
			$input = $values["configStringLimit"] : $input = 40;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "stringLimit"),
			"configStringLimit", $input, 10, 10) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "pagingLimit"), 'hint') . 
			" " . $this->html->span('*', 'required'));
		array_key_exists("configImgWidthLimit", $values) ?
			$input = $values["configImgWidthLimit"] : $input = 400;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "imgWidthLimit"),
			"configImgWidthLimit", $input, 10, 10) . $this->html->span('*', 'required'));
		array_key_exists("configImgHeightLimit", $values) ?
			$input = $values["configImgHeightLimit"] : $input = 400;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "imgHeightLimit"),
			"configImgHeightLimit", $input, 10, 10) . $this->html->span('*', 'required'));
		array_key_exists("configMaxPaste", $values) ?
			$input = $values["configMaxPaste"] : $input = 10;
		if(!$input)
			$input = 0;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "maxPaste"),
			"configMaxPaste", $input, 10, 10) . $this->html->span('*', 'required'));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		if(array_key_exists("configLastChangesType", $values) && ($values["configLastChangesType"] == 'N')) // set number
			$input = 1;
		else // Set no. days
			$input = 2;
		$array = array(1 => $this->messages->text("config", "lastChanges2"), 
			2 => $this->messages->text("config", "lastChanges3"));
		$td = $form->selectedBoxValue($this->messages->text("config", "lastChanges1") , "configLastChangesType", 
			$array, $input, 2, 80) . "&nbsp;&nbsp;";
		array_key_exists("configLastChanges", $values) ?
			$input = $values["configLastChanges"] : $input = 10;
		$td .= $form->textInput(FALSE, "configLastChanges", $input, 10, 10) . $this->html->span('*', 'required');
		$pString .= $this->table->td($td);
		array_key_exists("configLastChangesDayLimit", $values) ?
			$input = $values["configLastChangesDayLimit"] : $input = 10;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "lastChanges4"), 
			"configLastChangesDayLimit", $input, 10, 10) . $this->html->span('*', 'required'));
		$input = array_key_exists("configListlink", $values) ? "CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "listlink"), "configListlink", $input));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		$template = FACTORY_TEMPLATE::getInstance();
		$templates = $template->loadDir();
		array_key_exists("configTemplate", $values) ?
			$input = $values["configTemplate"] : $input = "default";
		$pString .= $this->table->td($form->selectedBoxValue($this->messages->text("config", "template") , "configTemplate", 
			$templates, $input) . " " . $this->html->span('*', 'required'));
		$language = FACTORY_MESSAGES::getInstance();
		$languages = $language->loadDir();
		array_key_exists("configLanguage", $values) ?
			$input = $values["configLanguage"] : $input = "en";
		$pString .= $this->table->td($form->selectedBoxValue($this->messages->text("config", "language") , "configLanguage", 
			$languages, $input) . " " . $this->html->span('*', 'required'));
		$styles = $this->loadStyle->loadDir("styles/bibliography");
// fetch array key name at element 0 in case we are running for first time
		$keys = array_keys($styles);
		array_key_exists("configStyle", $values) ? 
			$input = $values["configStyle"] : $input = $keys[0];
		$pString .= $this->table->td($form->selectedBoxValue($this->messages->text("config", "style") , "configStyle", 
			$styles, $input, 4) . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->messages->text("config", "missingXml"));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		array_key_exists("configTitle", $values) ?
			$input = $values["configTitle"] : $input = "WIKINDX";
		$td = $form->textInput($this->messages->text("config", "title"), "configTitle", $input, 30);
		$td .= $this->html->p();
		$input = array_key_exists("configContactEmail", $values) ?
			$values["configContactEmail"] : FALSE;
		$td .= $form->textInput($this->messages->text("config", "email"), "configContactEmail", $input, 30);
		$pString .= $this->table->td($td);
		$input = array_key_exists("configDescription", $values) ? $values["configDescription"] : FALSE;
		$pString .= $this->table->td($form->textareaInput($this->messages->text("config", "description"),
			"configDescription", $input, 75, 20));
		if($this->insert)
			$input = join(', ', array(
				'an', 'a', 'the', 'der', 'die', 'das',
				'ein', 'eine', 'einer', 'eines', 'le', 'la', 'las',
				'il', 'les', 'une', 'un', 'una', 'uno', 'lo', 'los',
				'i', 'gli', 'de', 'het', 'um', 'uma', 'o', 'os', 'as')); // default at first intall
		else
			$input = join(', ', $this->config->WIKINDX_NOSORT);
		$td = $this->html->p($form->textareaInput($this->messages->text("config", "noSort"),
			"configNoSort", $input, 50, 7) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "wordList"), 'hint'));
		if($this->insert)
			$input = join(', ', array('an', 'a', 'the', 'and', 'to')); // default at first install
		else
			$input = join(', ', $this->config->WIKINDX_SEARCHFILTER);
		$td .= $this->html->p($form->textareaInput($this->messages->text("config", "searchFilter"),
			"configSearchFilter", $input, 50, 7) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "wordList"), 'hint'));
		$pString .= $this->table->td($td);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		$input = array_key_exists("configDenyReadOnly", $values) ? "CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "denyReadOnly"), "configDenyReadOnly", $input));
		if($this->insert)
			$input = "CHECKED"; // NB checked by default
		else
			$input = array_key_exists("configReadOnlyAccess", $values) ? "CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "readOnlyAccess"), "configReadOnlyAccess", $input));
		$input = array_key_exists("configOriginatorEditOnly", $values) ? "CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "originatorEditOnly"), 
			"configOriginatorEditOnly", $input));
		$input = array_key_exists("configGlobalEdit", $values) ? "CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "globalEdit"), "configGlobalEdit", $input));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		$input = array_key_exists("configMultiUser", $values) ? "CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "multiUser"), "configMultiUser", $input));
		$input = array_key_exists("configUserRegistration", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "userRegistration"), 
				"configUserRegistration", $input) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "userRegistration"), 'hint'));
		$input = array_key_exists("configRegistrationModerate", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "registrationModerate"), 
				"configRegistrationModerate", $input) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "userRegistration"), 'hint'));
		$input = array_key_exists("configEmailNewRegistrations", $values) ?
			$values["configEmailNewRegistrations"] : FALSE;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "emailNewRegistrations"), 
			"configEmailNewRegistrations", $input, 30) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "userRegistration"), 'hint'));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		$input = array_key_exists("configCaptchaPublicKey", $values) ?$values["configCaptchaPublicKey"] : FALSE;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "captchaPublicKey"), 
			"configCaptchaPublicKey", $input, 45));
		$input = array_key_exists("configCaptchaPrivateKey", $values) ?$values["configCaptchaPrivateKey"] : FALSE;
		$pString .= $this->table->td($form->textInput($this->messages->text("config", "captchaPrivateKey"), 
			"configCaptchaPrivateKey", $input, 45));
		$input = array_key_exists("configErrorReport", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "errorReport"), "configErrorReport", $input));
		$input = array_key_exists("configPrintSql", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "printSql"), 
			"configPrintSql", $input));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		$input = array_key_exists("configNotify", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "notify"), "configNotify", $input) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "userRegistration"), 'hint'));
		$input = array_key_exists("configEmailStatistics", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "statistics"), "configEmailStatistics", $input) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "userRegistration"), 'hint'));
		$input = array_key_exists("configEmailNews", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "emailNews"), "configEmailNews", $input) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "userRegistration"), 'hint'));
		$input = array_key_exists("configQuarantine", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "quarantine"), "configQuarantine", $input));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->table->tableStart('generalTable', 1, 0, 0, "left");
		$pString .= $this->table->trStart();
		$input = array_key_exists("configFileAttach", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "fileAttach"), "configFileAttach", $input));
		$input = array_key_exists("configFileViewLoggedOnOnly", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "fileViewLoggedOnOnly"), 
			"configFileViewLoggedOnOnly", $input));
		$input = array_key_exists("configImportBib", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "importBib"), 
			"configImportBib", $input));
		if($this->insert)
			$input = "CHECKED"; // NB checked by default
		else
			$input = array_key_exists("configMetadataAllow", $values) ? "CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "metadataAllow"), 
			"configMetadataAllow", $input));
		$input = array_key_exists("configMetadataUserOnly", $values) ?
			"CHECKED" : FALSE;
		$pString .= $this->table->td($form->checkbox($this->messages->text("config", "metadataUserOnly"), 
			"configMetadataUserOnly", $input));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->html->p($form->formSubmit(), FALSE, "left");
		$pString .= $form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// write configuration details to WKX_config table and, if initial insert at wikindx installation, WKX_users
// Always write username, password, fullname and user email to WKX_users
	public function write()
	{
		$this->checkInput();
// If we get here, we're cleared to write to the database.  This user is always id = 1
		foreach($this->fields as $field)
		{
			if(!array_key_exists($field, $this->vars))
			{
// Checkboxes
				if((($field == 'configMultiUser') || ($field == "configUserRegistration") 
				|| ($field == "configNotify") || ($field == "configQuarantine") || ($field == "configListlink") 
				|| ($field == "configFileAttach") || ($field == "configFileViewLoggedOnOnly") 
				|| ($field == "configImportBib") || ($field == "configEmailNews") || ($field == "configMetadataAllow") 
				|| ($field == "configMetadataUserOnly") 
				|| ($field == "configRegistrationModerate") || ($field == 'configEmailStatistics')
				|| ($field == 'configDenyReadOnly') || ($field == 'configReadOnlyAccess') 
				|| ($field == 'configOriginatorEditOnly') || ($field == 'configGlobalEdit')
				|| ($field == 'configErrorReport') || ($field == 'configPrintSql')) && !$this->insert)
					$updateArray[$field] = 'N';
				continue;
			}
			$value = trim($this->vars[$field]);
// Description is a textarea so leave NL reduction until display.
			if($field == 'configDescription')
			{
				if(!trim($value))
					$nulls[] = $field;
			}
			else if($field == 'configNoSort')
			{
				if(!trim($value))
					$nulls[] = $field;
				else
				{
					$array = array();
					if(isset($this->config->WIKINDX_NOSORT))
						$oldNoSort = base64_encode(serialize($this->config->WIKINDX_NOSORT));
					foreach(explode(',', $value) as $word)
					{
						$word = $this->utf8->utf8_strtolower(stripslashes(trim($word)));
						if($word && array_search($word, $array) === FALSE)
							$array[] = $word;
					}
					$value = base64_encode(serialize($array));
					$this->config->WIKINDX_NOSORT = $array;
					if(isset($oldNoSort) && $oldNoSort != $value)
						$this->updateNoSort($oldNoSort);
				}
			}
			else if($field == 'configSearchFilter')
			{
				if(!trim($value))
					$nulls[] = $field;
				else
				{
					$array = array();
					foreach(explode(',', $value) as $word)
					{
						$word = $this->utf8->utf8_strtolower(stripslashes(trim($word)));
						if($word && array_search($word, $array) === FALSE)
							$array[] = $word;
					}
					$value = base64_encode(serialize($array));
					$this->config->WIKINDX_SEARCHFILTER = $array;
				}
			}
			else
			{
				$value = $this->html->removeNl($value);
				if(($field == 'configContactEmail') || ($field == "configEmailNewRegistrations")
					|| ($field == 'configCaptchaPublicKey') || ($field == "configCaptchaPrivateKey"))
				{
					if(!trim($value))
						$nulls[] = $field;
				}
				if(($field == "configRegistrationModerate") && 
					array_key_exists('configEmailNewRegistrations', $this->vars) && 
					!trim($this->vars['configEmailNewRegistrations']))
				{
					if($this->insert)
					{
						$valueArray[] = "N";
						$fieldArray[] = $field;
					}
					else
						$updateArray[$field] = "N";
					$this->session->delVar('config_configRegistrationModerate');
					continue;
				}
				if($field == 'configLastChangesType')
				{
					if($value == 2)	// Days
						$value = 'D';
					else	// Number
						$value = 'N';
				}
			}
			if($this->insert && !$value)
				continue;
// Checkboxes
			if(($field == 'configMultiUser') || ($field == "configUserRegistration") 
			|| ($field == "configNotify") || ($field == 'configEmailStatistics') || ($field == "configMetadataAllow") 
			|| ($field == "configMetadataUserOnly") 
			|| ($field == "configFileAttach") || ($field == "configFileViewLoggedOnOnly") 
			|| ($field == "configImportBib")|| ($field == "configEmailNews")  || ($field == "configListlink")
			|| ($field == "configRegistrationModerate") || ($field == "configQuarantine") 
			|| ($field == 'configDenyReadOnly') || ($field == 'configReadOnlyAccess') 
			|| ($field == 'configOriginatorEditOnly') || ($field == 'configGlobalEdit')
			|| ($field == 'configErrorReport') || ($field == 'configPrintSql'))
				$value = 'Y';
// Are we inserting the row and installing wikindx for the first time?
			if($this->insert)
			{
				$fieldArray[] = $field;
				$valueArray[] = "$value";
			}
			else
				$updateArray[$field] = "$value";
		}
//print_r($fieldArray); print "<P>";
//print_r($valueArray); print "<P>";
//print_r($updateArray); print "<P>";
//die;
		if($this->insert)
		{
			$fieldArray[] = 'configStatisticsCompiled';
			$valueArray[] = '2012-01-01 01:01:01';
			$this->db->insert('config', $fieldArray, $valueArray);
			$this->db->updateTimestamp('config', array('configStatisticsCompiled' => FALSE)); // i.e. NOW()
// if inserting after initial install, write superadmin's preferences to users table and create user session
			$usersFields = $this->db->listFields('users');
			foreach($fieldArray as $key)
			{
				$usersKey = str_replace('config', 'users', $key);
				$usersValue = array_shift($valueArray);
				if((array_search($usersKey, $usersFields) === FALSE) || 
					($key == 'configNotify') || ($key == 'configListlink') || ($key == 'configEmailStatistics'))
					continue; // do nothing
				$usersFieldArray[] = $usersKey;
				$usersValueArray[] = $usersValue;
			}
			$usersFieldArray[] = 'usersUsername';
			$usersValueArray[] = $this->vars['configUsername'];
			$usersFieldArray[] = 'usersPassword';
			$usersValueArray[] = crypt($this->vars['configPassword'], strrev(time()));
			$usersFieldArray[] = 'usersEmail';
			$usersValueArray[] = $this->vars['configEmail'];
			$usersFieldArray[] = 'usersAdmin';
			$usersValueArray[] = 'Y';
			if(array_key_exists('configFullname', $this->vars))
			{
				$usersFieldArray[] = 'usersFullname';
				$usersValueArray[] = $this->vars['configFullname'];
			}
			$this->db->insert('users', $usersFieldArray, $usersValueArray);
// set the new WKX_users.notifyTimestamp and WKX_users.timestamp to current date
			$this->db->formatConditions(array('usersId' => 1));
			$this->db->updateTimestamp('users', array('usersNotifyTimestamp' => FALSE, 'usersTimestamp' => FALSE));
			$this->user->writeSessionPreferences(1); // '1' == superAdmin
			$this->insert = FALSE;
		}
		else
		{
			$this->db->update('config', $updateArray);
			if(isset($nulls) && !empty($nulls))
				$this->db->updateNull('config', $nulls);
			$storedUpdateArray = $updateArray;
			$updateArray = $nulls = array();
			if(array_key_exists('configUsername', $this->vars))
				$updateArray['usersUsername'] = $this->vars['configUsername'];
			if(array_key_exists('configEmail', $this->vars))
				$updateArray['usersEmail'] = $this->vars['configEmail'];
			$this->db->formatConditions(array('usersId' => 1));
			$recordset = $this->db->select('users', 'usersPassword');
			if(array_key_exists('configPassword', $this->vars) && 
				($this->vars['configPassword'] != $this->db->fetchOne($recordset)))
				$updateArray['usersPassword'] = crypt($this->vars['configPassword'], strrev(time()));
			if(array_key_exists('configFullname', $this->vars))
				$updateArray['usersFullname'] = $this->vars['configFullname'];
			else
				$nulls[] = 'usersFullname';
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('usersId' => 1));
				$this->db->update('users', $updateArray);
			}
			if(!empty($nulls))
			{
				$this->db->formatConditions(array('usersId' => 1));
				$this->db->updateNull('users', $nulls);
			}
			$this->writeSetupSession($storedUpdateArray);
		}
		$this->init($this->success->text("config"));
	}
/**
* If the NoSort array has changed, need to update the database
*/
	private function updateNoSort($oldNoSort)
	{
		$oldArray = unserialize(base64_decode($oldNoSort));
//print 'OLD ARRAY: '; print_r($oldArray); print '<BR>';
		$arrayNewNoSort = array_diff($this->config->WIKINDX_NOSORT, $oldArray);
		$arrayRemoveNoSort = array_diff($oldArray, $this->config->WIKINDX_NOSORT);
//print 'NEW NO SORT: '; print_r($arrayNewNoSort); print '<BR>';
//print 'REMOVE NO SORT: '; print_r($arrayRemoveNoSort); print '<BR>';
		$newNSPattern = $newNSPatternBrace = FALSE;
		if(!empty($arrayNewNoSort))
		{
			foreach($arrayNewNoSort as $word)
			{
				$array[] = "^($word)\s(.*)";
				$arrayBrace[] = "^\{($word)\s(.*)";
			}
			$newNSPattern = join('|', $array);
			$newNSPatternBrace = join('|', $arrayBrace);
		}
//print "NEWNSPATTERN: $newNSPattern<P>";
		$resultset = $this->db->select('resource', array('resourceId', 'resourceTitle', 'resourceSubtitle', 'resourceNoSort', 'resourceTitleSort'));
		while($row = $this->db->fetchRow($resultset))
		{
//print 'ORIGINAL: '; print_r($row); print '<BR>';
			$update = array();
			if($newNSPattern && preg_match("/$newNSPattern/ui", $row['resourceTitle'], $matches))
			{
//print 'MATCHES: '; print_r($matches); print '<BR>';
				$update['resourceTitle'] = $resourceTitleSort = array_pop($matches);
				$update['resourceNoSort'] = array_pop($matches);
				if($row['resourceSubtitle'])
					$resourceTitleSort .= ' ' . $row['resourceSubtitle'];
				$update['resourceTitleSort'] = str_replace('{', '', str_replace('}', '', $resourceTitleSort));
			}
			else if($newNSPatternBrace && preg_match("/$newNSPatternBrace/ui", $row['resourceTitle'], $matches))
			{
//print 'MATCHES BRACES: '; print_r($matches); print '<BR>';
				$resourceTitleSort = array_pop($matches);
				$update['resourceTitle'] = '{' . $resourceTitleSort;
				$update['resourceNoSort'] = array_pop($matches);
				if($row['resourceSubtitle'])
					$resourceTitleSort .= ' ' . $row['resourceSubtitle'];
				$update['resourceTitleSort'] = str_replace('{', '', str_replace('}', '', $resourceTitleSort));
			}
			else if(array_search($this->utf8->utf8_strtolower($row['resourceNoSort']), $arrayRemoveNoSort) !== FALSE)
			{
				$update['resourceTitle'] = $row['resourceNoSort'] . ' ' . $row['resourceTitle'];
				$update['resourceTitleSort'] = $row['resourceNoSort'] . ' ' . $row['resourceTitleSort'];
				$this->db->formatConditions(array('resourceId' => $row['resourceId']));
				$this->db->updateNull('resource', array('resourceNoSort'));
//print 'NULL<BR>';
			}
			if(!empty($update))
			{
//print 'UPDATE: '; print_r($update); print '<BR>';
				$this->db->formatConditions(array('resourceId' => $row['resourceId']));
				$this->db->update('resource', $update);
			}
//print '<BR>';
		}
	}
/**
 * When updating the configuration, some settings will have an effect on the admin's view (e.g. no. resources to show on front page) -- write them to session
 */
	private function writeSetupSession($array)
	{
		$this->session->setVar('setup_LastChanges', $array['configLastChanges']);
		$this->session->setVar('setup_LastChangesDayLimit', $array['configLastChangesDayLimit']);
		$this->session->setVar('setup_LastChangesType', $array['configLastChangesType']);
		$this->session->setVar('setup_MultiUser', $array['configMultiUser']);
		$this->session->setVar('setup_Quarantine', $array['configQuarantine']);
		$this->session->setVar('setup_MetadataAllow', $array['configMetadataAllow']);
		$this->session->setVar('setup_MetadataUserOnly', $array['configMetadataUserOnly']);
	}
// Check input
	private function checkInput()
	{
// Store in session first and remove unrequired session variables
		foreach($this->fields as $key)
		{
			if(array_key_exists($key, $this->vars))
			{
				if(($key == 'configMultiUser') || ($key == 'configUserRegistration') || ($key == "configNotify") 
					 || ($key == "configFileAttach") || ($key == "configFileViewLoggedOnOnly") || ($key == 'configEmailStatistics') 
					 || ($key == "configImportBib") || ($key == "configEmailNews") || ($key == 'configListlink') 
					 || ($key == 'configRegistrationModerate') || ($key == 'configQuarantine') || ($key == 'configMetadataAllow') 
					 || ($key == 'configMetadataUserOnly') || ($key == 'configDenyReadOnly') || ($key == 'configReadOnlyAccess') 
					 || ($key == 'configOriginatorEditOnly') || ($key == 'configGlobalEdit')
					 || ($key == 'configErrorReport') || ($key == 'configPrintSql'))
					$array[$key] = 'Y';
				else if($key == 'configLastChangesType')
				{
					if($this->vars[$key] == 2) // Over last few days
						$array[$key] = 'D';
					else // Set number
						$array[$key] = 'N';
				}
				else
					$array[$key] = $this->vars[$key];
			}
			else if(($key == 'configMultiUser') || ($key == 'configUserRegistration') || ($key == "configNotify") 
				 || ($key == "configFileAttach") || ($key == "configFileViewLoggedOnOnly") || ($key == 'configEmailStatistics') 
				 || ($key == "configImportBib") || ($key == "configEmailNews") || ($key == 'configListlink') 
				 || ($key == 'configRegistrationModerate') || ($key == 'configQuarantine') || ($key == 'configMetadataAllow') 
				 || ($key == 'configMetadataUserOnly') || ($key == 'configDenyReadOnly') || ($key == 'configReadOnlyAccess') 
				 || ($key == 'configOriginatorEditOnly') || ($key == 'configGlobalEdit')
				 || ($key == 'configErrorReport') || ($key == 'configPrintSql'))
			{
				$this->session->delVar("config_" . $key);
				$this->session->delVar("setup_" . str_replace('config', 'setup', $key));
			}
		}
		if(array_key_exists('configFullname', $this->vars))
			$array['configFullname'] = trim($this->vars['configFullname']);
		else
			$this->session->delVar("config_configFullname");
		if(array_key_exists('configEmail', $this->vars))
			$array['configEmail'] = trim($this->vars['configEmail']);
		else
			$this->session->delVar("config_configEmail");
		$this->session->writeArray($array, "config");
// Only for superadmin who is always userid = 1
		if(!$this->session->issetVar('setup_UserId') || ($this->session->getVar('setup_UserId') == 1))
			$required = array("configFileDeleteSeconds", "configUsername", "configPassword", "configPasswordConfirm",
			"configPaging", "configPagingMaxLinks", "configPagingTagCloud", "configStringLimit", "configMaxPaste", "configLastChanges", 
			"configLastChangesType", "configLastChangesDayLimit", "configEmail");
		else
			$required = array("configFileDeleteSeconds", "configPaging", "configPagingMaxLinks", 
			"configStringLimit", "configMaxPaste", "configLastChanges", "configPagingTagCloud", 
			"configLastChangesType", "configLastChangesDayLimit");
		foreach($required as $value)
		{
// 'lastChanges' and 'lastChangesDayLimit' and 'maxPaste can be 0
			if(($value == 'configLastChanges') || ($value == 'configLastChangesDayLimit') || ($value == 'configMaxPaste'))
			{
				if(!array_key_exists($value, $this->vars) || (trim($this->vars[$value] == '')))
					$this->badInputLoad($this->errors->text("inputError", "missing", " ($value) "));
			}
			else if(!array_key_exists($value, $this->vars) || !trim($this->vars[$value]))
				$this->badInputLoad($this->errors->text("inputError", "missing", " ($value) "));
		}
		if(array_key_exists('configPassword', $this->vars) && 
		($this->vars['configPassword'] != $this->vars['configPasswordConfirm']))
			$this->badInputLoad($this->errors->text("inputError", "passwordMismatch"));
		$isNum = array("configFileDeleteSeconds", "configPaging", "configPagingMaxLinks", 
			"configPagingTagCloud", "configStringLimit", "configImgWidthLimit", 
			"configImgHeightLimit", "configMaxPaste", "configLastChanges", "configLastChangesDayLimit",);
		foreach($isNum as $value)
		{
			if(array_key_exists($value, $this->vars) && !is_numeric($this->vars[$value]))
				$this->badInputLoad($this->errors->text("inputError", "nan", " ($value) "));
		}
	}
// When updating, if the config variables are not in a session, grab from db table and write to session.
// Admin configuration session variables have prefix of 'config_'.
// Returns associative array: fieldnames => values.
	private function fromDbToSession()
	{
		if(!$row = $this->session->getArray("config"))
		{
			$recordset = $this->db->select('config', $this->fields);
			$row = $this->db->fetchRow($recordset);
// Remove noSort and searchFilter (they're in $this->config variables) and write the rest to session variables
			unset($row['configNoSort']);
			unset($row['configSearchFilter']);
			if(!$this->session->writeArray($row, "config"))
				print $this->errors->text("sessionError", "write");
		}
// MG 'multiUser' is a checkbox, too...
		if(array_key_exists("configMultiUser", $row) && ($row['configMultiUser'] == 'N'))
			unset($row['configMultiUser']);
// MG 'userRegistration' is a checkbox, too...
		if(array_key_exists("configUserRegistration", $row) && ($row['configUserRegistration'] == 'N'))
			unset($row['configUserRegistration']);
// MG 'registrationModerate' is a checkbox, too...
		if(array_key_exists("configRegistrationModerate", $row) && ($row['configRegistrationModerate'] == 'N'))
			unset($row['configRegistrationModerate']);
// MG 'notify' is a checkbox, too...
		if(array_key_exists("configNotify", $row) && ($row['configNotify'] == 'N'))
			unset($row['configNotify']);
// MG 'fileAttach' is a checkbox, too...
		if(array_key_exists("configFileAttach", $row) && ($row['configFileAttach'] == 'N'))
			unset($row['configFileAttach']);
// MG 'fileViewLoggedOnOnly' is a checkbox, too...
		if(array_key_exists("configFileViewLoggedOnOnly", $row) && ($row['configFileViewLoggedOnOnly'] == 'N'))
			unset($row['configFileViewLoggedOnOnly']);
// MG 'importBib' is a checkbox, too...
		if(array_key_exists("configImportBib", $row) && ($row['configImportBib'] == 'N'))
			unset($row['configImportBib']);
// MG 'emailNews' is a checkbox, too...
		if(array_key_exists("configEmailNews", $row) && ($row['configEmailNews'] == 'N'))
			unset($row['configEmailNews']);
// MG 'quarantine' is a checkbox, too...
		if(array_key_exists("configQuarantine", $row) && ($row['configQuarantine'] == 'N'))
			unset($row['configQuarantine']);
// MG 'metadataAllow' is a checkbox, too...
		if(array_key_exists("configMetadataAllow", $row) && ($row['configMetadataAllow'] == 'N'))
			unset($row['configMetadataAllow']);
// MG 'metadataUserOnly' is a checkbox, too...
		if(array_key_exists("configMetadataUserOnly", $row) && ($row['configMetadataUserOnly'] == 'N'))
			unset($row['configMetadataUserOnly']);
// MG 'listlink' is a checkbox, too...
		if(array_key_exists("configListlink", $row) && ($row['configListlink'] == 'N'))
			unset($row['configListlink']);
// MG 'emailStatistics' is a checkbox, too...
		if(array_key_exists("configEmailStatistics", $row) && ($row['configEmailStatistics'] == 'N'))
			unset($row['configEmailStatistics']);
// MG 'denyReadOnly' is a checkbox, too...
		if(array_key_exists("configDenyReadOnly", $row) && ($row['configDenyReadOnly'] == 'N'))
			unset($row['configDenyReadOnly']);
// MG 'ReadOnlyAccess' is a checkbox, too...
		if(array_key_exists("configReadOnlyAccess", $row) && ($row['configReadOnlyAccess'] == 'N'))
			unset($row['configReadOnlyAccess']);
// MG 'OriginatorEditOnly' is a checkbox, too...
		if(array_key_exists("configOriginatorEditOnly", $row) && ($row['configOriginatorEditOnly'] == 'N'))
			unset($row['configOriginatorEditOnly']);
// MG 'GlobalEdit' is a checkbox, too...
		if(array_key_exists("configGlobalEdit", $row) && ($row['configGlobalEdit'] == 'N'))
			unset($row['configGlobalEdit']);
// MG 'ErrorReport' is a checkbox, too...
		if(array_key_exists("configErrorReport", $row) && ($row['configErrorReport'] == 'N'))
			unset($row['configErrorReport']);
// MG 'PrintSql' is a checkbox, too...
		if(array_key_exists("configPrintSql", $row) && ($row['configPrintSql'] == 'N'))
			unset($row['configPrintSql']);
// user id is stored in session if user has already logged on.
		if($userId = $this->session->getVar('setup_UserId'))
		{
			$this->db->formatConditions(array('usersId' => $userId));
			$recordset = $this->db->select('users', array('usersUsername', 'usersPassword', 
				'usersFullname', 'usersEmail', 'usersAdmin'));
			$rowT = $this->db->fetchRow($recordset);
			$row['configUsername'] = $rowT['usersUsername'];
			$row['configPassword'] = $row['configPasswordConfirm'] = $rowT['usersPassword'];
			$row['configFullname'] = $rowT['usersFullname'];
			$row['configEmail'] = $rowT['usersEmail'];
		}
// 'lastChanges' can be 0 so may not exist if called from the session
		if(!$row['configLastChanges'])
			$row['configLastChanges'] = 0;
// 'lastChangesDayLimit' can be 0 so may not exist if called from the session
		if(!$row['configLastChangesDayLimit'])
			$row['configLastChangesDayLimit'] = 0;
// tidy up the $row elements for presentation to the browser.
		foreach($row as $key => $value)
			$tidy[$key] = $this->html->dbToFormTidy($value);
		return $tidy;
	}
// Error handling
	private function badInputLoad($error)
	{
		if($this->insert) // new install
			$this->badInput->closeType = 'closeNoMenu';
		$this->badInput->close($error, $this, 'init');
	}
}
?>
