<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2010
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* \brief Provides interface for external programs and for Wikindx AJAX.
* Currently it is the "default" class that API functions get put into that
* don't yet fit into other classes.  Eventually, this class may be removed altogether.
*/
class API
{
	private $db;
	private $vars;
	private $config;
	private $icons;
	private $stmt;
	private $errors;
	private $messages;
	private $table;
	private $form;
	private $html;
	private $bibtex;
	private $bibStyle;
	private $stats;
	private $session;
	private $user;
	private $template;
	private $commonBib;
	private $badInput;
	private $common;
	private $abstract;
	private $note;
	private $userId;
	private $nextDelete = FALSE;
	private $url;
	private $categories;
	private $allowEdit = FALSE;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->icons = FACTORY_LOADICONS::getInstance();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->bibtex = FACTORY_EXPORTBIBTEX::getInstance();
		$this->bibStyle = FACTORY_BIBSTYLE::getInstance();
		$this->stats = FACTORY_STATISTICS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->common = FACTORY_RESOURCECOMMON::getInstance();
		$this->url = FACTORY_URL::getInstance();
		include_once('core/modules/resource/RESOURCEABSTRACT.php');
		$this->abstract = new RESOURCEABSTRACT();
		include_once('core/modules/resource/RESOURCENOTE.php');
		$this->note = new RESOURCENOTE();
		include_once('core/modules/resource/RESOURCEMETA.php');
		$this->meta = new RESOURCEMETA();
		GLOBALS::setOutputHeading($this->messages->text("heading", "resources"));
	}
	public function fetchId()
	{
		if(!array_key_exists('bibtexKey', $this->vars) || !$this->vars['bibtexKey'])
			$this->badInput->close($this->errors->text("inputError", "missing"));

		$this->db->formatConditions(array('resourceBibtexKey' => $this->vars['bibtexKey']));
		$resultset = $this->db->select('resource', array('resourceId'));
		if(!$this->db->numRows($resultset))
			$this->badInput->close($this->messages->text("resources", "noResult"));
		$row = $this->db->fetchRow($resultset);

		GLOBALS::buildOutputString($row['resourceId']);
        FACTORY_CLOSERAW::getInstance();
	}
	public function getBibtex()
	{
		if(!array_key_exists('id', $this->vars) || !$this->vars['id'])
			$this->badInput->close($this->errors->text("inputError", "missing"));

		$this->stmt->listJoin();
		$this->db->formatConditions(array('resourceId' => $this->vars['id']));
		$resultset = $this->db->select('resource', $this->stmt->listFields());
		if(!$this->db->numRows($resultset))
			$this->badInput->close($this->messages->text("resources", "noResult"));
		$row = $this->db->fetchRow($resultset);
		$this->icons->setupIcons();
		$multiUser = $this->session->getVar('setup_MultiUser');

		$this->bibStyle->process($row);
		$body = $this->bibtex->export($row, $this->bibStyle->coinsCreators);

		GLOBALS::buildOutputString($body);
        FACTORY_CLOSERAW::getInstance();
	}
	private static function createBibtexRow( $varname, $val )
	{
		return "| $varname = $val\n";
	}
}
?>
