<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* ATTACHMENTS class
*
*/
class ATTACHMENTS
{
private $db;
private $vars;
private $html;
private $badInput;
private $session;
private $errors;
private $messages;
private $success;
private $form;
private $gatekeep;
private $attachment;
private $file;
private $resourceId;
private $embargoArray = array();
private $embargoNew;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->html = FACTORY_HTML::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->attachment = FACTORY_ATTACHMENT::getInstance();
		include_once("core/file/FILE.php");
		$this->file = new FILE();
	}
// add, edit, delete resource attachments
	public function init()
	{
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->gatekeep->init();
		if(!array_key_exists('resourceId', $this->vars) || !array_key_exists('function', $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$function = $this->vars['function'];
		$this->resourceId = $this->vars['resourceId'];
		$this->{$function}();
	}
// Initial editing/adding form
	private function editInit()
	{
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->session->delVar('attachLock');
		GLOBALS::setOutputHeading($this->messages->text("heading", "attach", $this->messages->text('misc', 'edit')));
		$fields = $this->attachment->listFiles($this->resourceId);
		if(!empty($fields)) // attachments exist for this resource
			GLOBALS::buildOutputString($this->fileAttachEdit($fields));
		else // add a new attachment
			GLOBALS::buildOutputString($this->fileAttachAddOne());
	}
// add an attachment
	private function add()
	{
		if($this->session->getVar('attachLock'))
			$this->badInput->close($this->errors->text("done", "attachAdd"));
		$this->getEmbargo();
		$navigate = FACTORY_NAVIGATE::getInstance();
		if(!$this->storeFile()) // FALSE if attachment already exists
		{
			$navigate->resource($this->resourceId, $this->errors->text("file", "attachmentExists"));
			return;
		}
// Lock re-uploading
		$this->session->setVar('attachLock', TRUE);
// send back to view this resource with success message
		$navigate->resource($this->resourceId, $this->success->text("attach", " " . $this->messages->text("misc", 'added') . " "));
	}
// edit attachments
	private function edit()
	{
// Get primary attachment if multiple attachments
		$primary = array_key_exists('attachmentPrimary', $this->vars) ? $this->vars['attachmentPrimary'] : FALSE;
// find any files to edit and files to delete
		foreach($this->vars as $key => $var)
		{
			$split = explode('_', $key);
			if($split[0] == 'attachmentEdit')
				$edits[$split[1]] = $var;
			if($key == 'embargo')
				$this->getEmbargo();
			else if($split[0] == 'embargo') // checkbox
				$this->getEmbargo($split[1]);
			if($split[0] == 'attachmentDelete')
				$deletes[$split[1]] = $var;
			if($split[0] == 'attachmentDescription')
				$descriptions[$split[1]] = $var;
		}
		$message = FALSE;
// Edit files
		if(isset($edits))
		{
			foreach($edits as $hash => $filename)
			{
				$updateArray = array();
				$updateArray['resourceattachmentsFileName'] = $filename;
				if(array_key_exists($hash, $this->embargoArray))
				{
					$updateArray['resourceattachmentsEmbargo'] = 'Y';
					$updateArray['resourceattachmentsEmbargoUntil'] = $this->embargoArray[$hash];
				}
				else
					$updateArray['resourceattachmentsEmbargo'] = 'N';
				if($descriptions[$hash])
					$updateArray['resourceattachmentsDescription'] = $descriptions[$hash];
				else
				{
					$this->db->formatConditions(array("resourceattachmentsResourceId" => $this->resourceId));
					$this->db->formatConditions(array("resourceattachmentsHashFilename" => $hash));
					$this->db->updateNull('resource_attachments', 'resourceattachmentsDescription');
				}
				$this->db->formatConditions(array("resourceattachmentsResourceId" => $this->resourceId));
				$this->db->formatConditions(array("resourceattachmentsHashFilename" => $hash));
				$this->db->update('resource_attachments', $updateArray);
			}
			$message = $this->success->text("attach", " " . $this->messages->text("misc", 'edited') . " ");
		}
// set primary attachment
		$this->setPrimaryAttachment($primary);
		$navigate = FACTORY_NAVIGATE::getInstance();
// Store any new file
		if($_FILES['file']['tmp_name'])
		{
			if($this->session->getVar('attachLock'))
				$message = $this->errors->text("done", "attachAdd");
			if(!$this->storeFile()) // FALSE if attachment already exists
			{
				$navigate->resource($this->resourceId, $this->errors->text("file", "attachmentExists"));
				return;
			}
// Lock re-uploading
			$this->session->setVar('attachLock', TRUE);
		}
		if(isset($deletes))
			$this->deleteConfirm($deletes, $message);
// send back to view this resource with success message (deleteConfirm breaks out before this)
		$navigate->resource($this->resourceId, $this->success->text("attach", " " . $this->messages->text("misc", 'edited') . " "));
	}
// Grab and sort embargo fields
	private function getEmbargo($hash = FALSE)
	{
		$year = FALSE;
		$day = $month = 1;
		if($hash)
		{
			$arrayIndex = $hash;
			$hash = "_$hash";
		}
		if(array_key_exists("year$hash", $this->vars) && $this->vars["year$hash"] && is_numeric($this->vars["year$hash"]))
			$year = trim($this->vars["year$hash"]);
		if(!$year)
			return;
		if($this->vars["month$hash"])
			$month = $this->vars["month$hash"];
		if($this->vars["day$hash"])
			$day = $this->vars["day$hash"];
		if(!$year || (!$day && !$year) || (!$day && $month && !$year))
			return;
		if(!$day)
			$day = 1;
		if(!$month)
			$month = 1;
		$timestamp = mktime(0, 0, 0, $month, $day, $year);
		if($timestamp === FALSE) // exceed UNIX timestamp capacity
		{
			if($day < 10)
				$day = '0' . $day;
			if($month < 10)
				$month = '0' . $month;
			if($hash)
				$this->embargoArray[$arrayIndex] = "$year-$month-$day 00:00:00";
			else
				$this->embargoNew = "$year-$month-$day 00:00:00";
			return;
		}
		else if($timestamp <= time())
			return;
		if($hash)
			$this->embargoArray[$arrayIndex] = $this->db->formatTimestamp($timestamp);
		else
			$this->embargoNew = $this->db->formatTimestamp($timestamp);
	}
// set primary attachment
	private function setPrimaryAttachment($primary)
	{
		$this->db->formatConditions(array("resourceattachmentsResourceId" => $this->resourceId));
		$recordSet = $this->db->update('resource_attachments', array('resourceattachmentsPrimary' => 'N'));
		$this->db->formatConditions(array('resourceattachmentsResourceId' => $this->resourceId));
		$this->db->formatConditions(array('resourceattachmentsHashFilename' => $primary));
		$recordSet = $this->db->update('resource_attachments', array('resourceattachmentsPrimary' => 'Y'));
	}
// confirm delete attachments
	private function deleteConfirm($deletes, $message)
	{
		$form = FACTORY_FORM::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "attach", $this->messages->text('misc', 'delete')));
		$pString = $message;
		$pString .= $form->formHeader("attachments_ATTACHMENTS_CORE");
		$pString .= $form->hidden('function', 'delete');
		$pString .= $form->hidden('resourceId', $this->resourceId);
		$pString .= $this->messages->text('resources', 'deleteConfirmAttach') . ':' . $this->html->br();
		$this->db->formatConditions(array("resourceattachmentsResourceId" => $this->resourceId));
		$recordSet = $this->db->select(array("resource_attachments"), 
			array('resourceattachmentsHashFilename', 'resourceattachmentsFileName'));
		while($row = $this->db->fetchRow($recordSet))
		{
			if(array_key_exists($row['resourceattachmentsHashFilename'], $deletes))
				$pString .= $form->checkBox(FALSE, 'attachmentDelete_' . $row['resourceattachmentsHashFilename'], TRUE) . 
					'&nbsp;' . $row['resourceattachmentsFileName'] . $this->html->br();
		}
		$pString .= $this->html->p('&nbsp;' . $this->html->br() . $form->formSubmit("Confirm"));
		$pString .= $form->formEnd();
		GLOBALS::buildOutputString($pString);
		FACTORY_CLOSE::getInstance();
	}
// delete attachments
	private function delete()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "attach", $this->messages->text('misc', 'delete')));
		foreach($this->vars as $key => $var)
		{
			$split = explode('_', $key);
			if($split[0] == 'attachmentDelete')
				$deletes[] = $split[1];
		}
		if(!isset($WIKINDX_ATTACHMENTS_DIR))
			$dirName = "attachments";
		else
			$dirName = $WIKINDX_ATTACHMENTS_DIR;
// remove reference from this resource first
		foreach($deletes as $hash)
		{
			$this->db->formatConditions(array("resourceattachmentsResourceId" => $this->resourceId));
			$this->db->formatConditions(array("resourceattachmentsHashFilename" => $hash));
			$this->db->delete('resource_attachments');
// remove file from attachments directory if there's no reference to it in any other resource
			$this->db->formatConditions(array("resourceattachmentsHashFilename" => $hash));
			if(!$this->db->numRows($this->db->select('resource_attachments', 'resourceattachmentsHashFilename')))
				@unlink($dirName . "/" . $hash);
		}
// send back to view this resource with success message
		$navigate = FACTORY_NAVIGATE::getInstance();
		$navigate->resource($this->resourceId, $this->success->text("attach", " " . 
			$this->messages->text("misc", 'deleted') . " "));
	}
// Store attachment
	private function storeFile()
	{
		include_once("core/file/FILE.php");
		list($filename, $hash, $type, $size) = FILE::fileUpload($this->vars['fileName']);
		if(!$hash)
			$this->badInput->close($this->errors->text("file", "upload"));
		if(!isset($WIKINDX_ATTACHMENTS_DIR))
			$dirName = "attachments";
		else
			$dirName = $WIKINDX_ATTACHMENTS_DIR;
		if(!FILE::fileStore($dirName, $hash))
			$this->badInput->close($this->errors->text("file", "upload"));
		$this->db->formatConditions(array("resourceattachmentsHashFilename" => $hash));
		$this->db->formatConditions(array("resourceattachmentsResourceId" => $this->resourceId));
		$recordSet = $this->db->select('resource_attachments', 'resourceattachmentsId');
		if($this->db->numRows($recordSet)) // attachment already exists for this resource
			return FALSE;
		else	// insert
		{
			$fields[] = 'resourceattachmentsResourceId';
			$values[] = $this->resourceId;
			$fields[] = 'resourceattachmentsHashFilename';
			$values[] = $hash;
			$fields[] = 'resourceattachmentsFileName';
			$values[] = $filename;
			$fields[] = 'resourceattachmentsFileType';
			$values[] = $type;
			$fields[] = 'resourceattachmentsFileSize';
			$values[] = $size;
			if(array_key_exists('embargo', $this->vars))
			{
				$fields[] = 'resourceattachmentsEmbargo';
				$values[] = 'Y';
				$fields[] = 'resourceattachmentsEmbargoUntil';
				$values[] = $this->embargoNew;
			}
			else
			{
				$fields[] = 'resourceattachmentsEmbargoUntil';
				$values[] = '2012-01-01 01:01:01';
			}
			$field[] = 'resourceattachmentsTimestamp';
			$value[] = '2012-01-01 01:01:01';
			if($this->vars['fileDescription'])
			{
				$fields[] = 'resourceattachmentsDescription';
				$values[] = $this->vars['fileDescription'];
			}
			$this->db->insert('resource_attachments', $fields, $values);
			$this->db->formatConditions(array('resourceattachmentsId' => $this->db->lastAutoID()));
			$this->db->updateTimestamp('resource_attachments', array('resourceattachmentsTimestamp' => FALSE)); // i.e. NOW()
			if(!array_key_exists('embargo', $this->vars))
				$this->db->updateTimestamp('resource_attachments', array('resourceattachmentsEmbargoUntil' => FALSE)); // i.e. NOW()
		}
		return TRUE;
	}
// form for editing, deleting and adding (another) attachments
	private function fileAttachEdit($fields)
	{
		include_once("core/file/FILE.php");
		$file = new FILE();
		$tinymce = FACTORY_LOADTINYMCE::getInstance();
		$tinyEditors[] = 'fileDescription';
		foreach($fields AS $hash => $null)
			$tinyEditors[] = $hash;
		$pString = $tinymce->loadBasicTextArea($tinyEditors, 400);
		$maxSize = $file->fileMaxSize();
		$this->session->setVar('attachMaxSize', $maxSize);
		$pString .= $this->form->formMultiHeader("attachments_ATTACHMENTS_CORE");
		$pString .= $this->form->hidden('function', 'edit');
		$pString .= $this->form->hidden('resourceId', $this->resourceId);
		$pString .= $this->form->hidden("MAX_FILE_SIZE", $maxSize);
// Form elements for adding another attachment
		$pString .= $this->table->tableStart('generalTable', 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->fileUpload($this->messages->text("resources", "fileAttach"), 
			"file", 50));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "fileName"), "fileName"));
		$pString .= $this->table->td($this->embargoForm());
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->textareaInput($this->messages->text('resources', 'attachmentDescription'), 
			"fileDescription", FALSE, 60), FALSE, FALSE, FALSE, 3);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->hr();
		$numFiles = sizeof($fields);
		$index = $count = 1;
		foreach($fields as $hash => $fileName)
		{
			$pString .= $this->table->tableStart('generalTable', 0, 0, 0, "left");
			$pString .= $this->table->trStart();
			$td = $this->form->textInput($this->messages->text('resources', 'fileName'), 
				"attachmentEdit_$hash", $fileName, 50);
			$td .= '&nbsp;&nbsp;' . $this->messages->text('misc', 'delete') . ':&nbsp;' . $this->form->checkBox(FALSE, 
				"attachmentDelete_$hash");
			if($numFiles > 1)
			{
				$td .= '&nbsp;&nbsp;' . $this->messages->text('resources', 'primaryAttachment') . ':&nbsp;';
				if(($index == 1) && !$this->attachment->primary)
				{	
					$td .= $this->form->radioButton(FALSE, 'attachmentPrimary', $hash, TRUE);
					++$index;
				}
				else if($this->attachment->primary == $hash)
					$td .= $this->form->radioButton(FALSE, 'attachmentPrimary', $hash, TRUE);
				else
					$td .= $this->form->radioButton(FALSE, 'attachmentPrimary', $hash);
			}
			$pString .= $this->table->td($td);
			$pString .= $this->table->td($this->embargoForm($hash));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->trStart();
			$this->db->formatConditions(array('resourceattachmentsHashFilename' => $hash));
			$this->db->formatConditions(array('resourceattachmentsResourceId' => $this->resourceId));
			$desc = $this->db->fetchOne($this->db->select('resource_attachments', 'resourceattachmentsDescription'));
			$pString .= $this->table->td($this->form->textareaInput($this->messages->text('resources', 'attachmentDescription'), 
				"attachmentDescription_$hash", $this->html->dbToHtmlTidy($desc), 60), FALSE, FALSE, FALSE, 2);
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			++$count;
		}
		$pString .= $this->html->p($this->form->formSubmit("Save"));
		$pString .= $this->form->formEnd();
		return $pString;
	}
// form input for adding an attachment
	private function fileAttachAddOne()
	{
		include_once("core/file/FILE.php");
		$file = new FILE();
		$tinymce = FACTORY_LOADTINYMCE::getInstance();
		$maxSize = $file->fileMaxSize();
		$this->session->setVar('attachMaxSize', $maxSize);
		$pString = $this->form->formMultiHeader("attachments_ATTACHMENTS_CORE");
		$pString .= $this->form->hidden('function', 'add');
		$pString .= $this->form->hidden('resourceId', $this->resourceId);
		$pString .= $this->form->hidden("MAX_FILE_SIZE", $maxSize);
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart(FALSE, FALSE, 'top');
		$pString .= $this->table->td($this->form->fileUpload($this->messages->text("resources", "fileAttach"), "file", 50));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "fileName"), "fileName"));
		$pString .= $this->table->td($this->embargoForm());
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
		$pString .= $tinymce->loadBasicTextArea(array("fileDescription"), 400);
		$pString .= $this->table->td($this->form->textareaInput($this->messages->text('resources', 'attachmentDescription'), 
			"fileDescription", FALSE, 60), FALSE, FALSE, FALSE, 3);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->formSubmit("Save"));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		return $pString;
	}
// construct table for embargo form items
	private function embargoForm($hash = FALSE)
	{
		if(!$this->session->getVar('setup_Superadmin'))
			return '&nbsp;';
		$day = $month = 0;
		$embargo = $year = FALSE;
		if($hash)
		{
			$this->db->formatConditions(array('resourceattachmentsHashFilename' => $hash));
			$this->db->formatConditions(array('resourceattachmentsResourceId' => $this->resourceId));
			$row = $this->db->fetchRow($this->db->select('resource_attachments', array('resourceattachmentsEmbargo', 'resourceattachmentsEmbargoUntil')));
			if($row['resourceattachmentsEmbargo'] == 'Y')
				$embargo = 'CHECKED';
			$hash = '_' . $hash;
			$split = explode(' ', $row['resourceattachmentsEmbargoUntil']);
			$date = explode('-', $split[0]);
			if($date[0] != '0000')
				$year = $date[0];
			if($date[1]!= '00')
				$month = $date[1];
			if($date[2] != '00')
				$day = $date[2];
		}
		$td = $this->table->tableStart(FALSE, 0, 0, 0, 'left', '95%');
		$td .= $this->table->trStart();
		$td .= $this->table->td($this->html->br() . 
			$this->messages->text("resources", 'attachEmbargo') . '&nbsp;' . $this->form->checkbox(FALSE, 'embargo' . $hash, $embargo));
		$td .= $this->table->td('&nbsp;');
		$td .= $this->table->td('&nbsp;');
		$days[] = $this->messages->text("misc", "ignore");
		$days = array_merge($days, range(1, 31));
		$td .= $this->table->td($this->form->selectedBoxValue($this->messages->text("resources", 'day'), 'day' . $hash, $days, $day, 1), FALSE, 'right');
		$languageDir = $this->session->getVar('setup_Language');
		if(!is_dir("languages/$languageDir"))
			$languageDir = "en";
		include_once("languages/$languageDir/CONSTANTS.php");
		$class = "CONSTANTS_" . $languageDir;
		$constant = new $class();
		$months[] = $this->messages->text("misc", "ignore");
		$months = array_merge($months, $constant->monthToLongName());
		$td .= $this->table->td($this->form->selectedBoxValue($this->messages->text("resources", 'month'), 'month' . $hash, $months, $month, 1), 
			FALSE, 'right');
		$td .= $this->table->td($this->form->textInput($this->messages->text("resources", "year"), 'year' . $hash, $year, 5, 4), FALSE, 'right');
		$td .= $this->table->trEnd();
		$td .= $this->table->tableEnd();
		return $td;
	}
// download an attachment to a user
	public function downloadAttachment()
	{
		$config = FACTORY_CONFIG::getInstance();
		if(!isset($config->WIKINDX_ATTACHMENTS_DIR))
			$dirName = "attachments";
		else
			$dirName = $config->WIKINDX_ATTACHMENTS_DIR;
		$hash = $this->vars['filename'];
		$this->db->formatConditions(array('resourceattachmentsId' => $this->vars['id']));
		$recordset = $this->db->select('resource_attachments', 
			array('resourceattachmentsFileType', 'resourceattachmentsFileSize', 
			'resourceattachmentsHashFilename', 'resourceattachmentsFileName'));
		$row = $this->db->fetchRow($recordset);
		$type = $row['resourceattachmentsFileType'];
		$size = $row['resourceattachmentsFileSize'];
		$filename = $row['resourceattachmentsFileName'];
		unset($row);
		if(file_exists($dirName . "/" . $hash) === FALSE)
		{
			$this->badInput->closeType = 'closePopup';
			$this->badInput->close($this->errors->text("file", "missing"));
			die;
		}
		$this->file->setHeaders($type, $size, $filename);
		$this->file->readfile_chunked($dirName . "/" . $hash);
		$this->attachment->incrementDownloadCounter($this->vars['id']);
		die;
	}
}
?>
