<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* BASKET class
*
* Management of resource baskets
*
*/
class BASKET
{
private $db;
private $vars;
private $messages;
private $success;
private $session;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
	}
// Add resource to basket
	public function init()
	{
		if($this->session->issetVar('basket_List'))
			$basket = unserialize($this->session->getVar('basket_List'));
		else
			$basket = array();
		$resourceId = $this->vars['resourceId'];
		if(array_search($resourceId, $basket) === FALSE)
			$basket[] = $resourceId;
// Ensure array is unique
		array_unique($basket);
		$this->session->setVar('basket_List', serialize($basket));
		$this->session->saveState('basket');
// send back to view this resource with success message
		include_once("core/modules/resource/RESOURCEVIEW.php");
		$resource = new RESOURCEVIEW();
		$resource->init($this->session->getVar('sql_LastSolo'));
		GLOBALS::buildOutputString($this->success->text("basketAdd"));
	}
// Remove resource from basket
	public function remove()
	{
		if($this->session->issetVar('basket_List'))
			$basket = unserialize($this->session->getVar('basket_List'));
		else
			$basket = array();
		$resourceId = $this->vars['resourceId'];
		if(($key = array_search($resourceId, $basket)) !== FALSE)
			unset($basket[$key]);
		$this->session->setVar('basket_List', serialize($basket));
		$this->session->saveState('basket');
// send back to view this resource with success message
		include_once("core/modules/resource/RESOURCEVIEW.php");
		$resource = new RESOURCEVIEW();
		$resource->init($this->session->getVar('sql_LastSolo'));
		GLOBALS::buildOutputString($this->success->text("basketRemove"));
	}
	public function reorder()
	{
		$this->session->setVar('list_AscDesc', $this->vars['list_AscDesc']);
		$this->view();
	}
// View the basket
	public function view()
	{
		$common = FACTORY_LISTCOMMON::getInstance();
		$stmt = FACTORY_SQLSTATEMENTS::getInstance();
		GLOBALS::setOutputHeading($this->messages->text('heading', 'basket'));
		$order = 'creator';
		if(array_key_exists('list_Order', $this->vars))
		{
			$this->session->setVar('list_Order', $this->vars['list_Order']);
			$order = $this->vars['list_Order'];
		}
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$queryString = 'action=basket_BASKET_CORE&method=view';
// set the lastMulti session variable for quick return to this process.
//		$this->session->setVar('sql_LastMulti', $queryString);
		$basket = unserialize($this->session->getVar('basket_List'));
		if(!empty($basket))
		{
			$sql_Stmt = $this->session->getVar('sql_Stmt');
			$stmt->execCondOneField['resourcemiscId'] = $basket;
			$stmt->createTotalResourceSubqueryList(FALSE, FALSE, $order, TRUE, $queryString);
			$sql = $stmt->listList(FALSE, FALSE, $order);
			$common->display($sql, 'basket');
			$this->session->setVar('sql_Stmt', $sql_Stmt);
			$this->session->setVar('sql_LastMulti', $queryString);
			$this->session->delVar('list_NextPreviousIds');
			$this->session->setVar('list_SubQuery', $this->session->getVar('list_SubQueryMulti'));
// set the lastMulti session variable for quick return to this process.
// Turn on the 'add bookmark' menu item
			$this->session->setVar("bookmark_DisplayAdd", TRUE);
			$this->session->saveState('sql');
			$this->session->saveState('bookmark');
			$this->session->saveState('basket');
		}
	}
// Delete the basket
	public function delete($confirm = FALSE)
	{
		if($confirm)
		{
			$this->session->clearArray('basket');
			include_once("core/front/FRONT.php");
			$obj = new FRONT($this->db);
			$obj->message = $this->success->text("basketDelete");
			return $obj->display();
		}
// Ask for confirmation first
		GLOBALS::setOutputHeading($this->messages->text('heading', 'basketDelete'));
		$template = FACTORY_TEMPLATE::getInstance();
		$form = FACTORY_FORM::getInstance();
		$pString = $form->formHeader('basket_BASKET_CORE');
		$pString .= $form->hidden('method', 'deleteConfirm');
		$pString .= $form->formSubmit('Confirm') . $form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
	public function deleteConfirm()
	{
		$this->session->clearArray('basket');
//		$this->session->delVar('sql_LastMulti');
		include_once("core/display/FRONT.php");
		$front = new FRONT($this->success->text("basketDelete")); // __construct() runs on autopilot
	}
}
?>
