<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
*	CHOOSEBIB class.
*
*	User bibliographies
*/
class CHOOSEBIB
{
private $db;
private $vars;
private $session;
private $form;
private $html;
private $template;
private $errors;
private $messages;
private $success;
private $badInput;
private $common;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->common = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
	}
// List user's bibliographies with options to use one of them or the WIKINDX master bibliography when listing, searching etc.
// Also option to view bibliography description and bibliography owner.
	public function init($message = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "bibs"));
		$pString = $message ? $message : '';
		$pString .= $this->form->formHeader("bibliography_CHOOSEBIB_CORE");
		$bibsArray = $this->common->getBibsArray();
		$selected = $this->session->getVar('mywikindx_Bibliography_use');
		$size = sizeof($bibsArray);
		if($size > 20)
			$size = 10;
		if($selected)
			$pString .= $this->html->p($this->form->selectedBoxValue(FALSE, "BibId", $bibsArray, $selected, $size));
		else
			$pString .= $this->html->p($this->form->selectFBoxValue(FALSE, "BibId", $bibsArray, $size));
// Radio buttons are the querystring's method
		if(!$this->session->getVar('mywikindx_Bibliography_method'))
			$checked = TRUE;
		else
			$checked = $this->session->getVar('mywikindx_Bibliography_method') == 'displayBib' ? TRUE : FALSE;
		$pString .= $this->html->p($this->form->radioButton(FALSE, "method", "displayBib", $checked) . "&nbsp;&nbsp;" . 
			$this->messages->text("user", "displayBib"));
		$checked = $this->session->getVar('mywikindx_Bibliography_method') == 'useBib' ? TRUE : FALSE;
		$pString .= $this->html->p($this->form->radioButton(FALSE, "method", "useBib", $checked) . "&nbsp;&nbsp;" . 
			$this->messages->text("user", "useBib"));
		$pString .= $this->html->p($this->form->formSubmit("Proceed"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Display bibliography details and owner's details
	public function displayBib()
	{
		$pString = '';
		if(!array_key_exists('BibId', $this->vars) || !$this->vars['BibId'])
			$pString .= $this->messages->text("user", "masterBib");
		else if(array_key_exists('BibId', $this->vars) && ($this->vars['BibId'] < 0))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		else
		{
			$this->db->leftJoin('user_bibliography_resource', $this->db->formatFields('userbibliographyresourceBibliographyId'), 
				$this->db->tidyInput($this->vars['BibId']), FALSE);
			$this->db->leftJoin('users', 'userbibliographyUserId', 'usersId');
			$this->db->formatConditions(array('userbibliographyId' => $this->vars['BibId']));
			$recordset = $this->db->selectCounts('user_bibliography', 'userbibliographyresourceBibliographyId', 
				array('userbibliographyTitle', 'usersFullname', 'usersUsername', 'userbibliographyDescription'));
			$row = $this->db->fetchRow($recordset);
			$text = $this->html->b($this->messages->text("user", "username") . ":&nbsp;&nbsp;") . 
				$this->html->dbToHtmlTidy($row['usersUsername']) . $this->html->br();
			$text .= $this->html->b($this->messages->text("user", "fullname") . ":&nbsp;&nbsp;") . 
				$this->html->dbToHtmlTidy($row['usersFullname']);
			$pString = $this->html->p($text);
			$text = '';
			if($row['count'])
				$text .= $this->html->b($this->messages->text("user", "numResources") . ":&nbsp;&nbsp;") . 
					$row['count'] . $this->html->br();
			$text .= $this->html->b($this->messages->text("user", "bibTitle") . ":&nbsp;&nbsp;") . 
				$this->html->dbToHtmlTidy($row['userbibliographyTitle']) . $this->html->br();
			$text .= $this->html->b($this->messages->text("user", "bibDescription") . ":&nbsp;&nbsp;") . 
				$this->html->dbToHtmlTidy($row['userbibliographyDescription']);
			$pString .= $this->html->p($text);
		}
		GLOBALS::setOutputHeading($this->messages->text("heading", "bibs"));
		$pString .= $this->form->formHeader("bibliography_CHOOSEBIB_CORE");
		$pString .= $this->form->hidden("method", "useBib");
		$pString .= $this->form->hidden("BibId", $this->vars['BibId']);
		$pString .= $this->html->p($this->messages->text("user", "useBib") . '&nbsp;&nbsp;' . 
			$this->form->formSubmit("Proceed"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Set a bibliography for browsing
	public function useBib()
	{
// bibId of 0 == master bibliography
// bibId of < 0 == a label
		if(!array_key_exists('BibId', $this->vars) || !$this->vars['BibId'])
			$this->session->delVar('mywikindx_Bibliography_use');
		else if(array_key_exists('BibId', $this->vars) && ($this->vars['BibId'] < 0))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		else
			$this->session->setVar('mywikindx_Bibliography_use', $this->vars['BibId']);
		$this->session->delVar('mywikindx_Bibliography_add');
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$this->session->delVar('sql_LastMulti');
		$this->init($this->success->text("bibliography", " " . $this->messages->text("misc", "set") . " "));
	}
}
?>
