<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net

**********************************************************************************/
/**
* BROWSEBIBLIOGRAPHY class
*
* Browse categories as a 'tag cloud'
*
*/
class BROWSEBIBLIOGRAPHY
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
private $sum;
private $bibs;
private $commonBib;
private $gatekeep;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseBibliography"));
	}
// display bibliographies
	public function init()
	{
		$this->gatekeep->init(); // No Read-only access allowed
		$this->sum = $this->bibs = array();
		$this->getBibliographies();
		if(empty($this->bibs))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noBibliographies"));
			return;
		}
		natcasesort($this->sum);
		$this->bibs = $this->common->paging($this->bibs);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process(), "center");
		$this->common->pagingLinks('action=browse_BROWSEBIBLIOGRAPHY_CORE');
		GLOBALS::buildOutputString($pString);
	}
// Get bibliogaphries from db
	public function getBibliographies()
	{
		$array = $this->commonBib->getUserBibs();
		foreach($array as $id => $title)
			$this->collate($id, $title);
		$array = $this->commonBib->getGroupBibs();
		foreach($array as $id => $title)
			$this->collate($id, $title);
	}
// Add bibliographies to array and sum totals
	private function collate($id, $title)
	{
		$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $id));
		$count = $this->db->fetchOne($this->db->selectCountDistinctField('user_bibliography_resource', 'userbibliographyresourceId'));
		if($count)
		{
			$this->bibs[$id] = preg_replace("/{(.*)}/U", "$1", $this->html->dbToHtmlTidy($title));
			$this->sum[$id] = $count;
		}
	}
// Process and display results
	private function process()
	{
		$lowestSum = current($this->sum);
		$highestSum = end($this->sum);
		foreach($this->bibs as $id => $name)
		{
			$colour = $this->common->colourText($lowestSum, $highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($lowestSum, $highestSum, $this->sum[$id]);
			$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=bibliographyProcess&id=' . $id)) . 
				"&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
}
?>