<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net

**********************************************************************************/
/**
* BROWSECITED class
*
* Browse creators cited in the wikindx as a 'tag cloud'
*
*/
class BROWSECITED
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
private $sum;
private $bib;
private $surname;
private $citedResourceIds;
private $prefix;
private $citeIds;
private $citeList;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseCited"));
	}
// display citations
	public function init()
	{
		$this->sum = $this->surname = $this->prefix = $this->citedResourceIds = $this->citeIds = $this->citeList = array();
		$this->getCitations();
		if(empty($this->citedResourceIds))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noCitations"));
			return;
		}
		$this->getCreators();
		$this->surname = $this->common->paging($this->surname);
		foreach($this->surname as $id => $surname)
		{
			if(!empty($this->prefix) && array_key_exists($id, $this->prefix))
				$this->surname[$id] = $this->prefix[$id] . ' ' . $surname;
		}
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process(), "center");
		$this->common->pagingLinks('action=browse_BROWSECITED_CORE');
		GLOBALS::buildOutputString($pString);
	}
// Get resource ids containing citations in their metadata
	private function getCitations()
	{
		$search = $this->db->like('%', '\\[cite\\]', '%');
// Abstract
		$this->common->userBibCondition('resourcetextId');
		$this->db->formatConditions($this->db->formatFields('resourcetextAbstract') . $search);
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
			array(array('resourcetextId' => 'rId'), array('resourcetextAbstract' => 'text'))));
//Note
		$this->common->userBibCondition('resourcetextId');
		$this->db->formatConditions($this->db->formatFields('resourcetextNote') . $search);
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
			array(array('resourcetextId' => 'rId'), array('resourcetextNote' => 'text'))));
// quotes
		$this->common->userBibCondition('resourcequoteResourceId');
		$this->db->formatConditions($this->db->formatFields('resourcequotetextText') . $search);
		$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
			array(array('resourcequoteResourceId' => 'rId'), array('resourcequotetextText' => 'text'))));
// paraphrases
		$this->common->userBibCondition('resourceparaphraseResourceId');
		$this->db->formatConditions($this->db->formatFields('resourceparaphrasetextText') . $search);
		$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
			array(array('resourceparaphraseResourceId' => 'rId'), array('resourceparaphrasetextText' => 'text'))));
		if($this->session->getVar('setup_Write'))
		{
// quote comments
			$this->common->userBibCondition('resourcequoteResourceId');
			$this->db->formatConditions($this->db->formatFields('resourcequotecommentComment') . $search);
			$this->db->formatConditions(array('resourcequotecommentAddUserIdQuote' => $this->session->getVar('setup_UserId')));
			$this->db->leftJoin('resource_quote_comment', 'resourcequotecommentQuoteId', 'resourcequoteId');
			$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
				array(array('resourcequoteResourceId' => 'rId'), array('resourcequotecommentComment' => 'text'))));
// paraphrase comments
			$this->common->userBibCondition('resourceparaphraseResourceId');
			$this->db->formatConditions($this->db->formatFields('resourceparaphrasecommentComment') . $search);
			$this->db->formatConditions(array('resourceparaphrasecommentAddUserIdParaphrase' => $this->session->getVar('setup_UserId')));
			$this->db->leftJoin('resource_paraphrase_comment', 'resourceparaphrasecommentParaphraseId', 'resourceparaphraseId');
			$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
				array(array('resourceparaphraseResourceId' => 'rId'), array('resourceparaphrasecommentComment' => 'text'))));
// musings
			$this->common->userBibCondition('resourcemusingResourceId');
			$this->db->formatConditions($this->db->formatFields('resourcemusingtextText') . $search);
			$this->db->formatConditions(array('resourcemusingtextAddUserIdMusing' => $this->session->getVar('setup_UserId')));
			$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
			$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_musing', 
				array(array('resourcemusingResourceId' => 'rId'), array('resourcemusingtextText' => 'text'))));
		}
		$resultset = $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, array('rId', 'text'), 
			$this->db->subQuery($this->db->union($unions), 'u')));
		while($row = $this->db->fetchRow($resultset))
		{
			$ids = array();
			preg_match_all('/\[cite\](\d+)/', $row['text'], $match);
			foreach($match[1] as $id)
			{
				if(!array_key_exists($id, $this->citeIds))
					$this->citeIds[$id][] = $row['rId'];
				else if(array_search($row['rId'], $this->citeIds[$id]) === FALSE)
					$this->citeIds[$id][] = $row['rId'];
				if(array_search($id, $ids) !== FALSE) // already cited in this resource
					continue;
				if(!array_key_exists($id, $this->citedResourceIds))
					$this->citedResourceIds[$id] = 1;
				else
					$this->citedResourceIds[$id]++;
				$ids[] = $id;
			}
		}
	}
// Process and display results
	private function process()
	{
		$sum = $this->sum;
		sort($sum);
		$highestSum = $sum[sizeof($sum) - 1];
		$lowestSum = $sum[0];
		foreach($this->surname as $id => $name)
		{
			$colour = $this->common->colourText($lowestSum, $highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($lowestSum, $highestSum, $this->sum[$id]);
			$citeIds = join(',', array_unique($this->citeList[$id]));
			$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=citeProcessCreator&id=' . $citeIds)) . 
				"&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
// Get creators from db -- only main authors
	private function getCreators()
	{
		$condition = $this->db->formatFields('resourcecreatorResourceId') . $this->db->equal . 
			join($this->db->or . $this->db->formatFields('resourcecreatorResourceId') . 
			$this->db->equal, array_keys($this->citedResourceIds));
		$this->db->formatConditions($condition);
		$this->db->formatConditions($this->db->formatFields('creatorId') . 'IS NOT NULL');
		$this->db->formatConditions(array('resourcecreatorRole' => 1));
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
		$this->db->orderBy('creatorSurname');
		$recordset = $this->db->select('resource_creator', 
			array('resourcecreatorResourceId', 'creatorId', 'creatorPrefix', 'creatorSurname'), TRUE);
		while($row = $this->db->fetchRow($recordset))
			$this->collate($row);
	}
// Add creators to array and sum totals
	private function collate($row)
	{
		$this->surname[$row['creatorId']] = preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorSurname']));
		$this->prefix[$row['creatorId']] = preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorPrefix']));
		if(array_key_exists($row['creatorId'], $this->citeList))
			$this->citeList[$row['creatorId']] = 
			array_merge($this->citeList[$row['creatorId']], $this->citeIds[$row['resourcecreatorResourceId']]);
		else
			$this->citeList[$row['creatorId']] = $this->citeIds[$row['resourcecreatorResourceId']];
		$this->citeList[$row['creatorId']] = array_unique($this->citeList[$row['creatorId']]);
		$this->sum[$row['creatorId']] = sizeof($this->citeList[$row['creatorId']]);
	}
}
?>