<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net

**********************************************************************************/
/**
* BROWSECOLLECTION class
*
* Browse collections as a 'tag cloud'
*
*/
class BROWSECOLLECTION
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
private $collections = array();
private $sum;
private $bib;
private $form;
private $map;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseCollection"));
	}
// display types of collections
	public function init()
	{
		$this->db->groupBy('collectionType');
		$recordset = $this->db->select('collection', 'collectionType');
		if(!$this->db->numRows($recordset))
		{
			$pString = $this->html->p("&nbsp;") . $this->messages->text("misc", "noCollections");
			GLOBALS::buildOutputString($pString);
		}
// Add 'ALL' to array
		$collections[0] = $this->messages->text("collection", 'all');
		while($row = $this->db->fetchRow($recordset))
		{
			if(!$row['collectionType'])
				continue;
			$collections[$row['collectionType']] = $this->messages->text("collection", $row['collectionType']);
		}
		$pString = $this->form->formHeader('browse_BROWSECOLLECTION_CORE');
		$pString .= $this->form->hidden('method', 'display');
		$pString .= $this->form->selectFBoxValue($this->messages->text("collection", 'chooseTypeBrowse'), 
			"CollectionType", $collections, 10);
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE);
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// display collections
	public function display()
	{
		$collectionType = $this->vars['CollectionType'];
		$queryString = 'action=browse_BROWSECOLLECTION_CORE&method=display&CollectionType=' . $collectionType;
		$this->sum = $this->collection = array();
		$this->getCollections($collectionType);
		if(empty($this->collections))
		{
			$pString = $this->html->p("&nbsp;") . $this->messages->text("misc", "noCollections");
			GLOBALS::buildOutputString($pString);
		}
		natcasesort($this->sum);
//		natcasesort($this->collections);
		$this->collections = $this->common->paging($this->collections);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process(), "center");
		$this->common->pagingLinks($queryString);
		GLOBALS::buildOutputString($pString);
	}
// Process and display results
	private function process()
	{
		$lowestSum = current($this->sum);
		$highestSum = end($this->sum);
		foreach($this->collections as $id => $name)
		{
			$colour = $this->common->colourText($lowestSum, $highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($lowestSum, $highestSum, $this->sum[$id]);
			$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=collectionProcess&id=' . $id)) . 
				"&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
// Get collections from db
	private function getCollections($collectionType)
	{
		$this->common->userBibCondition('resourcemiscId');
		if($collectionType)
			$this->db->formatConditions(array('collectionType' => $collectionType));
		$this->db->formatConditions(array('collectionId' => ' IS NOT NULL'));
		$this->db->leftJoin('collection', 'collectionId', 'resourcemiscCollection');
		$this->db->groupBy('collectionIdId');
		$this->db->orderBy('collectionTitle');
		$recordset = $this->db->selectCounts('resource_misc', 'collectionId', 
			array('resourcemiscCollection', 'collectionType', 'collectionTitle', 
			'collectionTitleShort'));
		while($row = $this->db->fetchRow($recordset))
		{
			if(array_key_exists($row['resourcemiscCollection'], $this->collections))
				continue;
			if(!$row['collectionType'])
				continue;
			$this->collate($row);
		}
	}
// Add collections to array and sum totals
	private function collate($row)
	{
		$this->sum[$row['resourcemiscCollection']] = $row['count'];
		$short = $row['collectionTitleShort'] ? " [" . $row['collectionTitleShort'] . ']' : FALSE;
		$title = $row['collectionTitle'] . $short;
		$thisType = $this->messages->text("collection", $row['collectionType']);
		if($thisType)
			$this->collections[$row['resourcemiscCollection']] = 
				preg_replace("/{(.*)}/U", "$1", $this->html->dbToHtmlTidy($title . " [$thisType]"));
		else
			$this->collections[$row['resourcemiscCollection']] = preg_replace("/{(.*)}/U", "$1", $this->html->dbToFormTidy($title));
	}
}
?>
