<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net

**********************************************************************************/
/**
* BROWSECREATOR class
*
* Browse creators as a 'tag cloud'
*
*/
class BROWSECREATOR
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
private $sum;
private $bib;
private $surname;
private $prefix;
private $collectedSurnames = array();
private $initials;
private $sameAs;
private $lowestSum = 1;
private $highestSum = 0;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseCreator"));
	}
// display creators
	public function init()
	{
		$this->sum = $this->surname = $this->prefix = $this->sameAs = array();
		$this->getCreators();
		if(empty($this->surname))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noCreators"));
			return;
		}
		$this->surname = $this->common->paging($this->surname);
		foreach($this->surname as $id => $surname)
		{
			if(!empty($this->prefix) && array_key_exists($id, $this->prefix))
			{
				$this->surname[$id] = $this->prefix[$id] . ' ' . $surname;
				$findName = $this->prefix[$id] . $surname;
			}
			else
				$findName = $surname;
			if(!array_key_exists($findName, $this->collectedSurnames) || ($this->collectedSurnames[$findName] == 1))
				unset($this->initials[$id]);
		}
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process(), "center");
		$this->common->pagingLinks('action=browse_BROWSECREATOR_CORE');
		GLOBALS::buildOutputString($pString);
	}
// Process and display results
	private function process()
	{
		foreach($this->surname as $id => $name)
		{
			if(array_key_exists($id, $this->initials))
				$name .= $this->initials[$id];
			$colour = $this->common->colourText($this->lowestSum, $this->highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($this->lowestSum, $this->highestSum, $this->sum[$id]);
			$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=creatorProcess&id=' . $id)) . 
				"&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
// Get creators from db with occurrences in resources
	private function getCreators()
	{
		$bib = $this->common->userBibCondition('resourcecreatorResourceId');
		$this->db->formatConditions(array('resourcecreatorCreatorId' => ' IS NOT NULL'));
		$subSql = $this->db->selectNoExecute('resource_creator', array('resourcecreatorResourceId', 'resourcecreatorCreatorId'), TRUE, TRUE, TRUE);
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
		$this->db->groupBy(array('resourcecreatorCreatorId', 'creatorPrefix', 'creatorSurname', 
			'creatorSameAs', 'creatorInitials', 'creatorFirstname'), TRUE, $this->db->count('resourcecreatorCreatorId') . 
			$this->db->greater . $this->db->tidyInput(0));
		$this->db->orderBy('creatorSurname');
		$recordset = $this->db->selectCounts(FALSE, 'resourcecreatorCreatorId', array('creatorPrefix', 'creatorSurname', 
			'creatorSameAs', 'creatorInitials', 'creatorFirstname'), $this->db->subQuery($subSql, 'rc', FALSE), FALSE);
		while($row = $this->db->fetchRow($recordset))
			$this->collate($row);
		if(!empty($this->sameAs))
		{
			foreach($this->sameAs as $id => $sameAsId)
			{
				if(!array_key_exists($sameAsId, $this->surname))
				{
					$this->db->formatConditions(array('creatorId' => $sameAsId));
					$row = $this->db->fetchRow($this->db->select('creator', array('creatorPrefix', 'creatorSurname')));
					$surname = preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorSurname']));
					$prefix = preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorPrefix']));
					$this->surname[$sameAsId] = $surname;
					$this->prefix[$sameAsId] = $prefix;
					$this->sum[$sameAsId] = 0;
				}
				$this->sum[$sameAsId] += $this->sum[$id];
			}
			foreach($this->sameAs as $id => $sameAsId)
				$this->sum[$id] = $this->sum[$sameAsId];
		}
		$this->sumSort = $this->sum;
		sort($this->sumSort, SORT_NUMERIC);
		$this->lowestSum = array_shift($this->sumSort);
		$this->highestSum = array_pop($this->sumSort);
	}
// Add creators to array and sum totals
	private function collate($row)
	{
		if(!trim($row['creatorSurname']))
			return;
		$surname = preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorSurname']));
		$prefix = preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorPrefix']));
		if(!array_key_exists($prefix . $surname, $this->collectedSurnames))
			$this->collectedSurnames[$prefix . $surname] = 1;
		else
			$this->collectedSurnames[$prefix . $surname]++;
		if($row['creatorFirstname'] || $row['creatorInitials'])
		{
			$firstname = FALSE;
			if($row['creatorFirstname'])
			{
				$split = preg_split('/(?<!^)(?!$)/u', preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorFirstname'])));
				$firstname = $split[0] . '.';
			}
			if($row['creatorInitials'])
				$this->initials[$row['resourcecreatorCreatorId']] = ', ' . $firstname . 
					str_replace(' ', '.', preg_replace("/{(.*)}/U", "$1", stripslashes($row['creatorInitials']))) . '.' ;
			else
				$this->initials[$row['resourcecreatorCreatorId']] = ', ' . $firstname;
		}
		$this->surname[$row['resourcecreatorCreatorId']] = $surname;
		$this->prefix[$row['resourcecreatorCreatorId']] = $prefix;
		$this->sum[$row['resourcecreatorCreatorId']] = $row['count'];
		if($row['creatorSameAs'])
			$this->sameAs[$row['resourcecreatorCreatorId']] = $row['creatorSameAs'];
	}
}
?>