<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net

**********************************************************************************/
/*****
* BROWSEYEAR class
*
* Browse publication years as a 'tag cloud'
*
*****/
class BROWSEYEAR
{
private $db;
private $vars;
private $common;
private $html;
private $template;
private $messages;
private $years = array();
private $sum;
private $bib;
private $form;
private $map;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "browseYear"));
	}
	public function init()
	{
		$this->sum = array();
		$this->getYears();
		if(empty($this->years))
		{
			GLOBALS::buildOutputString($this->messages->text("misc", "noResources"));
			return;
		}
		natcasesort($this->sum);
//		natcasesort($this->years);
		$this->years = $this->common->paging($this->years);
		$this->common->linksInfo();
		$pString = $this->html->pBrowse($this->process(), "center");
		$this->common->pagingLinks('action=browse_BROWSEYEAR_CORE');
		GLOBALS::buildOutputString($pString);
	}
// Process and display results
	private function process()
	{
		$lowestSum = current($this->sum);
		$highestSum = end($this->sum);
		foreach($this->years as $id => $name)
		{
			$colour = $this->common->colourText($lowestSum, $highestSum, $this->sum[$id]);
			$size = $this->common->sizeText($lowestSum, $highestSum, $this->sum[$id]);
			$links[] = $this->html->aBrowse($colour, $size, $name, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=yearProcess&id=' . base64_encode($id))) . 
				"&nbsp;[" . $this->sum[$id] . "]";
		}
		return join("&nbsp; ", $links);
	}
// Get collections from db
	private function getYears()
	{
		$this->common->userBibCondition('resourceyearId');
		$this->db->leftJoin('resource', 'resourceId', 'resourceyearId');
		$this->db->orderBy('resourceyearYear1', FALSE);
		$recordset = $this->db->selectCounts('resource_year', 'resourceyearYear1');
		while($row = $this->db->fetchRow($recordset))
			$this->collate($row);
	}
// Add collections to array and sum totals
	private function collate($row)
	{
		if(!$row['resourceyearYear1'])
			return;
		$this->years[$row['resourceyearYear1']] = $this->html->dbToHtmlTidy($row['resourceyearYear1']);
		$this->sum[$row['resourceyearYear1']] = $row['count'];
	}
}
?>