<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	CATEGORYTREE class.
*
*****/
class CATEGORYTREE
{
private $db;
private $common;
private $categories;
private $keywords;
private $subcategories;
private $html;
private $messages;
private $table;
private $template;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		include_once("core/browse/BROWSECOMMON.php");
		$this->common = new BROWSECOMMON();
		include_once("core/modules/browse/BROWSECATEGORY.php");
		$this->categories = new BROWSECATEGORY();
		include_once("core/modules/browse/BROWSEKEYWORD.php");
		$this->keywords = new BROWSEKEYWORD();
		include_once("core/modules/browse/BROWSESUBCATEGORY.php");
		$this->subcategories = new BROWSESUBCATEGORY();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
	}
	public function init()
	{
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('categoryTree'));
		GLOBALS::setOutputHeading($this->messages->text("heading", "categoryTree"));
		$this->categories->sum = $this->categories->category = array();
		$this->categories->getCategories();
		$list['info'] = $this->categories->bib;
		$this->template->tpl->assign('resourceListInfo', $list);
		$this->catSum = $this->categories->sum;
		$this->category = $this->categories->category;
// 3 columns in table
		$pString = $this->table->tableStart(FALSE, 1, 0, 0, "left");
		$index = 1;
		$rows = 0;
		foreach($this->category as $catId => $catName)
		{
			$tdKey = $tdSubcat = FALSE;
			$this->keywords->sum = $this->keywords->keyword = $this->subcategories->sum = $this->subcategories->subcategory = array();
			$this->getKeywords($catId);
			$this->kwSum = $this->keywords->sum;
			$keyword = $this->keywords->keyword;
			$this->getSubcategories($catId);
			$this->scSum = $this->subcategories->sum;
			$subcategory = $this->subcategories->subcategory;
			if(($index % 3) == 1)
				$pString .= $this->table->trStart();
			$td = $this->html->b($this->html->a("link", $catName, 'index.php?' . 
				htmlentities('action=list_LISTSOMERESOURCES_CORE&method=categoryProcess&id=' . $catId))) . 
				'&nbsp;[' . $this->catSum[$catId] . ']' . $this->html->br();
			if(!empty($keyword))
			{
				$tdKey = $this->messages->text("misc", "categoryTreeKeywords");
				foreach($keyword as $kwId => $kwName)
				{
					if(!$kwId)
						continue;
					$tdKey .= $this->html->br() . '&nbsp;&nbsp;&nbsp;&nbsp;' . 
						$this->html->a("link", $kwName, 'index.php?' . 
						htmlentities("action=list_LISTSOMERESOURCES_CORE&method=keywordProcess&id=$kwId&catId=$catId")) . 
						'&nbsp;[' . $this->kwSum[$kwId] . ']';
				}
			}
			if(!empty($subcategory))
			{
				$tdSubcat = $this->messages->text("misc", "categoryTreeSubcategories");
				foreach($subcategory as $scId => $scName)
				{
					if(!$scId)
						continue;
					$tdSubcat .= $this->html->br() . '&nbsp;&nbsp;&nbsp;&nbsp;' . 
						$this->html->a("link", $scName, 'index.php?' . 
						htmlentities("action=list_LISTSOMERESOURCES_CORE&method=subcategoryProcess&id=$scId&catId=$catId")) . 
						'&nbsp;[' . $this->scSum[$scId] . ']';
				}
			}
			if($tdKey && $tdSubcat) // two column table within TD element
			{
				$tdBoth = $this->table->tableStart(FALSE, 0, 0, 0, "left");
				$tdBoth .= $this->table->trStart();
				$tdBoth .= $this->table->td($tdKey);
				$tdBoth .= $this->table->td($tdSubcat);
				$tdBoth .= $this->table->trEnd() . $this->table->tableEnd();
				$pString .= $this->table->td($td . $tdBoth);
			}
			else
				$pString .= $this->table->td($td . $tdKey . $tdSubcat);
			if(!($index % 3))
			{
				$rows++;
				$pString .= $this->table->trEnd();
			}
			$index++;
		}
// If we are not still on the first row, add blank cells
		if($rows)
		{
			if(($index % 3) == 2) // 1 populated cell so add 2 blank cells
				$pString .= $this->table->td('&nbsp;') . $this->table->td('&nbsp;');
			else if(!($index % 3)) // 2 populated cells so add 1 blank cell
				$pString .= $this->table->td("&nbsp;");
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
	private function getKeywords($catId)
	{
		$bib = $this->common->userBibCondition('resourcecategoryResourceId');
		$this->db->formatConditions(array('resourcecategoryCategoryId' => $catId));
		$this->db->groupBy('resourcecategoryResourceId');
		$subSubQ = $this->db->subQuery($this->db->selectNoExecute('resource_category', 'resourcecategoryResourceId'), 'c', TRUE, TRUE);
		$this->db->formatConditions($this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL');
		$this->db->leftJoin('resource_keyword', 'resourcekeywordResourceId', 'resourcecategoryResourceId');
		$this->db->groupBy('resourcekeywordKeywordId');
		$subQ = $this->db->subQuery($this->db->selectCountsNoExecute(FALSE, 'resourcekeywordKeywordId', FALSE, $subSubQ, TRUE, TRUE), 'k');
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$this->db->orderBy('keywordKeyword');
		$recordset = $this->db->selectFromSubQuery(FALSE, array('resourcekeywordKeywordId', 'keywordKeyword', 'count'), $subQ);
		while($row = $this->db->fetchRow($recordset))
			$this->keywords->collate($row);
	}
	private function getSubcategories($catId)
	{
		$bib = $this->common->userBibCondition('resourcecategoryResourceId');
		$this->db->leftJoin('subcategory', 'subcategoryId', 'resourcecategorySubcategoryId');
		$this->db->leftJoin('category', 'categoryId', 'subcategoryCategoryId');
		$this->db->formatConditions(array('categoryId' => $catId));
		$having = $this->db->count('resourcecategorySubcategoryId') . $this->db->greater . $this->db->tidyInput(0);
		$this->db->groupBy(array('resourcecategorySubcategoryId', 'subcategorySubcategory'), TRUE, $having);
		$this->db->orderBy('subcategorySubcategory');
		$recordset = $this->db->selectCounts('resource_category', 'resourcecategorySubcategoryId', 'subcategorySubcategory', FALSE, FALSE);
		while($row = $this->db->fetchRow($recordset))
			$this->subcategories->collate($row);
	}
}
?>
