<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	INSERTCITATION class.
*
*	Insert a citation into a tinyMCE textarea
* 
*****/

/*
This is the main file to handle citations.

For users with javascript turned off, the hyperlinks to this page should not display at all.
*/

/**********************************************/


class INSERTCITATION
{
private $db;
private $vars = array();
private $config;
private $loadConfig;
private $stmt;
private $errors;
private $messages;
private $common;
private $session;
private $form;
private $table;
private $html;
private $template;
private $keyword;
private $keywords;
private $creator;
private $creators;
private $userTags;
private $badInput;
private $parsePhrase;
private $commonBib;
private $userBib;
private $input;
private $joinResourceId = 'resourcemiscId';
private $tAlias = 1;
private $words = FALSE;
public $wpMode = FALSE;  // If TRUE, class is called from word processor plugin so needs to deal with metadata too.

// Constructor
	public function __construct()
	{
		include_once("core/startup/FACTORY.php");
		include_once("core/startup/GLOBALS.php");
		$this->config = FACTORY_CONFIG::getInstance();
		$this->config->WIKINDX_PRINT_SQL = FALSE; // default for initial opening of DB 
		$this->db = FACTORY_DB::getInstance();
		if(!empty($_POST))
			$this->vars = $_POST;
		else if(!empty($_GET))
			$this->vars = $_GET;
		GLOBALS::setVars($this->vars, $this->vars);
		GLOBALS::buildOutputString('<script type="text/javascript" src="../../tiny_mce_popup.js"></script>');
		GLOBALS::buildOutputString('<script type="text/javascript" src="js/wikindxCitedialog.js"></script>');
		$this->loadConfig = FACTORY_LOADCONFIG::getInstance();
		$this->loadConfig->loadConfig();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->creator = FACTORY_CREATOR::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->badInput->closeType = 'closePopup';
		$this->parsePhrase = FACTORY_PARSEPHRASE::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		chdir($_SESSION['wikindxBasePath']); // tinyMCE changes the phpbasepath
		if($this->session->getVar('tinyMCE_mode') == 'wordProcessor')
			$this->wpMode = TRUE;
	}
// all methods pass through here
	public function init($error = FALSE)
	{
//First check, do we have resources?
		if(!$this->db->fetchOne($this->db->select('database_summary', 'databaseSummaryTotalResources')))
		{
			$pString = $this->messages->text('misc', 'noResources');
			GLOBALS::buildOutputString($pString);
			FACTORY_CLOSEPOPUP::getInstance();
		}
		if(!array_key_exists('PagingStart', $this->vars)) // reset paging counter and clear session
		{
			$this->session->delVar('mywikindx_PagingStart');
			$this->session->delVar('mywikindx_PagingStartAlpha');
			$this->session->clearArray("citeSearch");
		}
		$pString = $error ? $error : '';
		if(!array_key_exists('method', $this->vars))
			$pString .= $this->displayForm();
		else if(array_key_exists('method', $this->vars) && ($this->vars['method'] == 'reset'))
			$pString .= $this->reset();
		else if(array_key_exists('method', $this->vars) && ($this->vars['method'] == 'process'))
		{
			$this->input = $this->checkInput();
			$pString .= $this->displayForm();
			$pString .= $this->html->hr();
			$this->process();
		}
		GLOBALS::buildOutputString($pString);
		FACTORY_CLOSEPOPUP::getInstance();
	}

// display form
	private function displayForm()
	{
		$this->userBib = $this->session->getVar("mywikindx_Bibliography_use");
// load the session
		$sessionVars = $this->session->getArray("citeSearch");
		$pString = $this->html->h($this->messages->text("heading", "addCitation"), FALSE, 3);
		$pString .= $this->form->formHeaderVisibleAction("dialog.php", "searchInsertCitation");
		$pString .= $this->form->hidden("method", "process");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$this->grabCreators();
		if($this->creators)
			$pString .= $this->table->td($this->makeFormMultiple("creator", $this->creators));
		else
			$pString .= $this->table->td("&nbsp;");
// There may not yet be keywords in the database.
		$this->grabKeywords();
		if($this->keywords)
			$pString .= $this->table->td($this->makeFormMultiple("keyword", $this->keywords));
		else
			$pString .= $this->table->td("&nbsp;");
// There may not yet be user tags in the database.
		$this->grabUserTags();
		if(!empty($this->userTags))
			$pString .= $this->table->td($this->makeFormMultiple("userTag", $this->userTags));
		else
			$pString .= $this->table->td("&nbsp;");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->hr();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$fields = $this->searchFields();
		$this->makeRadioButtons('field');
		$pString .= $this->table->td($this->makeFormMultiple("field", $fields, TRUE));
		$this->radioButtons = FALSE;
		$word = $this->session->issetVar("citeSearch_Word") ? 
			htmlspecialchars(stripslashes($this->session->getVar("citeSearch_Word")), ENT_QUOTES) : FALSE;
		$checked = $this->session->issetVar("citeSearch_Partial") ? 'CHECKED' : FALSE;
		$pString .= $this->table->td($this->form->textInput($this->messages->text("search", "word"), "citeSearch_Word", $word, 40)
			. $this->html->p($this->messages->text("search", "partial")
			. ":&nbsp;&nbsp;"
			. $this->form->checkbox(FALSE, "citeSearch_Partial", $checked)));
		/*$reset = $this->html->p($this->form->formHeader("cite_INSERTCITATION_CORE") . $this->form->hidden("method", "reset") . 
			$this->form->formSubmit("Reset") . $this->form->formEnd());*/
		/*$reset = "<input type=\"reset\" value=\"Reset\">";*/
		$reset = $this->form->formSubmit("Reset", "onclick=\"document.forms['searchInsertCitation'].elements['method'].value='reset';\"", 'cancel');
		$pString .= $this->table->td($this->form->formSubmit("Search", FALSE, 'insert') . $reset, FALSE, FALSE, "bottom");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		return $pString;
	}
// Return array of database fields to perform the search on
	private function searchFields()
	{
		$userBib = $this->session->getVar("mywikindx_Bibliography_use");
		$fields = array("title"	=>	$this->messages->text("search", "title"));
		if($userBib)
			$this->commonBib->userBibCondition('resourcecreatorId');
		if($this->db->fetchOne($this->db->selectCount('resource_creator', 'resourcecreatorId')))
			$fields['creator'] = $this->messages->text("search", "creator");
		if($userBib)
			$this->commonBib->userBibCondition('resourcetextId');
		if($this->db->fetchOne($this->db->selectCount('resource_text', 'resourcetextId')))
			$fields['note'] = $this->messages->text("search", "note");
		if($this->db->fetchOne($this->db->selectCount('resource_text', 'resourcetextId')))
			$fields['abstract'] = $this->messages->text("search", "abstract");
		if($userBib)
			$this->commonBib->userBibCondition('resourcequoteId');
		if($this->db->fetchOne($this->db->selectCount('resource_quote', 'resourcequoteId')))
			$fields['quote'] = $this->messages->text("search", "quote");
		if($this->wpMode && $this->session->getVar('setup_UserId'))
		{
			if($userBib)
				$this->commonBib->userBibCondition('resourcequoteResourceId');
			if($this->db->fetchOne($this->db->selectCount('resource_quote_comment', 'resourcequotecommentId')))
				$fields['quoteComment'] = $this->messages->text("metadata", "quoteComments");
		}
		if($userBib)
			$this->commonBib->userBibCondition('resourceparaphraseId');
		if($this->db->fetchOne($this->db->selectCount('resource_paraphrase', 'resourceparaphraseId')))
			$fields['paraphrase'] = $this->messages->text("search", "paraphrase");
		if($this->wpMode && $this->session->getVar('setup_UserId'))
		{
			if($userBib)
				$this->commonBib->userBibCondition('resourceparaphraseResourceId');
			if($this->db->fetchOne($this->db->selectCount('resource_paraphrase_comment', 'resourceparaphrasecommentId')))
				$fields['paraphraseComment'] = $this->messages->text("metadata", "paraphraseComments");
		}
		if($userBib)
			$this->commonBib->userBibCondition('resourcemusingId');
		if($this->db->fetchOne($this->db->selectCount('resource_musing', 'resourcemusingId')))
			$fields['musing'] = $this->messages->text("search", "musing");
		if(is_array($this->keywords))
			$fields['keyword'] = $this->messages->text("resources", "keyword");
		if(!empty($this->userTags))
			$fields['userTag'] = $this->messages->text("select", "userTag");
// Add any used custom fields
		$subQ = $this->db->subQuery($this->db->selectNoExecute('resource_custom', 'resourcecustomCustomId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('customId') . $this->db->inClause($subQ));
		$recordset = $this->db->select('custom', array('customId', 'customLabel', 'customSize'));
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['customSize'] == 'S')
				$fields['Custom_S_' . $row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
			else
				$fields['Custom_L_' . $row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
		}
		return $fields;
	}
// Reset the form and clear the session
	private function reset()
	{
		$this->session->clearArray('citeSearch');
		$this->vars = array();
		$this->init();
	}
// make a multiple select box
	private function makeFormMultiple($type, $array, $noIgnore = FALSE)
	{
// add 0 => IGNORE to $array
		if(!$noIgnore)
		{
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($array as $key => $value)
				$temp[$key] = $value;
		}
		else
			$temp = $array;
		if($selected = $this->session->getVar("citeSearch_" . $type))
		{
			$selectedArray = explode(",", $selected);
			$pString = $this->form->selectedBoxValueMultiple($this->messages->text("select", $type), 
				"citeSearch_" . $type, $temp, $selectedArray, 5);
		}
		else
		{
			$pString = $this->form->selectFBoxValueMultiple($this->messages->text("select", $type), 
				"citeSearch_" . $type, $temp, 5);
		}
		if($type != 'field')
		{
			$pString .= $this->html->br() . $this->messages->text("hint", "multiples") . $this->html->br();
			$pString .= $this->radioButtons . $this->html->br();
		}
		return $pString;
	}
// get keywords from database.
	private function grabKeywords()
	{
		$this->keywords = $this->keyword->grabAll($this->userBib, 'resource');
		$this->makeRadioButtons('keyword');
	}
// get user tags from database.
	private function grabUserTags()
	{
		$userTagsObject = FACTORY_USERTAGS::getInstance();
		$this->userTags = $userTagsObject->grabAll($this->userBib, FALSE, TRUE);
		$this->makeRadioButtons('userTag');
	}
// get creators from database.
	private function grabCreators()
	{
		$this->creators = $this->creator->grabAll($this->userBib);
		$this->makeRadioButtons('creator');
	}
// Create radio buttons for AND and OR
	private function makeRadioButtons($type)
	{
		$type = 'citeSearch_' . $type . 'Method';
		if($this->session->getVar($type) == 'AND')
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR') . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
		}
// Default
		else
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . 
				$this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND') . " AND", "small");
		}
		$this->radioButtons = $pString;
	}

// Display results of search
	private function process()
	{
		$this->createTotalSql();
		$queryString = 'dialog.php?method=process';
		$sqlTotal = $this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, 'creator', TRUE, $queryString);
		$sql = $this->stmt->listList(FALSE, FALSE, 'creator');
		$this->common->display($sql, 'cite');
		$citeFields['formheader'] = $this->form->formHeaderName('', 'citeForm');
		$citeFields['pageStart'] = $this->messages->text("cite", "pages") . "&nbsp;&nbsp;" . 
			$this->form->textInput(FALSE, 'pageStart', FALSE, 6, 5);
		$citeFields['pageEnd'] = $this->form->textInput(FALSE, 'pageEnd', FALSE, 6, 5);
// preText and postText for the citation (usually appears within citation parentheses but will be ignored for endnote-style citations)
		$citeFields['preText'] = $this->messages->text("cite", "preText") . "&nbsp;&nbsp;" . 
			$this->form->textInput(FALSE, 'preText', FALSE, 12);
		$citeFields['postText'] = $this->messages->text("cite", "postText") . "&nbsp;&nbsp;" . 
			$this->form->textInput(FALSE, 'postText', FALSE, 12);
		$this->form->insertJscriptAction = "onclick=\"citedialog.insert();\"";
		$citeFields['cite'] = $this->form->formSubmitButton("Cite", "insert");
		$citeFields['formfooter'] = $this->form->formEnd();
		$this->template->tpl->assign('citeFields', $citeFields);
	}
// create the basic sql statement before we apply LIMIT
	private function createTotalSql()
	{
		$this->creatorSql();
		$this->keywordSql();
		$this->userTagSql();
		if(array_key_exists('Word', $this->input))
		{
			$this->parseWord();
			$this->fieldSql('creator');
		}
	}
// add to SQL conditional statement and add fields to fieldArray for creators
	private function creatorSql()
	{
		if(!array_key_exists('creator', $this->input))
			return;
		$selectArray = explode(",", $this->input['creator']);
		if($this->input['creatorMethod'] == 'OR')
		{
			foreach($selectArray as $creator)
			{
				$c[] = $this->db->formatFields('resourcecreatorCreatorId') . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($creator);
				$c[] = $this->db->formatFields('creatorSameAs') . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($creator);
			}
			$c = join(' ' . $this->db->or . ' ' , $c);
			$this->stmt->execCond[] = $c;
			$this->stmt->execJoin['c.resource_creator']['intField'] = 'c.resourcecreatorResourceId';
			$this->stmt->execJoin['c.resource_creator']['extField'] = $this->joinResourceId;
			$this->stmt->execJoin['creator']['intField'] = 'creatorId';
			$this->stmt->execJoin['creator']['extField'] = 'resourcecreatorCreatorId';
		}
		else
		{
			foreach($selectArray as $creator)
			{
				$this->stmt->execCond[] = $this->db->formatFields($this->tAlias . '.resourcecreatorCreatorId') . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($creator) . 
					' ' . $this->db->or . ' ' . 
					$this->db->formatFields('creatorSameAs') . ' ' . 
					$this->db->equal . ' ' . $this->db->tidyInput($creator);
				$this->stmt->execJoin[$this->tAlias . '.resource_creator']['intField'] = $this->tAlias . 			
					'.resourcecreatorResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_creator']['extField'] = $this->joinResourceId;
				$this->stmt->execJoin['creator']['intField'] = 'creatorId';
				$this->stmt->execJoin['creator']['extField'] = $this->tAlias . '.resourcecreatorCreatorId';
				++$this->tAlias;
			}
		}
	}
// add to SQL conditional statement and add fields to fieldArray for keywords
	private function keywordSql()
	{
		if(!array_key_exists('keyword', $this->input))
			return;
		$keywords = explode(",", $this->input['keyword']);
		if($this->input['keywordMethod'] == 'OR')
		{
			$this->stmt->execCondOneField['resourcekeywordKeywordId'] = $keywords;
			$this->stmt->execJoin['resource_keyword']['intField'] = 'resourcekeywordResourceId';
			$this->stmt->execJoin['resource_keyword']['extField'] = $this->joinResourceId;
		}
		else
		{
			foreach($keywords as $keyword)
			{
				$this->stmt->execCond[] = array($this->tAlias . '.resourcekeywordKeywordId' => $keyword);
				$this->stmt->execJoin[$this->tAlias . '.resource_keyword']['intField'] = $this->tAlias . '.resourcekeywordResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_keyword']['extField'] = $this->joinResourceId;
				++$this->tAlias;
			}
		}
	}
// add to SQL conditional statement and add fields to fieldArray for user tags
	private function userTagSql()
	{
		if(!array_key_exists('userTag', $this->input))
			return;
		$userTags = explode(",", $this->input['userTag']);
		if($this->input['userTagMethod'] == 'OR')
		{
			$this->stmt->execCondOneField['resourceusertagsTagId'] = $userTags;
			$this->stmt->execJoin['resource_user_tags']['intField'] = 'resourceusertagsResourceId';
			$this->stmt->execJoin['resource_user_tags']['extField'] = $this->joinResourceId;
		}
		else
		{
			foreach($userTags as $usertag)
			{
				$this->stmt->execCond[] = array($this->tAlias . '.resourceusertagsTagId' => $usertag);
				$this->stmt->execJoin[$this->tAlias . '.resource_user_tags']['intField'] = $this->tAlias . '.resourceusertagsResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_user_tags']['extField'] = $this->joinResourceId;
				++$this->tAlias;
			}
		}
	}
// parse the search word(s)
	private function parseWord()
	{
		$this->words = $this->parsePhrase->parse($this->input);
		if(!$this->words)
		{
			$this->vars = array();
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		}
	}
// add to SQL conditional statement and add fields to fieldArray for database fields
	private function fieldSql($order)
	{
		if(!array_key_exists('field', $this->input))
			return;
		$conditionArray = array();
		$fields = explode(",", $this->input['field']);
		if(sizeof($fields) > 1) // UNIONs
			$this->joinResourceId = 'id';
		else if($order == 'attachments')
			$this->joinResourceId = 'resourceattachmentsResourceId';
		else
			$this->joinResourceId = 'resourcemiscId';
// If $this->input['FieldMethod'] == 'OR', we use UNIONS for efficient SQL statements
		foreach($fields as $field)
		{
			if(strpos($field, 'Custom_') === 0)
			{
				$split = explode('_', $field);
				if($split[1] == 'S')
					$searchField = 'resourcecustomShort';
				else
					$searchField = 'resourcecustomLong';
				$cId = $split[2];
				if(sizeof($fields) > 1) // UNIONs
				{
					$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields($searchField), $this->words);
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_custom', 
						array(array('resourcecustomResourceId' => 'id'))));
				}
				else
				{
					$wc = str_replace('!WIKINDXFIELDWIKINDX!', 
						$this->db->formatFields($this->tAlias . '.' . $searchField), $this->words);
					$this->stmt->execCond[] = "($wc)";
					$this->stmt->execJoin[$this->tAlias . '.resource_custom']['intField'] = $this->tAlias . '.resourcecustomResourceId';
					$this->stmt->execJoin[$this->tAlias . '.resource_custom']['extField'] = $this->joinResourceId;
					++$this->tAlias;
				}
			}
			else if($field == 'title')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceTitleSort'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource', 
						array(array('resourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = "($wc)";
					$this->stmt->execJoin['resource']['intField'] = 'resourceId';
					$this->stmt->execJoin['resource']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'note')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextNote'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
						array(array('resourcetextId' => 'id'))));
				}
				else
				{
					$conditionArray[] = "($wc)";
					$this->stmt->execJoin['resource_text']['intField'] = 'resourcetextId';
					$this->stmt->execJoin['resource_text']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'abstract')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextAbstract'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
						array(array('resourcetextId' => 'id'))));
				}
				else
				{
					$conditionArray[] = "($wc)";
					$this->stmt->execJoin['resource_text']['intField'] = 'resourcetextId';
					$this->stmt->execJoin['resource_text']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'quote')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcequotetextText'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
						array(array('resourcequoteResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = "($wc)";
					$this->stmt->execJoin['resource_quote']['intField'] = 'resourcequoteResourceId';
					$this->stmt->execJoin['resource_quote']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_quote_text']['intField'] = 'resourcequotetextId';
					$this->stmt->execJoin['resource_quote_text']['extField'] = 'resourcequoteId';
				}
				$this->db->formatConditions("($wc)");
				$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
				$union = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourcequoteResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataTextCite[] = str_replace('DISTINCT', $this->db->formatFields('resourcequotetextText') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if(($field == 'quoteComment') && ($userId = $this->session->getVar('setup_UserId')))
			{
				$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
				$this->db->formatConditions($this->db->formatFields('usergroupsusersGroupId') . $this->db->equal . 
					$this->db->formatFields('resourcequotecommentPrivate'));
				$subSql = $this->db->selectNoExecute('user_groups_users', 'usergroupsusersId', FALSE, TRUE, TRUE);
				$subject = $this->db->formatFields('resourcequotecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('N') 
					. $this->db->and . 
					$this->db->formatFields('resourcequotecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('Y');
				$case1 = $this->db->caseWhen($subject, FALSE, $subSql, FALSE, FALSE);
				$subject = $this->db->formatFields('resourcequotecommentPrivate') . $this->db->equal . $this->db->tidyInput('Y');
				$result = $this->db->formatFields('resourcequotecommentAddUserIdQuote') . $this->db->equal . $this->db->tidyInput($userId);
				$case2 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
				$subject = $this->db->formatFields('resourcequotecommentPrivate') . $this->db->equal . $this->db->tidyInput('N');
				$result = $this->db->tidyInput(1);
				$case3 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
				$qCommentUserCheckCond = $case1 . $this->db->or . $case2 . $this->db->or . $case3;
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcequotecommentComment'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->leftJoin('resource_quote_comment', 'resourcequotecommentQuoteId', 'resourcequoteId');
					$this->db->formatConditions("($wc)" . $this->db->and . "($qCommentUserCheckCond)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
						array(array('resourcequoteResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = '(' . "($wc)" . $this->db->and . '(' . $qCommentUserCheckCond . '))';
					$this->stmt->execJoin['resource_quote']['intField'] = 'resourcequoteResourceId';
					$this->stmt->execJoin['resource_quote']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_quote_comment']['intField'] = 'resourcequotecommentQuoteId';
					$this->stmt->execJoin['resource_quote_comment']['extField'] = 'resourcequoteId';
				}
				$this->db->formatConditions($wc . $this->db->and . '(' . $qCommentUserCheckCond . ')');
				$this->db->leftJoin('resource_quote_comment', 'resourcequotecommentQuoteId', 'resourcequoteId');
				$union = $this->db->selectNoExecute('resource_quote', array(array('resourcequoteResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourcequoteResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataTextCite[] = str_replace('DISTINCT', $this->db->formatFields('resourcequotecommentComment') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if($field == 'paraphrase')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceparaphrasetextText'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
						array(array('resourceparaphraseResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = "($wc)";
					$this->stmt->execJoin['resource_paraphrase']['intField'] = 'resourceparaphraseResourceId';
					$this->stmt->execJoin['resource_paraphrase']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_paraphrase_text']['intField'] = 'resourceparaphrasetextId';
					$this->stmt->execJoin['resource_paraphrase_text']['extField'] = 'resourceparaphraseId';
				}
				$this->db->formatConditions("($wc)");
				$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
				$union = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourceparaphraseResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataTextCite[] = str_replace('DISTINCT', $this->db->formatFields('resourceparaphrasetextText') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if(($field == 'paraphraseComment') && ($userId = $this->session->getVar('setup_UserId')))
			{
				$this->db->formatConditions(array('usergroupsusersUserId' => $userId));
				$this->db->formatConditions($this->db->formatFields('usergroupsusersGroupId') . $this->db->equal . 
					$this->db->formatFields('resourceparaphrasecommentPrivate'));
				$subSql = $this->db->selectNoExecute('user_groups_users', 'usergroupsusersId', FALSE, TRUE, TRUE);
				$subject = $this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('N') 
					. $this->db->and . 
					$this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->notEqual . $this->db->tidyInput('Y');
				$case1 = $this->db->caseWhen($subject, FALSE, $subSql, FALSE, FALSE);
				$subject = $this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->equal . $this->db->tidyInput('Y');
				$result = $this->db->formatFields('resourceparaphrasecommentAddUserIdParaphrase') . $this->db->equal . $this->db->tidyInput($userId);
				$case2 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
				$subject = $this->db->formatFields('resourceparaphrasecommentPrivate') . $this->db->equal . $this->db->tidyInput('N');
				$result = $this->db->tidyInput(1);
				$case3 = $this->db->caseWhen($subject, FALSE, $result, FALSE, FALSE);
				$pCommentUserCheckCond = $case1 . $this->db->or . $case2 . $this->db->or . $case3;
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceparaphrasecommentComment'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->leftJoin('resource_paraphrase_comment', 'resourceparaphrasecommentParaphraseId', 'resourceparaphraseId');
					$this->db->formatConditions("($wc)" . $this->db->and . "($pCommentUserCheckCond)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
						array(array('resourceparaphraseResourceId' => 'id'))));
				}
				else
				{			
					$conditionArray[] = '(' . "($wc)" . $this->db->and . '(' . $pCommentUserCheckCond . '))';
					$this->stmt->execJoin['resource_paraphrase']['intField'] = 'resourceparaphraseResourceId';
					$this->stmt->execJoin['resource_paraphrase']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_paraphrase_comment']['intField'] = 'resourceparaphrasecommentParaphraseId';
					$this->stmt->execJoin['resource_paraphrase_comment']['extField'] = 'resourceparaphraseId';
				}
				$this->db->formatConditions($wc . $this->db->and . '(' . $pCommentUserCheckCond . ')');
				$this->db->leftJoin('resource_paraphrase_comment', 'resourceparaphrasecommentParaphraseId', 'resourceparaphraseId');
				$union = $this->db->selectNoExecute('resource_paraphrase', array(array('resourceparaphraseResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourceparaphraseResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataTextCite[] = str_replace('DISTINCT', $this->db->formatFields('resourceparaphrasecommentComment') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if($field == 'musing')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcemusingtextText'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_musing', 
						array(array('resourcemusingResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = "($wc)";
					$this->stmt->execJoin['resource_musing']['intField'] = 'resourcemusingResourceId';
					$this->stmt->execJoin['resource_musing']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_musing_text']['intField'] = 'resourcemusingtextId';
					$this->stmt->execJoin['resource_musing_text']['extField'] = 'resourcemusingId';
				}
				$this->db->formatConditions("($wc)");
				$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
				$union = $this->db->selectNoExecute('resource_musing', array(array('resourcemusingResourceId' => 'rId')), 
					TRUE, TRUE, TRUE);
				$union = str_replace('WHERE', 'WHERE (' . $this->db->formatFields('resourcemusingResourceId') . ' = RESID) AND ', $union);
				$this->common->metadataTextCite[] = str_replace('DISTINCT', $this->db->formatFields('resourcemusingtextText') . 
					' AS ' . $this->db->formatFields('text') . ', ', $union);
			}
			else if($field == 'keyword')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('keywordKeyword'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
					$this->db->formatConditions("($wc)");
// Need this check because resource_keyword tables also stores metadata keywords, not just resource keywords
					$this->db->formatConditions($this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL');
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_keyword', 
						array(array('resourcekeywordResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execCond[] = $this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL';
					$this->stmt->execJoin['resource_keyword']['intField'] = 'resourcekeywordResourceId';
					$this->stmt->execJoin['resource_keyword']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['keyword']['intField'] = 'keywordId';
					$this->stmt->execJoin['keyword']['extField'] = 'resourcekeywordKeywordId';
				}
			}
			else if($field == 'creator')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('creatorSurname'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_creator', 
						array(array('resourcecreatorResourceId' => 'id'))));
				}
				else
				{

					$creatorsCond = explode(' AND ', $wc);
					if(sizeof($creatorsCond) > 1)
					{
						$creatorStmts = $creatorAlias = array();
						foreach($creatorsCond AS $creatorCond)
						{
							$this->db->formatConditions("$creatorCond");
							$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
							$creatorStmts[] = $this->db->subQuery($this->db->selectNoExecute('resource_creator',

								array(array('resourcecreatorResourceId' => 'cId' . $this->tAlias)), FALSE, TRUE, TRUE),
								$this->tAlias, FALSE);
							$creatorAlias[] = 'cId' . $this->tAlias;
							++$this->tAlias;
						}
						$masterCreatorId = $this->db->formatFields(array_shift($creatorAlias));
						foreach($creatorAlias AS $cAlias)
							$creatorCondArray[] = $this->db->equal . $this->db->formatFields($cAlias);
						$this->db->formatConditions($masterCreatorId . join($this->db->and . $masterCreatorId, $creatorCondArray));
						$creatStmt = $this->db->selectNoExecuteFromSubQuery(FALSE, $masterCreatorId,
							$this->db->from . join(', ', $creatorStmts), FALSE, FALSE, TRUE);
						$this->stmt->execCond[] = $this->db->formatFields('resourcemiscId') . $this->db->inClause($creatStmt);
					}
					else
					{
						$this->stmt->execCond[] = $wc;
						$this->stmt->execJoin['c.resource_creator']['intField'] = 'c.resourcecreatorResourceId';
						$this->stmt->execJoin['c.resource_creator']['extField'] = $this->joinResourceId;
						$this->stmt->execJoin['creator']['intField'] = 'creatorId';
						$this->stmt->execJoin['creator']['extField'] = 'c.resourcecreatorCreatorId';
					}
				}
			}
			else if($field == 'userTag')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('usertagsTag'), $this->words);
				if(sizeof($fields) > 1) // UNIONs
				{
					$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
					$this->db->leftJoin('user_tags', 'usertagsId', 'resourceusertagsTagId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_user_tags', 
						array(array('resourceusertagsId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execCond[] = array('usertagsUserId' => $this->session->getVar('setup_UserId'));
					$this->stmt->execJoin['resource_user_tags']['intField'] = 'resourceusertagsResourceId';
					$this->stmt->execJoin['resource_user_tags']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['user_tags']['intField'] = 'usertagsId';
					$this->stmt->execJoin['user_tags']['extField'] = 'resourceusertagsTagId';
				}
			}
		}
		if(sizeof($fields) > 1)
			$this->stmt->unions = $this->db->union($fieldUnions);
		if(!empty($conditionArray))
		{
			$conditionJoin = $this->input['fieldMethod'] == 'OR' ? $this->db->or : $this->db->and;
			$this->stmt->execCond[] = ('(' . join($conditionJoin, $conditionArray) . ')');
		}
	}
// write input to session
	private function writeSession()
	{
// First, write all input with 'citeSearch_' prefix to session
		foreach($this->vars as $key => $value)
		{
			if(preg_match("/^citeSearch_/", $key))
			{
// Is this a multiple select box input?  If so, multiple choices are written to session as 
// comma-delimited string (no spaces).
// Don't write any FALSE or '0' values.
				if(is_array($value))
				{
					if(!$value[0] || ($value[0] == $this->messages->text("misc", "ignore")))
						unset($value[0]);
					if(empty($value))
					{
						$this->session->delVar($key);
						continue;
					}
					$value = implode(",", $value);
				}
				if(!trim($value))
					continue;
				$temp[$key] = trim($value);
			}
		}
// remove search_type, search_partial and search_word if not checked or empty
		if(isset($temp))
		{
			if(!array_key_exists('citeSearch_partial', $temp))
				$this->session->delVar('citeSearch_partial');
			if(!array_key_exists('citeSearch_word', $temp))
				$this->session->delVar('citeSearch_word');
			if(isset($temp))
				$this->session->writeArray($temp);
		}
	}
// validate user input
// Must have at least one of creator, keyword, userTag, searchWord.
	private function checkInput()
	{
		if(array_key_exists('PagingStart', $this->vars))
			return $this->session->getArray("citeSearch");
		$this->writeSession();
		$validInput = FALSE;
		if(!array_key_exists("citeSearch_method", $this->vars))
			$this->session->setVar('citeSearch_method', "OR");
		if(!array_key_exists("citeSearch_creatorMethod", $this->vars))
			$this->session->setVar('citeSearch_creatorMethod', "OR");
		if(!array_key_exists("citeSearch_fieldMethod", $this->vars))
			$this->session->setVar('citeSearch_fieldMethod', "OR");
		if(!array_key_exists("citeSearch_keywordMethod", $this->vars))
			$this->session->setVar('citeSearch_keywordMethod', "OR");
		if(!array_key_exists("citeSearch_userTagMethod", $this->vars))
			$this->session->setVar('citeSearch_userTagMethod', "OR");
		if(!array_key_exists("citeSearch_field", $this->vars))
			$this->session->setVar('citeSearch_field', "title");
		if(array_key_exists("citeSearch_Word", $this->vars) && trim($this->vars["citeSearch_Word"]))
			$validInput = TRUE;
		else if(array_key_exists("citeSearch_creator", $this->vars) && 
			((sizeof($this->vars['citeSearch_creator']) > 1) || $this->vars['citeSearch_creator'][0]))
			$validInput = TRUE;
		else if(array_key_exists("citeSearch_keyword", $this->vars) && 
			((sizeof($this->vars['citeSearch_keyword']) > 1) || $this->vars['citeSearch_keyword'][0]))
			$validInput = TRUE;
		else if(array_key_exists("citeSearch_userTag", $this->vars) && 
			((sizeof($this->vars['citeSearch_userTag']) > 1) || $this->vars['citeSearch_userTag'][0]))
			$validInput = TRUE;
		if(!$validInput)
		{
			$this->vars = array();
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		}
		return $this->session->getArray("citeSearch");
	}
}
?>
