<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* EDITCOLLECTION class
*
*/
class EDITCOLLECTION
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $collection;
private $html;
private $table;
private $gatekeep;
private $badInput;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->collection = FACTORY_COLLECTION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "collection") . ")"));
	}
// check we are allowed to edit and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->clearArray('edit');
		$this->db->groupBy('collectionType');
		$recordset = $this->db->select('collection', 'collectionType');
		if(!$this->db->numRows($recordset))
		{
			$pString = $this->html->p("&nbsp;") . $this->messages->text("misc", "noCollections");
			GLOBALS::buildOutputString($pString);
		}
// Add 'ALL' to array
		$collections[0] = $this->messages->text("collection", 'all');
		while($row = $this->db->fetchRow($recordset))
		{
			if(!$row['collectionType'])
				continue;
			$collections[$row['collectionType']] = $this->messages->text("collection", $row['collectionType']);
		}
		$pString = $message;
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$td = $this->form->formHeader('edit_EDITCOLLECTION_CORE');
		$td .= $this->form->hidden("method", "editChooseCollection");
		$td .= $this->form->selectFBoxValue(FALSE, "edit_CollectionType", $collections, 10);
		$td .= $this->html->p($this->form->formSubmit("Proceed"));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
// check we are allowed to edit and load appropriate method
	public function editChooseCollection()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$collectionType = $this->vars['edit_CollectionType'];
		include_once("core/collection/COLLECTIONMAP.php");
		$this->map = new COLLECTIONMAP();
		if($collectionType)
			$this->db->formatConditions(array('collectionType' => $collectionType));
		$this->db->formatConditions($this->db->formatFields('collectionId'), ' IS NOT NULL');
		$this->db->leftJoin('collection', 'collectionId', 'resourcemiscCollection');
		$this->db->groupBy('collectionId');
		$this->db->orderBy('collectionTitle');
		$recordset = $this->db->selectCounts('resource_misc', 'collectionId', 
			array('resourcemiscCollection', 'collectionTitle', 'collectionTitleShort'));
		while($row = $this->db->fetchRow($recordset))
		{
			$short = $row['collectionTitleShort'] ? 
				" [" . $row['collectionTitleShort'] . ']' : FALSE;
			$title = $row['collectionTitle'] . $short;
			$collections[$row['collectionId']] = $this->html->dbToFormTidy($title, TRUE);
		}
		$pString = $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$td = $this->form->formHeader('edit_EDITCOLLECTION_CORE');
		$td .= $this->form->hidden("method", "editDisplayCollection");
		$td .= $this->form->hidden("title", $title);
		$td .= $this->form->hidden("shortTitle", $short);
		$td .= $this->form->selectFBoxValue(FALSE, "edit_CollectionId", $collections, 20);
		$td .= $this->html->p($this->form->formSubmit("Proceed"));
		$td .= $this->form->formEnd();
		$pString .= $this->table->td($td);
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		GLOBALS::buildOutputString($pString);
	}
// Display interface to edit collection
	public function editDisplayCollection()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->setVar('editLock', FALSE);
		if(!array_key_exists('edit_CollectionId', $this->vars) || !$this->vars['edit_CollectionId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('title', $this->vars) || !$this->vars['title'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		else
			$title = $this->vars['title'];
		if(!array_key_exists('shortTitle', $this->vars) || !$this->vars['shortTitle'])
			$short = $this->vars['shortTitle'];
		else
			$short = FALSE;
		$tinymce = FACTORY_LOADTINYMCE::getInstance();
		$this->db->formatConditions(array('collectionId' => $this->vars['edit_CollectionId']));
		$recordset = $this->db->select('collection', array('collectionTitle', 'collectionTitleShort', 'collectionType'));
		$row = $this->db->fetchRow($recordset);
		$title = $this->html->dbToTinyMCE($row['collectionTitle']);
		$titleShort = $this->html->dbToTinyMCE($row['collectionTitleShort']);
		$pString = $this->form->formHeader('edit_EDITCOLLECTION_CORE');
		$pString .= $this->form->hidden("method", "edit");
		$pString .= $this->form->hidden("edit_CollectionId", $this->vars['edit_CollectionId']);
		$pString .= $this->form->hidden("edit_CollectionType", $row['collectionType']);
		$pString .= $tinymce->loadBasicTextinput(array('edit_CollectionTitle'), 400);
//		$pString .= $this->html->p($this->form->textInput($this->messages->text('resources', 'collection'), 
//			'edit_CollectionTitle', $title, 60) . ' ' . $this->html->span('*', 'required')); 
		$pString .= $this->html->p($this->form->textAreaInput($this->messages->text('resources', 'collection'), 
			'edit_CollectionTitle', $title, 60, 1) . ' ' . $this->html->span('*', 'required')); 
//		$pString .= $this->html->div('edit_CollectionTitle', $title); 
		$pString .= $this->html->p($this->form->textInput($this->messages->text('resources', 'collectionShort'), 
			'edit_CollectionTitleShort', $titleShort, 30)); 
		$pString .= $this->form->formSubmit("Edit") . $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// write to the database
	public function edit()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "collection"), $this, 'init');
		if(!array_key_exists('edit_CollectionId', $this->vars) || !$this->vars['edit_CollectionId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('edit_CollectionType', $this->vars)) // can be NULL
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$title = array_key_exists('edit_CollectionTitle', $this->vars) ? trim($this->vars['edit_CollectionTitle']) : FALSE;
		if(!$title)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$titleShort = array_key_exists('edit_CollectionTitleShort', $this->vars) ? 
			trim($this->vars['edit_CollectionTitleShort']) : FALSE;
		if($collectionExistId = $this->collection->checkExists($this->vars['edit_CollectionId'], $title, $titleShort, $this->vars['edit_CollectionType']))
		{
			if($collectionExistId != $this->vars['edit_CollectionId'])
				return $this->confirmDuplicate($collectionExistId, $title, $titleShort);
		}
		$this->db->formatConditions(array('collectionId' => $this->vars['edit_CollectionId']));
		$updateArray['collectionTitle'] = $title;
		if($titleShort)
			$updateArray['collectionTitleShort'] = $titleShort;
		else
		{
			$this->db->updateNull('collection', 'collectionTitleshort');
			$this->db->formatConditions(array('collectionId' => $this->vars['edit_CollectionId']));
		}
		$this->db->update('collection', $updateArray);
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// remove cache files for collections
		$this->db->deleteCache('cacheResourceCollections');
		$this->db->deleteCache('cacheMetadataCollections');
		$this->db->deleteCache('cacheResourceCollectionTitles');
		$this->db->deleteCache('cacheResourceCollectionShorts');
// send back to editDisplay with success message
		$this->init($this->success->text("collection"));
	}
// write to the database.
	function editConfirm()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if($this->session->getVar('editLock'))
			$this->badInput->close($this->errors->text("done", "collection"), $this, 'init');
		if(!array_key_exists('edit_CollectionId', $this->vars) || !$this->vars['edit_CollectionId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('edit_CollectionExistId', $this->vars) || !$this->vars['edit_CollectionExistId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$editId = $this->vars['edit_CollectionId'];
		$existId = $this->vars['edit_CollectionExistId'];
// Delete old creator
		$this->db->formatConditions(array('collectionId' => $editId));
		$this->db->delete('collection');
// remove cache files for collections
		$this->db->deleteCache('cacheResourceCollections');
		$this->db->deleteCache('cacheMetadataCollections');
		$this->db->deleteCache('cacheResourceCollectionTitles');
		$this->db->deleteCache('cacheResourceCollectionShorts');
// Select all resources referencing this old collection and replace reference with existing  collection
		$this->db->formatConditions(array('resourcemiscCollection' => $editId));
		$recordset = $this->db->select('resource_misc', array('resourcemiscId', 'resourcemiscCollection'));
		while($row = $this->db->fetchRow($recordset))
		{
			$this->db->formatConditions(array('resourcemiscId' => $row['resourcemiscId']));
			$this->db->update('resource_misc', array('resourcemiscCollection' => $existId));
		}
// lock reload
		$this->session->setVar('editLock', TRUE);
// Clear session
		$this->session->clearArray("edit");
// send back to editDisplay with success message
		$this->init($this->success->text("collection"));
	}
/**
* The new collection equals one already in the database. Confirm that this edited one is to be removed and 
* all references to it replaced by the existing one.
*/
	private function confirmDuplicate($collectionExistId, $title, $titleShort)
	{
		$pString = $this->errors->text("warning", "collectionExists");
		$pString .= $this->html->p($this->messages->text("misc", "collectionExists"));
		$pString .= $this->form->formHeader("edit_EDITCOLLECTION_CORE");
		$pString .= $this->form->hidden("edit_CollectionId", $this->vars['edit_CollectionId']);
		$pString .= $this->form->hidden("edit_CollectionExistId", $collectionExistId);
		$pString .= $this->form->hidden("method", 'editConfirm');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
}
?>