<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* EDITCREATOR class
*
*/
class EDITCREATOR
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $creator;
private $html;
private $table;
private $gatekeep;
private $badInput;
private $ajax;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->creator = FACTORY_CREATOR::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . $this->messages->text("resources", "creator") . ")"));
	}
// check we are allowed to edit and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$this->session->clearArray('edit');
		$creators = $this->creator->grabAll();
		if(!$creators)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noCreators'));
			return;
		}
		$pString = $message ? $this->html->p($message, "error", "center") : FALSE;
		$pString .= $this->form->formHeader('edit_EDITCREATOR_CORE');
		$pString .= $this->form->hidden("method", "edit");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->selectFBoxValue(FALSE, "creatorIds", $creators, 20));
		$pString .= $this->table->td($this->transferArrow());
		$pString .= $this->table->td($this->html->div('creatorDiv', $this->displayName(TRUE)));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit("Edit"));
		$pString .= $this->form->formEnd();
		$this->ajax->loadJavascript();
		GLOBALS::buildOutputString($pString);
	}
	private function transferArrow()
	{
		$jsonArray = array();
		$jScript = 'index.php?action=edit_EDITCREATOR_CORE&method=displayName';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'creatorIds',
			'targetDiv' => 'creatorDiv',
			);
		$image = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
		return $image;
	}
// Display interface to edit chose name
	public function displayName($initialDisplay = FALSE)
	{
		$initials = $prefix = $firstname = $surname = $creatorId = FALSE;
		if(!$initialDisplay)
		{
			$this->db->formatConditions(array('creatorId' => $this->vars['ajaxReturn']));
			$recordset = $this->db->select('creator', array("creatorSurname", "creatorFirstname", "creatorInitials", "creatorPrefix"));
			$row = $this->db->fetchRow($recordset);
			if($row['creatorInitials'])
				$initials = str_replace(" ", ".", $this->html->dbToFormTidy($row['creatorInitials']) . ".");
			if($row['creatorFirstname'])
				$firstname = $this->html->dbToFormTidy($row['creatorFirstname']);
			if($row['creatorPrefix'])
				$prefix = $this->html->dbToFormTidy($row['creatorPrefix']);
			$surname = $this->html->dbToFormTidy($row['creatorSurname']);
			$creatorId = $this->vars['ajaxReturn'];
		}
		$pString = $this->form->hidden("editCreatorId", $creatorId);
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "firstname"), 
			"firstname", $firstname, 30, 255));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "initials"), 
			"initials", $initials, 6, 255) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "initials")), 'hint');
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "prefix"), 
			"prefix", $prefix, 11, 10));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("resources", "surname") . ' ' . $this->html->span('*', 'required'), 
			"surname", $surname, 30, 255));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		if($initialDisplay)
			return $pString;
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $pString)));
		FACTORY_CLOSERAW::getInstance();
	}
// write to the database
	public function edit()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if(!array_key_exists('editCreatorId', $this->vars) || !$this->vars['editCreatorId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$surname = array_key_exists('surname', $this->vars) ? trim($this->vars['surname']) : FALSE;
		if(!$surname)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$firstname = trim($this->vars['firstname']);
		$initials = $this->creator->formatInitials(trim($this->vars['initials']));
		$prefix = trim($this->vars['prefix']);
		if($creatorExistId = $this->creator->checkExists($surname, $firstname, $initials, $prefix))
		{
			if($creatorExistId != $this->vars['editCreatorId'])
				return $this->confirmDuplicate($creatorExistId, $surname);
		}
		$updateArray['creatorSurname'] = $surname;
		if($firstname)
			$updateArray['creatorFirstname'] = $firstname;
		else
			$nulls[] = 'creatorFirstname';
		if($initials)
			$updateArray['creatorInitials'] = $initials;
		else
			$nulls[] = 'creatorInitials';
		if($prefix)
			$updateArray['creatorPrefix'] = $prefix;
		else
			$nulls[] = 'creatorPrefix';
		$this->db->formatConditions(array('creatorId' => $this->vars['editCreatorId']));
		$this->db->update('creator', $updateArray);
// set to NULL empty firstname, prefix and initials fields
		if(isset($nulls) && !empty($nulls))
		{
			$this->db->formatConditions(array('creatorId' => $this->vars['editCreatorId']));
			$this->db->updateNull('creator', $nulls);
		}
// remove cache files for creators
		$this->db->deleteCache('cacheResourceCreators');
		$this->db->deleteCache('cacheMetadataCreators');
// send back to editDisplay with success message
		$this->init($this->success->text("creator"));
	}
// write to the database.
	function editConfirm()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if(!array_key_exists('editCreatorId', $this->vars) || !$this->vars['editCreatorId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('editCreatorExistId', $this->vars) || !$this->vars['editCreatorExistId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('editCreatorSurname', $this->vars) || !$this->vars['editCreatorSurname'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$editId = $this->vars['editCreatorId'];
		$existId = $this->vars['editCreatorExistId'];
		$existSurname = $this->vars['editCreatorSurname'];
// Delete old creator
		$this->db->formatConditions(array('creatorId' => $editId));
		$this->db->delete('creator');
// remove cache files for creators
		$this->db->deleteCache('cacheResourceCreators');
		$this->db->deleteCache('cacheMetadataCreators');
// Select all resources referencing this old creator and replace reference with existing creator -- check if main or not.
// First update where old/new creator is not main creator, then update where old/new creator is main creator (and add newSurname).
		$this->db->formatConditions(array('resourcecreatorCreatorMain' => $editId), TRUE);
		$this->db->formatConditions(array('resourcecreatorCreatorId' => $editId));
		$this->db->update('resource_creator', array('resourcecreatorCreatorId' => $existId));
		$this->db->formatConditions(array('resourcecreatorCreatorMain' => $editId));
		$this->db->formatConditions(array('resourcecreatorCreatorId' => $editId));
		$updateArray['resourcecreatorCreatorId'] = $existId;
		$updateArray['resourcecreatorCreatorMain'] = $existId;
		$this->db->update('resource_creator', $updateArray);
// Update surname where resourcecreatorCreatorMain is $existId
		$this->db->formatConditions(array('resourcecreatorCreatorMain' => $existId));
		$this->db->update('resource_creator', array('resourcecreatorCreatorSurname' => $existSurname));
// send back to editDisplay with success message
		$this->init($this->success->text("creator"));
	}
/**
* The new name equals one already in the database. Confirm that this edited one is to be removed and 
* all references to it replaced by the existing one.
*/
	private function confirmDuplicate($creatorExistId, $surname)
	{
		$pString = $this->errors->text("warning", "creatorExists");
		$pString .= $this->html->p($this->messages->text("creators", "creatorExists"));
		$pString .= $this->form->formHeader("edit_EDITCREATOR_CORE");
		$pString .= $this->form->hidden("editCreatorId", $this->vars['editCreatorId']);
		$pString .= $this->form->hidden("editCreatorExistId", $creatorExistId);
		$pString .= $this->form->hidden("editCreatorSurname", $surname);
		$pString .= $this->form->hidden("method", 'editConfirm');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
}
?>