<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* EDITKEYWORD class
*
*/
class EDITKEYWORD
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $keyword;
private $html;
private $table;
private $gatekeep;
private $badInput;
private $ajax;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "keyword") . ")"));
	}
// check we are allowed to edit and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		$keywords = $this->keyword->grabAll();
		if(!$keywords)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noKeywords'));
			return;
		}
		$pString = $message ? $this->html->p($message, "error", "center") : FALSE;
		$pString .= $this->form->formHeader('edit_EDITKEYWORD_CORE');
		$pString .= $this->form->hidden("method", "edit");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->selectFBoxValue(FALSE, "keywordIds", $keywords, 20));
		$pString .= $this->table->td($this->transferArrow());
		$pString .= $this->table->td($this->html->div('keywordDiv', $this->displayKeyword(TRUE)));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit("Edit"));
		$pString .= $this->form->formEnd();
		$this->ajax->loadJavascript();
		GLOBALS::buildOutputString($pString);
	}
	private function transferArrow()
	{
		$jsonArray = array();
		$jScript = 'index.php?action=edit_EDITKEYWORD_CORE&method=displayKeyword';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'keywordIds',
			'targetDiv' => 'keywordDiv',
			);
		$image = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
		return $image;
	}
// Display interface to edit keyword
	public function displayKeyword($initialDisplay = FALSE)
	{
		$keyword = $keywordId = FALSE;
		if(!$initialDisplay)
		{
			$this->db->formatConditions(array('keywordId' => $this->vars['ajaxReturn']));
			$recordset = $this->db->select('keyword', 'keywordKeyword');
			$keyword = $this->html->dbToFormTidy($this->db->fetchOne($recordset));
			$keywordId = $this->vars['ajaxReturn'];
		}
		$pString = $this->form->hidden("editKeywordId", $keywordId);
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->textInput($this->messages->text('resources', 'keyword') . ' ' . $this->html->span('*', 'required'), 
			'keyword', $keyword, 30, 255));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		if($initialDisplay)
			return $pString;
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $pString)));
		FACTORY_CLOSERAW::getInstance();
	}
// write to the database
	public function edit()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if(!array_key_exists('editKeywordId', $this->vars) || !$this->vars['editKeywordId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$keyword = array_key_exists('keyword', $this->vars) ? trim($this->vars['keyword']) : FALSE;
		if(!$keyword)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if($keywordExistId = $this->keyword->checkExists($keyword))
		{
			if($keywordExistId != $this->vars['editKeywordId'])
				return $this->confirmDuplicate($keywordExistId, $keyword);
		}
		$updateArray['keywordKeyword'] = $keyword;
		$this->db->formatConditions(array('keywordId' => $this->vars['editKeywordId']));
		$this->db->update('keyword', $updateArray);
// remove cache files for keywords
		$this->db->deleteCache('cacheKeywords');
		$this->db->deleteCache('cacheResourceKeywords');
		$this->db->deleteCache('cacheMetadataKeywords');
		$this->db->deleteCache('cacheQuoteKeywords');
		$this->db->deleteCache('cacheParaphraseKeywords');
		$this->db->deleteCache('cacheMusingKeywords');
// send back to editDisplay with success message
		$this->init($this->success->text("keyword"));
	}
// write to the database.
	function editConfirm()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if(!array_key_exists('editKeywordId', $this->vars) || !$this->vars['editKeywordId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('editKeywordExistId', $this->vars) || !$this->vars['editKeywordExistId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$editId = $this->vars['editKeywordId'];
		$existId = $this->vars['editKeywordExistId'];
// Delete old keyword
		$this->db->formatConditions(array('keywordId' => $editId));
		$this->db->delete('keyword');
// Update references to keyword
		$this->db->formatConditions(array('resourcekeywordKeywordId' => $editId));
		$this->db->update('resource_keyword', array('resourcekeywordKeywordId' => $existId));
// remove cache files for keywords
		$this->db->deleteCache('cacheKeywords');
		$this->db->deleteCache('cacheResourceKeywords');
		$this->db->deleteCache('cacheMetadataKeywords');
		$this->db->deleteCache('cacheQuoteKeywords');
		$this->db->deleteCache('cacheParaphraseKeywords');
		$this->db->deleteCache('cacheMusingKeywords');
// send back to editDisplay with success message
		$this->init($this->success->text("keyword"));
	}
/**
* The new keyword equals one already in the database. Confirm that this edited one is to be removed and 
* all references to it replaced by the existing one.
*/
	private function confirmDuplicate($keywordExistId, $keyword)
	{
		$pString = $this->errors->text("warning", "keywordExists");
		$pString .= $this->html->p($this->messages->text("misc", "keywordExists"));
		$pString .= $this->form->formHeader("edit_EDITKEYWORD_CORE");
		$pString .= $this->form->hidden("editKeywordId", $this->vars['editKeywordId']);
		$pString .= $this->form->hidden("editKeywordExistId", $keywordExistId);
		$pString .= $this->form->hidden("method", 'editConfirm');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
}
?>
