<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* EDITPUBLISHER class
*
*/
class EDITPUBLISHER
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $config;
private $form;
private $publisher;
private $html;
private $table;
private $gatekeep;
private $badInput;
private $ajax;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->publisher = FACTORY_PUBLISHER::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "edit", " (" . 
			$this->messages->text("resources", "publisher") . ")"));
	}
// check we are allowed to edit and load appropriate method
	public function init($message = FALSE)
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if(array_key_exists('PublisherType', $this->vars))
			$publisherType = $this->vars['PublisherType'];
		else
			$publisherType = FALSE;
		$publishers = $this->publisher->grabAll($publisherType);
		if(!$publishers)
		{
			GLOBALS::buildOutputString($this->messages->text('misc', 'noPublishers'));
			return;
		}
		$pString = $message ? $this->html->p($message, "error", "center") : FALSE;
		$pString .= $this->form->formHeader('edit_EDITPUBLISHER_CORE');
		$pString .= $this->form->hidden("method", "edit");
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->selectFBoxValue(FALSE, "publisherIds", $publishers, 20));
		$pString .= $this->table->td($this->transferArrow());
		$pString .= $this->table->td($this->html->div('publisherDiv', $this->displayPublisher(TRUE)));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit("Edit"));
		$pString .= $this->form->formEnd();
		$this->ajax->loadJavascript();
		GLOBALS::buildOutputString($pString);
	}
	private function transferArrow()
	{
		$jsonArray = array();
		$jScript = 'index.php?action=edit_EDITPUBLISHER_CORE&method=displayPublisher';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'publisherIds',
			'targetDiv' => 'publisherDiv',
			);
		$image = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
		return $image;
	}
// Display interface to edit publisher
	public function displayPublisher($initialDisplay = FALSE)
	{
		$name = $location = $publisherId = FALSE;
		if(!$initialDisplay)
		{
			$this->db->formatConditions(array('publisherId' => $this->vars['ajaxReturn']));
			$recordset = $this->db->select('publisher', array('publisherName', 'publisherLocation'));
			$row = $this->db->fetchRow($recordset);
			$location = $this->html->dbToFormTidy($row['publisherLocation']);
			$name = $this->html->dbToFormTidy($row['publisherName']);
			$publisherId = $this->vars['ajaxReturn'];
		}
		$pString = $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->form->hidden("editPublisherId", $publisherId);
		$pString .= $this->table->td($this->form->textInput($this->messages->text('resources', 'publisherName'), 'publisherName', $name, 30, 255));
		$pString .= $this->table->td($this->form->textInput($this->messages->text('resources', 'publisherLocation'),
			'publisherLocation', $location, 30, 255));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		if($initialDisplay)
			return $pString;
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $pString)));
		FACTORY_CLOSERAW::getInstance();
	}
// write to the database
	public function edit()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if(!array_key_exists('editPublisherId', $this->vars) || !$this->vars['editPublisherId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$name = array_key_exists('publisherName', $this->vars) ? trim($this->vars['publisherName']) : FALSE;
		$location = array_key_exists('publisherLocation', $this->vars) ? 
			trim($this->vars['publisherLocation']) : FALSE;
		if(!$name & !$location)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if($publisherExistId = $this->publisher->checkExists($name, $location))
		{
			if($publisherExistId != $this->vars['editPublisherId'])
				return $this->confirmDuplicate($publisherExistId);
		}
		if($name)
			$updateArray['publisherName'] = $name;
		else
			$nulls[] = 'publisherName';
		if($location)
			$updateArray['publisherLocation'] = $location;
		else
			$nulls[] = 'publisherLocation';
		if(isset($updateArray))
		{
			$this->db->formatConditions(array('publisherId' => $this->vars['editPublisherId']));
			$this->db->update('publisher', $updateArray);
		}
		if(isset($nulls))
		{
			$this->db->formatConditions(array('publisherId' => $this->vars['editPublisherId']));
			$this->db->updateNull('publisher', $nulls);
		}
// remove cache files for keywords
		$this->db->deleteCache('cacheResourcePublishers');
		$this->db->deleteCache('cacheMetadataPublishers');
// send back to editDisplay with success message
		$this->init($this->success->text("publisher"));
	}
// write to the database.
	function editConfirm()
	{
		$this->gatekeep->init(TRUE); // write access requiring config.php's WIKINDX_GLOBAL_EDIT to be TRUE
		if(!array_key_exists('editPublisherId', $this->vars) || !$this->vars['editPublisherId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('editPublisherExistId', $this->vars) || !$this->vars['editPublisherExistId'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$editId = $this->vars['editPublisherId'];
		$existId = $this->vars['editPublisherExistId'];
// Delete old publisher
		$this->db->formatConditions(array('publisherId' => $editId));
		$this->db->delete('publisher');
// remove cache files for publishers
		$this->db->deleteCache('cacheResourcePublishers');
		$this->db->deleteCache('cacheMetadataPublishers');
// Select all resources referencing this old publisher and replace reference with existing publisher
		$this->db->conditionSeparator = $this->db->or;
		$this->db->formatConditions(array('resourcemiscPublisher' => $editId, 
			'resourcemiscField1' => $editId));
		$this->db->leftJoin('resource', 'resourceId', 'resourcemiscId');
		$recordset = $this->db->select('resource_misc', 
			array('resourcemiscId', 'resourcemiscPublisher', 'resourcemiscField1', 'resourceType'));
		while($row = $this->db->fetchRow($recordset))
		{
			$updateArray = array();
			if((($row['resourceType'] == 'proceedings_article') || ($row['resourceType'] == 'proceedings_article'))
				&& $row['resourcemiscField1'])
				$updateArray['resourcemiscField1'] = $existId;
			else if(($row['resourceType'] != 'proceedings_article') && ($row['resourceType'] != 'proceedings_article')
				&& $row['resourcemiscPublisher'])
				$updateArray['resourcemiscPublisher'] = $existId;
			if(!empty($updateArray))
			{
				$this->db->formatConditions(array('resourcemiscId' => $row['resourcemiscId']));
				$this->db->update('resource_misc', $updateArray);
			}
		}
// send back to editDisplay with success message
		$this->init($this->success->text("publisher"));
	}
/**
* The new publisher equals one already in the database. Confirm that this edited one is to be removed and 
* all references to it replaced by the existing one.
*/
	private function confirmDuplicate($publisherExistId)
	{
		$pString = $this->errors->text("warning", "publisherExists");
		$pString .= $this->html->p($this->messages->text("misc", "publisherExists"));
		$pString .= $this->form->formHeader("edit_EDITPUBLISHER_CORE");
		$pString .= $this->form->hidden("editPublisherId", $this->vars['editPublisherId']);
		$pString .= $this->form->hidden("editPublisherExistId", $publisherExistId);
		$pString .= $this->form->hidden("method", 'editConfirm');
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
}
?>