<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	BIBTEXFILE -- Import a bibtex file
*
*****/
class BIBTEXFILE
{
private $db;
private $vars;
private $messages;
private $session;
private $template;
private $gatekeep;
private $form;
private $html;
private $table;
private $import;
private $tag;

// Constructor
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->import = FACTORY_IMPORT::getInstance();
		$this->tag = FACTORY_TAG::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "bibtexImport"));
	}
// check we are allowed to delete and load appropriate method
	public function init()
	{
		if(!$this->session->getVar("setup_ImportBib"))
			$this->gatekeep->requireSuper = TRUE;
		$this->gatekeep->init();
		if(array_key_exists('function', $this->vars))
		{
			$function = $this->vars['function'];
			$this->{$function}();
		}
		else
			$this->display();
	}
/*
* Display form for pasting bibtex
*/
	public function display($message = FALSE)
	{
		$this->session->delVar('importLock');
		$category = FACTORY_CATEGORY::getInstance();
		$categories = $category->grabAll();
		$pString = $message;
		if(sizeof($categories) > 1)
			$pString .= $this->html->p($this->messages->text("import", "categoryPrompt"));
		$pString .= $this->form->formMultiHeader("import_IMPORTBIBTEX_CORE");
		$pString .= $this->form->hidden('method', 'stage1');
		$pString .= $this->form->hidden('type', 'file');
		$pString .= $this->table->tableStart('generalTable', 0, 10, 0, "");
		$pString .= $this->table->trStart();
// Load tags
		$tags = $this->tag->grabAll();
		$tagInput = $this->form->textInput($this->messages->text("import", "tag"), "import_Tag", FALSE, 30, 255);
		if($tags)
		{
// add 0 => IGNORE to tags array
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($tags as $key => $value)
				$temp[$key] = $value;
			$tags = $temp;
			$sessionTag = $this->session->issetVar('import_TagId') ? 
				$this->session->getVar('import_TagId') : FALSE;
			if($sessionTag)
				$element = $this->form->selectedBoxValue(FALSE, 'import_TagId', $tags, 5);
			else
				$element = $this->form->selectFBoxValue(FALSE, 'import_TagId', $tags, 5);
			$pString .= $this->table->td($tagInput . '&nbsp;&nbsp;' . $element);
		}
		else
			$pString .= $this->table->td($tagInput);
		$categoryTd = FALSE;
		if(sizeof($categories) > 1)
		{
			if($sessionCategories = $this->session->getVar('import_Categories'))
			{
				$sCategories = explode(",", $sessionCategories);
				$element = $this->form->selectedBoxValueMultiple($this->messages->text("import", 
					"category"), 'import_Categories', $categories, $sCategories, 5);
			}
			else
				$element = $this->form->selectFBoxValueMultiple($this->messages->text("import", 
					"category"), 'import_Categories', $categories, 5);
			$pString .= $this->table->td($element . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$categoryTd = TRUE;
		}
		if($bibs = $this->import->bibliographySelect())
			$pString .= $this->table->td($bibs . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, "left", "bottom");
		$pString .= $this->table->td($this->form->fileUpload($this->messages->text("import", "file"), 
			"import_File", 30), FALSE, "left", "bottom");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
		if($categoryTd)
			$pString .= $this->table->td("&nbsp;");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->tableStart('generalTable', 0, 10, 0, "");
		$pString .= $this->table->td($this->messages->text("import", "importDuplicates") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, 'import_ImportDuplicates'));
		$td = $this->messages->text("import", "storeRawBibtex");
		$pString .= $this->table->td($td . "&nbsp;&nbsp;" . $this->messages->text("import", "storeRawLabel") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, 'import_Raw'));
		$pString .= $this->table->td($this->import->keywordSeparator());
		$pString .= $this->table->td($this->import->titleSubtitleSeparator());
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit(), FALSE, "right");
		$this->session->clearArray("import");
		GLOBALS::buildOutputString($pString);
	}
}
?>