<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
*	PASTEBIBTEX -- Users can copy 'n' paste bibtex entries into wikindx
*
*****/
class PASTEBIBTEX
{
private $db;
private $vars;
private $messages;
private $session;
private $template;
private $gatekeep;
private $form;
private $html;
private $table;
private $import;

// Constructor
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->session = FACTORY_SESSION::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->import = FACTORY_IMPORT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "bibtexImport"));
	}
// check we are allowed to delete and load appropriate method
	public function init()
	{
		$this->gatekeep->init();
		if(array_key_exists('function', $this->vars))
		{
			$function = $this->vars['function'];
			$this->{$function}();
		}
		else
			$this->display();
	}
/*
* Display form for pasting bibtex
*/
	public function display($message = FALSE)
	{
		if(!$message)
			$this->session->delVar('import_Paste');
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('pasteBibtex'));
		GLOBALS::setOutputHeading($this->messages->text("heading", "bibtexImport"));
		$this->session->delVar('importLock');
		$category = FACTORY_CATEGORY::getInstance();
		$categories = $category->grabAll();
		$pString = $message;
		if(sizeof($categories) > 1)
			$pString .= $this->html->p($this->messages->text("import", "categoryPrompt"));
		$pString .= $this->form->formHeader("import_IMPORTBIBTEX_CORE");
		$pString .= $this->form->hidden('method', 'stage1');
		$pString .= $this->form->hidden('type', 'paste');
		if(!$this->session->getVar("setup_Superadmin")) // Admin can paste unlimited entries
			$pString .= $this->html->p($this->messages->text("import", "pasteBibtex", 
			' ' . $this->session->getVar('setup_MaxPaste') . " "));
		$pString .= $this->table->tableStart(FALSE, 0, 10, 0, "");
		$pString .= $this->table->trStart();
		$categoryTd = FALSE;
		if(sizeof($categories) > 1)
		{
			if($sessionCategories = $this->session->getVar('import_Categories'))
			{
				$sCategories = explode(",", $sessionCategories);
				$element = $this->form->selectedBoxValueMultiple($this->messages->text("import", 
					"category"), 'import_Categories', $categories, $sCategories, 5);
			}
			else
				$element = $this->form->selectFBoxValueMultiple($this->messages->text("import", 
					"category"), 'import_Categories', $categories, 5);
			$pString .= $this->table->td($element . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			$categoryTd = TRUE;
		}
		if($bibs = $this->import->bibliographySelect())
			$pString .= $this->table->td($bibs . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, "left", "bottom");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
		$paste = $this->session->issetVar('import_Paste') ? 
			unserialize(base64_decode($this->session->getVar('import_Paste'))) : FALSE;
		$pString .= $this->table->td($this->html->br() . "&nbsp;" . $this->html->br() . $this->form->textareaInput(FALSE, 
			"import_Paste", $paste, 80, 20), FALSE, "left", "bottom");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		
		$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
		
		$pString .= $this->table->tableStart(FALSE, 0, 10, 0, "");
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->messages->text("import", "importDuplicates") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, 'import_ImportDuplicates'));
		$td = $this->messages->text("import", "storeRawBibtex");
		$pString .= $this->table->td($td . "&nbsp;&nbsp;" . $this->messages->text("import", "storeRawLabel") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, 'import_Raw'));
		$pString .= $this->table->td($this->import->keywordSeparator());
		$pString .= $this->table->td($this->import->titleSubtitleSeparator());
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit(), FALSE, "right");
		$pString .= $this->form->formEnd();
		$this->session->clearArray("import");
		GLOBALS::buildOutputString($pString);
	}
}
?>