<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* LISTADDTO class
*
* Add selected resources in list to user bibliographies, categories, keywords etc.
* 
*/
class LISTADDTO
{
private $db;
private $vars;
private $messages;
private $errors;
private $session;
private $form;
private $table;
private $html;
private $template;
private $commonBib;
private $user;
private $badInput;
private $navigate;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->navigate = FACTORY_NAVIGATE::getInstance();
	}
	public function init()
	{
		if(!array_key_exists('resourceSelectedTo', $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$this->session->setVar("resourceSelectedTo", $this->vars['resourceSelectedTo']);
		if($this->vars['resourceSelectedTo'] == 1)	// add to categories
			return $this->organizeInit();
		else if($this->vars['resourceSelectedTo'] == 3)	// remove resources from user bibliography
			return $this->deleteFromBib();
		else if($this->vars['resourceSelectedTo'] == 4)	// delete resources from WIKINDX
			return $this->deleteFromWikindx();
		else if($this->vars['resourceSelectedTo'] == 7)	// add to basket
			return $this->addToBasket();
		else if($this->vars['resourceSelectedTo'] == 8)	// remove from basket
			return $this->RemoveFromBasket();
		else	// 0 -> add to user bibs
			return $this->addToUserBibInit();
	}
// Check for proper resource ID input and produce hidden fields
	private function checkIdInput()
	{
		if(array_key_exists("selectWhat", $this->vars) && ($this->vars['selectWhat'] != 'checked'))
		{
			if($this->vars['selectWhat'] == 'all')
				return array(TRUE, 'all');
			if($this->vars['selectWhat'] == 'display')
				return array(TRUE, 'display');
			return array(FALSE, FALSE);
		}
		foreach($this->vars as $key => $value)
		{
			if(strstr($key, "bib_"))
				$string[] = substr($key, 4);
		}
		if(isset($string))
			return array(TRUE, base64_encode(serialize($string)));
		else
			return array(FALSE, FALSE);
	}
// Add selected resources to the basket
	private function addToBasket()
	{
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		if($this->session->issetVar('basket_List'))
			$basket = unserialize($this->session->getVar('basket_List'));
		else
			$basket = array();
		if($string == 'display')
			$resourceIds = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		else if(($string == 'all') && ($query = $this->session->getVar('list_SubQuery')))
		{
			$resultset = $this->db->query(base64_decode($query));
			while($row = $this->db->fetchRow($resultset))
				$resourceIds[] = $row['resourcemiscId'];
		}
		else
			$resourceIds = unserialize(base64_decode($string));
		foreach($resourceIds as $resourceId)
		{
			if(array_search($resourceId, $basket) === FALSE)
				$basket[] = $resourceId;
		}
// Ensure array is unique
		array_unique($basket);
		$this->session->setVar('basket_List', serialize($basket));
		$this->session->saveState('basket');
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("basketAdd"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Remove selected resources from the basket
	private function removeFromBasket()
	{
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		if($this->session->issetVar('basket_List'))
			$basket = unserialize($this->session->getVar('basket_List'));
		else
			$basket = array();
		if($string == 'display')
			$resourceIds = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		else if(($string == 'all') && ($query = $this->session->getVar('list_SubQuery')))
		{
			$resultset = $this->db->query(base64_decode($query));
			while($row = $this->db->fetchRow($resultset))
				$resourceIds[] = $row['resourcemiscId'];
		}
		else
			$resourceIds = unserialize(base64_decode($string));
		foreach($resourceIds as $resourceId)
		{
			if(($key = array_search($resourceId, $basket)) !== FALSE)
				unset($basket[$key]);
		}
		if(empty($basket))
			$this->session->delVar('basket_List');
		else
			$this->session->setVar('basket_List', serialize($basket));
		$this->session->saveState('basket');
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("basketRemove"));
		FACTORY_CLOSE::getInstance(); // die
	}
// add/remove selected resources to categories etc.  Display the form
	private function organizeInit()
	{
		$this->session->clearArray('organize');
		include_once('core/modules/resource/RESOURCECATEGORYEDIT.php');
		$catForm = new RESOURCECATEGORYEDIT();
		$ajax = FACTORY_AJAX::getInstance();
		$pString = $this->html->b($this->messages->text("resources", "warningOrganize"));
		$pString .= $this->form->formHeader('list_LISTADDTO_CORE', "onsubmit=\"selectAll();return true;\"");
		$pString .= $this->form->hidden("method", "organize");
		$display = array('categoryDisplay' => $this->messages->text("resources", "categories"), 
			'subcategoryDisplay' => $this->messages->text("resources", "subcategories"), 
			'languageDisplay' => $this->messages->text("resources", "languages"), 
			'keywordDisplay' => $this->messages->text("resources", "keywords"));
		if(!empty($catForm->userTags))
			$display['usertagDisplay'] = $this->messages->text("resources", "usertags");
		list($idFound, $string) = $this->checkIdInput();
		if(!$idFound)
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$pString .= $this->form->hidden("ids", $string);
		$pString .= $catForm->getTable(TRUE);
		$pString .= $this->html->p($this->messages->text("resources", "replaceExisting") . "&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "replaceExisting") . '&nbsp;&nbsp;' . $this->form->formSubmit("Save"));
		$pString .= $this->form->formEnd();
		$ajax->loadJavascript('core/modules/resource/resourceCategoryEdit.js');
		GLOBALS::buildOutputString($pString);
	}
// hide or display fields
	public function displayCategory()
	{
		include_once('core/modules/resource/RESOURCECATEGORYEDIT.php');
		$catForm = new RESOURCECATEGORYEDIT();
		$ajax = FACTORY_AJAX::getInstance();
		if($this->session->getVar('organize_CategoryHide')) // currently hidden so show
		{
			$div = $catForm->displayCategory(TRUE);
			$this->session->setVar('organize_CategoryHide', FALSE);
		}
		else // currently visible so hide
		{
			$div = $this->table->tableStart('generalTable', FALSE, FALSE, 3);
			$div .= $this->table->trStart();
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayCategory';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displayCategory',
				'targetDiv' => 'categoryDiv',
				);
			$ajax->jActionForm('onchange', $jsonArray);
			$div .= $this->table->td($this->messages->text('select', "availableCategory") . "&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, "displayCategory"));
			$div .= $this->table->trEnd();
			$div .= $this->table->tableEnd();
			$this->session->setVar('organize_CategoryHide', TRUE);
		}
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => "$div")));
		FACTORY_CLOSERAW::getInstance();
	}
// hide or display fields
	public function displaySubcategory()
	{
		include_once('core/modules/resource/RESOURCECATEGORYEDIT.php');
		$catForm = new RESOURCECATEGORYEDIT();
		$ajax = FACTORY_AJAX::getInstance();
		if($this->session->getVar('organize_SubcategoryHide')) // currently hidden so show
		{
			$div = $catForm->displaySubcategory(TRUE);
			$this->session->setVar('organize_SubcategoryHide', FALSE);
		}
		else // currently visible so hide
		{
			$div = $this->table->tableStart('generalTable', FALSE, FALSE, 3);
			$div .= $this->table->trStart();
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displaySubcategory';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displaySubcategory',
				'targetDiv' => 'subcategoryDiv',
				);
			$ajax->jActionForm('onchange', $jsonArray);
			$div .= $this->table->td($this->messages->text('select', "availableSubcategory") . "&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, "displaySubcategory"));
			$div .= $this->table->trEnd();
			$div .= $this->table->tableEnd();
			$this->session->setVar('organize_SubcategoryHide', TRUE);
		}
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => "$div")));
		FACTORY_CLOSERAW::getInstance();
	}
// hide or display fields
	public function displayLanguage()
	{
		include_once('core/modules/resource/RESOURCECATEGORYEDIT.php');
		$catForm = new RESOURCECATEGORYEDIT();
		$ajax = FACTORY_AJAX::getInstance();
		if($this->session->getVar('organize_LanguageHide')) // currently hidden so show
		{
			$div = $catForm->displayLanguage(TRUE);
			$this->session->setVar('organize_LanguageHide', FALSE);
		}
		else // currently visible so hide
		{
			$div = $this->table->tableStart();
			$div .= $this->table->trStart();
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayLanguage';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displayLanguage',
				'targetDiv' => 'languageDiv',
				);
			$ajax->jActionForm('onchange', $jsonArray);
			$div .= $this->table->td($this->messages->text('select', "language") . "&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, "displayLanguage"));
			$div .= $this->table->trEnd();
			$div .= $this->table->tableEnd();
			$this->session->setVar('organize_LanguageHide', TRUE);
		}
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => "$div")));
		FACTORY_CLOSERAW::getInstance();
	}
// hide or display fields
	public function displayKeyword()
	{
		include_once('core/modules/resource/RESOURCECATEGORYEDIT.php');
		$catForm = new RESOURCECATEGORYEDIT();
		$ajax = FACTORY_AJAX::getInstance();
		if($this->session->getVar('organize_KeywordHide')) // currently hidden so show
		{
			$div = $catForm->displayKeyword(TRUE);
			$this->session->setVar('organize_KeywordHide', FALSE);
		}
		else // currently visible so hide
		{
			$div = $this->table->tableStart('generalTable', FALSE, FALSE, 3);
			$div .= $this->table->trStart();
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayKeyword';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displayKeyword',
				'targetDiv' => 'keywordDiv',
				);
			$ajax->jActionForm('onchange', $jsonArray);
			$div .= $this->table->td($this->messages->text('select', "availableKeyword") . "&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, "displayKeyword"));
			$div .= $this->table->trEnd();
			$div .= $this->table->tableEnd();
			$this->session->setVar('organize_KeywordHide', TRUE);
		}
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => "$div")));
		FACTORY_CLOSERAW::getInstance();
	}
// hide or display fields
	public function displayUsertag()
	{
		include_once('core/modules/resource/RESOURCECATEGORYEDIT.php');
		$catForm = new RESOURCECATEGORYEDIT();
		$ajax = FACTORY_AJAX::getInstance();
		if($this->session->getVar('organize_UsertagHide')) // currently hidden so show
		{
			$div = $catForm->displayUserTags(TRUE);
			$this->session->setVar('organize_UsertagHide', FALSE);
		}
		else // currently visible so hide
		{
			$div = $this->table->tableStart('generalTable', FALSE, FALSE, 3);
			$div .= $this->table->trStart();
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayUsertag';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displayUsertag',
				'targetDiv' => 'usertagDiv',
				);
			$ajax->jActionForm('onchange', $jsonArray);
			$div .= $this->table->td($this->messages->text('resources', 'availableUserTags') . "&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, "displayUsertag"));
			$div .= $this->table->trEnd();
			$div .= $this->table->tableEnd();
			$this->session->setVar('organize_UsertagHide', TRUE);
		}
		GLOBALS::buildOutputString($ajax->encode_jArray(array('innerHTML' => "$div")));
		FACTORY_CLOSERAW::getInstance();
	}
// Update resources with new or removed selections of categories, subcategories, languages, keywords and usertags
	public function organize()
	{
		$replaceExisting = FALSE;
		if(!array_key_exists("languageIds", $this->vars) && !array_key_exists("categoryIds", $this->vars) && 
			!array_key_exists("subcategoryIds", $this->vars) && !array_key_exists("keywords", $this->vars) && 
			!array_key_exists("usertags", $this->vars) && !array_key_exists("replaceExisting", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$resourceIds = $this->getHiddenIds();
// Categories
		$categoryIds = array_key_exists('categoryIds', $this->vars) ? $this->vars['categoryIds'] : array();
		if(array_key_exists("replaceExisting", $this->vars) && array_key_exists("categoryRadio", $this->vars) && 
		($this->vars['categoryRadio'] == 'add'))
		{
// remove all old categories and subcategories from resource if adding categories
			$this->db->formatConditionsOneField($resourceIds, 'resourcecategoryResourceId');
			$this->db->delete('resource_category');
		}
// remove categories
		if(array_key_exists("categoryRadio", $this->vars) && $this->vars['categoryRadio'] == 'remove')
		{	
			foreach($categoryIds as $cId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcecategoryResourceId' => $rId));
					$this->db->formatConditions(array('resourcecategoryCategoryId' => $cId));
					$this->db->delete('resource_category');
				}
			}
		}
// add new categories
		else
		{	
			foreach($categoryIds as $cId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcecategoryResourceId' => $rId));
					$this->db->formatConditions(array('resourcecategoryCategoryId' => $cId));
					$resultset = $this->db->select('resource_category', 'resourcecategoryId');
					if(!$this->db->fetchOne($resultset))
						$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
						array($rId, $cId));
				}
			}
		}
// Subcategories
		$subcategoryIds = array_key_exists('subcategoryIds', $this->vars) ? $this->vars['subcategoryIds'] : array();
		if(array_key_exists("replaceExisting", $this->vars) && array_key_exists("subcategoryRadio", $this->vars) && 
		($this->vars['subcategoryRadio'] == 'add'))
		{
// remove all old subcategories from resource if adding subcategories
			$this->db->formatConditionsOneField($resourceIds, 'resourcecategoryResourceId');
			$this->db->formatConditionsOneField($subcategoryIds, 'resourcecategorySubcategoryId');
			$this->db->delete('resource_category');
		}
// remove subcategories
		if(array_key_exists("subcategoryRadio", $this->vars) && $this->vars['subcategoryRadio'] == 'remove')
		{
			foreach($subcategoryIds as $cId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcecategoryResourceId' => $rId));
					$this->db->formatConditions(array('resourcecategorySubcategoryId' => $cId));
					$this->db->delete('resource_category');
				}
			}
		}
// add subcategories
		else
		{
// Find categories subcategories belong to in case category needs adding to resource_category.CategoryId
			foreach($subcategoryIds as $subcategoryId)
			{
				$this->db->formatConditions(array('subcategoryId' => $subcategoryId));
				$scIds[] = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
				foreach($resourceIds as $rId)
					$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategorySubcategoryId'), 
					array($rId, $subcategoryId));
			}
		}
// If category of which subcategory is a member is not in table row, add it
		if(isset($scIds))
		{
			foreach(array_unique($scIds) as $cId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcecategoryCategoryId' => $cId));
					$this->db->formatConditions(array('resourcecategoryResourceId' => $rId));
					$resultset = $this->db->select('resource_category', 'resourcecategoryId');
					if(!$this->db->fetchOne($resultset))
						$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
							array($rId, $cId));
				}
			}
		}
// Languages
		$languageIds = array_key_exists('languageIds', $this->vars) ? $this->vars['languageIds'] : array();
		if(array_key_exists("replaceExisting", $this->vars) && array_key_exists("languageRadio", $this->vars) && 
		($this->vars['languageRadio'] == 'add'))
		{
// remove all old languages from resource when adding
			$this->db->formatConditionsOneField($resourceIds, 'resourcelanguageResourceId');
			$this->db->delete('resource_language');
		}
// remove languages
		if(array_key_exists("languageRadio", $this->vars) && $this->vars['languageRadio'] == 'remove')
		{
			foreach($languageIds as $lId)
			{
				if(!$lId) // IGNORE
					continue;
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcelanguageResourceId' => $rId));
					$this->db->formatConditions(array('resourcelanguageLanguageId' => $lId));
					$this->db->delete('resource_language');
				}
			}
		}
// add new languages
		else
		{
			foreach($languageIds as $lId)
			{
				if(!$lId)
					continue;
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcelanguageResourceId' => $rId));
					$this->db->formatConditions(array('resourcelanguageLanguageId' => $lId));
					$resultset = $this->db->select('resource_language', 'resourcelanguageId');
					if(!$this->db->fetchOne($resultset))
						$this->db->insert('resource_language', array('resourcelanguageResourceId', 'resourcelanguageLanguageId'), 
						array($rId, $lId));
				}
			}
		}
// Keywords
		$keyword = FACTORY_KEYWORD::getInstance();
		$keywordIds = array();
		if(array_key_exists('keywords', $this->vars) && trim($this->vars['keywords']))
			$keywordTexts = explode(',', trim($this->vars['keywords']));
		else
			$keywordTexts = array();
		foreach($keywordTexts as $text)
		{
			$keywordText = trim($text);
			if($keywordText)
			{
				if($id = $keyword->checkExists($keywordText))
				{
					if(array_search($id, $keywordIds) === FALSE)
						$keywordIds[] = $id;
				}
				else
				{
// given keyword doesn't exist so now write to db
					$fields = $values = array();
					$fields[] = "keywordKeyword";
					$values[] = $keywordText;
					$this->db->insert('keyword', $fields, $values);
					$keywordIds[] = $this->db->lastAutoId('keyword');
				}
			}
		}
		if(array_key_exists("replaceExisting", $this->vars) && array_key_exists("keywordRadio", $this->vars) && 
		($this->vars['keywordRadio'] == 'add'))
		{
// remove old keywords from all selected resources when adding
			$this->db->formatConditionsOneField($resourceIds, 'resourcekeywordResourceId');
			$this->db->delete('resource_keyword');
		}
// remove keywords
		if(array_key_exists("keywordRadio", $this->vars) && $this->vars['keywordRadio'] == 'remove')
		{	
			foreach($keywordIds as $kId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcekeywordResourceId' => $rId));
					$this->db->formatConditions(array('resourcekeywordKeywordId' => $kId));
					$this->db->delete('resource_keyword');
				}
			}
		}
// add new keywords
		else
		{	
			foreach($keywordIds as $kId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourcekeywordResourceId' => $rId));
					$this->db->formatConditions(array('resourcekeywordKeywordId' => $kId));
					$resultset = $this->db->select('resource_keyword', 'resourcekeywordId');
					if(!$this->db->fetchOne($resultset))
						$this->db->insert('resource_keyword', array('resourcekeywordResourceId', 'resourcekeywordKeywordId'), array($rId, $kId));
				}
			}
		}
// remove keyword(s) if they no longer have resources attached to them
		$keyword->removeHanging();
// remove cache files for keywords
		$this->db->deleteCache('cacheResourceKeywords');
		$this->db->deleteCache('cacheMetadataKeywords');
		$this->db->deleteCache('cacheQuoteKeywords');
		$this->db->deleteCache('cacheParaphraseKeywords');
		$this->db->deleteCache('cacheMusingKeywords');
		$this->db->deleteCache('cacheKeywords');
// User tags
		$usertag = FACTORY_USERTAGS::getInstance();
		$usertagIds = array();
		if(array_key_exists('userTags', $this->vars) && trim($this->vars['userTags']))
			$usertagTexts = explode(',', trim($this->vars['userTags']));
		else
			$usertagTexts = array();
		foreach($usertagTexts as $text)
		{
			$usertagText = trim($text);
			if($usertagText)
			{
				if($id = $usertag->checkExists($usertagText))
				{
					if(array_search($id, $usertagIds) === FALSE)
						$usertagIds[] = $id;
				}
				else
				{
// given usertag doesn't exist so now write to db
					$fields = $values = array();
					$fields[] = 'usertagsTag';
					$values[] = $usertagText;
					$fields[] = 'usertagsUserId';
					$values[] = $this->session->getVar('setup_UserId');
					$this->db->insert('user_tags', $fields, $values);
					$usertagIds[] = $this->db->lastAutoId('user_tags');
				}
			}
		}
		if(array_key_exists("replaceExisting", $this->vars) && array_key_exists("usertagRadio", $this->vars) && 
		($this->vars['usertagRadio'] == 'add'))
		{
// remove old usertags from all selected resources
			$this->db->formatConditionsOneField($resourceIds, 'resourceusertagsResourceId');
			$this->db->delete('resource_user_tags');
		}
// remove usertags
		if(array_key_exists("usertagRadio", $this->vars) && $this->vars['usertagRadio'] == 'remove')
		{
			foreach($usertagIds as $uId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourceusertagsResourceId' => $rId));
					$this->db->formatConditions(array('resourceusertagsTagId' => $uId));
					$this->db->delete('resource_user_tags');
				}
			}
		}
// add new usertags
		else
		{
			foreach($usertagIds as $uId)
			{
				foreach($resourceIds as $rId)
				{
					$this->db->formatConditions(array('resourceusertagsResourceId' => $rId));
					$this->db->formatConditions(array('resourceusertagsTagId' => $uId));
					$resultset = $this->db->select('resource_user_tags', 'resourceusertagsId');
					if(!$this->db->fetchOne($resultset))
						$this->db->insert('resource_user_tags', array('resourceusertagsResourceId', 'resourceusertagsTagId'), 
						array($rId, $uId));
				}
			}
		}
// Check that each of these resources still belongs to at least one category, if not, set to 'General'
		foreach($resourceIds as $rId)
		{
			$this->db->formatConditions(array('resourcecategoryResourceId' => $rId));
			$this->db->formatConditions(array('resourcecategoryCategoryId' => ' IS NOT NULL'));
			$resultset = $this->db->select('resource_category', 'resourcecategoryId');
			if(!$this->db->fetchOne($resultset))
				$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
				array($rId, 1));
		}
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("organized"));
		FACTORY_CLOSE::getInstance(); // die
	}
	private function addToUserBibInit()
	{
		$usingBib = $pString = FALSE;
		$bibs = array();
		$useBib = $this->session->getVar('mywikindx_Bibliography_use');
		if($useBib)
		{
			$this->db->formatConditions(array('userbibliographyId' => $useBib));
			$recordset = $this->db->select('user_bibliography', 'userbibliographyTitle');
			$row = $this->db->fetchRow($recordset);
			$usingBib = $row['userbibliographyTitle'];
		}
		$tempU = $this->commonBib->getUserBibs();
		$tempUG = $this->commonBib->getGroupBibs();
		if(!empty($tempU))
		{
			$bibs[0] = $this->messages->text('user', 'userBibs');
			foreach($tempU as $key => $value)
				$bibs[$key] = $value;
		}
		if(!empty($tempUG))
		{
			$bibs[-1] = $this->messages->text('user', 'userGroupBibs');
			foreach($tempUG as $key => $value)
				$bibs[$key] = $value;
		}
		if(!empty($bibs))
		{
			$pString = $this->form->formHeader('list_LISTADDTO_CORE');
				$pString .= $this->form->hidden("method", "addResourceToBib");
			list($idFound, $string) = $this->checkIdInput();
			if(!$idFound)
				$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
			$pString .= $this->form->hidden("ids", $string);
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
			$sessVar = $this->session->getVar("mywikindx_Bibliography_add");
			if($sessVar)
				$pString .= $this->table->td($this->form->selectedBoxValue(FALSE, "bibId", $bibs, $sessVar, 10) . 
					$this->html->p($this->form->formSubmit("Add")));
			else
				$pString .= $this->table->td($this->form->selectFBoxValue(FALSE, "bibId", $bibs, 10) . 
					$this->html->p($this->form->formSubmit("Add")));
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->form->formEnd();
		}
//		$this->session->setVar("addToKeywordCategory", TRUE);
		GLOBALS::setOutputHeading($this->messages->text("heading", "addToBib"));
		GLOBALS::buildOutputString($pString);
	}
// Get resource ids ready for writing to database
	private function getHiddenIds()
	{
		if(!array_key_exists("ids", $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		if($this->vars['ids'] == 'display')
			$ids = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		else if(($this->vars['ids'] == 'all') && ($query = $this->session->getVar('list_SubQuery')))
		{
			$resultset = $this->db->query(base64_decode($query));
			while($row = $this->db->fetchRow($resultset))
				$ids[] = $row['resourcemiscId'];
		}
		else
			$ids = unserialize(base64_decode($this->vars['ids']));
		if(!isset($ids))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		return $ids;
	}
// Add resources to a bibliography
	public function addResourceToBib()
	{
		if(!array_key_exists('bibId', $this->vars) || ($this->vars['bibId'] < 1))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$ids = $this->getHiddenIds();
// valid user?
		$userBibs = $this->commonBib->getUserBibs();
		$groupBibs = $this->commonBib->getGroupBibs();
		if(!array_key_exists($this->vars['bibId'], $userBibs) && !array_key_exists($this->vars['bibId'], $groupBibs))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this->navigate, 'listView');
		foreach($ids as $id)
		{
			$this->db->formatConditions(array('userbibliographyresourceResourceId' => $id));
			$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $this->vars['bibId']));
			if(!$this->db->numRows($this->db->select('user_bibliography_resource', 'userbibliographyresourceId')))
				$this->db->insert('user_bibliography_resource', 
					array('userbibliographyresourceBibliographyId', 'userbibliographyresourceResourceId'), 
					array($this->vars['bibId'], $id));
		}
		$this->session->setVar('mywikindx_Bibliography_add', $this->vars['bibId']);
		$this->session->setVar("setup_Bibliographies", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("addBib"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Delete resources from a user bibliogaphy
	private function deleteFromBib()
	{
		if(array_key_exists("selectAll", $this->vars))
			$inputIds = unserialize(base64_decode($this->session->getVar('list_AllIds')));
		foreach($this->vars as $key => $value)
		{
			if(strstr($key, "bib_"))
				$inputIds[] = substr($key, 4);
		}
		if(!isset($inputIds))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $this->session->getVar("mywikindx_Bibliography_use")));
		$this->db->formatConditionsOneField($inputIds, 'userbibliographyresourceResourceId');
		$this->db->delete('user_bibliography_resource');
		$bibs = $this->commonBib->getUserBibs();
		if(empty($bibs))
			$bibs = $this->commonBib->getUserBibs();
		if(!empty($bibs))
			$this->session->setVar("setup_Bibliographies", TRUE);
		$this->session->setVar("addToKeywordCategory", TRUE);
		$success = FACTORY_SUCCESS::getInstance();
		$this->navigate->listView($success->text("deleteFromBib"));
		FACTORY_CLOSE::getInstance(); // die
	}
// Delete resources from WIKINDX
	public function deleteFromWikindx()
	{
		$gatekeep = FACTORY_GATEKEEP::getInstance();
		$gatekeep->requireSuper = TRUE; // only admins can delete resources
		$gatekeep->init();
		if(array_key_exists("selectAll", $this->vars))
		{
			if(!$sql = base64_decode($this->session->getVar('sql_Stmt')))
				$this->badInput->close($this->errors->text("file", "noSql"), $this->navigate, 'listView');
			$recordset = $this->db->query($sql);
			while($row = $this->db->fetchRow($recordset))
				$inputIds[] = $row['resourceId'];
		}
		else
		{
			foreach($this->vars as $key => $value)
			{
				if(strstr($key, "bib_"))
					$inputIds[] = substr($key, 4);
			}
		}
		if(!isset($inputIds))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this->navigate, 'listView');
		$this->session->setVar("addToKeywordCategory", TRUE);
		include_once("core/modules/admin/DELETERESOURCE.php");
		$deleteResource = new DELETERESOURCE();
		$deleteResource->resourceIds['resource_id'] = $inputIds;
		$deleteResource->navigate = 'list';
		return $deleteResource->deleteResourceConfirm();
	}
}
?>