<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* LISTRESOURCES class
*
* List all database resources.
* 
*/
class LISTRESOURCES
{
private $db;
private $vars;
private $stmt;
private $messages;
private $session;
private $form;
private $table;
private $html;
private $common;
private $commonBib;
private $user;
private $count = 0;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$errors = FACTORY_ERRORS::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$template = FACTORY_TEMPLATE::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$badInput = FACTORY_BADINPUT::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "list"));
		if(!$this->common->resourcesExist())
			$badInput->close($this->messages->text("misc", "noResources"));
		$sq = $this->session->getVar('list_SubQuery');
		$this->session->clearArray('list');
		$this->session->setVar('list_SubQuery', $sq);
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$linksInfo['info'] = $this->commonBib->displayBib();
		$template->tpl->assign('resourceListInfo', $linksInfo);
		GLOBALS::setOutputHeading($this->messages->text("heading", "list"));
		if(!array_key_exists('method', $this->vars))
			$badInput->close($errors->text("inputError", "missing"));
		if(array_key_exists('list_Order', $this->vars))
			$this->session->setVar('list_Order', $this->vars['list_Order']);
		else if(!array_key_exists('type', $this->vars) || ($this->vars['type'] != 'lastMulti'))
			$badInput->close($errors->text("inputError", "missing"));
		$method = $this->vars['method'];
		if(!method_exists($this, $method))
			$badInput->close($errors->text("inputError", "missing"));
		if($method != 'reorder')
		{
			switch($this->session->getVar('list_Order'))
			{
				case 'creator':		$this->session->setVar('list_AscDesc', $this->db->asc); break;
				case 'title':		$this->session->setVar('list_AscDesc', $this->db->asc); break;
				case 'publisher':		$this->session->setVar('list_AscDesc', $this->db->asc); break;
				case 'year':		$this->session->setVar('list_AscDesc', $this->db->desc); break;
				case 'timestamp':		$this->session->setVar('list_AscDesc', $this->db->desc); break;
				case 'viewsIndex':		$this->session->setVar('list_AscDesc', $this->db->desc); break;
				case 'popularityIndex':		$this->session->setVar('list_AscDesc', $this->db->desc); break;
				case 'downloadsIndex':		$this->session->setVar('list_AscDesc', $this->db->desc); break;
				case 'maturityIndex':		$this->session->setVar('list_AscDesc', $this->db->desc); break;
				default: 				$this->session->setVar('list_AscDesc', $this->db->asc); break;
			}
		}
		$this->{$method}();
	}
// With a reorder list request, which type of process do we want?
	public function reorder()
	{
		if(array_key_exists("list_Order", $this->vars) && $this->vars["list_Order"])
		{
			$this->session->setVar('search_Order', $this->vars["list_Order"]);
			$this->session->setVar('sql_LastOrder', $this->vars["list_Order"]);
		}
		if(array_key_exists('list_AscDesc', $this->vars))
			$this->session->setVar('list_AscDesc', $this->vars['list_AscDesc']);
		$this->processGeneral();
	}
// Display titles....
	public function processGeneral()
	{
// April 2013: For some reason I haven't figured out yet, this method and all its time/processing overhead is called twice.  This little routine stops that.
		if($this->count)
			return;
		++$this->count;
		$queryString = 'action=list_LISTRESOURCES_CORE&method=processGeneral&list_Order=' . $this->session->getVar('list_Order');
		$this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, $this->session->getVar('list_Order'), TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$this->session->setVar("bookmark_DisplayAdd", TRUE);
			$this->common->lastMulti('list');
			return;
		}
		$sql = $this->stmt->listList(FALSE, FALSE, $this->session->getVar('list_Order'));
		$this->common->display($sql, 'list');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		$this->session->saveState('list');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
}
?>
