<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* LISTSHOWHIDE class
*
* AJAX for LIST -- show or hide select boxes according to actions on other form elements
*/
class LISTSHOWHIDE
{
private $categories;
private $subcategories;
private $category;
private $keyword;
private $keywords;
private $types;
private $creator;
private $creators;
private $publisher;
private $publishers;
private $collection;
private $collections;
private $usertag;
private $usertags;
private $session;
private $form;
private $table;
private $html;
private $messages;
private $vars;
private $radioButtons;
private $type;
private $previousSelect = array();
private $ajax;
private $typeObj;
private $metadataTypeArray = array('quote', 'paraphrase', 'quoteComment', 'paraphraseComment', 'musing');

	public function __construct()
	{
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->typeObj = FACTORY_TYPE::getInstance();
	}
/**
* AJAX-based DIV content creator for resource types -- only used for select and search metadata
*/
	public function initTypes()
	{
// if no ajaxReturn, quietly exit
		$div = $this->html->div('type', $this->table->td("&nbsp;")); // default
		$this->type = $this->vars['type'];
		if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all metadata types
		{
			$this->checkMetadataExists();
			$this->types = $this->typeObj->grabAll(TRUE, $this->metadataTypeArray);
		}
		else
			$this->types = $this->typeObj->grabAll(TRUE, explode(',', $this->vars['ajaxReturn']));
		$this->grabPreviouslySelected('ajaxReturn2');
		if(is_array($this->types))
			$div = $this->html->div('type', $this->table->td($this->makeFormMultiple($this->types, 'type', 'Type')));
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for categories
*/
	public function initCategories()
	{
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->type = $this->vars['type'];
		if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
			$this->categories = $this->category->grabAll($this->session->getVar("mywikindx_Bibliography_use"), 
				TRUE, array_keys($this->typeObj->grabAll()));
		else
			$this->categories = $this->category->grabAll($this->session->getVar("mywikindx_Bibliography_use"), 
				TRUE, explode(',', $this->vars['ajaxReturn']));
		$this->grabPreviouslySelected('ajaxReturn2');
		$this->makeRadioButtons('CategoryMethod');
		if(is_array($this->categories))
		{
			$categoryTable = $this->table->tableStart('generalTable');
			$categoryTable .= $this->table->trStart();
			$selectedCategories = array();
			if($selected = $this->session->getVar($this->type . "_Category"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(!array_key_exists($key, $this->categories))
						continue;
					$selectedCategories[$key] = $this->categories[$key];
					unset($this->categories[$key]);
				}
			}
			$categoryTable .= $this->table->td($this->makeFormMultiple($this->categories, "availableCategory", "AvailableCategory", TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '18%');
			list($toRightImage, $toLeftImage) = $this->category->transferArrowsCat();
			$categoryTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$categoryTable .= $this->table->td($this->makeFormMultiple($selectedCategories, "category", "Category", TRUE), FALSE, 'left', FALSE, FALSE, '18%');
			$categoryTable .= $this->table->trEnd();
			$categoryTable .= $this->table->tableEnd();
		}
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => "$categoryTable")));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for languages
*/
	public function initLanguages()
	{
// if no ajaxReturn, quietly exit
		$div = $this->table->td($this->html->div('language', "&nbsp;")); // default
		$this->type = $this->vars['type'];
		$db = FACTORY_DB::getInstance();
		if(array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean specific resource types
		{
			$db->leftJoin('resource', 'resourceId', 'resourcelanguageResourceId');
			$db->formatConditionsOneField(explode(',', $this->vars['ajaxReturn']), 'resourceType');
		}
		$languages = array();
		$db->orderBy('languageLanguage');
		$db->leftJoin('language', 'languageId', 'resourcelanguageLanguageId');
		$resultset = $db->select('resource_language', array('resourcelanguageLanguageId', 'languageLanguage'), TRUE);
		while($row = $db->fetchRow($resultset))
			$languages[$row['resourcelanguageLanguageId']] = $row['languageLanguage'];
		$this->makeRadioButtons('LanguageMethod', FALSE);
		if(!empty($languages))
			$div = $this->table->td($this->html->div('language', $this->makeFormMultiple($languages, 'language', 'Language')));
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for keywords
*/
	public function initKeywords()
	{
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->type = $this->vars['type'];
		if(array_key_exists('metadata', $this->vars))
		{
			if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all metadata types
			{
				$this->checkMetadataExists();
				$this->keywords = $this->keyword->grabAll(TRUE, $this->metadataTypeArray);
			}
			else
				$this->keywords = $this->keyword->grabAll(TRUE, explode(',', $this->vars['ajaxReturn']));
		}
		else
		{
			if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
				$this->keywords = $this->keyword->grabAll($this->session->getVar("mywikindx_Bibliography_use"),
					'resource', array_keys($this->typeObj->grabAll()));
			else
				$this->keywords = $this->keyword->grabAll($this->session->getVar("mywikindx_Bibliography_use"),
					'resource', explode(',', $this->vars['ajaxReturn']));
		}
		$this->grabPreviouslySelected('ajaxReturn2');
		$this->makeRadioButtons('KeywordMethod');
		if(is_array($this->keywords))
		{
			$keywordTable = $this->table->tableStart('generalTable');
			$keywordTable .= $this->table->trStart();
			$selectedKeywords = array();
			if($selected = $this->session->getVar($this->type . "Keyword"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(array_key_exists($key, $this->keywords))
						$selectedKeywords[$key] = $this->keywords[$key];
				}
			}
			$keywordTable .= $this->table->td($this->makeFormMultiple($this->keywords, "availableKeyword", "AvailableKeyword", TRUE, TRUE), 
				FALSE, 'left', FALSE, FALSE, '20%');
			list($toRightImage, $toLeftImage) = $this->keyword->transferArrows();
			$keywordTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$keywordTable .= $this->table->td($this->makeFormMultiple($selectedKeywords, "keyword", 'Keyword', TRUE), FALSE, 'left', FALSE, FALSE, '20%');
		}
		else
		{
			$keywordTable = $this->table->tableStart();
			$keywordTable .= $this->table->trStart();
			$keywordTable .= $this->table->td("&nbsp;"); // default
		}
		$keywordTable .= $this->table->trEnd();
		$keywordTable .= $this->table->tableEnd();
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $keywordTable)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for subcategories
*/
	public function initSubcategories()
	{
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->type = $this->vars['type'];
		if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
			$categories = $this->category->grabAll($this->session->getVar("mywikindx_Bibliography_use"), 
				TRUE, array_keys($this->typeObj->grabAll()), FALSE);
		else
			$categories = $this->category->grabAll($this->session->getVar("mywikindx_Bibliography_use"), 
				TRUE, explode(',', $this->vars['ajaxReturn']), FALSE);
		$this->subcategories = $this->category->grabSubAll(TRUE, $this->session->getVar("mywikindx_Bibliography_use"), 
			array_keys($categories), TRUE);
		if($this->subcategories)
		{
			$this->grabPreviouslySelected('ajaxReturn2');
			$this->makeRadioButtons('SubcategoryMethod');
			$subcatTable = $this->table->tableStart('generalTable');
			$subcatTable .= $this->table->trStart();
			$selectedSubcats = array();
			if($selected = $this->session->getVar("search_Subcategory"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(!array_key_exists($key, $this->subcategories))
						continue;
					$selectedSubcats[$key] = $this->subcategories[$key];
					unset($this->subcategories[$key]);
				}
			}
			$subcatTable .= $this->table->td($this->makeFormMultiple($this->subcategories, "availableSubcategory", "AvailableSubcategory", TRUE, TRUE), 
				FALSE, 'left', FALSE, FALSE, '18%');
			list($toRightImage, $toLeftImage) = $this->category->transferArrowsSubcat();
			$subcatTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$subcatTable .= $this->table->td($this->makeFormMultiple($selectedSubcats, "subcategory", "Subcategory", TRUE), 
				FALSE, 'left', FALSE, FALSE, '18%');
		}
		else
		{
			$subcatTable = $this->table->tableStart();
			$subcatTable .= $this->table->trStart();
			$subcatTable .= $this->table->td("&nbsp;");
		}
		$subcatTable .= $this->table->trEnd();
		$subcatTable .= $this->table->tableEnd();
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $subcatTable)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* DIV content creator for subcategories because subcategories are dependent upon categories
*/
	public function initSubcategoriesDependent($matchCategoryIds)
	{
		$div = $this->html->div('subcategory', $this->table->td("&nbsp;")); // default
		if(empty($matchCategoryIds))
			return array('subcategory', $div);
		$this->type = $this->vars['type'];
			$this->subcategories = $this->category->grabSubAll(TRUE, $this->session->getVar("mywikindx_Bibliography_use"), 
				$matchCategoryIds, TRUE);
		if(is_array($this->subcategories))
			$div = $this->html->div('subcategory', 
			$this->table->td($this->makeFormMultiple($this->subcategories, 'subcategory', 'Subcategory')));
		return array('subcategory', $div);
	}
/**
* AJAX-based DIV content creator for creators
*/
	public function initCreators()
	{
		$this->creator = FACTORY_CREATOR::getInstance();
		$this->type = $this->vars['type'];
		if(array_key_exists('metadata', $this->vars))
		{
			if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all metadata types
			{
				$this->checkMetadataExists();
				$this->creators = $this->creator->grabAll(TRUE, FALSE, $this->metadataTypeArray);
			}
			else
				$this->creators = $this->creator->grabAll(TRUE, FALSE, explode(',', $this->vars['ajaxReturn']));
		}
		else
		{
			if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
				$this->creators = $this->creator->grabAll($this->session->getVar("mywikindx_Bibliography_use"),
					array_keys($this->typeObj->grabAll()));
			else
				$this->creators = $this->creator->grabAll($this->session->getVar("mywikindx_Bibliography_use"),
					explode(',', $this->vars['ajaxReturn']), FALSE, TRUE);
		}
		$this->grabPreviouslySelected('ajaxReturn2');
		$this->makeRadioButtons('CreatorMethod');
		if(is_array($this->creators))
		{
			$creatorTable = $this->table->tableStart('generalTable');
			$creatorTable .= $this->table->trStart();
			$selectedCreators = array();
			if($selected = $this->session->getVar($this->type . "_Creator"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(!array_key_exists($key, $this->creators))
						continue;
					$selectedCreators[$key] = $this->creators[$key];
					unset($this->creators[$key]);
				}
			}
			$creatorTable .= $this->table->td($this->makeFormMultiple($this->creators, "availableCreator", "AvailableCreator", TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '20%');
			list($toRightImage, $toLeftImage) = $this->creator->transferArrows();
			$creatorTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$creatorTable .= $this->table->td($this->makeFormMultiple($selectedCreators, "creator", "Creator", TRUE), FALSE, 'left', FALSE, FALSE, '20%');
			$creatorTable .= $this->table->trEnd();
			$creatorTable .= $this->table->tableEnd();
		}
		else
		{
			$creatorTable = $this->table->tableStart();
			$creatorTable .= $this->table->trStart();
			$creatorTable .= $this->table->td("&nbsp;"); // default
		}
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => "$creatorTable")));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for publishers
*/
	public function initPublishers()
	{
		$this->publisher = FACTORY_PUBLISHER::getInstance();
		$this->type = $this->vars['type'];
		if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
			$this->publishers = $this->publisher->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"),
				array_keys($this->typeObj->grabAll()));
		else
			$this->publishers = $this->publisher->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"),
				explode(',', $this->vars['ajaxReturn']));
		$this->grabPreviouslySelected('ajaxReturn2');
		if(is_array($this->publishers))
		{
			$publisherTable = $this->table->tableStart('generalTable');
			$publisherTable .= $this->table->trStart();
			$selectedPublishers = array();
			if($selected = $this->session->getVar($this->type . "_Publisher"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(!array_key_exists($key, $this->publishers))
						continue;
					$selectedPublishers[$key] = $this->publishers[$key];
					unset($this->publishers[$key]);
				}
			}
			$publisherTable .= $this->table->td($this->makeFormMultiple($this->publishers, "availablePublisher", "AvailablePublisher", TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '20%');
			list($toRightImage, $toLeftImage) = $this->publisher->transferArrows();
			$publisherTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$publisherTable .= $this->table->td($this->makeFormMultiple($selectedPublishers, "publisher", "Publisher", TRUE), FALSE, 'left', FALSE, FALSE, '20%');
			$publisherTable .= $this->table->trEnd();
			$publisherTable .= $this->table->tableEnd();
		}
		else
		{
			$publisherTable = $this->table->tableStart();
			$publisherTable .= $this->table->trStart();
			$publisherTable .= $this->table->td("&nbsp;"); // default
		}
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => "$publisherTable")));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for collections
*/
	public function initCollections()
	{
		$this->collection = FACTORY_COLLECTION::getInstance();
		$this->type = $this->vars['type'];
		if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
			$this->collections = $this->collection->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"),
				array_keys($this->typeObj->grabAll()));
		else
			$this->collections = $this->collection->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"),
				explode(',', $this->vars['ajaxReturn']));
		$this->grabPreviouslySelected('ajaxReturn2');
		if(is_array($this->collections))
		{
			$collectionTable = $this->table->tableStart('generalTable');
			$collectionTable .= $this->table->trStart();
			$selectedCollections = array();
			if($selected = $this->session->getVar($this->type . "_Collection"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(!array_key_exists($key, $this->collections))
						continue;
					$selectedCollections[$key] = $this->collections[$key];
					unset($this->collections[$key]);
				}
			}
			$collectionTable .= $this->table->td($this->makeFormMultiple($this->collections, "availableCollection", "AvailableCollection", TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '30%');
			list($toRightImage, $toLeftImage) = $this->collection->transferArrows();
			$collectionTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$collectionTable .= $this->table->td($this->makeFormMultiple($selectedCollections, "collection", "Collection", TRUE), FALSE, 'left', FALSE, FALSE, '30%');
			$collectionTable .= $this->table->trEnd();
			$collectionTable .= $this->table->tableEnd();
		}
		else
		{
			$collectionTable = $this->table->tableStart();
			$collectionTable .= $this->table->trStart();
			$collectionTable .= $this->table->td("&nbsp;"); // default
		}
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => "$collectionTable")));
		FACTORY_CLOSERAW::getInstance();
		
		
// if no ajaxReturn, quietly exit
		$div = $this->html->div('collection', $this->table->td("&nbsp;")); // default
		$this->type = $this->vars['type'];
		if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
			$this->collections = $this->collection->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"),
				array_keys($this->typeObj->grabAll()));
		else
			$this->collections = $this->collection->grabAll(FALSE, $this->session->getVar("mywikindx_Bibliography_use"),
				explode(',', $this->vars['ajaxReturn']));
		$this->grabPreviouslySelected('ajaxReturn2');
		if(is_array($this->collections))
			$div = $this->html->div('collection', $this->table->td($this->makeFormMultiple($this->collections, 
				'collection', 'Collection')));
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for user tags
*/
	public function initUsertags()
	{
		$this->userTag = FACTORY_USERTAGS::getInstance();
		$this->type = $this->vars['type'];
		if(!array_key_exists('ajaxReturn', $this->vars)) // i.e. we mean all resource types
			$this->userTags = $this->userTag->grabAll($this->session->getVar("mywikindx_Bibliography_use"),
				array_keys($this->typeObj->grabAll()), TRUE);
		else
			$this->userTags = $this->userTag->grabAll($this->session->getVar("mywikindx_Bibliography_use"),
				explode(',', $this->vars['ajaxReturn']), TRUE);
		$this->grabPreviouslySelected('ajaxReturn2');
		$this->makeRadioButtons('UserTagMethod');
		if(!empty($this->userTags))
		{
			$utTable = $this->table->tableStart('generalTable');
			$utTable .= $this->table->trStart();
			$selectedUTs = array();
			if($selected = $this->session->getVar($this->type . "_UserTag"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(!array_key_exists($key, $this->userTags))
						continue;
					$selectedUTs[$key] = $this->userTags[$key];
					unset($this->userTags[$key]);
				}
			}
			$utTable .= $this->table->td($this->makeFormMultiple($this->userTags, "availableUserTag", "AvailableUserTag", TRUE, TRUE), 
				FALSE, 'left', FALSE, FALSE, '20%');
			list($toRightImage, $toLeftImage) = $this->userTag->transferArrows();
			$utTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$utTable .= $this->table->td($this->makeFormMultiple($selectedUTs, "userTag", 'UserTag', TRUE), FALSE, 'left', FALSE, FALSE, '20%');
		}
		else
		{
			$utTable = $this->table->tableStart();
			$utTable .= $this->table->trStart();
			$utTable .= $this->table->td("&nbsp;"); // default
		}
		$utTable .= $this->table->trEnd();
		$utTable .= $this->table->tableEnd();
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $utTable)));
		FACTORY_CLOSERAW::getInstance();
	}
// make a self-contained form with a multiple select box
	private function makeFormMultiple($array, $message, $box, $noIgnore = FALSE, $noRadio = FALSE)
	{
// add 0 => IGNORE to $array
		if(!$noIgnore)
		{
			$temp[0] = $this->messages->text("misc", "ignore");
			$temp += $array;
		}
		else
			$temp = $array;
//print_r($this->previousSelect); print '\n\n';
		$this->previousSelect = array_intersect($this->previousSelect, array_keys($temp));
//print_r($this->previousSelect); 
		if($this->type == 'search')
		{
			if(!empty($this->previousSelect))
				$pString = $this->form->selectedBoxValueMultiple($this->messages->text("search", $message), "search_" . $box, 
				$temp, $this->previousSelect, "5");
			else
				$pString = $this->form->selectFBoxValueMultiple($this->messages->text("search", $message), "search_" . $box, $temp, "5");
		}
		else
		{
			if(!empty($this->previousSelect))
				$pString = $this->form->selectedBoxValueMultiple($this->messages->text("select", $message), 
					"select_" . $box, $temp, $this->previousSelect, "5");
			else
				$pString = $this->form->selectFBoxValueMultiple($this->messages->text("select", $message), "select_" . $box, $temp, "5");
		}
		$pString .= $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
		if($this->radioButtons && !$noRadio)
			$pString .= $this->html->br() . $this->radioButtons . $this->html->br();
		return $pString;
	}
// Create radio buttons for AND and OR
	private function makeRadioButtons($method, $andDefault = TRUE)
	{
		$type = $this->type . '_' . $method;
		if($andDefault)
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR') . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
		}
		else
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND') . " AND", "small");
		}
		$this->radioButtons = $pString;
	}
/**
* Store previously selected options
*/
	private function grabPreviouslySelected($qsElement)
	{
		if(array_key_exists($qsElement, $this->vars))
		{
			$this->previousSelect = explode(',', $this->vars[$qsElement]);
			if(($index = array_search(0, $this->previousSelect)) !== FALSE)
				unset($this->previousSelect[$index]); // remove 'IGNORE' selected
		}
	}
/**
* Check we have metadata
*/
	private function checkMetadataExists()
	{
		$db = FACTORY_DB::getInstance();
		$commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$commonBib->userBibCondition('resourcequoteResourceId');
		$recordset = $db->select('resource_quote', 'resourcequoteId');
		if(!$db->numRows($recordset))
			unset($this->metadataTypeArray['quote']);
		$commonBib->userBibCondition('resourceparaphraseResourceId');
		$recordset = $db->select('resource_paraphrase', 'resourceparaphraseId');
		if(!$db->numRows($recordset))
			unset($this->metadataTypeArray['paraphrase']);
		$commonBib->userBibCondition('resourcemusingResourceId');
		$recordset = $db->select('resource_musing', 'resourcemusingId');
		if(!$db->numRows($recordset))
			unset($this->metadataTypeArray['musing']);
		$db->leftJoin('resource_quote', 'resourcequoteId', 'resourcequotecommentQuoteId');
		$commonBib->userBibCondition('resourcequoteResourceId');
		$recordset = $db->select('resource_quote_comment', 'resourcequotecommentId');
		if(!$db->numRows($recordset))
			unset($this->metadataTypeArray['quoteComment']);
		$db->leftJoin('resource_paraphrase', 'resourceparaphraseId', 'resourceparaphrasecommentParaphraseId');
		$commonBib->userBibCondition('resourceparaphraseResourceId');
		$recordset = $db->select('resource_paraphrase_comment', 'resourceparaphrasecommentId');
		if(!$db->numRows($recordset))
			unset($this->metadataTypeArray['paraphraseComment']);
		return TRUE;
	}
}
?>
