<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
*	QUICKSEARCH class
*
*	Quickly search database
*
*/
class QUICKSEARCH
{
private $db;
private $vars;
private $stmt;
private $errors;
private $messages;
private $common;
private $session;
private $form;
private $table;
private $html;
private $template;
private $keyword;
private $keywords;
private $input;
private $badInput;
private $initCells;
private $parsePhrase;
private $words = '';
private $typeArray;
private $joinResourceId;
private $tAlias = 1;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->common->quickSearch = TRUE;
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->parsePhrase = FACTORY_PARSEPHRASE::getInstance();
	}
// display form options
	public function init($error = FALSE, $tableBorder = FALSE, $returnString = FALSE)
	{
///First check, do we have resources?
		if(!$this->common->resourcesExist())
			return;
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('search'));
		if(!$returnString)
			GLOBALS::setOutputHeading($this->messages->text("heading", "search"));
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		if(!$tableBorder && $returnString) // FRONT.php display
			$this->session->clearArray('search');
		$pString = $error ? $error : FALSE;
		$pString .= $this->form->formHeader("list_QUICKSEARCH_CORE");
		$pString .= $this->form->hidden("method", "process");
		if($tableBorder)
			$pString .= $this->table->tableStart('generalTable', 1, 0, 4);
		else
			$pString .= $this->table->tableStart(FALSE, 0, 0, 0, FALSE, '50%');
		$pString .= $this->table->trStart();
		$fields = $this->searchFields();
		$this->makeRadioButtons('Field');
		$pString .= $this->table->td($this->makeFormMultiple($fields));
		$this->radioButtons = FALSE;
		$word = $this->session->issetVar("search_Word") ? 
			htmlspecialchars(stripslashes($this->session->getVar("search_Word")), ENT_QUOTES) : FALSE;
		$checked = $this->session->issetVar("search_Partial") ? 'CHECKED' : FALSE;
		$hint = $this->html->br() . $this->html->span($this->messages->text("hint", "wordLogic"), 'hint');
		$pString .= $this->table->td($this->form->textInput($this->messages->text("search", "word"), "search_Word", 
			$word, 40) . $hint . $this->html->p($this->messages->text("search", "partial") . ":&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "search_Partial", $checked) . '&nbsp;&nbsp;&nbsp;' . $this->form->formSubmit("Search")));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		if($returnString)
			return $pString; // cf FRONT.php or process() below.
		else
			GLOBALS::buildOutputString($pString);
	}
// Return array of database fields to perform the search on
	private function searchFields()
	{
		$fields = array("title"	=>	$this->messages->text("search", "title"));
		if($this->db->fetchOne($this->db->selectCount('resource_creator', 'resourcecreatorId')))
			$fields['creator'] = $this->messages->text("search", "creator");
		if($this->db->fetchOne($this->db->selectCount('resource_text', 'resourcetextId')))
			$fields['note'] = $this->messages->text("search", "note");
		if($this->db->fetchOne($this->db->selectCount('resource_text', 'resourcetextId')))
			$fields['abstract'] = $this->messages->text("search", "abstract");
		if($this->db->fetchOne($this->db->selectCount('resource_quote', 'resourcequoteId')))
			$fields['quote'] = $this->messages->text("search", "quote");
		if($this->db->fetchOne($this->db->selectCount('resource_paraphrase', 'resourceparaphraseId')))
			$fields['paraphrase'] = $this->messages->text("search", "paraphrase");
		if($this->db->fetchOne($this->db->selectCount('resource_musing', 'resourcemusingId')))
			$fields['musing'] = $this->messages->text("search", "musing");
		$this->grabKeywords();
		if(is_array($this->keywords))
			$fields['keyword'] = $this->messages->text("resources", "keyword");
// Add any used custom fields
		$subQ = $this->db->subQuery($this->db->selectNoExecute('resource_custom', 'resourcecustomCustomId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('customId') . $this->db->inClause($subQ));
		$recordset = $this->db->select('custom', array('customId', 'customLabel', 'customSize'));
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['customSize'] == 'S')
				$fields['Custom_S_' . $row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
			else
				$fields['Custom_L_' . $row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
		}
		return $fields;
	}
// Reset the form and clear the session
	public function reset()
	{
		$this->session->clearArray('search');
		$this->init();
	}
// make a multiple select box
	private function makeFormMultiple($array)
	{
		$temp = $array;
		if(($selected = $this->session->getVar("search_Field")) && array_key_exists($selected, $temp))
		{
			$selectedArray = explode(",", $selected);
			$pString = $this->form->selectedBoxValueMultiple($this->messages->text("search", 'field'), "search_Field", $temp, $selectedArray, 2);
		}
		else
		{
// If $type == 'field', select all fields as default
			$pString = $this->form->selectFBoxValueMultiple($this->messages->text("search", 'field'), "search_Field", $temp, 2);
		}
		$pString .= $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
			$this->html->br();
		$pString .= $this->radioButtons . $this->html->br();
		return $pString;
	}
// get keywords from database.
	private function grabKeywords()
	{
		$this->keywords = $this->keyword->grabAll($this->session->getVar("mywikindx_Bibliography_use"), 
			'resource', $this->typeArray);
	}
// Create radio buttons for AND and OR
	private function makeRadioButtons($type)
	{
		$type = 'search_' . $type . 'Method';
		if($this->session->getVar($type) == 'AND')
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR') . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
		}
// Default
		else
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . 
				$this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND') . " AND", "small");
		}
		$this->radioButtons = $pString;
	}
// parse the search word(s)
	private function parseWord()
	{
		$this->words = $this->parsePhrase->parse($this->input);
		if(!$this->words)
		{
			$this->template->tpl->assign('resourceListSearchForm', FALSE);
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
		}
	}
// add to SQL conditional statement and add fields to fieldArray for database fields
	private function fieldSql()
	{
		if(!array_key_exists('Field', $this->input))
			return;
		$conditionArray = array();
		$fields = explode(",", $this->input['Field']);
		if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
			$this->joinResourceId = 'id';
		else
			$this->joinResourceId = 'resourcemiscId';
// If $this->input['FieldMethod'] == 'OR', we use UNIONS for efficient SQL statements
		foreach($fields as $field)
		{
			if(strpos($field, 'Custom_') === 0)
			{
				$split = explode('_', $field);
				if($split[1] == 'S')
					$searchField = 'resourcecustomShort';
				else
					$searchField = 'resourcecustomLong';
				$cId = $split[2];
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields($searchField), $this->words);
					$this->db->formatConditions("($wc)");
					$this->db->formatConditions(array('resourcecustomCustomId' => $cId));
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_custom', 
						array(array('resourcecustomResourceId' => 'id'))));
				}
				else
				{
					$wc = str_replace('!WIKINDXFIELDWIKINDX!', 
						$this->db->formatFields($this->tAlias . '.' . $searchField), $this->words);
					$this->stmt->execCond[] = "($wc)";
					$this->stmt->execCond[] = array($this->tAlias . '.' . 'resourcecustomCustomId' => $cId);
					$this->stmt->execJoin[$this->tAlias . '.resource_custom']['intField'] = $this->tAlias . '.resourcecustomResourceId';
					$this->stmt->execJoin[$this->tAlias . '.resource_custom']['extField'] = $this->joinResourceId;
					++$this->tAlias;
				}
			}
			else if($field == 'title')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceTitleSort'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource', 
						array(array('resourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource']['intField'] = 'resourceId';
					$this->stmt->execJoin['resource']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'note')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextNote'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
						array(array('resourcetextId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_text']['intField'] = 'resourcetextId';
					$this->stmt->execJoin['resource_text']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'abstract')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextAbstract'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
						array(array('resourcetextId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_text']['intField'] = 'resourcetextId';
					$this->stmt->execJoin['resource_text']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'quote')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcequotetextText'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
						array(array('resourcequoteResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_quote']['intField'] = 'resourcequoteResourceId';
					$this->stmt->execJoin['resource_quote']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_quote_text']['intField'] = 'resourcequotetextId';
					$this->stmt->execJoin['resource_quote_text']['extField'] = 'resourcequoteId';
				}
			}
			else if($field == 'paraphrase')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceparaphrasetextText'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
						array(array('resourceparaphraseResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_paraphrase']['intField'] = 'resourceparaphraseResourceId';
					$this->stmt->execJoin['resource_paraphrase']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_paraphrase_text']['intField'] = 'resourceparaphrasetextId';
					$this->stmt->execJoin['resource_paraphrase_text']['extField'] = 'resourceparaphraseId';
				}
			}
			else if($field == 'musing')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcemusingtextText'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_musing', 
						array(array('resourcemusingResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_musing']['intField'] = 'resourcemusingResourceId';
					$this->stmt->execJoin['resource_musing']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_musing_text']['intField'] = 'resourcemusingtextId';
					$this->stmt->execJoin['resource_musing_text']['extField'] = 'resourcemusingId';
				}
			}
			else if($field == 'keyword')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('keywordKeyword'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
					$this->db->formatConditions("($wc)");
// Need this check because resource_keyword tables also stores metadata keywords, not just resource keywords
					$this->db->formatConditions($this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL');
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_keyword', 
						array(array('resourcekeywordResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execCond[] = $this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL';
					$this->stmt->execJoin['resource_keyword']['intField'] = 'resourcekeywordResourceId';
					$this->stmt->execJoin['resource_keyword']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['keyword']['intField'] = 'keywordId';
					$this->stmt->execJoin['keyword']['extField'] = 'resourcekeywordKeywordId';
				}
			}
			else if($field == 'creator')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('creatorSurname'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_creator', 
						array(array('resourcecreatorResourceId' => 'id'))));
				}
				else
				{
					$creatorsCond = explode(' AND ', $wc);
					if(sizeof($creatorsCond) > 1)
					{
						$creatorStmts = $creatorAlias = array();
						foreach($creatorsCond AS $creatorCond)
						{
							$this->db->formatConditions("$creatorCond");
							$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
							$creatorStmts[] = $this->db->subQuery($this->db->selectNoExecute('resource_creator',
								array(array('resourcecreatorResourceId' => 'cId' . $this->tAlias)), FALSE, TRUE, TRUE),
								$this->tAlias, FALSE);
							$creatorAlias[] = 'cId' . $this->tAlias;
							++$this->tAlias;
						}
						$masterCreatorId = $this->db->formatFields(array_shift($creatorAlias));
						foreach($creatorAlias AS $cAlias)
							$creatorCondArray[] = $this->db->equal . $this->db->formatFields($cAlias);
						$this->db->formatConditions($masterCreatorId . join($this->db->and . $masterCreatorId, $creatorCondArray));
						$creatStmt = $this->db->selectNoExecuteFromSubQuery(FALSE, $masterCreatorId,
							$this->db->from . join(', ', $creatorStmts), FALSE, FALSE, TRUE);
						$this->stmt->execCond[] = $this->db->formatFields('resourcemiscId') . $this->db->inClause($creatStmt);
					}
					else
					{
						$this->stmt->execCond[] = $wc;
						$this->stmt->execJoin['c.resource_creator']['intField'] = 'c.resourcecreatorResourceId';
						$this->stmt->execJoin['c.resource_creator']['extField'] = $this->joinResourceId;
						$this->stmt->execJoin['creator']['intField'] = 'creatorId';
						$this->stmt->execJoin['creator']['extField'] = 'c.resourcecreatorCreatorId';
					}
				}
			}
			else if($field == 'userTag')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('usertagsTag'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
					$this->db->leftJoin('user_tags', 'usertagsId', 'resourceusertagsTagId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_user_tags', 
						array(array('resourceusertagsId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execCond[] = array('usertagsUserId' => $this->session->getVar('setup_UserId'));
					$this->stmt->execJoin['resource_user_tags']['intField'] = 'resourceusertagsResourceId';
					$this->stmt->execJoin['resource_user_tags']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['user_tags']['intField'] = 'usertagsId';
					$this->stmt->execJoin['user_tags']['extField'] = 'resourceusertagsTagId';
				}
			}
		}
		if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1))
			$this->stmt->unions = $this->db->union($fieldUnions);
		if(!empty($conditionArray))
		{
			$conditionJoin = $this->input['FieldMethod'] == 'OR' ? $this->db->or : $this->db->and;
			$this->stmt->execCond[] = ('(' . join($conditionJoin, $conditionArray) . ')');
		}
	}
/**
* For re-ordering or paging
*/
	public function reprocess()
	{
		$this->input = $this->session->getArray("search");
		if(array_key_exists("search_Order", $this->vars) && $this->vars["search_Order"])
		{
			$this->input['order'] = $this->vars["search_Order"];
			$this->session->setVar('search_Order', $this->input['order']);
			$this->session->setVar('sql_LastOrder', $this->input['order']);
			$this->session->setVar('search_AscDesc', $this->vars['search_AscDesc']);
		}
		$this->process(TRUE);
	}
	public function process($reprocess = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "search"));
		$this->stmt->listMethodAscDesc = 'search_AscDesc';
		$this->stmt->listType = 'search';
		$queryString = 'action=list_QUICKSEARCH_CORE&method=reprocess';
		if(!$reprocess)
			$this->input = $this->checkInput();
		if(array_key_exists('order', $this->input))
			$order = $this->input['order'];
		else
			$order = 'creator';
		$this->template->tpl->assign('resourceListSearchForm', $this->init(FALSE, TRUE, TRUE));
		$this->parseWord();
		$this->fieldSql();
		$sqlTotal = $this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, $order, TRUE, $queryString);
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$this->common->lastMulti('search');
			return;
		}
		$searchTerms = explode(",", $this->session->getVar('search_Highlight'));
		foreach($searchTerms as $term)
			$patterns[] = "/($term)(?=[^>]*(<|$))/i";
		$this->common->patterns = $patterns;
		$this->common->keepHighlight = TRUE;
		if($this->session->getVar('search_DisplayAttachment'))
			$sql = $sqlTotal;
		else
			$sql = $this->stmt->listList(FALSE, FALSE, $order);
		$this->common->display($sql, 'search');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->saveState('search');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
// write input to session
	private function writeSession()
	{
// First, write all input with 'search_' prefix to session
		foreach($this->vars as $key => $value)
		{
			if(preg_match("/^search_/", $key))
			{
				$key = str_replace('search_', '', $key);
// Is this a multiple select box input?  If so, multiple choices are written to session as 
// comma-delimited string (no spaces).
// Don't write any FALSE or '0' values.
				if(is_array($value))
				{
					if(!$value[0] || ($value[0] == $this->messages->text("misc", "ignore")))
						unset($value[0]);
					$value = implode(",", $value);
				}
				if(!trim($value))
					continue;
				$temp[$key] = trim($value);
			}
		}
		$this->session->clearArray("search");
		$this->session->writeArray($temp, 'search');
	}
// validate user input - method, word and field are required.
// Input comes either from form input or, when paging, from the session.
	private function checkInput()
	{
		$this->writeSession();
		$type = FALSE;
		if(array_key_exists("search_Field", $this->vars) && $this->vars["search_Field"])
			$type = $this->vars["search_Field"];
		else if($this->session->issetVar('search_Field'))
			$type = $this->session->getVar("search_Field");
		if(!$type)
			$this->session->setVar('search_Field', 'title'); // force to default title search
		if((array_key_exists("search_Word", $this->vars) && !trim($this->vars["search_Word"])) 
		|| !$this->session->getVar("search_Word"))
		{
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		}
		return $this->session->getArray("search");
	}
}
?>
