<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
*	SEARCHRESOURCES class
*
*	Search database resources on a variety of parameters
*
*/
class SEARCHRESOURCES
{
private $db;
private $vars;
private $stmt;
private $errors;
private $messages;
private $common;
private $session;
private $form;
private $table;
private $html;
private $template;
private $type;
private $category;
private $keyword;
private $userTag;
private $categories;
private $subcategories;
private $keywords;
private $languages = array();
private $userTags;
private $types;
private $input;
private $badInput;
private $commonBib;
private $parsePhrase;
private $words = '';
private $typeArray;
private $joinResourceId;
private $tAlias = 1;
private $resourceTableJoined = FALSE;
private $ajax;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->stmt = FACTORY_SQLSTATEMENTS::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->type = FACTORY_TYPE::getInstance();
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->userTag = FACTORY_USERTAGS::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->parsePhrase = FACTORY_PARSEPHRASE::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "search"));
	}
// display form options
	public function init($error = FALSE)
	{
///First check, do we have resources?
		if(!$this->common->resourcesExist())
			return;
		$searchAttachmentString = FALSE;
		include_once("core/modules/help/HELPMESSAGES.php");
		$help = new HELPMESSAGES();
		GLOBALS::setOutputHelp($help->createLink('search'));
		GLOBALS::setOutputHeading($this->messages->text("heading", "search"));
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$linksInfo['info'] = $this->commonBib->displayBib();
		$this->template->tpl->assign('resourceListInfo', $linksInfo);
		$pString = $error ? $error : FALSE;
		$this->ajax = FACTORY_AJAX::getInstance();
// Always do a full reset because of AJAX controlling some select boxes
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->formHeader("list_SEARCHRESOURCES_CORE") . $this->form->hidden("method", "reset") . 
			$this->form->formSubmit("Reset") . $this->form->formEnd());
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formHeader("list_SEARCHRESOURCES_CORE", "onsubmit=\"selectAll();return true;\"");
		$pString .= $this->form->hidden("method", "process");
		$pString .= $this->table->tableStart(FALSE, FALSE, FALSE, 3);
		$pString .= $this->table->trStart();
		$this->grabTypes();
		$jsonArray = array();
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initLanguages&type=search';
// Amend languages list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'search_Type',
			'targetDiv' => 'language',
			'restoreField' => 'search_Language'
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initCategories&type=search';
// Amend category list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'search_Type',
			'targetDiv' => 'category',
			'restoreField' => 'search_Category'
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initSubcategories&type=search';
// Amend subcategory list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'search_Type',
			'targetDiv' => 'subcategory',
			'restoreField' => 'search_Subcategory'
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initKeywords&type=search';
// Amend keyword list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'search_Type',
			'targetDiv' => 'keyword',
			'restoreField' => 'search_Keyword',
			);
		$jScript = 'index.php?action=list_LISTSHOWHIDE_CORE&method=initUsertags&type=search';
// Amend usertags list depending upon which resource types are chosen
		$jsonArray[] = array(
			'startFunction' => 'triggerFromMultiSelect', 
			'script' => "$jScript",
			'triggerField' => 'search_Type',
			'targetDiv' => 'usertag',
			'restoreField' => 'search_UserTag',
			);
		$this->ajax->jActionForm('onclick', $jsonArray);
		$temp[0] = $this->messages->text("misc", "ignore");
		foreach($this->types as $key => $value)
			$temp[$key] = $value;
		if($selected = $this->session->getVar("search_Type"))
		{
			$this->typeArray = explode(",", $selected);
			$pString .= $this->table->td($this->form->selectedBoxValueMultiple(
				$this->messages->text("search", "type"), "search_Type", $temp, $this->typeArray, 5) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->br(), FALSE, 'left', FALSE, FALSE, '20%');
		}
		else
		{
			$this->typeArray = array();
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple(
				$this->messages->text("search", "type"), "search_Type", $temp, 5) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
				$this->html->br(), FALSE, 'left', FALSE, FALSE, '20%');
		}
		$this->grabLanguages();
		if(!empty($this->languages))
			$pString .= $this->table->td($this->html->div('language', $this->makeFormMultiple("language", $this->languages)), 
			FALSE, 'left', FALSE, FALSE, '20%');
		else
			$pString .= $this->table->td($this->html->div('language', "&nbsp;"), FALSE, 'left', FALSE, FALSE, '20%');
		$fields = $this->searchFields();
		$this->makeRadioButtons('Field', FALSE);
		$pString .= $this->table->td($this->makeFormMultiple("field", $fields, TRUE), FALSE, 'left', FALSE, FALSE, '20%');
		$this->radioButtons = FALSE;
		$word = $this->session->issetVar("search_Word") ? 
			htmlspecialchars(stripslashes($this->session->getVar("search_Word")), ENT_QUOTES) : FALSE;
		$checked = $this->session->issetVar("search_Partial") ? 'CHECKED' : FALSE;
		$hint = $this->html->br() . $this->html->span($this->messages->text("hint", "wordLogic"), 'hint');
		$pString .= $this->table->td($this->html->span('*', 'required') . '&nbsp;' . 
			$this->form->textInput($this->messages->text("search", "word"), "search_Word", $word, 40) . $hint . 
			$this->html->p($this->messages->text("search", "partial") . ":&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, "search_Partial", $checked)), FALSE, 'left', FALSE, FALSE, '20%');
		$pString .= $this->table->td($this->common->displayOrder('search'), FALSE, 'left', FALSE, FALSE, '20%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->hr();
		$pString .= $this->table->tableStart(FALSE, FALSE, FALSE, 3);
		$pString .= $this->table->trStart();
		$this->grabCategories();
		$categoryTable = $this->table->tableStart('generalTable');
		$categoryTable .= $this->table->trStart();
		$selectedCategories = array();
		if($selected = $this->session->getVar("search_Category"))
		{
			$selected = explode(',', $selected);
			foreach($selected as $key)
			{
				$selectedCategories[$key] = $this->categories[$key];
				unset($this->categories[$key]);
			}
		}
		$categoryTable .= $this->table->td($this->makeFormMultiple("availableCategory", $this->categories, TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '18%');
		list($toRightImage, $toLeftImage) = $this->category->transferArrowsCat();
		$categoryTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
		$categoryTable .= $this->table->td($this->makeFormMultiple("category", $selectedCategories, TRUE), FALSE, 'left', FALSE, FALSE, '18%');
		$categoryTable .= $this->table->trEnd();
		$categoryTable .= $this->table->tableEnd();
// There may not yet be subcategories in the database.
		$this->grabSubcategories();
		$pString .= $this->table->td($this->html->div('category', $categoryTable), FALSE, 'left', FALSE, FALSE, '41%');
		if(is_array($this->subcategories))
		{
			$subcatTable = $this->table->tableStart('generalTable');
			$subcatTable .= $this->table->trStart();
			$selectedSubcats = array();
			if($selected = $this->session->getVar("search_Subcategory"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					$selectedSubcats[$key] = $this->subcategories[$key];
					unset($this->subcategories[$key]);
				}
			}
			$subcatTable .= $this->table->td($this->makeFormMultiple("availableSubcategory", $this->subcategories, TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '18%');
			list($toRightImage, $toLeftImage) = $this->category->transferArrowsSubcat();
			$subcatTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$subcatTable .= $this->table->td($this->makeFormMultiple("subcategory", $selectedSubcats, TRUE), FALSE, 'left', FALSE, FALSE, '18%');
		}
		else
		{
			$subcatTable = $this->table->tableStart();
			$subcatTable .= $this->table->trStart();
			$subcatTable .= $this->table->td("&nbsp;");
		}
		$subcatTable .= $this->table->trEnd();
		$subcatTable .= $this->table->tableEnd();
		$pString .= $this->table->td($this->html->div('subcategory', $subcatTable), FALSE, 'left', FALSE, FALSE, '41%');
		if($this->session->getVar("setup_ReadOnly") && $this->session->getVar("setup_FileViewLoggedOnOnly"))
			$extras = array();
		else if(!$this->db->fetchOne($this->db->selectCount('resource_attachments', 'resourceattachmentsId')))
			$extras = array();
		else
		{
			$attach = array(
					'ignore'	=>	$this->messages->text("misc", "ignore"),
					'no'		=>	$this->messages->text('select', 'noAttachment'),
					'yes'		=>	$this->messages->text('select', 'attachment')
				);
			if($selected = $this->session->getVar("search_Attachment"))
				$searchAttachmentString = $this->form->selectedBoxValue(FALSE, "search_Attachment", $attach, $selected, 3);
			else
				$searchAttachmentString = $this->html->p($this->form->selectFBoxValue(FALSE, "search_Attachment", $attach, 3));
			$checked = $this->session->getVar('search_DisplayAttachment') ? TRUE : FALSE;
			$extras[] = $this->messages->text('select', 'displayAttachment') . ":&nbsp;&nbsp;" . 
			$this->form->checkbox(FALSE, 'search_DisplayAttachment', $checked);
			$checked = $this->session->getVar('search_DisplayAttachmentZip') ? TRUE : FALSE;
			$extras[] = $this->messages->text('select', 'displayAttachmentZip') . ":&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, 'search_DisplayAttachmentZip', $checked);
		}
		$checked = $this->session->getVar('search_DisplayPeerReviewed') ? TRUE : FALSE;
		if($searchAttachmentString)
		{
			$extras[] = $this->messages->text('select', 'displayPeerReviewed') .  ":&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, 'search_DisplayPeerReviewed', $checked);
			$pString .= $this->table->td($searchAttachmentString . $this->html->p(join($this->html->br(), $extras)), 
			FALSE, 'left', FALSE, FALSE, '18%');
		}
		else
			$pString .= $this->table->td($this->messages->text('select', 'displayPeerReviewed') .  ":&nbsp;&nbsp;" . 
				$this->form->checkbox(FALSE, 'search_DisplayPeerReviewed', $checked), FALSE, 'left', FALSE, FALSE, '18%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->hr();
		$pString .= $this->table->tableStart(FALSE, FALSE, FALSE, 3);
		$pString .= $this->table->trStart();
// There may not yet be keywords in the database.
		$this->grabKeywords();
		if(is_array($this->keywords))
		{
			$keywordTable = $this->table->tableStart('generalTable');
			$keywordTable .= $this->table->trStart();
			$selectedKeywords = array();
			if($selected = $this->session->getVar("search_Keyword"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					if(array_key_exists($key, $this->keywords))
					{
						$selectedKeywords[$key] = $this->keywords[$key];
						unset($this->keywords[$key]);
					}
				}
			}
			$keywordTable .= $this->table->td($this->makeFormMultiple("availableKeyword", $this->keywords, TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '20%');
			list($toRightImage, $toLeftImage) = $this->keyword->transferArrows();
			$keywordTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$keywordTable .= $this->table->td($this->makeFormMultiple("keyword", $selectedKeywords, TRUE), FALSE, 'left', FALSE, FALSE, '20%');
		}
		else
		{
			$keywordTable = $this->table->tableStart();
			$keywordTable .= $this->table->trStart();
			$keywordTable .= $this->table->td("&nbsp;");
		}
		$keywordTable .= $this->table->trEnd();
		$keywordTable .= $this->table->tableEnd();
		$pString .= $this->table->td($this->html->div('keyword', $keywordTable), FALSE, 'left', FALSE, FALSE, '45%');
// There may not yet be user tags in the database.
		$this->grabUserTags();
		if(!empty($this->userTags))
		{
			$utTable = $this->table->tableStart('generalTable');
			$utTable .= $this->table->trStart();
			$selectedUTs = array();
			if($selected = $this->session->getVar("search_UserTag"))
			{
				$selected = explode(',', $selected);
				foreach($selected as $key)
				{
					$selectedUTs[$key] = $this->userTags[$key];
					unset($this->userTags[$key]);
				}
			}
			$utTable .= $this->table->td($this->makeFormMultiple("availableUserTag", $this->userTags, TRUE, TRUE), FALSE, 'left', FALSE, FALSE, '20%');
			list($toRightImage, $toLeftImage) = $this->userTag->transferArrows();
			$utTable .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage), FALSE, 'left', FALSE, FALSE, '5%');
			$utTable .= $this->table->td($this->makeFormMultiple("userTag", $selectedUTs, TRUE), FALSE, 'left', FALSE, FALSE, '20%');
		}
		else
		{
			$utTable = $this->table->tableStart();
			$utTable .= $this->table->trStart();
			$utTable .= $this->table->td("&nbsp;");
		}
		$utTable .= $this->table->trEnd();
		$utTable .= $this->table->tableEnd();
		$pString .= $this->table->td($this->html->div('usertag', $utTable), FALSE, 'left', FALSE, FALSE, '45%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->formSubmit("Search"), FALSE, 'left');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
// Load javascript at end after form elements have printed
		$this->ajax->loadJavascript('core/modules/list/searchSelect.js');
	}
// Return array of database fields to perform the search on
	private function searchFields()
	{
		$userBib = $this->session->getVar("mywikindx_Bibliography_use");
		$fields = array("title"	=>	$this->messages->text("search", "title"));
		if($userBib)
			$this->commonBib->userBibCondition('resourcecreatorId');
		if($this->db->fetchOne($this->db->selectCount('resource_creator', 'resourcecreatorId')))
			$fields['creator'] = $this->messages->text("search", "creator");
		if($userBib)
			$this->commonBib->userBibCondition('resourcetextId');
		if($this->db->fetchOne($this->db->selectCount('resource_text', 'resourcetextId')))
			$fields['note'] = $this->messages->text("search", "note");
		if($this->db->fetchOne($this->db->selectCount('resource_text', 'resourcetextId')))
			$fields['abstract'] = $this->messages->text("search", "abstract");
		if($userBib)
			$this->commonBib->userBibCondition('resourcequoteId');
		if($this->db->fetchOne($this->db->selectCount('resource_quote', 'resourcequoteId')))
			$fields['quote'] = $this->messages->text("search", "quote");
		if($userBib)
			$this->commonBib->userBibCondition('resourceparaphraseId');
		if($this->db->fetchOne($this->db->selectCount('resource_paraphrase', 'resourceparaphraseId')))
			$fields['paraphrase'] = $this->messages->text("search", "paraphrase");
		if($userBib)
			$this->commonBib->userBibCondition('resourcemusingId');
		if($this->db->fetchOne($this->db->selectCount('resource_musing', 'resourcemusingId')))
			$fields['musing'] = $this->messages->text("search", "musing");
		if(is_array($this->keywords))
			$fields['keyword'] = $this->messages->text("resources", "keyword");
		if(!empty($this->userTags))
			$fields['userTag'] = $this->messages->text("select", "userTag");
// Add any used custom fields
		$subQ = $this->db->subQuery($this->db->selectNoExecute('resource_custom', 'resourcecustomCustomId'), FALSE, FALSE, TRUE);
		$this->db->formatConditions($this->db->formatFields('customId') . $this->db->inClause($subQ));
		$recordset = $this->db->select('custom', array('customId', 'customLabel', 'customSize'));
		while($row = $this->db->fetchRow($recordset))
		{
			if($row['customSize'] == 'S')
				$fields['Custom_S_' . $row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
			else
				$fields['Custom_L_' . $row['customId']] = $this->html->dbToFormTidy($row['customLabel']);
		}
		return $fields;
	}
// Reset the form and clear the session
	public function reset()
	{
		$this->session->clearArray('search');
		$this->init();
	}
// make a multiple select box
	private function makeFormMultiple($type, $array, $noIgnore = FALSE, $noRadio = FALSE)
	{ 
// add 0 => IGNORE to $array
		if(!$noIgnore)
		{
			$temp[0] = $this->messages->text("misc", "ignore");
			foreach($array as $key => $value)
				$temp[$key] = $value;
		}
		else
			$temp = $array;
		if($selected = $this->session->getVar("search_" . ucfirst($type)))
		{
			$selectedArray = explode(",", $selected);
			$pString = $this->form->selectedBoxValueMultiple($this->messages->text("search", $type), 
				"search_" . ucfirst($type), $temp, $selectedArray, 5);
		}
		else
		{
// If $type == 'field', select all fields as default
			if($type == 'field')
			{
				$pString = $this->form->selectedBoxValueMultiple($this->messages->text("search", $type), 
					"search_" . 'Field', $temp, array_keys($array), 5);
			}
			else
				$pString = $this->form->selectFBoxValueMultiple($this->messages->text("search", $type), 
					"search_" . ucfirst($type), $temp, 5);
		}
		$pString .= $this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint');
		if(!$noRadio)
			$pString .= $this->html->br() . $this->radioButtons;
		return $pString;
	}
// get languages from database.
	private function grabLanguages()
	{
		$this->db->orderBy('languageLanguage');
		$this->db->leftJoin('language', 'languageId', 'resourcelanguageLanguageId');
		$resultset = $this->db->select('resource_language', array('resourcelanguageLanguageId', 'languageLanguage'), TRUE);
		while($row = $this->db->fetchRow($resultset))
			$this->languages[$row['resourcelanguageLanguageId']] = $row['languageLanguage'];
		if(empty($this->languages))
			return;
// add 0 => IGNORE to array
		$temp[0] = $this->messages->text("misc", "ignore");
		foreach($this->languages as $key => $value)
			$temp[$key] = $value;
		$this->makeRadioButtons('Language', FALSE);
		$this->languages = $temp;
	}
// get types from database.
	private function grabTypes()
	{
		$this->types = $this->type->grabAll();
	}
// get categories from database.
	private function grabCategories()
	{
		$this->categories = $this->category->grabAll($this->session->getVar("mywikindx_Bibliography_use"), TRUE, $this->typeArray, FALSE);
		$this->makeRadioButtons('Category');
	}
// get subcategories from database.
	private function grabSubcategories($matchCategoryIds = FALSE)
	{
		$this->subcategories = 
			$this->category->grabSubAll(TRUE, $this->session->getVar("mywikindx_Bibliography_use"), $matchCategoryIds, TRUE);
		$this->makeRadioButtons('Subcategory');
	}
// get keywords from database.
	private function grabKeywords()
	{
		$this->keywords = $this->keyword->grabAll($this->session->getVar("mywikindx_Bibliography_use"), 'resource', $this->typeArray);
		$this->makeRadioButtons('Keyword');
	}
// get user tags from database.
	private function grabUserTags()
	{
		$this->userTags = $this->userTag->grabAll($this->session->getVar("mywikindx_Bibliography_use"), FALSE, TRUE);
		$this->makeRadioButtons('UserTag');
	}
// Create radio buttons for AND and OR
	private function makeRadioButtons($type, $andDefault = TRUE)
	{
		$type = 'search_' . $type . 'Method';
		if(!$this->session->issetVar($type))
		{
			if($andDefault)
			{
				$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR') . " OR", "small") . $this->html->br();
				$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
			}
			else
			{
				$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . $this->html->br();
				$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND') . " AND", "small");
			}
		}
		else if($this->session->getVar($type) == 'AND')
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR') . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND', TRUE) . " AND", "small");
		}
// Session 'OR'
		else
		{
			$pString = $this->html->span($this->form->radioButton(FALSE, $type, 'OR', TRUE) . " OR", "small") . $this->html->br();
			$pString .= $this->html->span($this->form->radioButton(FALSE, $type, 'AND') . " AND", "small");
		}
		$this->radioButtons = $pString;
	}
// parse the search word(s)
	private function parseWord()
	{
		$this->words = $this->parsePhrase->parse($this->input);
		if(empty($this->words))
			$this->badInput->close($this->errors->text("inputError", "invalid"), $this, 'init');
	}
// add to SQL conditional statement and add fields to fieldArray for database fields
	private function fieldSql($order)
	{
		if(!array_key_exists('Field', $this->input))
			return;
		$conditionArray = array();
		$fields = explode(",", $this->input['Field']);
		if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
			$this->joinResourceId = 'id';
		else if($order == 'attachments')
			$this->joinResourceId = 'resourceattachmentsResourceId';
		else
			$this->joinResourceId = 'resourcemiscId';
// If $this->input['FieldMethod'] == 'OR', we use UNIONS for efficient SQL statements
		foreach($fields as $field)
		{
			if(strpos($field, 'Custom_') === 0)
			{
				$split = explode('_', $field);
				if($split[1] == 'S')
					$searchField = 'resourcecustomShort';
				else
					$searchField = 'resourcecustomLong';
				$cId = $split[2];
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields($searchField), $this->words);
					$this->db->formatConditions("($wc)");
					$this->db->formatConditions(array('resourcecustomCustomId' => $cId));
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_custom', 
						array(array('resourcecustomResourceId' => 'id'))));
				}
				else
				{
					$wc = str_replace('!WIKINDXFIELDWIKINDX!', 
						$this->db->formatFields($this->tAlias . '.' . $searchField), $this->words);
					$this->stmt->execCond[] = "($wc)";
					$this->stmt->execCond[] = array($this->tAlias . '.' . 'resourcecustomCustomId' => $cId);
					$this->stmt->execJoin[$this->tAlias . '.resource_custom']['intField'] = $this->tAlias . '.resourcecustomResourceId';
					$this->stmt->execJoin[$this->tAlias . '.resource_custom']['extField'] = $this->joinResourceId;
					++$this->tAlias;
				}
			}
			else if($field == 'title')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceTitleSort'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource', 
						array(array('resourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource']['intField'] = 'resourceId';
					$this->stmt->execJoin['resource']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'note')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextNote'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
						array(array('resourcetextId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_text']['intField'] = 'resourcetextId';
					$this->stmt->execJoin['resource_text']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'abstract')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcetextAbstract'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', 
						array(array('resourcetextId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_text']['intField'] = 'resourcetextId';
					$this->stmt->execJoin['resource_text']['extField'] = $this->joinResourceId;
				}
			}
			else if($field == 'quote')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcequotetextText'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
						array(array('resourcequoteResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_quote']['intField'] = 'resourcequoteResourceId';
					$this->stmt->execJoin['resource_quote']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_quote_text']['intField'] = 'resourcequotetextId';
					$this->stmt->execJoin['resource_quote_text']['extField'] = 'resourcequoteId';
				}
			}
			else if($field == 'paraphrase')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourceparaphrasetextText'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
						array(array('resourceparaphraseResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_paraphrase']['intField'] = 'resourceparaphraseResourceId';
					$this->stmt->execJoin['resource_paraphrase']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_paraphrase_text']['intField'] = 'resourceparaphrasetextId';
					$this->stmt->execJoin['resource_paraphrase_text']['extField'] = 'resourceparaphraseId';
				}
			}
			else if($field == 'musing')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('resourcemusingtextText'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_musing', 
						array(array('resourcemusingResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execJoin['resource_musing']['intField'] = 'resourcemusingResourceId';
					$this->stmt->execJoin['resource_musing']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['resource_musing_text']['intField'] = 'resourcemusingtextId';
					$this->stmt->execJoin['resource_musing_text']['extField'] = 'resourcemusingId';
				}
			}
			else if($field == 'keyword')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('keywordKeyword'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
					$this->db->formatConditions("($wc)");
// Need this check because resource_keyword tables also stores metadata keywords, not just resource keywords
					$this->db->formatConditions($this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL');
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_keyword', 
						array(array('resourcekeywordResourceId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execCond[] = $this->db->formatFields('resourcekeywordResourceId') . ' IS NOT NULL';
					$this->stmt->execJoin['resource_keyword']['intField'] = 'resourcekeywordResourceId';
					$this->stmt->execJoin['resource_keyword']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['keyword']['intField'] = 'keywordId';
					$this->stmt->execJoin['keyword']['extField'] = 'resourcekeywordKeywordId';
				}
			}
			else if($field == 'creator')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('creatorSurname'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions("($wc)");
					$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_creator', 
						array(array('resourcecreatorResourceId' => 'id'))));
				}
				else
				{

					$creatorsCond = explode(' AND ', $wc);
					if(sizeof($creatorsCond) > 1)
					{
						$creatorStmts = $creatorAlias = array();
						foreach($creatorsCond AS $creatorCond)
						{
							$this->db->formatConditions("$creatorCond");
							$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
							$creatorStmts[] = $this->db->subQuery($this->db->selectNoExecute('resource_creator',

								array(array('resourcecreatorResourceId' => 'cId' . $this->tAlias)), FALSE, TRUE, TRUE),
								$this->tAlias, FALSE);
							$creatorAlias[] = 'cId' . $this->tAlias;
							++$this->tAlias;
						}
						$masterCreatorId = $this->db->formatFields(array_shift($creatorAlias));
						foreach($creatorAlias AS $cAlias)
							$creatorCondArray[] = $this->db->equal . $this->db->formatFields($cAlias);
						$this->db->formatConditions($masterCreatorId . join($this->db->and . $masterCreatorId, $creatorCondArray));
						$creatStmt = $this->db->selectNoExecuteFromSubQuery(FALSE, $masterCreatorId,
							$this->db->from . join(', ', $creatorStmts), FALSE, FALSE, TRUE);
						$this->stmt->execCond[] = $this->db->formatFields('resourcemiscId') . $this->db->inClause($creatStmt);
					}
					else
					{
						$this->stmt->execCond[] = $wc;
						$this->stmt->execJoin['c.resource_creator']['intField'] = 'c.resourcecreatorResourceId';
						$this->stmt->execJoin['c.resource_creator']['extField'] = $this->joinResourceId;
						$this->stmt->execJoin['creator']['intField'] = 'creatorId';
						$this->stmt->execJoin['creator']['extField'] = 'c.resourcecreatorCreatorId';
					}
				}
			}
			else if($field == 'userTag')
			{
				$wc = str_replace('!WIKINDXFIELDWIKINDX!', $this->db->formatFields('usertagsTag'), $this->words);
				if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1)) // UNIONs
				{
					$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
					$this->db->leftJoin('user_tags', 'usertagsId', 'resourceusertagsTagId');
					$this->db->formatConditions("($wc)");
					$fieldUnions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_user_tags', 
						array(array('resourceusertagsId' => 'id'))));
				}
				else
				{
					$conditionArray[] = $wc;
					$this->stmt->execCond[] = array('usertagsUserId' => $this->session->getVar('setup_UserId'));
					$this->stmt->execJoin['resource_user_tags']['intField'] = 'resourceusertagsResourceId';
					$this->stmt->execJoin['resource_user_tags']['extField'] = $this->joinResourceId;
					$this->stmt->execJoin['user_tags']['intField'] = 'usertagsId';
					$this->stmt->execJoin['user_tags']['extField'] = 'resourceusertagsTagId';
				}
			}
		}
		$this->session->setVar('search_Wordclause', base64_encode($wc));
		if($this->session->getVar('search_DisplayPeerReviewed'))
		{
			if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1))
			{
				if($order != 'attachments')
					$this->stmt->execCond[] = array('resourcemiscPeerReviewed' => 'Y');
			}
			else
				$this->stmt->execCond[] = array('resourcemiscPeerReviewed' => 'Y');
		}
		if(($this->input['FieldMethod'] == 'OR') && (sizeof($fields) > 1))
			$this->stmt->unions = $this->db->union($fieldUnions);
		if(!empty($conditionArray))
		{
			$conditionJoin = $this->input['FieldMethod'] == 'OR' ? $this->db->or : $this->db->and;
			$this->stmt->execCond[] = ('(' . join($conditionJoin, $conditionArray) . ')');
		}
	}
// add to SQL conditional statement and add fields to fieldArray for resource types
	private function typeSql()
	{
		if(!array_key_exists('Type', $this->input))
			return;
		$types = explode(",", $this->input['Type']);
		foreach($types as $type)
			$conditionArray[] = $this->db->formatFields('resourceType') . "=" . $this->db->tidyInput($type);
		$this->stmt->execCond[] = join($this->db->or, $conditionArray);
		$this->stmt->execJoin['resource']['intField'] = 'resourceId';
		$this->stmt->execJoin['resource']['extField'] = $this->joinResourceId;
	}
// add to SQL conditional statement and add fields to fieldArray for resource languages
	private function languageSql()
	{
		if(!array_key_exists('Language', $this->input))
			return;
		$languages = explode(",", $this->input['Language']);
		if($this->input['LanguageMethod'] == 'OR')
		{
			$this->stmt->execCondOneField[$this->tAlias . '.resourcelanguageLanguageId'] = $languages;
			$this->stmt->execJoin[$this->tAlias . '.resource_language']['intField'] = $this->tAlias . '.resourcelanguageResourceId';
			$this->stmt->execJoin[$this->tAlias . '.resource_language']['extField'] = $this->joinResourceId;
			++$this->tAlias;
		}
		else
		{
			foreach($languages as $language)
			{
				$this->stmt->execCond[] = array($this->tAlias . '.resourcelanguageLanguageId' => $language);
				$this->stmt->execJoin[$this->tAlias . '.resource_language']['intField'] = $this->tAlias . '.resourcelanguageResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_language']['extField'] = $this->joinResourceId;
				++$this->tAlias;
			}
		}
	}
// add to SQL conditional statement and add fields to fieldArray for categories
	function categorySql()
	{
		if(!array_key_exists('Category', $this->input))
			return;
		$categories = explode(",", $this->input['Category']);
		if($this->input['CategoryMethod'] == 'OR')
		{
			$this->stmt->execCondOneField[$this->tAlias . '.resourcecategoryCategoryId'] = $categories;
			$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
			$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
			++$this->tAlias;
		}
		else
		{
			foreach($categories as $category)
			{
				$this->stmt->execCond[] = array($this->tAlias . '.resourcecategoryCategoryId' => $category);
				$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
				++$this->tAlias;
			}
		}
	}
// add to SQL conditional statement and add fields to fieldArray for subcategories
	function subcategorySql()
	{
		if(!array_key_exists('Subcategory', $this->input))
			return;
		$subcategories = explode(",", $this->input['Subcategory']);
		if($this->input['SubcategoryMethod'] == 'OR')
		{
			$this->stmt->execCondOneField[$this->tAlias . '.resourcecategorySubcategoryId'] = $subcategories;
			$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
			$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
			++$this->tAlias;
		}
		else
		{
			foreach($subcategories as $subcategory)
			{
				$this->stmt->execCond[] = array($this->tAlias . '.resourcecategorySubcategoryId' => $subcategory);
				$this->stmt->execJoin[$this->tAlias . '.resource_category']['intField'] = $this->tAlias . '.resourcecategoryResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_category']['extField'] = $this->joinResourceId;
				++$this->tAlias;
			}
		}
	}
// add to SQL conditional statement and add fields to fieldArray for keywords
	private function keywordSql()
	{
		if(!array_key_exists('Keyword', $this->input))
			return;
		$keywords = explode(",", $this->input['Keyword']);
		if($this->input['KeywordMethod'] == 'OR')
		{
			$this->stmt->execCondOneField['resourcekeywordKeywordId'] = $keywords;
			$this->stmt->execJoin['resource_keyword']['intField'] = 'resourcekeywordResourceId';
			$this->stmt->execJoin['resource_keyword']['extField'] = $this->joinResourceId;
		}
		else
		{
			foreach($keywords as $keyword)
			{
				$this->stmt->execCond[] = array($this->tAlias . '.resourcekeywordKeywordId' => $keyword);
				$this->stmt->execJoin[$this->tAlias . '.resource_keyword']['intField'] = $this->tAlias . '.resourcekeywordResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_keyword']['extField'] = $this->joinResourceId;
				++$this->tAlias;
			}
		}
	}
// add to SQL conditional statement and add fields to fieldArray for user tags
	private function userTagSql()
	{
		if(!array_key_exists('UserTag', $this->input))
			return;
		$userTags = explode(",", $this->input['UserTag']);
		if($this->input['UserTagMethod'] == 'OR')
		{
			$this->stmt->execCondOneField['resourceusertagsTagId'] = $userTags;
			$this->stmt->execJoin['resource_user_tags']['intField'] = 'resourceusertagsResourceId';
			$this->stmt->execJoin['resource_user_tags']['extField'] = $this->joinResourceId;
		}
		else
		{
			foreach($userTags as $usertag)
			{
				$this->stmt->execCond[] = array($this->tAlias . '.resourceusertagsTagId' => $usertag);
				$this->stmt->execJoin[$this->tAlias . '.resource_user_tags']['intField'] = $this->tAlias . '.resourceusertagsResourceId';
				$this->stmt->execJoin[$this->tAlias . '.resource_user_tags']['extField'] = $this->joinResourceId;
				++$this->tAlias;
			}
		}
	}
// create the basic sql statement before we apply LIMIT
	private function createTotalSql()
	{
		$attach = $this->session->getVar('search_Attachment');
		$order = $this->session->getVar('search_DisplayAttachment') ? 'attachments' : $this->input['Order'];
		$this->parseWord();
		$this->fieldSql($order);
		$this->typeSql();
		$this->languageSql();
		$this->categorySql();
		$this->subcategorySql();
		$this->keywordSql();
		$this->userTagSql();
		if($attach)
		{
			if($attach == 'no')
				$this->stmt->execCond[] = array('resourceattachmentsId' => ' IS NULL');
			else if($attach == 'yes')
				$this->stmt->execCond[] = array('resourceattachmentsId' => ' IS NOT NULL');
			if(($attach != 'ignore') && ($order != 'attachments'))
			{
				$this->stmt->execJoin['resource_attachments']['intField'] = 'resourceattachmentsResourceId';
				$this->stmt->execJoin['resource_attachments']['extField'] = $this->joinResourceId;
			}
		}
		return $order;
	}
/**
* For re-ordering or paging
*/
	public function reprocess()
	{
		$this->input = $this->session->getArray("search");
		if(array_key_exists('search_Order', $this->vars) && $this->vars['search_Order'])
		{
			$this->input['Order'] = $this->vars['search_Order'];
			$this->session->setVar('search_Order', $this->input['Order']);
			$this->session->setVar('sql_LastOrder', $this->input['Order']);
		}
		if(array_key_exists('search_AscDesc', $this->vars) && $this->vars['search_AscDesc'])
		{
			$this->input['AscDesc'] = $this->vars['search_AscDesc'];
			$this->session->setVar('search_AscDesc', $this->input['AscDesc']);
		}
		$this->process(TRUE);
	}
	public function process($reprocess = FALSE)
	{
		$this->stmt->listMethodAscDesc = 'search_AscDesc';
		$this->stmt->listType = 'search';
		$queryString = 'action=list_SEARCHRESOURCES_CORE&method=reprocess';
		if(!$reprocess)
			$this->input = $this->checkInput();
		$order = $this->createTotalSql();
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$this->session->delVar('mywikindx_PagingStart');
			$this->session->delVar('mywikindx_PagingStartAlpha');
		}
		$sqlTotal = $this->stmt->createTotalResourceSubqueryList(FALSE, FALSE, $order, TRUE, $queryString);
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'lastMulti'))
		{
			$this->common->lastMulti('search');
			return;
		}
		$searchTerms = explode(",", $this->session->getVar('search_Highlight'));
		foreach($searchTerms as $term)
			$patterns[] = "/($term)(?=[^>]*(<|$))/i";
		$this->common->patterns = $patterns;
		$this->common->keepHighlight = TRUE;
		if($this->session->getVar('search_DisplayAttachment'))
			$sql = $sqlTotal;
		else
			$sql = $this->stmt->listList(FALSE, FALSE, $order);
		$this->common->display($sql, 'search');
// set the lastMulti session variable for quick return to this process.
		$this->session->setVar('sql_LastMulti', $queryString);
// Turn on the 'add bookmark' menu item
		$this->session->setVar("bookmark_DisplayAdd", TRUE);
		$this->session->saveState('search');
		$this->session->saveState('sql');
		$this->session->saveState('bookmark');
	}
// write input to session
	private function writeSession()
	{
// First, write all input with 'search_' prefix to session
		foreach($this->vars as $key => $value)
		{
			if(preg_match("/^search_/", $key))
			{
				$key = str_replace('search_', '', $key);
// Is this a multiple select box input?  If so, multiple choices are written to session as 
// comma-delimited string (no spaces).
// Don't write any FALSE or '0' values.
				if(is_array($value))
				{
					if(!$value[0] || ($value[0] == $this->messages->text("misc", "ignore")))
						unset($value[0]);
					$value = implode(",", $value);
				}
				if(!trim($value))
					continue;
				$temp[$key] = trim($value);
			}
		}
		$this->session->clearArray("search");
		$this->session->writeArray($temp, 'search');
	}
// validate user input - method, word and field are required.
// Input comes either from form input or, when paging, from the session.
	private function checkInput()
	{
		if(array_key_exists('search_DisplayAttachment', $this->vars))
		{
			if(array_key_exists('search_Attachment', $this->vars) && 
				(($this->vars['search_Attachment'] == 'ignore') || ($this->vars['search_Attachment'] == 'no')))
				unset($this->vars['search_DisplayAttachment']);
		}
		$this->writeSession();
		if(array_key_exists("search_Field", $this->vars) && $this->vars["search_Field"])
			$type = $this->vars["search_Field"];
		else if($this->session->issetVar('search_Field'))
			$type = $this->session->getVar("search_Field");
		else
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if((array_key_exists("search_Word", $this->vars) && !trim($this->vars["search_Word"])) 
		|| !$this->session->getVar("search_Word"))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		return $this->session->getArray("search");
	}
}
?>
