<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/

/**
* Edit Metadata:  notes, abstract, quotes, paraphrases, musings.
*/
class EDITMETADATA
{
private $db;
private $vars;
private $badInput;
private $form;
private $html;
private $gatekeep;
private $errors;
private $messages;
private $success;
private $session;
private $tinymce;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->tinymce = FACTORY_LOADTINYMCE::getInstance();
	}
	public function init()
	{
		if(!array_key_exists('type', $this->vars) || !array_key_exists('id', $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"));
		if(($this->vars['type'] == 'abstractInit') || ($this->vars['type'] == 'noteInit') || 
			($this->vars['type'] == 'noteDeleteInit') || ($this->vars['type'] == 'abstractDeleteInit') || 
			($this->vars['type'] == 'noteDelete') || ($this->vars['type'] == 'abstractDelete') || 
			($this->vars['type'] == 'abstractEdit') || ($this->vars['type'] == 'noteEdit'))
			$this->gatekeep->init();
		else
			$this->badInput->close($this->errors->text("inputError", "missing"));
// proceed
		$type = $this->vars['type'];
		$this->{$type}();
	}
/**
* Display form for editing abstract
*/
	private function abstractInit()
	{
		$this->session->delVar('metadataLock');
		GLOBALS::setOutputHeading($this->messages->text("heading", "abstract"));
// Create citation link for this type
//		include_once("core/cite/CITE.php");
//		$citeLink = CITE::makeCiteLink('abstract');
		$pString = $this->form->formHeader('metadata_EDITMETADATA_CORE');
		$pString .= $this->form->hidden("type", "abstractEdit");
		$pString .= $this->form->hidden("id", $this->vars['id']);
		$pString .= $this->tinymce->loadMetadataTextarea();
		$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
		$recordset = $this->db->select('resource_text', 'resourcetextAbstract');
		$row = $this->db->fetchRow($recordset);
		if(!$this->db->numRows($recordset))
			$pString .= $this->form->hidden('sql', 'insert');
		else
			$pString .= $this->form->hidden('sql', 'update');
		$pString .= $this->form->textareaInput(FALSE, 
			"text", $this->html->dbToFormTidy($row['resourcetextAbstract']), 80, 15);
		$pString .= $this->html->p($this->form->formSubmit("Save"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
/**
* Display form for editing note
*/
	private function noteInit()
	{
		$this->session->delVar('metadataLock');
		GLOBALS::setOutputHeading($this->messages->text("heading", "notes"));
// Create citation link for this type
//		include_once("core/cite/CITE.php");
//		$citeLink = CITE::makeCiteLink('note');
		$pString = $this->form->formHeader('metadata_EDITMETADATA_CORE');
		$pString .= $this->form->hidden("type", "noteEdit");
		$pString .= $this->form->hidden("id", $this->vars['id']);
		$pString .= $this->tinymce->loadMetadataTextarea();
		$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
		$recordset = $this->db->select('resource_text', 'resourcetextNote');
		$row = $this->db->fetchRow($recordset);
		if(!$this->db->numRows($recordset))
			$pString .= $this->form->hidden('sql', 'insert');
		else
			$pString .= $this->form->hidden('sql', 'update');
		$pString .= $this->form->textareaInput(FALSE, 
			"text", $this->html->dbToFormTidy($row['resourcetextNote']), 80, 15);
		$pString .= $this->html->p($this->form->formSubmit("Save"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
/**
* Edit abstract
*/
	private function abstractEdit()
	{
		if(!array_key_exists('type', $this->vars) || !array_key_exists('id', $this->vars) || 
			!array_key_exists('sql', $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"));
		if($this->session->getVar('metadataLock'))
			$this->navigate($this->errors->text("done", "abstract"));
		$abstract = trim($this->vars['text']);
		$userId = $this->session->getVar('setup_UserId');
		if($this->vars['sql'] == 'insert')
		{
			if(!$abstract)
				$this->navigate($this->errors->text("inputError", "missing"));
			$addEdit = 'added';
			$fields[] = "resourcetextId";
			$values[] = $this->vars['id'];
			$fields[] = "resourcetextAbstract";
			$values[] = $abstract;
			if($userId)
			{
				$fields[] = "resourcetextAddUserIdAbstract";
				$values[] = $userId;
			}
			$this->db->insert('resource_text', $fields, $values);
		}
		else // update
		{
			$addEdit = 'edited';
			$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
// if abstractText is empty, set fields to null
			if(!$abstract)
			{
				$addEdit = 'deleted';
				$this->db->updateNull('resource_text', array('resourcetextAbstract', 'resourcetextAddUserIdAbstract', 
					'resourcetextEditUserIdAbstract'));
				$this->checkDeleteRow();
			}
			else
			{
				$addEdit = 'edited';
				if(!$this->db->fetchOne($this->db->select('resource_text', 'resourcetextAddUserIdAbstract')) && $userId)
					$updateArray["resourcetextAddUserIdAbstract"] = $userId;
				$updateArray["resourcetextAbstract"] = $abstract;
				if($userId)
					$updateArray["resourcetextEditUserIdAbstract"] = $userId;
				$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
				$this->db->update('resource_text', $updateArray);
			}
		}
// update resource timestamp
		$this->db->formatConditions(array('resourcetimestampId' => $this->vars['id']));
		$this->db->update('resource_timestamp', array('resourcetimestampTimestamp' => $this->db->formatTimestamp()));
		$this->notify();
// lock reload
		$this->session->setVar('metadataLock', TRUE);
		$this->navigate($this->success->text("abstract", " " . $this->messages->text("misc", $addEdit) . " "));
	}
/**
* Edit note
*/
	private function noteEdit()
	{
		if(!array_key_exists('type', $this->vars) || !array_key_exists('id', $this->vars) || 
			!array_key_exists('sql', $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"));
		if($this->session->getVar('metadataLock'))
			$this->navigate($this->errors->text("done", "note"));
		$note = trim($this->vars['text']);
		$userId = $this->session->getVar('setup_UserId');
		if($this->vars['sql'] == 'insert')
		{
			if(!$note)
				$this->navigate($this->errors->text("inputError", "missing"));
			$addEdit = 'added';
			$fields[] = "resourcetextId";
			$values[] = $this->vars['id'];
			$fields[] = "resourcetextNote";
			$values[] = $note;
			if($userId)
			{
				$fields[] = "resourcetextAddUserIdNote";
				$values[] = $userId;
			}
			$this->db->insert('resource_text', $fields, $values);
		}
		else // update
		{
			$addEdit = 'edited';
			$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
			if(!$this->db->fetchOne($this->db->select('resource_text', 'resourcetextAddUserIdNote')) && $userId)
				$updateArray["resourcetextAddUserIdNote"] = $userId;
// if noteText is empty, set fields to null
			if(!$note)
			{
				$addEdit = 'deleted';
				$this->db->updateNull('resource_text', array('resourcetextNote', 'resourcetextAddUserIdNote', 
					'resourcetextEditUserIdNote'));
				$this->checkDeleteRow();
			}
			else
			{
				$addEdit = 'edited';
				$updateArray["resourcetextNote"] = $note;
				if($userId)
					$updateArray["resourcetextEditUserIdNote"] = $userId;
				$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
				$this->db->update('resource_text', $updateArray);
			}
		}
// update resource timestamp
		$this->db->formatConditions(array('resourcetimestampId' => $this->vars['id']));
		$this->db->update('resource_timestamp', array('resourcetimestampTimestamp' => $this->db->formatTimestamp()));
		$this->notify();
// lock reload
		$this->session->setVar('metadataLock', TRUE);
		$this->navigate($this->success->text("notes", " " . $this->messages->text("misc", $addEdit) . " "));
	}
/**
* If all abstract, note and url fields are null, delete the row
*/
	private function checkDeleteRow()
	{
		$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
		$recordset = $this->db->select('resource_text', array('resourcetextAbstract', 'resourcetextNote', 
			'resourcetextUrls'));
		$row = $this->db->fetchRow($recordset);
		if(!$row['resourcetextAbstract'] && !$row['resourcetextNote'] && !$row['resourcetextUrls'])
		{
			$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
			$this->db->delete('resource_text');
		}
	}
/** 
* Ask for confirmation for abstract to be deleted
*/
	public function abstractDeleteInit()
	{
		$this->session->delVar('metadataLock');
		GLOBALS::setOutputHeading($this->messages->text("heading", "abstractDelete"));
		$pString = $this->form->formHeader('metadata_EDITMETADATA_CORE');
		$pString .= $this->form->hidden("type", "abstractDelete");
		$pString .= $this->form->hidden("id", $this->vars['id']);
		$pString .= $this->html->p($this->form->formSubmit("Confirm"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
/**
* Delete abstract following confirmation
*/
	public function abstractDelete()
	{
		$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
		$this->db->updateNull('resource_text', array('resourcetextAbstract', 'resourcetextAddUserIdAbstract', 
			'resourcetextEditUserIdAbstract'));
		$this->checkDeleteRow();
// update resource timestamp
		$this->db->formatConditions(array('resourcetimestampId' => $this->vars['id']));
		$this->db->update('resource_timestamp', array('resourcetimestampTimestamp' => $this->db->formatTimestamp()));
		$this->notify();
// lock reload
		$this->session->setVar('metadataLock', TRUE);
		$this->navigate($this->success->text("abstract", " " . $this->messages->text("misc", 'deleted') . " "));
	}
/** 
* Ask for confirmation for note to be deleted
*/
	public function noteDeleteInit()
	{
		$this->session->delVar('metadataLock');
		GLOBALS::setOutputHeading($this->messages->text("heading", "notesDelete"));
		$pString = $this->form->formHeader('metadata_EDITMETADATA_CORE');
		$pString .= $this->form->hidden("type", "noteDelete");
		$pString .= $this->form->hidden("id", $this->vars['id']);
		$pString .= $this->html->p($this->form->formSubmit("Confirm"));
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
/**
* Delete notes following confirmation
*/
	public function noteDelete()
	{
		$this->db->formatConditions(array('resourcetextId' => $this->vars['id']));
		$this->db->updateNull('resource_text', array('resourcetextNote', 'resourcetextAddUserIdNote', 
			'resourcetextEditUserIdNote'));
		$this->checkDeleteRow();
// update resource timestamp
		$this->db->formatConditions(array('resourcetimestampId' => $this->vars['id']));
		$this->db->update('resource_timestamp', array('resourcetimestampTimestamp' => $this->db->formatTimestamp()));
		$this->notify();
// lock reload
		$this->session->setVar('metadataLock', TRUE);
		$this->navigate($this->success->text("notes", " " . $this->messages->text("misc", 'deleted') . " "));
	}
/**
* email notifications
*/
	private function notify()
	{
		include_once("core/modules/email/EMAIL.php");
		$email = new EMAIL();
		$email->notify($this->vars['id']);
	}
/** 
* Navigate to last resource with message (success or error)
*/
	private function navigate($message)
	{
		$navigate = FACTORY_NAVIGATE::getInstance();
		$navigate->resource($this->vars['id'], $message);
		FACTORY_CLOSE::getInstance();
	}
}
?>