<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* RESOURCECATEGORYEDIT class
*
* Edit category, keyword and user tags for a single resource..
*
*/
class RESOURCECATEGORYEDIT
{
private $db;
private $vars;
private $config;
private $icons;
private $errors;
private $messages;
private $success;
private $table;
private $form;
private $html;
private $session;
private $badInput;
private $category;
private $keyword;
private $ajax;
private $userTagsObject;
private $categories = array();
private $subcategories = array();
private $keywords = array();
public $userTags = array();
private $subcatArray = array();

	public function __construct()
	{
		$gatekeep = FACTORY_GATEKEEP::getInstance();
		$gatekeep->init();
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->session->clearArray('edit');
		$this->categories = $this->category->grabAll();
		$this->keywords = $this->keyword->grabAll();
		$this->userTagsObject = FACTORY_USERTAGS::getInstance();
		$this->userTags = $this->userTagsObject->grabAll();
		GLOBALS::setOutputHeading($this->messages->text("heading", "resourceEdit"));
	}
// Display form for editing
	public function init($message = FALSE)
	{
		if(!array_key_exists('id', $this->vars) || !$this->vars['id'])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$pString = $message;
		$pString .= $this->form->formHeader('resource_RESOURCECATEGORYEDIT_CORE', "onsubmit=\"selectAll();return true;\"");
		$pString .= $this->form->hidden('method', 'edit');
		$pString .= $this->form->hidden("id", $this->vars['id']);
		$pString .= $this->getTable();
		$pString .= $this->html->p($this->form->formSubmit("Save"));
		$pString .= $this->form->formEnd();
		$this->ajax->loadJavascript('core/modules/resource/resourceCategoryEdit.js');
		GLOBALS::buildOutputString($pString);
		$this->session->delVar('resourceLock');
	}
// Display categories etc. in a table.
// Can be called from modules/list/LISTADDTO in which case $multiIds = TRUE
	public function getTable($multiIds = FALSE)
	{
		$pString = $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->html->div('categoryDiv', $this->displayCategory($multiIds)));
		$pString .= $this->table->td($this->html->div('subcategoryDiv', $this->displaySubcategory($multiIds)));
		$pString .= $this->table->td($this->html->div('languageDiv', $this->displayLanguage($multiIds)));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->html->hr(), FALSE, FALSE, FALSE, 2) . $this->table->trEnd() . $this->table->trStart();
		$pString .= $this->table->td($this->html->div('keywordDiv', $this->displayKeyword($multiIds)));
		if(!empty($this->userTags))
			$pString .= $this->table->td($this->html->div('usertagDiv', $this->displayUserTags($multiIds)));
		$pString .= $this->table->trEnd();
		$pString.= $this->table->tableEnd();
		return $pString;
	}
// edit
	public function edit()
	{
		if($this->session->getVar('resourceLock'))
			$this->badInput->close($this->errors->text("done", "resource"));
		if(!array_key_exists('id', $this->vars) || !$this->vars['id'])
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		if(!array_key_exists('categoryIds', $this->vars) || empty($this->vars['categoryIds']))
			$this->badInput->close($this->errors->text("inputError", "missing"), $this, 'init');
		$this->writeLanguageTable();
		$this->writeCategoryTable();
		$this->writeKeywordTable();
		$this->writeUserTagsTable();
		$this->session->setVar('resourceLock', TRUE);
		$this->navigate($this->success->text("categoryKeyword", " " . $this->messages->text("misc", 'edited') . " "));
	}
// write to resource_language
	private function writeLanguageTable()
	{
		if(!array_key_exists('languageIds', $this->vars))
			return;
// remove all old languages from resource
		$this->db->formatConditions(array('resourcelanguageResourceId' => $this->vars['id']));
		$this->db->delete('resource_language');
// add new languages
		foreach($this->vars['languageIds'] as $lId)
		{
			if(!$lId) // IGNORE
				continue;
			$this->db->insert('resource_language', array('resourcelanguageResourceId', 'resourcelanguageLanguageId'), 
				array($this->vars['id'], $lId));
		}
	}
// write to resource_category
	private function writeCategoryTable()
	{
// remove all old categories and subcategories from resource
		$this->db->formatConditions(array('resourcecategoryResourceId' => $this->vars['id']));
		$this->db->delete('resource_category');
// add new categories
		if(empty($this->vars['categoryIds']))
			$this->vars['categoryIds'][] = 1; // force to 'General' category
		foreach($this->vars['categoryIds'] as $cId)
			$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
				array($this->vars['id'], $cId));
		if(array_key_exists('subcategoryIds', $this->vars))
		{
// Remove 'IGNORE' field
			if(array_search(0, $this->vars['subcategoryIds']) === 0)
				unset($this->vars['subcategoryIds'][0]);
			$this->writeSubcategoryTable();
		}
	}
	private function writeSubcategoryTable()
	{
// No subcategories to be added
		if(empty($this->vars['subcategoryIds']))
			return;
// Find categories subcategories belong to in case category needs adding to resource_category.CategoryId
		foreach($this->vars['subcategoryIds'] as $subcategoryId)
		{
			$this->db->formatConditions(array('subcategoryId' => $subcategoryId));
			$scIds[] = $this->db->fetchOne($this->db->select('subcategory', 'subcategoryCategoryId'));
			$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategorySubcategoryId'), 
				array($this->vars['id'], $subcategoryId));
		}
// If category of which subcategory is a member is not in table row, add it
		if(isset($scIds))
		{
			foreach(array_unique($scIds) as $cId)
			{
				$this->db->formatConditions(array('resourcecategoryResourceId' => $this->vars['id']));
				$this->db->formatConditions(array('resourcecategoryCategoryId' => $cId));
				$resultset = $this->db->select('resource_category', 'resourcecategoryId');
				if(!$this->db->fetchOne($resultset))
					$this->db->insert('resource_category', array('resourcecategoryResourceId', 'resourcecategoryCategoryId'), 
						array($this->vars['id'], $cId));
			}
		}
	}
// write to resource_keyword
	private function writeKeywordTable()
	{
		$keywordIds = $this->keyword->writeKeywordTable($this->vars);
// remove old keywords from resource
		$this->db->formatConditions(array('resourcekeywordResourceId' => $this->vars['id']));
		$this->db->delete('resource_keyword');
// add new keywords
		if(is_array($keywordIds))
		{
			foreach($keywordIds as $kId)
			{
				$this->db->formatConditions(array('resourcekeywordResourceId' => $this->vars['id']));
				$this->db->formatConditions(array('resourcekeywordKeywordId' => $kId));
				$resultset = $this->db->select('resource_keyword', 'resourcekeywordId');
				if(!$this->db->fetchOne($resultset))
					$this->db->insert('resource_keyword', array('resourcekeywordResourceId', 'resourcekeywordKeywordId'), 
					array($this->vars['id'], $kId));
			}
		}
// remove keyword(s) if they no longer have resource attached to them
		$this->keyword->removeHanging();
	}
// write to resource_user_tags
	private function writeUserTagsTable()
	{
		if(!array_key_exists('userTags', $this->vars))
			return;
		$tagIds = array();
		if(trim($this->vars['userTags']))
		{
			foreach(explode(',', $this->vars['userTags']) as $inputTag)
			{
				$fields = $values = array();
				if(!$inputTag = trim($inputTag))
					continue;
				if($id = $this->userTagsObject->checkExists($inputTag))
					$tagIds[] = $id;
				else // write to user_tags table
				{
					$fields[] = 'usertagsTag';
					$values[] = $inputTag;
					$fields[] = 'usertagsUserId';
					$values[] = $this->session->getVar('setup_UserId');
					$this->db->insert('user_tags', $fields, $values);
					$tagIds[] = $this->db->lastAutoID();
				}
			}
		}
// Delete current user tags for this resource
		$this->db->formatConditions(array('resourceusertagsResourceId' => $this->vars['id']));
		$this->db->delete('resource_user_tags');
		if(empty($tagIds))
		{
			$this->db->optimize('resource_user_tags');
			return;
		}
		foreach($tagIds as $tId)
		{
			$fields = $values = array();
			$fields[] = 'resourceusertagsTagId';
			$values[] = $tId;
			$fields[] = "resourceusertagsResourceId";
			$values[] = $this->vars['id'];
			$this->db->insert('resource_user_tags', $fields, $values);
		}
		$this->db->optimize('resource_user_tags');
	}
// display language form
	public function displayLanguage($multiIds)
	{
		$this->db->orderBy('languageLanguage');
		$resultset = $this->db->select('language', array('languageId', 'languageLanguage'));
		if($this->db->numRows($resultset))
		{
			$pString = FALSE;
			if(!$multiIds)
			{
				$pString .= $this->table->tableStart() . $this->table->trStart() . $this->table->td('&nbsp;', FALSE, FALSE, FALSE, 3) . 
					$this->table->trEnd() . $this->table->tableEnd();
				$pString .= $this->table->tableStart(FALSE, FALSE, FALSE, FALSE, 3);
			}
			else
				$pString .= $this->table->tableStart('generalTable', FALSE, FALSE, FALSE, 3);
			$pString .= $this->table->trStart();
			$languages[0] = $this->messages->text("misc", "ignore");
			while($row = $this->db->fetchRow($resultset))
				$languages[$row['languageId']] = $this->html->dbToFormtidy($row['languageLanguage']);
			$selected = array();
			if(!$multiIds)
			{
				$this->db->formatConditions(array('resourcelanguageResourceId' => $this->vars['id']));
				$resultset = $this->db->select('resource_language', 'resourcelanguageLanguageId');
				while($row = $this->db->fetchRow($resultset))
					$selected[] = $row['resourcelanguageLanguageId'];
				if(!empty($selected))
					$pString .= $this->table->td($this->form->selectedBoxValueMultiple($this->messages->text('resources', 
						'languages'), 'languageIds', $languages, $selected, 6) . $this->html->br() . 
						$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
				else
					$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 
						'languages'), 'languageIds', $languages, 6) . $this->html->br() . 
						$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			}
			else
			{
				$checkbox = FALSE;
				if($multiIds)
				{
					$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayLanguage';
					$jsonArray[] = array(
						'startFunction' => 'triggerFromSelect', 
						'script' => "$jScript",
						'triggerField' => 'displayLanguage',
						'targetDiv' => 'languageDiv',
						);
					$this->ajax->jActionForm('onchange', $jsonArray);
					$checkbox = "&nbsp;&nbsp;" . $this->form->checkbox(FALSE, "displayLanguage", TRUE);
				}
				$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 
						'languages') . $checkbox, 'languageIds', $languages, 6) . $this->html->br() . 
						$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			}
			if($multiIds)
			{
				$radios = $this->html->span($this->form->radioButton(FALSE, 'languageRadio', 'add', TRUE) . 
					$this->messages->text('misc', 'add'), "small") . $this->html->br();
				$radios .= $this->html->span($this->form->radioButton(FALSE, 'languageRadio', 'remove') . 
					$this->messages->text('misc', 'remove'), "small");
				$pString .= $this->table->td($radios);
			}
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			return $pString;
		}
		return FALSE;
	}
// display category form
	public function displayCategory($multiIds)
	{
		if(!$multiIds)
		{
			$this->db->formatConditions(array('resourcecategoryResourceId' => $this->vars['id']));
			$resultset = $this->db->select('resource_category', array('resourcecategoryCategoryId', 'resourcecategorySubcategoryId'));
			while($row = $this->db->fetchRow($resultset))
			{
				$catArray[] = $row['resourcecategoryCategoryId'];
				if($row['resourcecategorySubcategoryId'])
					$this->subcatArray[] = $row['resourcecategorySubcategoryId'];
			}
			$selectedCategories = array();
			foreach($catArray as $key)
			{
				if(!array_key_exists($key, $this->categories))
					continue;
				$selectedCategories[$key] = $this->categories[$key];
				unset($this->categories[$key]);
			}
		}
		$hint = $td = FALSE;
		if(!$multiIds)
		{
			$td .= $this->table->tableStart() . $this->table->trStart() . 
				$this->table->td($this->messages->text("hint", "categories"), FALSE, FALSE, FALSE, 3) . 
				$this->table->trEnd() . $this->table->tableEnd();
			$radios = FALSE;
		}
		$td .= $this->table->tableStart('generalTable', FALSE, FALSE, 3);
		$td .= $this->table->trStart();
		$checkbox = FALSE;
		if($multiIds)
		{
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayCategory';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displayCategory',
				'targetDiv' => 'categoryDiv',
				);
			$this->ajax->jActionForm('onchange', $jsonArray);
			$checkbox = "&nbsp;&nbsp;" . $this->form->checkbox(FALSE, "displayCategory", TRUE);
			$radios = $this->html->span($this->form->radioButton(FALSE, 'categoryRadio', 'add', TRUE) . 
				$this->messages->text('misc', 'add'), "small") . $this->html->br();
			$radios .= $this->html->span($this->form->radioButton(FALSE, 'categoryRadio', 'remove') . 
				$this->messages->text('misc', 'remove'), "small");
		}
		$td .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('select', "availableCategory") . $checkbox, 
			'availableCategory', $this->categories, 6) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, 'left', FALSE, FALSE, '18%');
		list($toRightImage, $toLeftImage) = $this->category->transferArrowsCat();
		$td .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage) . $radios, FALSE, 'left', FALSE, FALSE, '5%');
		if(!$multiIds)
			$td .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('select', "category"), 
				'categoryIds', $selectedCategories, 6), FALSE, 'left', FALSE, FALSE, '18%');
		else
			$td .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('select', "category"), 
				'categoryIds', array(), 6), FALSE, 'left', FALSE, FALSE, '18%');
		$td .= $this->table->trEnd();
		$td .= $this->table->tableEnd();
		return $td;
	}
// display subcategory form
	public function displaySubcategory($multiIds)
	{
		if(!is_array($this->subcategories = $this->category->grabSubAll(TRUE)))
			return FALSE;
		$pString = FALSE;
		$checkbox = FALSE;
		if(!$multiIds)
		{
			$selectedSubcategories = array();
			foreach($this->subcatArray as $key)
			{
				if(!array_key_exists($key, $this->subcategories))
					continue;
				$selectedSubcategories[$key] = $this->subcategories[$key];
				unset($this->subcategories[$key]);
			}
			$pString .= $this->table->tableStart() . $this->table->trStart() . $this->table->td('&nbsp;', FALSE, FALSE, FALSE, 3) . 
				$this->table->trEnd() . $this->table->tableEnd();
			$radios = FALSE;
		}
		else
		{
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displaySubcategory';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displaySubcategory',
				'targetDiv' => 'subcategoryDiv',
				);
			$this->ajax->jActionForm('onchange', $jsonArray);
			$checkbox = "&nbsp;&nbsp;" . $this->form->checkbox(FALSE, "displaySubcategory", TRUE);
			$radios = $this->html->span($this->form->radioButton(FALSE, 'subcategoryRadio', 'add', TRUE) . 
				$this->messages->text('misc', 'add'), "small") . $this->html->br();
			$radios .= $this->html->span($this->form->radioButton(FALSE, 'subcategoryRadio', 'remove') . 
				$this->messages->text('misc', 'remove'), "small");
		}
		$pString .= $this->table->tableStart('generalTable', FALSE, FALSE, 3);
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('select', "availableSubcategory") . $checkbox, 
			'availableSubcategory', $this->subcategories, 6) . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, 'left', FALSE, FALSE, '18%');
		list($toRightImage, $toLeftImage) = $this->category->transferArrowsSubcat();
		$pString .= $this->table->td($this->html->p($toRightImage) . $this->html->p($toLeftImage) . $radios, FALSE, 'left', FALSE, FALSE, '5%');
		if(!$multiIds)
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('select', "subcategory"), 
				'subcategoryIds', $selectedSubcategories, 6), FALSE, 'left', FALSE, FALSE, '18%');
		else
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('select', "subcategory"), 
				'subcategoryIds', array(), 6), FALSE, 'left', FALSE, FALSE, '18%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		return $pString;
	}
// display keyword form
	public function displayKeyword($multiIds)
	{
		$checkbox = FALSE;
		$pString = $this->table->tableStart('generalTable', FALSE, FALSE, 3);
		$pString .= $this->table->trStart();
		if(!empty($this->keywords))
		{
// If preferences reduce long keywords, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($this->keywords as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$keywords[$key] = html_entity_decode($value);
			}
			if(!$multiIds)
			{
				$this->db->formatConditions(array('resourcekeywordResourceId' => $this->vars['id']));
				$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
				$this->db->orderBy('keywordKeyword');
				$resultset = $this->db->select('resource_keyword', array('resourcekeywordKeywordId', 'keywordKeyword'));
				while($row = $this->db->fetchRow($resultset))
					$keywordArray[] = $row['keywordKeyword'];
				$keywordString = isset($keywordArray) ? join(', ', $keywordArray) : FALSE;
				$radios = FALSE;
			}
			else
			{
				$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayKeyword';
				$jsonArray[] = array(
					'startFunction' => 'triggerFromSelect', 
					'script' => "$jScript",
					'triggerField' => 'displayKeyword',
					'targetDiv' => 'keywordDiv',
					);
				$this->ajax->jActionForm('onchange', $jsonArray);
				$checkbox = "&nbsp;&nbsp;" . $this->form->checkbox(FALSE, "displayKeyword", TRUE);
				$radios = $this->html->span($this->form->radioButton($this->messages->text('misc', 'add'), 
					'keywordRadio', 'add', TRUE), "small") . $this->html->br();
				$radios .= $this->html->span($this->form->radioButton(
					$this->messages->text('misc', 'remove'), 'keywordRadio', 'remove'), "small");
				$radios = $this->html->p($radios);
			}
			$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 'availableKeywords') . $checkbox, 
				'fromKeywords', $keywords, 6) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, 'left', FALSE, FALSE, '20%');
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'transferKeyword', 
				);
			$toLeftImage = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
			$pString .= $this->table->td($toLeftImage . $radios, FALSE, 'left', FALSE, FALSE, '5%');
// Entry text area
			if(!$multiIds)
				$pString .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "keywords"), 
					"keywords", $this->html->dbToFormTidy($keywordString), 50, 5) . $this->html->br() . 
					$this->html->span($this->messages->text("hint", "keywords"), 'hint'), FALSE, 'left', FALSE, FALSE, '20%');
			else
				$pString .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "keywords"), 
					"keywords", FALSE, 45, 5) . $this->html->br() . 
					$this->html->span($this->messages->text("hint", "keywords"), 'hint'), FALSE, 'left', FALSE, FALSE, '20%');
		}
		else
			$pString .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "keywords"), 
				"keywords", FALSE, 50, 5) . $this->html->br() . $this->html->span($this->messages->text("hint", "keywordsAlt"), 'hint'));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		return $pString;
	}
// display userTags form
	public function displayUserTags($multiIds)
	{
		$checkbox = FALSE;
		if(!$multiIds)
		{
			$this->db->formatConditions(array('resourceusertagsResourceId' => $this->vars['id']));
			$radios = FALSE;
		}
		else
		{
			$jScript = 'index.php?action=list_LISTADDTO_CORE&method=displayUsertag';
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'displayUsertag',
				'targetDiv' => 'usertagDiv',
				);
			$this->ajax->jActionForm('onchange', $jsonArray);
			$checkbox = "&nbsp;&nbsp;" . $this->form->checkbox(FALSE, "displayUsertag", TRUE);
				$radios = $this->html->span($this->form->radioButton($this->messages->text('misc', 'add'), 
					'usertagRadio', 'add', TRUE) , "small") . $this->html->br();
				$radios .= $this->html->span($this->form->radioButton(
					$this->messages->text('misc', 'remove'), 'usertagRadio', 'remove'), "small");
				$radios = $this->html->p($radios);
		}
		$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
		$this->db->leftJoin('user_tags', 'usertagsId', 'resourceusertagsTagId');
		$resultset = $this->db->select('resource_user_tags', 'resourceusertagsTagId');
		$tagIds = array();
		while($row = $this->db->fetchRow($resultset))
			$tagIds[] = $row['resourceusertagsTagId'];
		foreach($this->userTags as $userTagId => $userTagName)
		{
			if(array_search($userTagId, $tagIds) !== FALSE)
				$thisResourceTags[$userTagId] = $userTagName;
// If preferences reduce long userTags, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
				$userTagId = $userTagId . '_' . base64_encode($userTagName);
				$userTags[$userTagId] = html_entity_decode($userTagName);
		}
		$userTagString = isset($thisResourceTags) ? join(', ', $thisResourceTags) : FALSE;
		$pString = $this->table->tableStart('generalTable', FALSE, FALSE, 3);
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 'availableUserTags') . $checkbox, 
			'fromUserTags', $userTags, 6) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, 'left', FALSE, FALSE, '20%');
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'transferUserTag', 
			);
		$toLeftImage = $this->ajax->jActionIcon('toRight', 'onclick', $jsonArray);
		$pString .= $this->table->td($toLeftImage . $radios, FALSE, 'left', FALSE, FALSE, '5%');
// Entry text area
		if(!$multiIds)
			$pString .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "userTags"), 
				"userTags", $userTagString, 50, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "userTags"), 'hint'), FALSE, 'left', FALSE, FALSE, '20%');
		else
			$pString .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "userTags"), 
				"userTags", FALSE, 45, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "userTags"), 'hint'), FALSE, 'left', FALSE, FALSE, '20%');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		return $pString;
	}
/** 
* Navigate to last resource with message (success or error)
*/
	private function navigate($message)
	{
		$navigate = FACTORY_NAVIGATE::getInstance();
		$navigate->resource($this->vars['id'], $message);
		FACTORY_CLOSE::getInstance();
	}
}
?>