<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* RESOURCEFORM -- Enter or edit a new resource via a form
*/
class RESOURCEFORM
{
private $db;
private $vars;
private $messages;
private $session;
private $form;
private $table;
private $html;
private $ajax;
private $type;
private $types;
private $gatekeep;
private $sessionVars;
private $resourceMap;
private $typeMaps;
private $resourceType = 'book'; // default for new entry
private $category;
private $keyword;
private $userTagObj;
private $publisher;
private $collection;
private $collectionMap;
private $publisherMap;
private $creator;
private $commonBib;
private $eds = array();
private $tdLabelWidth = '15%';
private $tdContentWidth = '85%';
private $edit = FALSE;
private $creatorsArray = array();
public $seriesFill = FALSE;
public $translationFill = FALSE;
public $conferenceFill = FALSE;
private $error = FALSE;
private $tinymce;
		
	public function __construct()
	{
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->gatekeep->init();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->type = FACTORY_TYPE::getInstance();
		$this->resourceMap = FACTORY_RESOURCEMAP::getInstance();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->category = FACTORY_CATEGORY::getInstance();
		$this->keyword = FACTORY_KEYWORD::getInstance();
		$this->userTagObj = FACTORY_USERTAGS::getInstance();
		$this->publisher = FACTORY_PUBLISHER::getInstance();
		$this->collection = FACTORY_COLLECTION::getInstance();
		$this->collectionMap = FACTORY_COLLECTIONMAP::getInstance();
		$this->publisherMap = FACTORY_PUBLISHERMAP::getInstance();
		$this->creator = FACTORY_CREATOR::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->tinymce = FACTORY_LOADTINYMCE::getInstance();
		$this->typeMaps = $this->resourceMap->getTypeMap();
		$this->loadElementDefinitions();
		$this->session->delVar('resourceLock');
	}
/**
* Start the process of entering a new resource by asking for the choice of resource type and number of authors
*/
	public function init($error = FALSE)
	{
		if(!$error)
		{
			$this->session->clearArray('resourceForm');
			$this->session->clearArray('resourceFormValidate');
			$this->sessionVars = array();
		}
		else
		{
			$this->error = TRUE;
			if(!$this->sessionVars = $this->session->getArray('resourceForm'))
				$this->sessionVars = array();
			$this->resourceType = array_key_exists("resourceType", $this->sessionVars) ? $this->sessionVars['resourceType'] : FALSE;
		}
		if(array_key_exists('type', $this->vars) && ($this->vars['type'] == 'edit'))
		{
			GLOBALS::setOutputHeading($this->messages->text('heading', 'editResource'));
			if(!$error)
				$this->getEditSession();
			$this->session->setVar('resourceFormType', 'edit');
		}
		else
		{
			GLOBALS::setOutputHeading($this->messages->text('heading', 'newResource'));
			$this->session->setVar('resourceFormType', 'new');
		}
		$pString = $this->html->p($this->messages->text('resources', 'new', '&nbsp;' . $this->required()));
		$pString .= $this->html->hr();
		$pString .= $this->tinymce->loadBasicTextinput(array('resourceTitle', 'resourceSubtitle'), 400);
		$pString .= $this->tinymce->loadMetadataTextarea(array('resourcetextAbstract', 'resourcetextNote'));
		$requiredArray = $this->ajax->encode_jArray($this->resourceMap->getRequired(), TRUE);
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=validate';
		$jsonArray[] = array(
			'startFunction' => 'validate', 
			'script' => "$jScript",
			'targetDiv' => 'validateOuter',
			'requiredArray'	=>	"$requiredArray",
			);
		$this->ajax->jActionForm('onsubmit', $jsonArray, TRUE);
		$pString .= $this->form->formHeaderName('resource_RESOURCEWRITE_CORE', 'resourceForm');
		if($this->edit)
			$pString .= $this->form->hidden('resourceId', $this->vars['id']);
// resource type and title
		$pString .= $this->typeTitleCell();
// creators
		$array = array();
		if(array_key_exists('resourceType', $this->sessionVars))
		{
			if(array_key_exists('resourcecreator', $this->typeMaps[$this->sessionVars['resourceType']]))
				$array = $this->typeMaps[$this->sessionVars['resourceType']]['resourcecreator'];
		}
		else
			$array = $this->typeMaps[$this->resourceType]['resourcecreator']; // default when 'new resource' selected
		if(!empty($array))
		{
			$cell = $this->table->tableStart('generalTable', 1);
			foreach($array as $key => $creatorMsg)
				$cell .= $this->blankCreatorCell($key, $creatorMsg);
			$cell .= $this->table->tableEnd();
// Creator tables all wrapped up in 'creatorsOuter' DIV which is hidden if no creators selected or available for this resource type.
			$pString .= $this->html->div('creatorsOuter', $cell);
		}
		else
			$pString .= $this->html->div('creatorsOuter', FALSE);
// optional cells
		$pString .= $this->optionalCells();
// custom fields
		$pString .= $this->customCells();
// Abstract, notes, categories, keywords etc.
		$pString .= $this->otherCell();
// Must be here but is initially hidden
		$pString .= $this->html->div('validateOuter', $error, 'error');
		$pString .= $this->html->p($this->form->formSubmit("Save"));
		
		$pString .= $this->form->formEnd();
		
		GLOBALS::buildOutputString($pString);
// Load at end because .js initialization needs various DIVs to be in the page before they are made invisible
		$this->ajax->loadJavascript('core/modules/resource/resourceForm.js');
	}
/**
* We're editing a resource so get resource details and place in $this->sessionVars
*/
	private function getEditSession()
	{
		$badInput = FACTORY_BADINPUT::getInstance();
		if(!array_key_exists('id', $this->vars) || !$this->vars['id'])
		{
			$errors = FACTORY_ERRORS::getInstance();
			$badInput->close($errors->text("inputError", "missing"));
		}
		$this->edit = TRUE;
		$res = FACTORY_RESOURCECOMMON::getInstance();
		$resultset = $res->getResource($this->vars['id']);
		if(!$this->db->numRows($resultset))
			$badInput->close($this->messages->text("resources", "noResult"));
		$row = $this->db->fetchRow($resultset);
		$this->resourceType = $this->sessionVars['resourceType'] = $row['resourceType'];
		$this->sessionVars['resourceTitle'] = $row['resourceTitle'];
		if(array_key_exists('resourceNoSort', $row) && $row['resourceNoSort'])
			$this->sessionVars['resourceNoSort'] = $row['resourceNoSort'];
		if(array_key_exists('resourceSubtitle', $row) && $row['resourceSubtitle'])
			$this->sessionVars['resourceSubtitle'] = $row['resourceSubtitle'];
		if(array_key_exists('resourceShortTitle', $row) && $row['resourceShortTitle'])
			$this->sessionVars['resourceShortTitle'] = $row['resourceShortTitle'];
		if(array_key_exists('resourceTransNoSort', $row) && $row['resourceTransNoSort'])
			$this->sessionVars['resourceTransNoSort'] = $row['resourceTransNoSort'];
		if(array_key_exists('resourcetextUrls', $row) && $row['resourcetextUrls'])
		{
			$this->sessionVars['resourcetextUrl'] = array_shift(unserialize(base64_decode($row['resourcetextUrls'])));
			if(array_key_exists('resourcetextUrlText', $row) && $row['resourcetextUrlText'])
				$this->sessionVars['resourcetextUrlText'] = array_shift(unserialize(base64_decode($row['resourcetextUrlText'])));
		}
		$ids = array();
		foreach($this->resourceMap->getTables($this->resourceType) as $table)
		{
			foreach($this->resourceMap->getOptional() as $optional)
			{
				if(!array_key_exists($optional, $this->typeMaps[$this->resourceType]['optional']))
					continue;
				if(array_key_exists($table, $this->typeMaps[$this->resourceType]['optional'][$optional]))
				{
					foreach($this->typeMaps[$this->resourceType]['optional'][$optional][$table] as $key => $value)
					{
						$rowKey = $table . $key;
						if(array_key_exists($rowKey, $row) && $row[$rowKey])
							$this->sessionVars[$rowKey] = $row[$rowKey];
					}
				}
			}
			if(array_key_exists('virtual', $this->typeMaps[$this->resourceType]))
			{
				if(array_key_exists($table, $this->typeMaps[$this->resourceType]['virtual']))
				{
					foreach($this->typeMaps[$this->resourceType]['virtual'][$table] as $key => $value)
					{
						$rowKey = $table . $key;
						if(array_key_exists($rowKey, $row) && $row[$rowKey])
						{
							$this->sessionVars[$value] = $row[$rowKey];
							$ids[] = $value;
						}
					}
				}
			}
		}
// abstract, note, isbn, doi
		if(array_key_exists('resourcetextAbstract', $row) && $row['resourcetextAbstract'])
			$this->sessionVars['resourcetextAbstract'] = $row['resourcetextAbstract'];
		if(array_key_exists('resourcetextNote', $row) && $row['resourcetextNote'])
			$this->sessionVars['resourcetextNote'] = $row['resourcetextNote'];
		if(array_key_exists('resourceIsbn', $row) && $row['resourceIsbn'])
			$this->sessionVars['resourceIsbn'] = $row['resourceIsbn'];
		if(array_key_exists('resourceDoi', $row) && $row['resourceDoi'])
			$this->sessionVars['resourceDoi'] = $row['resourceDoi'];
// creators
		if(array_key_exists('resourcecreator', $this->typeMaps[$this->resourceType]))
		{
			$array = array();
			$this->db->formatConditions(array('resourcecreatorResourceId' => $this->vars['id']));
			$this->db->orderBy(array('resourcecreatorRole', 'resourcecreatorOrder'));
			$resultset = $this->db->select('resource_creator', array('resourcecreatorCreatorId', 'resourcecreatorRole', 'resourcecreatorOrder'));
			while($row = $this->db->fetchRow($resultset))
				$array[$row['resourcecreatorRole']][] = $row['resourcecreatorCreatorId'];
			foreach($this->typeMaps[$this->resourceType]['resourcecreator'] as $key => $value)
			{
				foreach($array as $role => $cArray)
				{
					foreach($cArray as $index => $cId)
						$this->sessionVars["Creator$role" . "_$index" . '_select'] = $cId;
				}
			}
		}
// Get keywords, categories, subcategories, userTags
		$array = array();
		$this->db->formatConditions(array('resourcekeywordResourceId' => $this->vars['id']));
		$this->db->leftJoin('keyword', 'keywordId', 'resourcekeywordKeywordId');
		$resultset = $this->db->select('resource_keyword', 'keywordKeyword');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $row['keywordKeyword'];
		if(!empty($array))
			$this->sessionVars['resourcekeywordKeywords'] = join(',', $array);
		$array = array();
		$this->db->formatConditions(array('resourcecategoryCategoryId' => ' IS NOT NULL'));
		$this->db->formatConditions(array('resourcecategoryResourceId' => $this->vars['id']));
		$resultset = $this->db->select('resource_category', 'resourcecategoryCategoryId');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $row['resourcecategoryCategoryId'];
		if(!empty($array))
			$this->sessionVars['resourcecategoryCategories'] = join(',', $array);
		$array = array();
		$this->db->formatConditions(array('resourcecategoryResourceId' => $this->vars['id']));
		$this->db->formatConditions(array('resourcecategorySubcategoryId' => ' IS NOT NULL'));
		$resultset = $this->db->select('resource_category', 'resourcecategorySubcategoryId');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $row['resourcecategorySubcategoryId'];
		if(!empty($array))
			$this->sessionVars['resourcecategorySubcategories'] = join(',', $array);
		$array = array();
		$this->db->formatConditions(array('resourceusertagsResourceId' => $this->vars['id']));
		$this->db->leftJoin('user_tags', 'usertagsId', 'resourceusertagsTagId');
		$resultset = $this->db->select('resource_user_tags', 'usertagsTag');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $row['usertagsTag'];
		if(!empty($array))
			$this->sessionVars['resourceusertagsTagId'] = join(',', $array);
// User bibliographies
		$this->db->formatConditions(array('userbibliographyUserId' => $this->session->getVar('setup_UserId')));
		$resultset = $this->db->select('user_bibliography', 'userbibliographyId');
		$array = array();
		while($row = $this->db->fetchRow($resultset))
			$array[] = $row['userbibliographyId'];
		if(!empty($array))
		{
			$this->db->formatconditionsOneField($array, 'userbibliographyresourceBibliographyId');
			$this->db->formatConditions(array('userbibliographyresourceResourceId' => $this->vars['id']));
			$resultset = $this->db->select('user_bibliography_resource', 'userbibliographyresourceBibliographyId');
			$array = array();
			while($row = $this->db->fetchRow($resultset))
				$array[] = $row['userbibliographyresourceBibliographyId'];
			if(!empty($array))
				$this->sessionVars['bibliographies'] = join(',', $array);
		}
// Languages
		$array = array();
		$this->db->formatConditions(array('resourcelanguageResourceId' => $this->vars['id']));
		$resultset = $this->db->select('resource_language', 'resourcelanguageLanguageId');
		while($row = $this->db->fetchRow($resultset))
			$array[] = $row['resourcelanguageLanguageId'];
		if(!empty($array))
			$this->sessionVars['resourcelanguageLanguages'] = join(',', $array);
// Remove 'virtualFields' values (see RESOURCEMAP.php)
		foreach($ids as $value)
		{
			if(array_key_exists('virtualFields', $this->typeMaps[$this->resourceType]) && 
				array_key_exists($value, $this->typeMaps[$this->resourceType]['virtualFields']))
			{
				foreach($this->typeMaps[$this->resourceType]['virtualFields'][$value] as $removeField)
				{
					if(array_key_exists($removeField, $this->sessionVars))
						unset($this->sessionVars[$removeField]);
				}
			}
		}
		if(array_key_exists('resourcemiscPeerReviewed', $this->sessionVars) && ($this->sessionVars['resourcemiscPeerReviewed'] == 'N'))
			unset($this->sessionVars['resourcemiscPeerReviewed']);
		$this->session->writeArray($this->sessionVars, 'resourceForm');
	}
/**
* Print custom fields if any exist and we're not editing
*/
	private function customCells()
	{
		$array = $small = $large = array();
		$this->db->orderBy('customSize'); // 'L' before 'S'
		$resultset = $this->db->select('custom', array('customSize', 'customLabel', 'customId'));
		while($row = $this->db->fetchRow($resultset))
		{
			$array[$row['customId']] = array($row['customLabel'], $row['customSize']);
			if($row['customSize'] == 'L')
				$large[$row['customId']] = $this->html->dbToHtmlTidy($row['customLabel']);
			else
				$small[$row['customId']] = $this->html->dbToHtmlTidy($row['customLabel']);
		}
		if(empty($array))
			return;
		$this->sessionVars['customFields'] = serialize($array);
		$pString = $this->table->td($this->html->h($this->messages->text('resources', 'customFields'), 4), 
			FALSE, FALSE, FALSE, FALSE, $this->tdLabelWidth);
		$tdContent = $this->table->tableStart();
		$count = 1;
		$rowStart = TRUE;
		foreach($small as $id => $label)
		{
			if($rowStart)
			{
				$tdContent .= $this->table->trStart();
				$rowStart = FALSE;
			}
			$value = FALSE;
			if($this->error)
				$value = $this->session->getVar('resourceForm_customId' . $id) ? 
					$this->session->getVar('resourceForm_customId' . $id) : FALSE;
			else if($this->edit)
			{
				$this->db->formatConditions(array('resourcecustomResourceId' => $this->vars['id']));
				$this->db->formatConditions(array('resourcecustomCustomId' => $id));
				$row = $this->db->fetchRow($this->db->select('resource_custom', 'resourcecustomShort'));
				$value = $row['resourcecustomShort'];
			}
			$tdContent .= $this->table->td($this->form->textInput($label, 'customId' . $id, $value, 30, 255));
			if(!($count%3))
			{
				$tdContent .= $this->table->trEnd();
				$rowStart = TRUE;
			}
			++$count;
		}
		if(!$rowStart)
		{
			while(($count%3))
			{
				$tdContent .= $this->table->td('&nbsp;');
				++$count;
			}
			$tdContent .= $this->table->trEnd();
		}
		$tdContent .= $this->table->tableEnd() . $this->table->tableStart();
		$idArray = array();
		foreach($large as $id => $label)
		{
			$idArray[] = 'customId' . $id;
			$tdContent .= $this->table->trStart();
			$value = FALSE;
			if($this->error)
				$value = $this->session->getVar('resourceForm_customId' . $id) ? 
					$this->session->getVar('resourceForm_customId' . $id) : FALSE;
			else if($this->edit)
			{
				$this->db->formatConditions(array('resourcecustomResourceId' => $this->vars['id']));
				$this->db->formatConditions(array('resourcecustomCustomId' => $id));
				$row = $this->db->fetchRow($this->db->select('resource_custom', 'resourcecustomLong'));
				$value = $row['resourcecustomLong'];
			}
			$tdContent .= $this->table->td($this->form->textareaInput($label, 'customId' . $id, $value, 60, 10));
			$tdContent .= $this->table->trEnd();
		}
		if(!$rowStart)
		{
			while(($count%2))
			{
				$tdContent .= $this->table->td('&nbsp;');
				++$count;
			}
			$tdContent .= $this->table->trEnd();
		}
		$tdContent .= $this->table->tableEnd();
		$pString .= $this->tinymce->loadMetadataTextarea($idArray);
		$pString .= $this->table->td($tdContent, FALSE, FALSE, FALSE, FALSE, $this->tdContentWidth);
		return $this->table->tableStart('generalTable', 1) . $this->table->trStart() . 
			$pString . $this->table->trEnd() . $this->table->tableEnd();
	}
/**
* Print optional cells such as translated works, collection, publisher
*/
	public function optionalCells($cellFunction = FALSE)
	{
		if($cellFunction && array_key_exists($cellFunction, $this->typeMaps[$this->resourceType]['optional']) && 
			method_exists($this, $cellFunction))
			return $this->{$cellFunction}();
		else if($cellFunction)
		{
			if($cellFunction == 'publisher')
				return $this->blankPublisher();
			else if($cellFunction == 'conference')
				return $this->blankConference();
			else if($cellFunction == 'series')
				return $this->blankSeries();
			else if($cellFunction == 'collection')
				return $this->blankCollection();
			else if($cellFunction == 'translation')
				return $this->blankTranslation();
			else if($cellFunction == 'miscellaneous')
				return $this->blankMiscellaneous();
		}
		$pString = '';
		foreach($this->resourceMap->getOptional() as $cellFunction)
		{
			if(array_key_exists($cellFunction, $this->typeMaps[$this->resourceType]['optional']) && 
				method_exists($this, $cellFunction))
				$pString .= $this->{$cellFunction}();
			else if($cellFunction == 'publisher')
				$pString .= $this->blankPublisher();
			else if($cellFunction == 'conference')
				$pString .= $this->blankConference();
			else if($cellFunction == 'series')
				$pString .= $this->blankSeries();
			else if($cellFunction == 'collection')
				$pString .= $this->blankCollection();
			else if($cellFunction == 'translation')
				$pString .= $this->blankTranslation();
			else if($cellFunction == 'miscellaneous')
				$pString .= $this->blankMiscellaneous();
		}
		return $pString;
	}
/**
* Create table cells for resource type
*/
	private function typeTitleCell()
	{
		$tdLabel = $this->messages->text("resources", "type");
		$tdContent = $this->table->tableStart();
		$tdContent .= $this->table->trStart();
		foreach($this->typeMaps as $type => $null)
			$array[$type] = $this->messages->text("resourceType", $type);
		$jsonArray = array();
// Amend creators lists depending upon which resource types are chosen
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=initCreators';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'resourceType',
			'targetDiv' => 'creatorsOuter',
			);
// Display collections details depending upon which resource types are chosen
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=initCollection';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'resourceType',
			'targetDiv' => 'collectionOuter',
			);
// Display series details depending upon which resource types are chosen
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=initSeries';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'resourceType',
			'targetDiv' => 'seriesOuter',
			);
// Display conference details depending upon which resource types are chosen
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=initConference';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'resourceType',
			'targetDiv' => 'conferenceOuter',
			);
// Display publisher details depending upon which resource types are chosen
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=initPublisher';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'resourceType',
			'targetDiv' => 'publisherOuter',
			);
// Display translation details depending upon which resource types are chosen
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=initTranslation';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'resourceType',
			'targetDiv' => 'translationOuter',
			);
// Display miscellaneous details depending upon which resource types are chosen
		$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=initMiscellaneous';
		$jsonArray[] = array(
			'startFunction' => 'triggerFromSelect', 
			'script' => "$jScript",
			'triggerField' => 'resourceType',
			'targetDiv' => 'miscellaneousOuter',
			);
		$this->ajax->jActionForm('onchange', $jsonArray);
		$tdContent .= $this->table->td($this->form->selectedBoxValue(FALSE, "resourceType", $array, $this->resourceType, 15));
		$title = array_key_exists("resourceTitle", $this->sessionVars) ? 
			$this->html->dbToTinyMCE($this->sessionVars['resourceTitle']) : FALSE;
		$title = array_key_exists("resourceNoSort", $this->sessionVars) ? 
			$this->html->dbToTinyMCE($this->sessionVars['resourceNoSort']) . " $title" : FALSE . $title;
		$subtitle = array_key_exists("resourceSubtitle", $this->sessionVars) ? 
			$this->html->dbToTinyMCE($this->sessionVars['resourceSubtitle']) : FALSE;
		$shortTitle = array_key_exists("resourceShortTitle", $this->sessionVars) ? 
			$this->html->dbToFormTidy($this->sessionVars['resourceShortTitle']) : FALSE;
		$tdContent .= $this->table->td($this->required() . $this->form->textInput($this->messages->text("resources", "title"), 
			"resourceTitle", $title, 75) . $this->html->br() . 
			$this->form->textInput($this->messages->text("resources", "subtitle"), "resourceSubtitle", 
			$subtitle, 75) . $this->html->br() . 
			$this->form->textInput($this->messages->text("resources", "shortTitle"), 
			"resourceShortTitle", $shortTitle, 35));
		$tdContent .= $this->table->trEnd() . $this->table->tableEnd();
		$div = $this->table->td($this->html->h($tdLabel, FALSE, 4) . 
			$this->html->span($this->messages->text("hint", "capitals"), 'hint'), FALSE, FALSE, FALSE, FALSE, $this->tdLabelWidth)
			. $this->table->td($tdContent, FALSE, FALSE, FALSE, FALSE, $this->tdContentWidth);
		return $this->html->div('typeOuter', $this->table->tableStart('generalTable', 1) . $this->table->trStart() . 
			$div . $this->table->trEnd() . $this->table->tableEnd());
	}
/**
* Parse [type]['optional'] in RESOURCEMAP and produce required table and form elements.
*
* @param $optionalArray -- the array element in [type]['optional'] to process (e.g. 'translation', 'collection' etc.)
* @param $outerDiv -- the encapsulating DIV
* @param $insertBefore -- array of inserts
* @param $insertAfter -- array of inserts
* @param $tdLabel -- left hand td cell label for table
* @return formatted string
*/
	private function optionalCellsFormat($optionalArray, $outerDiv, $insertBefore, $insertAfter, $tdLabel)
	{
		if((($this->resourceType == 'web_site') || ($this->resourceType == 'web_article') || ($this->resourceType == 'database') || 
			($this->resourceType == 'web_encyclopedia') || ($this->resourceType == 'web_encyclopedia_article')) && 
			($optionalArray == 'miscellaneous'))
			$date = getdate();
		if(array_key_exists('title', $this->typeMaps[$this->resourceType]['optional'][$optionalArray]))
		{
			$tdLabel = $this->messages->text("resources", $this->typeMaps[$this->resourceType]['optional'][$optionalArray]['title']);
			unset($this->typeMaps[$this->resourceType]['optional'][$optionalArray]['title']);
		}
		else
			$tdLabel = $this->messages->text("resources", $tdLabel);
		if(array_key_exists('titleHint', $this->typeMaps[$this->resourceType]['optional'][$optionalArray]))
		{
			$tdHint = $this->html->p($this->html->span($this->messages->text("hint", 
				$this->typeMaps[$this->resourceType]['optional'][$optionalArray]['titleHint'])), 'hint');
			unset($this->typeMaps[$this->resourceType]['optional'][$optionalArray]['titleHint']);
		}
		else
			$tdHint = FALSE;
		$contentTable = $this->table->tableStart() . $this->table->trStart();
		$tds = $tdElements = $hints = array();
		foreach($this->typeMaps[$this->resourceType]['optional'][$optionalArray] as $tableKey => $tableArray)
		{
			if(($tableKey == 'insertBefore') || ($tableKey == 'insertAfter')
				 || ($tableKey == 'insertTitle')) // should have been dealt with and removed earlier
				continue;
			if(strpos($tableKey, 'hint_') === 0) // found at start
			{
				$hintArray = explode('_', $tableKey);
				$hints[$hintArray[1]] = $this->html->br() . $this->html->span($this->messages->text('hint', 
					$this->typeMaps[$this->resourceType]['optional'][$optionalArray][$tableKey]), 'hint');
				continue;
			}
			$continue = FALSE;
			foreach($tableArray as $key => $value)
			{
				if(!is_bool($value) && array_key_exists($value, $this->eds))
				{
					$required = FALSE;
					if(array_key_exists('required', $this->typeMaps[$this->resourceType]))
						$required = array_search($tableKey . $key, $this->typeMaps[$this->resourceType]['required']) !== FALSE ? 
							$this->required() : FALSE;
					$sessionVar = array_key_exists($tableKey . $key, $this->sessionVars) ? 
						$this->html->dbToFormTidy($this->sessionVars[$tableKey . $key]) : FALSE;
					$hint = array_key_exists($key, $hints) ? $hints[$key] : FALSE;
					if(array_key_exists($key, $insertBefore))
						$tdElements[] = $insertBefore[$key];
					if($this->eds[$value]['type'] == 'textInput')
					{
						if(($key == 'TransTitle') && (array_key_exists($tableKey . 'TransNoSort', $this->sessionVars)))
							$sessionVar = $this->html->dbToFormTidy($this->sessionVars[$tableKey . 'TransNoSort']) . ' ' . $sessionVar;
						if(isset($date) && !$sessionVar && ($key == 'Year2'))
							$sessionVar = $date['year'];
						$tdElements[] = $required . $this->form->textInput($this->messages->text("resources", $value), 
							$tableKey . $key, $sessionVar, $this->eds[$value]['width'], $this->eds[$value]['max']) . $hint;
					}
					else if($this->eds[$value]['type'] == 'textareaInput')
						$tdElements[] = $required . $this->form->textareaInput($this->messages->text("resources", $value), 
							$tableKey . $key, $sessionVar, $this->eds[$value]['width'], $this->eds[$value]['max']) . $hint;
					else if($this->eds[$value]['type'] == 'checkbox')
						$tdElements[] = $this->form->checkbox($this->messages->text("resources", $value), $tableKey . $key, $sessionVar) . $hint;
					else if($this->eds[$value]['type'] == 'day_selectbox')
					{
						if(!isset($days))
						{
							$days[] = $this->messages->text("misc", "ignore");
							$days = array_merge($days, range(1, 31));
						}
						if(isset($date) && !$sessionVar)
							$sessionVar = $date['mday'];
						if($sessionVar)
							$tdElements[] = $required . $this->form->selectedBoxValue($this->messages->text("resources", $value), 
								$tableKey . $key, $days, $sessionVar, 1) . $hint;
						else
							$tdElements[] = $required . $this->form->selectFBoxValue($this->messages->text("resources", $value), 
								$tableKey . $key, $days, 1) . $hint;
					}
					else if($this->eds[$value]['type'] == 'month_selectbox')
					{
						$languageDir = $this->session->getVar('setup_Language');
						if(!is_dir("languages/$languageDir"))
							$languageDir = "en";
						include_once("languages/$languageDir/CONSTANTS.php");
						$class = "CONSTANTS_" . $languageDir;
						$constant = new $class();
						if(!isset($months))
						{
							$months[] = $this->messages->text("misc", "ignore");
							$months = array_merge($months, $constant->monthToLongName());
						}
						if(!isset($days))
						{
							$days[] = $this->messages->text("misc", "ignore");
							$days = array_merge($days, range(1, 31));
						}
						if(isset($date) && !$sessionVar)
							$sessionVar = $date['mon'];
						if($sessionVar)
							$tdElements[] = $required . $this->form->selectedBoxValue($this->messages->text("resources", $value), 
								$tableKey . $key, $months, $sessionVar, 1) . $hint;
						else
							$tdElements[] = $required . $this->form->selectFBoxValue($this->messages->text("resources", $value), 
								$tableKey . $key, $months, 1) . $hint;
					}
					if(array_key_exists($key, $insertAfter))
						$tdElements[] = $insertAfter[$key];
				}
				else if($key == 'continue')
					$continue = TRUE;
				else if(strpos($key, 'break') === 0)
				{
					$tds[] = $tdElements;
					$tdElements = array();
				}
			}
			if(!$continue)
			{
				$tds[] = $tdElements;
				$tdElements = array();
			}
		}
		$tdWidth = floor(100/sizeof($tds)) . '%';
		foreach($tds as $td)
		{
			$tdString = join($this->html->br(), $td);
			$contentTable .= $this->table->td($tdString, FALSE, FALSE, FALSE, FALSE, $tdWidth);
		}
		$contentTable .= $this->table->trEnd() . $this->table->tableEnd();
		$table = $this->table->td($this->html->h($tdLabel, FALSE, 4) . $tdHint, FALSE, FALSE, FALSE, FALSE, $this->tdLabelWidth)
			. $this->table->td($contentTable, FALSE, FALSE, FALSE, FALSE, $this->tdContentWidth);
		return $this->html->div($outerDiv, $this->table->tableStart('generalTable', 1) . $this->table->trStart() . 
			$table . $this->table->trEnd() . $this->table->tableEnd());
	}
/**
** Publisher
*/
	public function publisher()
	{
		if($this->conferenceFill)
			$insert = $this->publisherInsertFill();
		else
			$insert = $this->publisherInsert();
		if(empty($insert))
			$insert = '';
		$insertAfter = $insertBefore = array();
		if(array_key_exists('insertBefore', $this->typeMaps[$this->resourceType]['optional']['publisher']))
		{
			$insertBefore[$this->typeMaps[$this->resourceType]['optional']['publisher']['insertBefore']] = $insert;
			unset($this->typeMaps[$this->resourceType]['optional']['publisher']['insertBefore']);
		}
		if(array_key_exists('insertAfter', $this->typeMaps[$this->resourceType]['optional']['publisher']))
		{
			$insertAfter[$this->typeMaps[$this->resourceType]['optional']['publisher']['insertAfter']] = $insert;
			unset($this->typeMaps[$this->resourceType]['optional']['publisher']['insertAfter']);
		}
		return $this->optionalCellsFormat('publisher', 'publisherOuter', $insertBefore, $insertAfter, 'publisher');
	}
/**
* Select box publisher insert for conferences -- fields filled in after AJAX select conference
*/
	private function publisherInsertFill()
	{
// Get a resourceId to fill in conference details from
		$this->db->formatConditions(array('resourcemiscCollection' => $this->conferenceFill));
		$this->db->formatConditions(array('resourcemiscField1' => ' IS NOT NULL'));
		$this->db->limit(1, 0); // pick just the first one
		$resultset = $this->db->select('resource_misc', array('resourcemiscId', 'resourcemiscField1'));
//print_r($row);
		if(!$this->db->numRows($resultset))
		{
			unset($this->sessionVars['publisherId']);
			return $this->publisherInsert();
		}
		$row = $this->db->fetchRow($resultset);
		$publisherId = $row['resourcemiscField1'];
		$resourceId = $row['resourcemiscId'];
		foreach($this->typeMaps[$this->resourceType]['optional']['publisher'] as $tableKey => $tableArray)
		{
			if(($tableKey == 'insertBefore') || ($tableKey == 'insertAfter') || ($tableKey == 'title') 
				 || ($tableKey == 'insertTitle') || ($tableKey == 'publisher') || ($tableKey == 'resourcepage')
				 || (strpos($tableKey, 'hint_') === 0))
				continue;
			if(array_key_exists('break', $tableArray))
				unset($tableArray['break']);
			if(array_key_exists('break2', $tableArray))
				unset($tableArray['break2']);
			if(array_key_exists('continue', $tableArray))
				unset($tableArray['continue']);
			if($tableKey == 'resourcemisc')
				$table = 'resource_misc';
			else if($tableKey == 'resourceyear')
				$table = 'resource_year';
			else
				$table = $tableKey;
			$fieldArray = array_keys($tableArray);
//print_r($fieldArray); print '<P>';
			$fieldArrayDB = $this->db->prependTableToField($tableKey, $fieldArray);
			$this->db->formatConditions(array($tableKey . 'Id' => $resourceId));
			$row = $this->db->fetchRow($this->db->select($table, $fieldArrayDB));
//print_r($row);
			foreach($fieldArrayDB as $field)
			{
				if($row[$field])
					$this->sessionVars[$field] = $row[$field];
				else
					unset($this->sessionVars[$field]);
			}
		}
		if(($this->resourceType == 'proceedings_article') || ($this->resourceType == 'proceedings'))
			$this->sessionVars['organizerId'] = $publisherId;
		else
			$this->sessionVars['publisherId'] = $publisherId;
//print_r($this->sessionVars); print '<P>';
		return $this->publisherInsert();
	}
/**
* Select box publishers insert for the publishers table
*/
	private function publisherInsert()
	{
		if(array_key_exists($this->resourceType, $this->publisherMap->publisherTypes))
			$temp = $this->publisher->grabAll($this->publisherMap->publisherTypes[$this->resourceType]);
		else
			$temp = $this->publisher->grabAll();
		if(is_array($temp))
		{
			if(array_key_exists('insertTitle', $this->typeMaps[$this->resourceType]['optional']['publisher']))
			{
				if($this->typeMaps[$this->resourceType]['optional']['publisher']['insertTitle'])
					$title =
					$this->messages->text("resources", $this->typeMaps[$this->resourceType]['optional']['publisher']['insertTitle']);
			}
			else
				$title = $this->messages->text("resources", "publisher");
			$publishers[0] = $this->messages->text("misc", "ignore");
			foreach($temp as $key => $value)
				$publishers[$key] = $value;
			if(($this->resourceType == 'proceedings_article') || ($this->resourceType == 'proceedings'))
				$id = 'organizerId';
			else
				$id = 'publisherId';
			$pId = array_key_exists($id, $this->sessionVars) ? 
				$this->html->dbToFormTidy($this->sessionVars[$id]) : FALSE;
			if($pId)
				return $this->form->selectedBoxValue($title, $id, $publishers, $pId, 1);
			else
				return $this->form->selectFBoxValue($title, $id, $publishers, 1);
		}
		return array();
	}
/**
** For book, book_article, book_chapter, translated work details
*/
	public function translation()
	{
		if(($this->resourceType != 'book') && ($this->resourceType != 'book_article') && ($this->resourceType != 'book_chapter'))
			return '';
		if($this->translationFill)
			list($titInsert, $pubInsert) = $this->translationInsertFill();
		else
		{
			$pubInsert = $this->translationPubInsert();
			$titInsert = $this->translationTitleInsert();
		}
		if(!$pubInsert)
			$pubInsert = '';
		if(!$titInsert)
			$titInsert = '';
		$insertAfter = $insertBefore = array();
// In RESOURCEMAP, we stipulate that insert_after is used for title select box and insert_before for publisher select box
		if(array_key_exists('insertBefore', $this->typeMaps[$this->resourceType]['optional']['translation']))
		{
			$insertBefore[$this->typeMaps[$this->resourceType]['optional']['translation']['insertBefore']] = $pubInsert;
			unset($this->typeMaps[$this->resourceType]['optional']['translation']['insertBefore']);
		}
		if(array_key_exists('insertAfter', $this->typeMaps[$this->resourceType]['optional']['translation']))
		{
			$insertAfter[$this->typeMaps[$this->resourceType]['optional']['translation']['insertAfter']] = $titInsert;
			unset($this->typeMaps[$this->resourceType]['optional']['translation']['insertAfter']);
		}
		return $this->optionalCellsFormat('translation', 'translationOuter', $insertBefore, $insertAfter, 'translatedFrom');
	}
/**
* Select box translation insert for translation -- fields filled in after AJAX select translation
*/
	private function translationInsertFill()
	{
// Get a resourceId to fill in translation details from
		$array = unserialize(base64_decode($this->translationFill));
		if($array[0])
			$this->db->formatConditions(array('resourceTransNoSort' => $array[0])); // noSort
		else
			$this->db->formatConditions(array('resourceTransNoSort' => ' IS NULL')); // noSort
		$this->db->formatConditions(array('resourceTransTitle' => $array[1])); // title
		if(sizeof($array) > 2)
			$this->db->formatConditions(array('resourceTransSubtitle' => $array[2])); // subtitle
		else
			$this->db->formatConditions(array('resourceTransSubtitle' => ' IS NULL')); // subtitle
		$this->db->limit(1, 0); // pick just the first one
		$resourceId = $this->db->fetchOne($this->db->select('resource', 'resourceId'));
		foreach($this->typeMaps[$this->resourceType]['optional']['translation'] as $tableKey => $tableArray)
		{
			if(($tableKey == 'insertBefore') || ($tableKey == 'insertAfter') || ($tableKey == 'title') 
				 || ($tableKey == 'insertTitle') || ($tableKey == 'publisher') || (strpos($tableKey, 'hint_') === 0))
				continue;
			if(array_key_exists('break', $tableArray))
				unset($tableArray['break']);
			if(array_key_exists('continue', $tableArray))
				unset($tableArray['continue']);
			if($tableKey == 'resourcemisc')
				$table = 'resource_misc';
			else if($tableKey == 'resourceyear')
				$table = 'resource_year';
			else
				$table = $tableKey;
			$fieldArray = array_keys($tableArray);
			$fieldArrayDB = $this->db->prependTableToField($tableKey, $fieldArray);
			$this->db->formatConditions(array($tableKey . 'Id' => $resourceId));
			$row = $this->db->fetchRow($this->db->select($table, $fieldArrayDB));
			foreach($fieldArrayDB as $field)
			{
				if($row[$field])
					$this->sessionVars[$field] = $row[$field];
				else
					unset($this->sessionVars[$field]);
			}
		}
		$this->db->formatConditions(array('resourcemiscId' => $resourceId));
		$resultset = $this->db->select('resource_misc', 'resourcemiscField1');
		$row = $this->db->fetchRow($resultset);
		if($row['resourcemiscField1'])
			$this->sessionVars['transPublisherId'] = $row['resourcemiscField1'];
		else
			unset($this->sessionVars['transPublisherId']);
		return array($this->translationTitleInsert(), $this->translationPubInsert()); // sets title and publisher select boxes back to IGNORE
	}
/**
* Select box titles insert for the translation table
*/
	private function translationTitleInsert()
	{
		$temp = array();
		$this->db->formatConditions(array('resourceType' => $this->resourceType));
		$this->db->formatConditions(array('resourceTransTitle' => ' IS NOT NULL'));
		$this->db->orderBy('resourceTransTitle');
		$resultset = $this->db->select('resource', array('resourceTransTitle', 'resourceTransSubtitle', 'resourceTransShortTitle', 
			'resourceTransNoSort'), TRUE);
		while($row = $this->db->fetchRow($resultset))
		{
			if($row['resourceTransNoSort'])
				$noSort = $row['resourceTransNoSort'] . ' ';
			else
				$noSort = '';
			if($row['resourceTransSubtitle'])
				$temp[] = array(array($noSort, $row['resourceTransTitle'], $row['resourceTransSubtitle']), 
					$noSort . $row['resourceTransTitle'] . ': ' . $row['resourceTransSubtitle']);
			else
				$temp[] = array(array($noSort, $row['resourceTransTitle']), $noSort . $row['resourceTransTitle']);
		}
		if(!empty($temp))
		{
			if(array_key_exists('insertTitle', $this->typeMaps[$this->resourceType]['optional']['translation']))
			{
				if($this->typeMaps[$this->resourceType]['optional']['translation']['insertTitle'])
					$title = 
					$this->messages->text("resources", $this->typeMaps[$this->resourceType]['optional']['translation']['insertTitle']);
			}
			else
				$title = $this->messages->text("resources", "originalTitle");
// Fill in trans DIV depending upon which trans is chosen
			$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=fillTrans&resourceType=' . $this->resourceType;
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'transTitles',
				'targetDiv' => 'translationOuter',
				);
			$this->ajax->jActionForm('onchange', $jsonArray);
			$titles[base64_encode('IGNORE')] = $this->messages->text("misc", "ignore");
			foreach($temp as $value)
				$titles[base64_encode(serialize($value[0]))] = $value[1];
			$tId = array_key_exists("transTitles", $this->sessionVars) ? 
				$this->html->dbToFormTidy($this->sessionVars['transTitles']) : FALSE;
			if($tId)
				return $this->form->selectedBoxValue($title, 'transTitles', $titles, $tId, 1);
			else
				return $this->form->selectFBoxValue($title, 'transTitles', $titles, 1);
		}
		return array();
	}
/**
* Select box publishers insert for the translation table
*/
	private function translationPubInsert()
	{
		$temp = $this->publisher->grabAll();
		if(is_array($temp))
		{
			if(array_key_exists('insertTitle', $this->typeMaps[$this->resourceType]['optional']['translation']))
			{
				if($this->typeMaps[$this->resourceType]['optional']['translation']['insertTitle'])
					$title = 
					$this->messages->text("resources", $this->typeMaps[$this->resourceType]['optional']['translation']['insertTitle']);
			}
			else
				$title = $this->messages->text("resources", "publisher");
			$publishers[0] = $this->messages->text("misc", "ignore");
			foreach($temp as $key => $value)
				$publishers[$key] = $value;
			$pId = array_key_exists("transPublisherId", $this->sessionVars) ? 
				$this->html->dbToFormTidy($this->sessionVars['transPublisherId']) : FALSE;
			if($pId)
				return $this->form->selectedBoxValue($title, 'transPublisherId', $publishers, $pId, 1);
			else
				return $this->form->selectFBoxValue($title, 'transPublisherId', $publishers, 1);
		}
		return array();
	}
/**
** Series
*/
	public function series()
	{
		if($this->seriesFill)
			$insert = $this->seriesInsertFill();
		else
			$insert = $this->seriesInsert();
		if(!$insert)
			$insert = '';
		$insertAfter = $insertBefore = array();
		if(array_key_exists('insertBefore', $this->typeMaps[$this->resourceType]['optional']['series']))
		{
			$insertBefore[$this->typeMaps[$this->resourceType]['optional']['series']['insertBefore']] = $insert;
			unset($this->typeMaps[$this->resourceType]['optional']['series']['insertBefore']);
		}
		if(array_key_exists('insertAfter', $this->typeMaps[$this->resourceType]['optional']['series']))
		{
			$insertAfter[$this->typeMaps[$this->resourceType]['optional']['series']['insertAfter']] = $insert;
			unset($this->typeMaps[$this->resourceType]['optional']['series']['insertAfter']);
		}
		return $this->optionalCellsFormat('series', 'seriesOuter', $insertBefore, $insertAfter, 'series');
	}
/**
* Select box series insert for series -- fields filled in after AJAX select series
*/
	private function seriesInsertFill()
	{
// Get a resourceId to fill in series details from
		$this->db->formatConditions(array('resourceType' => $this->resourceType));
		$this->db->formatConditions(array('resourceField1' => base64_decode($this->seriesFill)));
		$this->db->limit(1, 0); // pick just the first one
		$resourceId = $this->db->fetchOne($this->db->select('resource', 'resourceId'));
		foreach($this->typeMaps[$this->resourceType]['optional']['series'] as $tableKey => $tableArray)
		{
			if(($tableKey == 'insertBefore') || ($tableKey == 'insertAfter') || ($tableKey == 'title') 
				 || ($tableKey == 'insertTitle') || (strpos($tableKey, 'hint_') === 0))
				continue;
			if(array_key_exists('break', $tableArray))
				unset($tableArray['break']);
			if(array_key_exists('continue', $tableArray))
				unset($tableArray['continue']);
			if($tableKey == 'resourcemisc')
				$table = 'resource_misc';
			else if($tableKey == 'resourceyear')
				$table = 'resource_year';
			else
				$table = $tableKey;
			$fieldArray = array_keys($tableArray);
			$fieldArrayDB = $this->db->prependTableToField($tableKey, $fieldArray);
			$this->db->formatConditions(array($tableKey . 'Id' => $resourceId));
			$row = $this->db->fetchRow($this->db->select($table, $fieldArrayDB));
			foreach($fieldArrayDB as $field)
			{
				if($row[$field])
					$this->sessionVars[$field] = $row[$field];
				else
					unset($this->sessionVars[$field]);
			}
		}
		return $this->seriesInsert(); // sets select box back to IGNORE
	}
/**
* Select box series insert for series -- initial blank fields
*/
	private function seriesInsert()
	{
		$this->db->formatConditions(array('resourceType' => $this->resourceType));
		$this->db->formatConditions(array('resourceField1' => ' IS NOT NULL'));
		$this->db->orderBy('resourceField1');
		$resultset = $this->db->select('resource', 'resourceField1', TRUE);
		while($row = $this->db->fetchRow($resultset))
		{
			if($row['resourceField1'])
				$temp[] = $row['resourceField1'];
		}
		if(isset($temp))
		{
// Fill in series DIV depending upon which series is chosen
			$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=fillSeries&resourceType=' . $this->resourceType;
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'series',
				'targetDiv' => 'seriesOuter',
				);
			$this->ajax->jActionForm('onchange', $jsonArray);
			$series[base64_encode('IGNORE')] = $this->messages->text("misc", "ignore");
			foreach($temp as $value)
				$series[base64_encode($value)] = $value;
			$selected = array_key_exists('series', $this->sessionVars) ? 
				$this->sessionVars['series'] : FALSE;
			if($selected)
				return $this->form->selectedBoxValue(FALSE, "series", $series, $selected, 1);
			else
				return $this->form->selectFBoxValue(FALSE, "series", $series, 1);
		}
		return FALSE;
	}
/**
** Conference
*/
	public function conference()
	{
		if($this->conferenceFill !== FALSE)
			$insert = $this->conferenceInsertFill();
		else
			$insert = $this->conferenceInsert();
		if(empty($insert))
			$insert = '';
		$insertAfter = $insertBefore = array();
		if(array_key_exists('insertBefore', $this->typeMaps[$this->resourceType]['optional']['conference']))
		{
			$insertBefore[$this->typeMaps[$this->resourceType]['optional']['conference']['insertBefore']] = 
				$insert;
			unset($this->typeMaps[$this->resourceType]['optional']['conference']['insertBefore']);
		}
		if(array_key_exists('insertAfter', $this->typeMaps[$this->resourceType]['optional']['conference']))
		{
			$insertAfter[$this->typeMaps[$this->resourceType]['optional']['conference']['insertAfter']] = 
				$insert;
			unset($this->typeMaps[$this->resourceType]['optional']['conference']['insertAfter']);
		}
		return $this->optionalCellsFormat('conference', 'conferenceOuter', $insertBefore, $insertAfter, 'conference');
	}
/**
* Select box conference insert for conferences -- fields filled in after AJAX select conference
*/
	private function conferenceInsertFill()
	{
// Get a resourceId to fill in conference details from
		$this->db->formatConditions(array('resourcemiscCollection' => $this->conferenceFill));
		$this->db->limit(1, 0); // pick just the first one
		$resourceId = $this->db->fetchOne($this->db->select('resource_misc', 'resourcemiscId'));
		foreach($this->typeMaps[$this->resourceType]['optional']['conference'] as $tableKey => $tableArray)
		{
			if(($tableKey == 'insertBefore') || ($tableKey == 'insertAfter') || ($tableKey == 'title') || ($tableKey == 'titleHint') 
				 || ($tableKey == 'insertTitle') || ($tableKey == 'collection') || (strpos($tableKey, 'hint_') === 0))
				continue;
			if(array_key_exists('break', $tableArray))
				unset($tableArray['break']);
			if(array_key_exists('break2', $tableArray))
				unset($tableArray['break2']);
			if(array_key_exists('continue', $tableArray))
				unset($tableArray['continue']);
			if($tableKey == 'resourcemisc')
				$table = 'resource_misc';
			else if($tableKey == 'resourceyear')
				$table = 'resource_year';
			else
				$table = $tableKey;
			$fieldArray = array_keys($tableArray);
//print_r($fieldArray); print '<P>';
			$fieldArrayDB = $this->db->prependTableToField($tableKey, $fieldArray);
			$this->db->formatConditions(array($tableKey . 'Id' => $resourceId));
			$row = $this->db->fetchRow($this->db->select($table, $fieldArrayDB));
//print_r($row);
			foreach($fieldArrayDB as $field)
			{
				if($row[$field])
				{
//					if(($field == 'resourcemiscPeerReviewed') && ($row[$field] == 'N'))
//						unset($this->sessionVars[$field]);
//					else
						$this->sessionVars[$field] = $row[$field];
				}
				else
					unset($this->sessionVars[$field]);
			}
		}
		$this->sessionVars['conferenceId'] = $this->conferenceFill;
		return $this->conferenceInsert();
	}
/**
* Select box conference insert for the conference table
*/
	private function conferenceInsert()
	{
		if(!array_key_exists('conference', $this->typeMaps[$this->resourceType]['optional']))
			return;
// check there are collections/conferences
		if((($this->resourceType == 'proceedings_article') || ($this->resourceType == 'conference_paper') || 
			($this->resourceType == 'conference_poster') || ($this->resourceType == 'proceedings')) 
			&& array_key_exists($this->resourceType, $this->collectionMap->collectionTypes))
			$temp = $this->collection->grabAll($this->collectionMap->collectionTypes[$this->resourceType]);
		else 
			$temp = $this->collection->grabAll();
		if(is_array($temp))
		{
// Fill in conference DIV depending upon which conference is chosen
			$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=fillConference&resourceType=' . $this->resourceType;
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'conferenceId',
				'targetDiv' => 'conferenceOuter',
				);
// Fill in publisher DIV depending upon which conference is chosen
			$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=fillPublisher&resourceType=' . $this->resourceType;
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'conferenceId',
				'targetDiv' => 'publisherOuter',
				);
// Fill in collection/organizer DIV depending upon which conference is chosen
			$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=fillCollection&resourceType=' . $this->resourceType;
			$jsonArray[] = array(
				'startFunction' => 'triggerFromSelect', 
				'script' => "$jScript",
				'triggerField' => 'conferenceId',
				'targetDiv' => 'collectionOuter',
				);
			$this->ajax->jActionForm('onchange', $jsonArray);
			if(array_key_exists('insertTitle', $this->typeMaps[$this->resourceType]['optional']['conference']))
			{
				if($this->typeMaps[$this->resourceType]['optional']['conference']['insertTitle'])
					$title =  
					$this->messages->text("resources", $this->typeMaps[$this->resourceType]['optional']['conference']['insertTitle']);
			}
			else
				$title = $this->messages->text("resources", "collection");
			$collections[0] = $this->messages->text("misc", "ignore");
			foreach($temp as $key => $value)
				$collections[$key] = $value;
			$selected = array_key_exists('conferenceId', $this->sessionVars) ? $this->sessionVars['conferenceId'] : FALSE;
//print 'SELECT: ' . $this->sessionVars['conferenceId'] . ", $selected<P>";
//			if($selected)
				return $this->form->selectedBoxValue($title, "conferenceId", $collections, $selected, 1);
//			else
//				return $this->form->selectFBoxValue($title, "conferenceId", $collections, 1);
		}
		return array();
	}
/**
* Select box organizer insert for conferences -- fields filled in after AJAX select conference
*/
	private function collectionInsertFill()
	{
		$this->db->formatConditions(array('resourcemiscCollection' => $this->conferenceFill));
		$this->db->formatConditions(array('resourcemiscPublisher' => ' IS NOT NULL'));
		$this->db->limit(1, 0); // pick just the first one
		$resultset = $this->db->select('resource_misc', 'resourcemiscPublisher');
		if(!$this->db->numRows($resultset))
			unset($this->sessionVars['organizerId']);
		else
			$this->sessionVars['organizerId'] = $this->db->fetchOne($resultset);
//print_r($this->sessionVars);
		return $this->collectionInsert();
	}
/**
* Select box collection insert for the collection table
*/
	private function collectionInsert()
	{
// check there are collections
		if((($this->resourceType == 'proceedings_article') || ($this->resourceType == 'conference_paper') || 
			($this->resourceType == 'conference_poster') || ($this->resourceType == 'proceedings')) 
			&& array_key_exists($this->resourceType, $this->publisherMap->publisherTypes))
			$temp = $this->publisher->grabAll($this->publisherMap->publisherTypes[$this->resourceType]);
		else if(($this->resourceType == 'proceedings_article') || ($this->resourceType == 'conference_paper') || 
			($this->resourceType == 'conference_poster') || ($this->resourceType == 'proceedings'))
			$temp = $this->publisher->grabAll();
		else if(array_key_exists($this->resourceType, $this->collectionMap->collectionTypes))
			$temp = $this->collection->grabAll($this->collectionMap->collectionTypes[$this->resourceType]);
		else
			$temp = $this->collection->grabAll();
		if(is_array($temp))
		{
			if(array_key_exists('insertTitle', $this->typeMaps[$this->resourceType]['optional']['collection']))
			{
				if($this->typeMaps[$this->resourceType]['optional']['collection']['insertTitle'])
					$title =  
					$this->messages->text("resources", $this->typeMaps[$this->resourceType]['optional']['collection']['insertTitle']);
			}
			else
				$title = $this->messages->text("resources", "collection");
			$collections[0] = $this->messages->text("misc", "ignore");
			foreach($temp as $key => $value)
				$collections[$key] = $value;
			if(($this->resourceType == 'conference_paper') || ($this->resourceType == 'conference_poster'))
				$id = 'organizerId';
			else if(($this->resourceType == 'proceedings_article') || ($this->resourceType == 'proceedings'))
				$id = 'publisherId';
			else
				$id = 'collectionId';
			$selected = array_key_exists($id, $this->sessionVars) ? $this->sessionVars[$id] : FALSE;
			if($selected)
				return $this->form->selectedBoxValue($title, $id, $collections, $selected, 1);
			else
				return $this->form->selectFBoxValue($title, $id, $collections, 1);
		}
		return array();
	}
/**
** Collection
*/
	public function collection()
	{
		if($this->conferenceFill)
			$insert = $this->collectionInsertFill();
		else
			$insert = $this->collectionInsert();
		if(empty($insert))
			$insert = '';
		$insertAfter = $insertBefore = array();
		if(array_key_exists('insertBefore', $this->typeMaps[$this->resourceType]['optional']['collection']))
		{
			$insertBefore[$this->typeMaps[$this->resourceType]['optional']['collection']['insertBefore']] = 
				$insert;
			unset($this->typeMaps[$this->resourceType]['optional']['collection']['insertBefore']);
		}
		if(array_key_exists('insertAfter', $this->typeMaps[$this->resourceType]['optional']['collection']))
		{
			$insertAfter[$this->typeMaps[$this->resourceType]['optional']['collection']['insertAfter']] = 
				$insert;
			unset($this->typeMaps[$this->resourceType]['optional']['collection']['insertAfter']);
		}
		return $this->optionalCellsFormat('collection', 'collectionOuter', $insertBefore, $insertAfter, 'collection');
	}
/**
** Miscellaneous details -- may not have 'insertBefore' or 'insertAfter'
*/
	public function miscellaneous()
	{
		$insertAfter = $insertBefore = array(); // empty arrays
		if(array_key_exists('insertBefore', $this->typeMaps[$this->resourceType]['optional']['miscellaneous']))
			unset($this->typeMaps[$this->resourceType]['optional']['miscellaneous']['insertBefore']);
		if(array_key_exists('insertAfter', $this->typeMaps[$this->resourceType]['optional']['miscellaneous']))
			unset($this->typeMaps[$this->resourceType]['optional']['miscellaneous']['insertAfter']);
		return $this->optionalCellsFormat('miscellaneous', 'miscellaneousOuter', $insertBefore, $insertAfter, 'miscellaneous');
	}
/**
* Populated creator cells -- used for new resource entry
*/
	public function creatorFields($type, $inputArray)
	{
		$fields = $label = '';
		if(empty($this->creatorsArray))
		{
			$temp = $this->creator->grabAll();
			$this->creatorsArray[0] = $this->messages->text("misc", "ignore");
			if(is_array($temp))
				$this->creatorsArray = $this->creatorsArray + $temp;
//			if(is_array($temp))
//			{
//				foreach($temp as $key => $value)
//					$this->creatorsArray[$key] = $value;
//			}
		}
		$this->writeSessionCreators($inputArray);
		for($index = 0; $index < $inputArray['index']; $index++)
		{
			if($index == 0)
				$label = $this->makeCreatorLabel();
			$this->session->setVar('resourceFormValidate_' . $type . '_' . $index, TRUE);
			$entry = $type . '_' . $index . '_firstname';
			$text = array_key_exists($entry, $this->sessionVars) ? $this->sessionVars[$entry] : FALSE;
			$fields .= $this->table->trStart();
			$fields .= $this->table->td($this->form->textInput(FALSE, $entry, $this->html->dbToFormTidy($text), 30, 255));
			$entry = $type . '_' . $index . '_initials';
			$text = array_key_exists($entry, $this->sessionVars) ? $this->sessionVars[$entry] : FALSE;
			$fields .= $this->table->td($this->form->textInput(FALSE, $entry, $this->html->dbToFormTidy($text), 6, 255));
			$entry = $type . '_' . $index . '_prefix';
			$text = array_key_exists($entry, $this->sessionVars) ? $this->sessionVars[$entry] : FALSE;
			$fields .= $this->table->td($this->form->textInput(FALSE, $entry, $this->html->dbToFormTidy($text), 11, 255));
			$entry = $type . '_' . $index . '_surname';
			$text = array_key_exists($entry, $this->sessionVars) ? $this->sessionVars[$entry] : FALSE;
			$fields .= $this->table->td($this->form->textInput(FALSE, $entry, $this->html->dbToFormTidy($text), 30, 255));
			$entry = $type . '_' . $index . '_select';
			$selected = array_key_exists($entry, $this->sessionVars) ? $this->sessionVars[$entry] : FALSE;
			if($selected)
				$fields .= $this->table->td($this->form->selectedBoxValue(FALSE, $entry, $this->creatorsArray, $selected, 1));
			else
				$fields .= $this->table->td($this->form->selectFBoxValue(FALSE, $entry, $this->creatorsArray, 1));
			$fields .= $this->table->trEnd();
		}
		if(!$label)
			return FALSE;
		return $label . $fields;
	}
/**
* Populated creator cells -- used for editing a resource entry
*/
	public function creatorFieldsEdit($type)
	{
		$fields = $label = '';
		if(empty($this->creatorsArray))
		{
			$temp = $this->creator->grabAll();
			$this->creatorsArray[0] = $this->messages->text("misc", "ignore");
			if(is_array($temp))
				$this->creatorsArray = $this->creatorsArray + $temp;
//			if(is_array($temp))
//			{
//				foreach($temp as $key => $value)
//					$this->creatorsArray[$key] = $value;
//			}
		}
		for($index = 0; ; $index++)
		{
			$found = FALSE;
			$thisRow = FALSE;
			$select = $type . '_' . $index . '_select';
			if(!array_key_exists($select, $this->sessionVars) && !$this->error)
				break;
			$this->session->setVar('resourceFormValidate_' . $type . '_' . $index, TRUE);
			$entry = $type . '_' . $index . '_firstname';
			$value = $this->session->getVar('resourceForm_' . $entry) ? $this->session->getVar('resourceForm_' . $entry) : FALSE;
			$found = $value ? TRUE : FALSE;
			$thisRow .= $this->table->trStart();
			$thisRow .= $this->table->td($this->form->textInput(FALSE, $entry, $value, 30, 255));
			$entry = $type . '_' . $index . '_initials';
			$value = $this->session->getVar('resourceForm_' . $entry) ? $this->session->getVar('resourceForm_' . $entry) : FALSE;
			$found = $value ? TRUE : FALSE;
			$thisRow .= $this->table->td($this->form->textInput(FALSE, $entry, $value, 6, 255));
			$entry = $type . '_' . $index . '_prefix';
			$value = $this->session->getVar('resourceForm_' . $entry) ? $this->session->getVar('resourceForm_' . $entry) : FALSE;
			$found = $value ? TRUE : FALSE;
			$thisRow .= $this->table->td($this->form->textInput(FALSE, $entry, $value, 11, 255));
			$entry = $type . '_' . $index . '_surname';
			$value = $this->session->getVar('resourceForm_' . $entry) ? $this->session->getVar('resourceForm_' . $entry) : FALSE;
			$found = $value ? TRUE : FALSE;
			$thisRow .= $this->table->td($this->form->textInput(FALSE, $entry, $value, 30, 255));
			if(array_key_exists($select, $this->sessionVars))
			{
				$found = TRUE;
				$thisRow .= $this->table->td($this->form->selectedBoxValue(FALSE, $select, $this->creatorsArray, 
					$this->sessionVars[$select], 1));
			}
			else
				$thisRow .= $this->table->td($this->form->selectFBoxValue(FALSE, $select, $this->creatorsArray, 1));
			$thisRow .= $this->table->trEnd();
			if(!$found)
				break;
			else
				$fields .= $thisRow;
			if($index == 0)
				$label = $this->makeCreatorLabel();
		}
		if(!$label)
			return array($index, $this->table->tableStart('generalTable', 1) . FALSE . $this->table->tableEnd());
		return array($index, $this->table->tableStart('generalTable', 1) . $label . $fields . $this->table->tableEnd());
	}
/**
* Blank creator cells
*/
	public function blankCreatorCell($key, $creatorMsg)
	{
		$jsonArray = array();
		$jScript = "index.php?action=resource_RESOURCEFORMAJAX_CORE&method=addCreatorField&creatorType=$key";
		if($this->edit || $this->error || ($this->session->getVar('resourceFormType') == 'edit'))
		{
			list($index, $editCell) = $this->creatorFieldsEdit($key);
			$jsonArray[] = array(
				'startFunction' => 'addCreator', 
				'script' => "$jScript",
				'creatorType' => "$key",
				'type' => 'edit',
				'index' => "$index",
				);
		}
		else
			$jsonArray[] = array(
				'startFunction' => 'addCreator', 
				'script' => "$jScript",
				'creatorType' => "$key",
				'type' => 'new',
				);
		$addImage = $this->ajax->jActionIcon('add', 'onclick', $jsonArray);
		$jsonArray = array();
		$jScript = "index.php?action=resource_RESOURCEFORMAJAX_CORE&method=removeCreatorField&creatorType=$key";
		if($this->edit)
			$jsonArray[] = array(
				'startFunction' => 'removeCreator', 
				'script' => "$jScript",
				'creatorType' => "$key",
				'type' => 'edit',
				'index' => "$index",
				);
		else
			$jsonArray[] = array(
				'startFunction' => 'removeCreator', 
				'script' => "$jScript",
				'creatorType' => "$key",
				'type' => 'new',
				);
		$removeImage = $this->ajax->jActionIcon('remove', 'onclick', $jsonArray);
		$images = '&nbsp;&nbsp;' . $addImage . '&nbsp;&nbsp;' . $removeImage;
		$creatorCells = $this->table->trStart();
		$creatorCells .= $this->table->td($this->html->h($this->messages->text('creators', $creatorMsg) . $images, '', 4), 
			FALSE, FALSE, FALSE, FALSE, $this->tdLabelWidth);
		if($this->edit || $this->error || ($this->session->getVar('resourceFormType') == 'edit'))
			$creatorCells .= $this->table->td($this->html->div($key . '_Inner', $editCell), 
				FALSE, FALSE, FALSE, FALSE, $this->tdContentWidth);
		else
			$creatorCells .= $this->table->td($this->html->div($key . '_Inner', '&nbsp;'), 
				FALSE, FALSE, FALSE, FALSE, $this->tdContentWidth);
		$creatorCells .= $this->table->trEnd();
		return $creatorCells;
	}
/**
* Blank collection DIV
*/
	public function blankCollection()
	{
		return $this->html->div('collectionOuter', FALSE);
	}
/**
* Blank series DIV
*/
	public function blankSeries()
	{
		return $this->html->div('seriesOuter', FALSE);
	}
/**
* Blank conference DIV
*/
	public function blankConference()
	{
		return $this->html->div('conferenceOuter', FALSE);
	}
/**
* Blank translation DIV
*/
	public function blankTranslation()
	{
		return $this->html->div('translationOuter', FALSE);
	}
/**
* Blank publisher DIV
*/
	public function blankPublisher()
	{
		return $this->html->div('publisherOuter', FALSE);
	}
/**
* Blank miscellaneous DIV
*/
	public function blankMiscellaneous()
	{
		return $this->html->div('miscellaneousOuter', FALSE);
	}
/**
* Abstract, notes, categories, subcategories, keywords, user bibliographies, URL, ISBN, DOI etc.
*/
	private function otherCell()
	{
		$pString = $this->table->td($this->html->h($this->messages->text('resources', 'commonDetails'), 4) . 
			$this->html->span($this->messages->text("hint", "keywordsUserTags"), 'hint'), 
			FALSE, FALSE, FALSE, FALSE, $this->tdLabelWidth);
		$tdContent1 = $this->table->tableStart() . $this->table->trStart();
		$text = array_key_exists('resourcetextAbstract', $this->sessionVars) ? 
			$this->html->dbToFormTidy($this->sessionVars['resourcetextAbstract']) : FALSE;
		$tdContent1 .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "abstract"), 
			"resourcetextAbstract", $text, 60, 10));
		$tdContent1 .= $this->table->trEnd() . $this->table->trStart();
		$text = array_key_exists('resourcetextNote', $this->sessionVars) ? 
			$this->html->dbToFormTidy($this->sessionVars['resourcetextNote']) : FALSE;
		$tdContent1 .= $this->table->td($this->form->textareaInput($this->messages->text("resources", "note"), 
			"resourcetextNote", $text, 60, 10));
		$tdContent1 .= $this->table->trEnd() . $this->table->tableEnd();
		$tdContent1 .= $this->html->br() . '&nbsp;' . $this->html->br();
		$tdContent2 = $this->table->tableStart() . $this->table->trStart();
		$text = array_key_exists('resourcetextUrl', $this->sessionVars) ? $this->sessionVars['resourcetextUrl'] : 'http://';
		$urlContent = $this->form->textInput($this->messages->text("resources", "url"), 
		"resourcetextUrl", $text, 70, 255) . $this->html->br() . $this->html->span($this->messages->text("hint", "url"), 'hint');
		$text = array_key_exists('resourcetextUrlText', $this->sessionVars) ? 
			$this->html->dbToFormTidy($this->sessionVars['resourcetextUrlText']) : FALSE;
		$urlContent .= $this->html->br() . '&nbsp;' . $this->html->br();
		$urlContent .= $this->form->textInput($this->messages->text("resources", "urlLabel"), 
			"resourcetextUrlText", $text, 70, 255);
		$tdContent2 .= $this->table->td($urlContent, FALSE, FALSE, FALSE, FALSE, '50%');
		$text = array_key_exists('resourceIsbn', $this->sessionVars) ? 
			$this->html->dbToFormTidy($this->sessionVars['resourceIsbn']) : FALSE;
		$tdContent2 .= $this->table->td($this->form->textInput($this->messages->text("resources", "isbn"), 
			"resourceIsbn", $text, 30, 255), FALSE, FALSE, FALSE, FALSE, '25%');
		$text = array_key_exists('resourceDoi', $this->sessionVars) ? 
			$this->html->dbToFormTidy($this->sessionVars['resourceDoi']) : '';
		$tdContent2 .= $this->table->td($this->form->textInput($this->messages->text("resources", "doi"), 
			"resourceDoi", $text, 30, 255) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "doi"), 'hint'), FALSE, FALSE, FALSE, FALSE, '25%');
		$tdContent2 .= $this->table->trEnd() . $this->table->tableEnd();
		$tdContent2 .= $this->html->br() . '&nbsp;' . $this->html->br();
		$tdContent3 = $this->table->tableStart(FALSE, 0, 4) . $this->table->trStart();
// categories
		$categories = $this->category->grabAll();
		$selectedCategories = array();
		if(array_key_exists('resourcecategoryCategories', $this->sessionVars))
		{
			$selected = explode(',', $this->sessionVars['resourcecategoryCategories']);
			foreach($selected as $key)
			{
				$selectedCategories[$key] = $categories[$key];
				unset($categories[$key]);
			}
		}
		$td = $this->form->selectFBoxValueMultiple($this->messages->text('resources', 'categories'), 
			"availableCategory", $categories, 5) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "multiples"), 'hint');
		list($toBottomImage, $toTopImage) = $this->category->transferArrowsCatUpDown();
		$td .= $this->html->p($toBottomImage . '&nbsp;&nbsp;' . $toTopImage);
		$td .= $this->html->p($this->form->selectFBoxValueMultiple($this->messages->text('search', 'category'),
			"resourcecategoryCategories", $selectedCategories, 5) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
		$tdContent3 .= $this->table->td($td, FALSE, FALSE, FALSE, FALSE, '33%');
		$temp = $this->category->grabSubAll(TRUE);
		$tdContent3 .= $this->subcategoryBox($temp);
		
// Resource languages
		$this->db->orderBy('languageLanguage');
		$resultset = $this->db->select('language', array('languageId', 'languageLanguage'));
		if($this->db->numRows($resultset))
		{
			$this->languages[0] = $this->messages->text("misc", "ignore");
			while($row = $this->db->fetchRow($resultset))
				$this->languages[$row['languageId']] = $this->html->dbToFormtidy($row['languageLanguage']);
			if(array_key_exists('resourcelanguageLanguages', $this->sessionVars))
			{
				$selectedLanguages = explode(',', $this->sessionVars['resourcelanguageLanguages']);
				$tdContent3 .= $this->table->td($this->form->selectedBoxValueMultiple($this->messages->text('resources', 
					'languages'), 'language', $this->languages, $selectedLanguages, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, FALSE, FALSE, FALSE, '33%');
			}
			else
				$tdContent3 .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 
					'languages'), 'language', $this->languages, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, FALSE, FALSE, FALSE, '33%');
		}
		$tdContent3 .= $this->table->trEnd() . $this->table->tableEnd();
		$tdContent3 .= $this->html->br() . '&nbsp;' . $this->html->br();
		
		
		$tdContent4 = $this->table->tableStart();
		$tdContent4 .= $this->table->trStart();
		
// keywords
		$text = FALSE;
		$keywords = $this->keyword->grabAll();
		if(!empty($keywords))
		{
// If preferences reduce long keywords, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($keywords as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$keywordArray[$key] = html_entity_decode($value);
			}
			$jsonArray = array();
			$jsonArray[] = array(
				'startFunction' => 'transferKeyword', 
				);
			$toBottom = $this->ajax->jActionIcon('toBottom', 'onclick', $jsonArray);
			$text = array_key_exists('resourcekeywordKeywords', $this->sessionVars) ? 
				stripslashes($this->sessionVars['resourcekeywordKeywords']) : FALSE;
			$this->form->class = 'mceNoEditor';
			$keywordText = $this->html->p($this->form->textareaInput(FALSE, "keywordList", $text, 40, 2) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "keywordList"), 'hint'));
			$this->form->class = 'formElements';
			$tdContent4 .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 'keywords'), 
				'resourcekeywordKeywords', $keywordArray, 5) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "multiples"), 'hint') . 
			$this->html->br() . $toBottom . $keywordText, FALSE, FALSE, FALSE, FALSE, '33%');
		}
		else
		{
			$this->form->class = 'mceNoEditor';
			$keywordText = $this->form->textareaInput($this->messages->text('resources', 'keywords'), "keywordList", $text, 40, 2) . 
				$this->html->br() . $this->html->span($this->messages->text("hint", "keywordList"), 'hint');
			$tdContent4 .= $this->table->td($keywordText, FALSE, FALSE, FALSE, FALSE, '33%');
			$this->form->class = 'formElements';
		}
// user tags
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'transferUserTag', 
			);
		$toBottom = $this->ajax->jActionIcon('toBottom', 'onclick', $jsonArray);
		$text = array_key_exists('resourceusertagsTagId', $this->sessionVars) ? stripslashes($this->sessionVars['resourceusertagsTagId']) : FALSE;
		$this->form->class = 'mceNoEditor';
		$userTagText = $this->html->p($this->form->textareaInput(FALSE, "userTagList", $text, 40, 2) . $this->html->br() . 
			$this->html->span($this->messages->text("hint", "keywordList"), 'hint'));
		$this->form->class = 'formElements';
		$userTags = $this->userTagObj->grabAll();
		if(!empty($userTags))
		{
// If preferences reduce long userTags, we want to transfer the original rather than the condensed version. 
// Store the base64-encoded value for retrieval in the javascript.
			foreach($userTags as $key => $value)
			{
				$key = $key . '_' . base64_encode($value);
				$userTagArray[$key] = html_entity_decode($value);
			}
			$tdContent4 .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 'userTags'), 
				'resourceusertagsTagId', $userTagArray, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint') . $this->html->br() . 
				$toBottom . $userTagText, FALSE, FALSE, FALSE, FALSE, '33%');
		}
		
// bibliographies
		$bibs[0] = $this->messages->text("misc", "ignore");
// Get this user's bibliographies
		if($this->session->getVar('mywikindx_Bibliographies'))
		{
			$bibsRaw = unserialize($this->session->getVar("mywikindx_Bibliographies"));
			foreach($bibsRaw as $key => $value)
				$bibsU[$key] = $this->html->dbToFormTidy($value);
		}
// Get this user's user group bibliographies
		if($this->session->getVar('mywikindx_Groupbibliographies'))
		{
			$bibsRaw = unserialize($this->session->getVar("mywikindx_Groupbibliographies"));
			foreach($bibsRaw as $key => $value)
				$bibsUG[$key] = $this->html->dbToFormTidy($value);
		}
		$bibsU = $this->commonBib->getUserBibs();
		if(!empty($bibsU))
		{
			foreach($bibsU as $key => $value)
				$bibs[$key] = $value;
		}
		$bibsUG = $this->commonBib->getGroupBibs();
		if(!empty($bibsUG))
		{
			foreach($bibsUG as $key => $value)
				$bibs[$key] = $value;
		}
		if(array_key_exists('bibliographies', $this->sessionVars) && (sizeof($bibs) > 1))
		{
			$selected = explode(',', $this->sessionVars['bibliographies']);
			$tdContent4 .= $this->table->td($this->form->selectedBoxValueMultiple($this->messages->text('resources',
				'addNewResourceToBib'), 'bibliographies', $bibs, $selected, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, FALSE, FALSE, FALSE, '33%');
		}
		else if(sizeof($bibs) > 1)
			$tdContent4 .= $this->table->td($this->form->selectFBoxValueMultiple($this->messages->text('resources', 
				'addNewResourceToBib'), 'bibliographies', $bibs, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'),
				FALSE, FALSE, FALSE, FALSE, '33%');
		$tdContent4 .= $this->table->trEnd();
		$tdContent4 .= $this->table->tableEnd();
		
		
		$pString .= $this->table->td($tdContent1 . $tdContent2 . $tdContent3 . $tdContent4, FALSE, FALSE, FALSE, FALSE, $this->tdContentWidth);
		
		return $this->html->div('miscOuter', 
			$this->table->tableStart('generalTable', 1)
			. $this->table->trStart()
			. $pString
			. $this->table->trEnd()
			. $this->table->tableEnd()
		);
	}
/**
* Make select box for subcategories
*/
	public function subcategoryBox($temp)
	{
		if(is_array($temp))
		{
			$selectedSubcategories = array();
			if(array_key_exists('resourcecategorySubcategories', $this->sessionVars))
			{
				$selected = explode(',', $this->sessionVars['resourcecategorySubcategories']);
				foreach($selected as $key)
				{
					$selectedSubcategories[$key] = $temp[$key];
					unset($temp[$key]);
				}
			}
			$td = $this->form->selectFBoxValueMultiple($this->messages->text('resources', 'subcategories'), 
				"availableSubcategory", $temp, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint');
			list($toBottomImage, $toTopImage) = $this->category->transferArrowsSubcatUpDown();
			$td .= $this->html->p($toBottomImage . '&nbsp;&nbsp;' . $toTopImage);
			$td .= $this->html->p($this->form->selectFBoxValueMultiple($this->messages->text('search', 'subcategory'),
				"resourcecategorySubcategories", $selectedSubcategories, 5) . $this->html->br() . 
				$this->html->span($this->messages->text("hint", "multiples"), 'hint'));
			return $this->table->td($this->html->div('subcategory', $td, FALSE, FALSE, FALSE, FALSE, '33%'));
		
			$subcategories[0] = $this->messages->text("misc", "ignore");
			foreach($temp as $key => $value)
				$subcategories[$key] = $value;
			if(array_key_exists('resourcecategorySubcategories', $this->sessionVars))
			{
				$selected = explode(',', $this->sessionVars['resourcecategorySubcategories']);
				return $this->table->td($this->html->div('subcategory', 
					$this->form->selectedBoxValueMultiple($this->messages->text('resources', 'subcategories'), 
					'resourcecategorySubcategories', $subcategories, $selected, 5) . $this->html->br() . 
					$this->html->span($this->messages->text("hint", "multiples"), 'hint'), FALSE, FALSE, FALSE, FALSE, '20%'));
			}
			else
				return $this->table->td($this->html->div('subcategory', 
					$this->form->selectFBoxValueMultiple($this->messages->text('resources', 'subcategories'), 
					'resourcecategorySubcategories', $subcategories, 5) . $this->html->br() . 
					$this->html->span($this->messages->text("hint", "multiples"), 'hint'),
					FALSE, FALSE, FALSE, FALSE, '20%'));
		}
		else
			return $this->table->td($this->html->div('subcategory', "&nbsp;"));
	}
/**
* Make the labels for the creator fields
*/
	private function makeCreatorLabel()
	{
		$label = $this->table->trStart();
		$label .= $this->table->td($this->messages->text("resources", "firstname"));
		$label .= $this->table->td($this->messages->text("resources", "initials") . 
			$this->html->br() . $this->html->span($this->messages->text("hint", "initials"), 'hint'));
		$label .= $this->table->td($this->messages->text("resources", "prefix"));
		$label .= $this->table->td($this->messages->text("resources", "surname"));
		$label .= $this->table->td('&nbsp;');
		$label .= $this->table->trEnd();
		return $label;
	}
/**
* Write session variables for creators to session array 'resourceForm'
* @param -- array of session names and values.
* @return -- array of session names and values.
*/
	private function writeSessionCreators($array)
	{
		$temp = $this->session->getArray('resourceForm');
		if(!empty($array['creatorFields']))
		{
// $array['creatorFields'] does not store fields set to IGNORE or blank so, if creator fields don't exist, remove them from the session.
			if(is_array($temp))
			{
				foreach($temp as $key => $value)
				{
					if(!array_key_exists($key, $array['creatorFields']) && (strpos($key, 'Creator') === 0))
						$this->session->delVar("resourceForm_$key");
				}
			}
			$this->session->writeArray($array['creatorFields'], 'resourceForm');
		}
// else remove all creator fields
		else
		{
			if(is_array($temp))
			{
				foreach($temp as $key => $value)
				{
					if(strpos($key, 'Creator') === 0)
						$this->session->delVar("resourceForm_$key");
				}
			}
		}
		if(!$this->sessionVars = $this->session->getArray('resourceForm'))
			$this->sessionVars = array();
	}
/**
* Required indicator as HTML element
*/
	private function required()
	{
		return $this->html->span('*', 'required');
	}
/**
* Create form element definitions and store in $this->eds array -- form element type and size(s).
* This is used for RESOURCEMAP's $type['optional'] arrays.  e.g., if RESOURCEMAP has:
* 
*		$this->book['optional']['transPublisher'] = 
*			array(
*				'resource' => array(
*									'TransTitle'	=>	'title',
*									'TransSubtitle'	=>	'subtitle',
*									'TransShortTitle' => 'shortTitle',
*								),
*				);
*
* 'title', 'subtitle' and 'shortTitle' would each be $element below and large textInput.
*/
	private function loadElementDefinitions()
	{
		$ti_width_large = 255;
		$ti_width_medLarge = 140;
		$ti_width_med = 75;
		$ti_width_medSmall = 35;
		$ti_width_small = 10;
		$ti_max_large = 255;
		$ti_max_medLarge = 255;
		$ti_max_med = 255;
		$ti_max_medSmall = 255;
		$ti_max_small = 255;
		$ta_width = 60;
		$ta_height = 10;
		$ti_large = array('type' => 'textInput', 'width' => $ti_width_large, 'max' => $ti_max_large);
		$ti_medLarge = array('type' => 'textInput', 'width' => $ti_width_medLarge, 'max' => $ti_max_medLarge);
		$ti_med = array('type' => 'textInput', 'width' => $ti_width_med, 'max' => $ti_max_med);
		$ti_medSmall = array('type' => 'textInput', 'width' => $ti_width_medSmall, 'max' => $ti_max_medSmall);
		$ti_small = array('type' => 'textInput', 'width' => $ti_width_small, 'max' => $ti_max_small);
		$ta = array('type' => 'textareaInput', 'width' => $ta_width, 'height' => $ta_height);
		$cb = array('type' => 'checkbox');
		$day_sb = array('type' => 'day_selectbox');
		$month_sb = array('type' => 'month_selectbox');
// textInput large
		foreach(array() as $element)
			$this->eds[$element] = $ti_large;
// textInput medium large
		foreach(array() as $element)
			$this->eds[$element] = $ti_medLarge;
// textInput medium
		foreach(array('title', 'subtitle', 'seriesTitle',) as $element)
			$this->eds[$element] = $ti_med;
// textInput medium small
		foreach(array('shortTitle', 'section', 'city', 'publisherName', 'publisherLocation',
			'institutionName', 'institutionLocation', 'department', 'distributor', 'country', 'channel', 
			'channelLocation', 'typeOfSoftware', 'medium', 'court', 'reporter', 'reporterVolume', 'code', 
			'codeVolume', 'legislativeLocation', 'section', 'volume', 'conferenceOrganiser', 'conferenceLocation',
			'hearing', 'legislativeBody', 'committee', 'nameOfFile', 'publishedSource', 'patentType', 'intPatentTitle',
			'intPatentClassification', 'legalStatus', 'typeOfCommunication', 'typeOfWork', 'recordLabel') as $element)
			$this->eds[$element] = $ti_medSmall;
// textInput small
		foreach(array('edition', 'publicationYear', 'reprintYear', 'volumeYear', 'pageStart', 'pageEnd', 
			'seriesNumber', 'numPages', 'bookVolumeNumber', 'numberOfVolumes', 'journalVolumeNumber', 
			'journalIssueNumber', 'startPublicationYear', 'endPublicationYear', 'year', 'revisionYear', 'thesisYear', 
			'minutes', 'hours', 'proceedingsVolumeNumber', 'thesis', 'thesisLabel', 'version', 'caseYear', 
			'session', 'sessionYear', 'billNumber', 'startYear', 'endYear', 'ruleType', 'ruleNumber', 
			'issueNumber', 'typeOfReport', 'documentNumber', 'typeOfArticle', 'typeOfManuscript', 'manuscriptNumber',
			'typeOfMap', 'imageProgram', 'imageType', 'imageSize', 'number', 'publicLawNumber', 'codeNumber', 
			'applicationNumber', 'patentVersionNumber', 'intPatentNumber', 'patentNumber',) as $element)
			$this->eds[$element] = $ti_small;
// textarea input
		foreach(array() as $element)
			$this->eds[$element] = $ta;
// checkbox input
		foreach(array('peerReviewed') as $element)
			$this->eds[$element] = $cb;
// day selectbox
		foreach(array('day', 'startDay', 'endDay', 'publicationDay') as $element)
			$this->eds[$element] = $day_sb;
// month selectbox
		foreach(array('month', 'startMonth', 'endMonth', 'publicationMonth') as $element)
			$this->eds[$element] = $month_sb;
	}
/**
* Setter for private property $this->resourceType
*/
	public function setResourceType($resourceType)
	{
		$this->resourceType = $resourceType;
	}
/**
* Setter for loading session variables into $this->sessionVars when editing and using AJAX
*/
	public function setSessionVars()
	{
		if(!$this->sessionVars = $this->session->getArray('resourceForm'))
			$this->sessionVars = array();
//print '<p>'; print_r($this->sessionVars); print '<P>';
	}
}
?>
