<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* RESOURCEFORMAJAX -- AJAX for resource input form
*/
class RESOURCEFORMAJAX
{
private $ajax;
private $table;
private $session;
private $html;
private $messages;
private $errors;
private $resourceMap;
private $typeMaps;
private $vars;
private $db;
private $form;
private $resourceForm;
private $tdLabelWidth = '15%';
private $tdContentWidth = '85%';

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->ajax = FACTORY_AJAX::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->resourceMap = FACTORY_RESOURCEMAP::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		include('core/modules/resource/RESOURCEFORM.php');
		$this->resourceForm = new RESOURCEFORM();
		$this->resourceForm->setSessionVars();
		$this->typeMaps = $this->resourceMap->getTypeMap();
	}
	public function initCreators()
	{
		if(!array_key_exists('resourcecreator', $this->typeMaps[$this->vars['ajaxReturn']]) || 
			!$this->typeMaps[$this->vars['ajaxReturn']]['resourcecreator'] ||
			!array_key_exists('ajaxReturn', $this->vars) || !$this->vars['ajaxReturn'])
		{
			$div = $this->html->div('creatorsOuter', '&nbsp;');
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => "$div")));
			FACTORY_CLOSERAW::getInstance(); // die;
		}
		$creatorCells = FALSE;
		foreach($this->typeMaps[$this->vars['ajaxReturn']]['resourcecreator'] as $key => $creatorMsg)
			$creatorCells .= $this->resourceForm->blankCreatorCell($key, $creatorMsg);
		if($creatorCells)
			$div = $this->html->div('creatorsOuter', $this->table->tableStart(FALSE, 1) . $creatorCells .  $this->table->tableEnd());
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'creatorsOuter',
			'targetState'	=>	'visible'
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Add a creator input field
*/
	public function addCreatorField()
	{
		if($fields = $this->creatorFields())
			$div = $this->html->div($this->vars['creatorType'] . '_Inner', $this->table->tableStart(FALSE, 1) . 
				$fields . $this->table->tableEnd());
		else
			$div = $this->html->div($this->vars['creatorType'] . '_Inner', '&nbsp;');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* remove a creator input field
*/
	public function removeCreatorField()
	{
		if($fields = $this->creatorFields())
			$div = $this->html->div($this->vars['creatorType'] . '_Inner', $this->table->tableStart(FALSE, 1) . 
				$fields . $this->table->tableEnd());
		else
			$div = $this->html->div($this->vars['creatorType'] . '_Inner', '&nbsp;');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Cycle creator fields and make label row
*/
	private function creatorFields()
	{
		$jArray = $this->ajax->decode_jString($this->vars['ajaxReturn']);
//print_r($jArray);
		return $this->resourceForm->creatorFields($this->vars['creatorType'], $jArray);
	}
/**
* Fill in publisher detail if a conference is selected in the conference select box
*/
	public function fillPublisher()
	{
		$this->resourceForm->setResourceType($this->vars['resourceType']);
		$this->resourceForm->conferenceFill = $this->vars['ajaxReturn'];
		$div = $this->resourceForm->optionalCells('publisher');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'publisherOuter',
			'targetState'	=>	"visible"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Publisher
*/
	public function initPublisher()
	{
		$this->resourceForm->setResourceType($this->vars['ajaxReturn']);
		if(array_key_exists('publisher', $this->typeMaps[$this->vars['ajaxReturn']]['optional']))
			$visibility = 'visible';
		else
			$visibility = 'hidden';
		$div = $this->resourceForm->optionalCells('publisher');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'publisherOuter',
			'targetState'	=>	"$visibility"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Fill in series detail if a series is selected in the series select box
*/
	public function fillSeries()
	{
		$this->resourceForm->setResourceType($this->vars['resourceType']);
		$this->resourceForm->seriesFill = $this->vars['ajaxReturn'];
		$div = $this->resourceForm->optionalCells('series');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'seriesOuter',
			'targetState'	=>	"visible"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Series DIV
*/
	public function initSeries()
	{
		$this->resourceForm->setResourceType($this->vars['ajaxReturn']);
		if(array_key_exists('series', $this->typeMaps[$this->vars['ajaxReturn']]['optional']))
			$visibility = 'visible';
		else
			$visibility = 'hidden';
		$div = $this->resourceForm->optionalCells('series');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'seriesOuter',
			'targetState'	=>	"$visibility"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Fill in collection detail if a conference is selected in the conference select box
*/
	public function fillCollection()
	{
		$this->resourceForm->setResourceType($this->vars['resourceType']);
		$this->resourceForm->conferenceFill = $this->vars['ajaxReturn'];
		$div = $this->resourceForm->optionalCells('collection');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'collectionOuter',
			'targetState'	=>	"visible"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Collections
*/
	public function initCollection()
	{
		$this->resourceForm->setResourceType($this->vars['ajaxReturn']);
		if(array_key_exists('collection', $this->typeMaps[$this->vars['ajaxReturn']]['optional']))
			$visibility = 'visible';
		else
			$visibility = 'hidden';
		$div = $this->resourceForm->optionalCells('collection');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'collectionOuter',
			'targetState'	=>	"$visibility"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Fill in conference detail if a conference is selected in the conference select box
*/
	public function fillConference()
	{
		$this->resourceForm->setResourceType($this->vars['resourceType']);
		$this->resourceForm->conferenceFill = $this->vars['ajaxReturn'];
		$div = $this->resourceForm->optionalCells('conference');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'conferenceOuter',
			'targetState'	=>	"visible"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Conferences
*/
	public function initConference()
	{
		$this->resourceForm->setResourceType($this->vars['ajaxReturn']);
		if(array_key_exists('conference', $this->typeMaps[$this->vars['ajaxReturn']]['optional']))
			$visibility = 'visible';
		else
			$visibility = 'hidden';
		$div = $this->resourceForm->optionalCells('conference');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'conferenceOuter',
			'targetState'	=>	"$visibility"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Fill in translation detail if a translated book is selected in the translated book select box
*/
	public function fillTrans()
	{
		$this->resourceForm->setResourceType($this->vars['resourceType']);
		$this->resourceForm->translationFill = $this->vars['ajaxReturn'];
		$div = $this->resourceForm->optionalCells('translation');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'translationOuter',
			'targetState'	=>	"visible"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
** For book, book_article, book_chapter:  translated work details
*/
	public function initTranslation()
	{
		$this->resourceForm->setResourceType($this->vars['ajaxReturn']);
		if(array_key_exists('translation', $this->typeMaps[$this->vars['ajaxReturn']]['optional']))
			$visibility = 'visible';
		else
			$visibility = 'hidden';
		$div = $this->resourceForm->optionalCells('translation');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'translationOuter',
			'targetState'	=>	"$visibility"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
** Miscellaneous details
*/
	public function initMiscellaneous()
	{
		$this->resourceForm->setResourceType($this->vars['ajaxReturn']);
		if(array_key_exists('miscellaneous', $this->typeMaps[$this->vars['ajaxReturn']]['optional']))
			$visibility = 'visible';
		else
			$visibility = 'hidden';
		$div = $this->resourceForm->optionalCells('miscellaneous');
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			'next'	=>	'TRUE',
			'startFunction'	=>	'divVisibility',
			'targetDiv'	=>	'miscellaneousOuter',
			'targetState'	=>	"$visibility"
			);
		if(isset($php_errormsg))
		{
// NB E_STRICT in PHP5 gives warning about use of GLOBALS below.  E_STRICT cannot be controlled through WIKINDX
			$error = $php_errormsg;
			GLOBALS::buildOutputString($this->ajax->encode_jArray(array('ERROR' => $error)));
		}
		else
			GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* AJAX-based DIV content creator for subcategories
*/
	public function initSubcategories()
	{
		$this->category = FACTORY_CATEGORY::getInstance();
// if no ajaxReturn, quietly exit
		$div = $this->table->td($this->html->div('subcategory', "&nbsp;")); // default
		if(array_key_exists('ajaxReturn', $this->vars))
		{
			$this->subcategories = $this->category->grabSubAll(TRUE, FALSE, explode(',', $this->vars['ajaxReturn']));
			$this->grabPreviouslySelected('ajaxReturn2');
			if(is_array($this->subcategories))
				$div = $this->resourceForm->subcategoryBox($this->subcategories);
		}
		GLOBALS::buildOutputString($this->ajax->encode_jArray(array('innerHTML' => $div)));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Store previously selected options
*/
	private function grabPreviouslySelected($qsElement)
	{
		if(array_key_exists($qsElement, $this->vars))
		{
			$this->previousSelect = explode(',', $this->vars[$qsElement]);
			if(($index = array_search(0, $this->previousSelect)) !== FALSE)
				unset($this->previousSelect[$index]); // remove 'IGNORE' selected
		}
	}
/**
* RESOURCEFORM input validation -- NOT USED
*/
	public function validate2()
	{
		$error = FALSE;
		$jArray1 = $this->ajax->decode_jString($this->vars['ajaxReturn']);
		$jArray2 = $this->ajax->decode_jString($this->vars['ajaxReturn2']);
		if(($id = array_search('resourceType', $jArray1)) !== FALSE)
			$resourceType = $jArray2[$id];
		if((($id = array_search('resourceTitle', $jArray1)) !== FALSE) && (trim($jArray2[$id])))
			$resourceTitle = $jArray2[$id];
		else
			$error = TRUE;
		if($error)
		{
			$jScript = 'index.php?action=resource_RESOURCEFORMAJAX_CORE&method=validateError';
			$div = $this->errors->text('inputError', 'missing');
			$jsonResponseArray = array(
				'innerHTML' => "$div",
				'script' => "$jScript",
				'next'	=>	'TRUE',
				'startFunction'	=>	'validateError',
				'targetDiv'	=>	'validateOuter',
				);
		}
		else
		{
			$div = 'SUCCESS';
			$jScript = 'index.php?action=resource_RESOURCEWRITE_CORE';
			$jsonResponseArray = array(
				'innerHTML' => "$div",
				'script'	=>	"$jScript",
				'next'	=>	'TRUE',
				'startFunction'	=>	'writeResource',
				'targetDiv'	=>	'validateOuter',
			);
		}
		GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
/**
* Return an error code to sendback to AJAX when RESOURCEFORM fails to validate
*/
	public function validate()
	{
		$field = $this->vars['field'];
		$error = $this->vars['error'];
		$div = $this->errors->text('inputError', $error, '&nbsp;&nbsp;' . $field);
		$jsonResponseArray = array(
			'innerHTML' => "$div",
			);
		GLOBALS::buildOutputString($this->ajax->encode_jArray($jsonResponseArray));
		FACTORY_CLOSERAW::getInstance();
	}
}
?>
