<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/*****
* RESOURCEQUOTE class
*
* Deal with resource's quotes
*****/
class RESOURCEQUOTE
{
private $gatekeep;
private $db;
private $vars;
private $textqp;
private $session;
private $html;
private $messages;
private $errors;
private $success;
private $form;
private $table;
private $navigate;
private $badInput;
private $template;

// Constructor
	public function __construct()
	{
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->gatekeep->init();
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		include_once("core/modules/resource/TEXTQP.php");
		$this->textqp = new TEXTQP();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->navigate = FACTORY_NAVIGATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "quotes"));
		if(!array_key_exists('resourceId', $this->vars) || !$this->vars['resourceId'] || 
			!array_key_exists('method', $this->vars) || !$this->vars['method'])
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$function = $this->vars['method'];
		if(!method_exists($this, $function))
			$this->navigate->resource($this->vars['resourceId'], $this->errors->text("inputError", "invalid"));
	}
// display the editing form:
// Only the original posters can edit their comments
	public function quoteEdit()
	{
		$this->checkInput();
		$tinymce = FACTORY_LOADTINYMCE::getInstance();
		$pString = $this->form->formHeader('resource_RESOURCEQUOTE_CORE');
		$pString .= $this->form->hidden('method', 'edit');
		$pString .= $tinymce->loadMetadataTextarea(array('resourcequotetextText', 'resourcequotecommentComment'));
		$metadata = $this->textqp->editdisplay('quote');
		$metadata['hidden'] = $pString;
		$metadata['formfoot'] = $this->form->formEnd();
		$this->template->tpl->assign('metadata', $metadata);
		$this->session->delVar('resourceQuoteLock');
	}
// write to the database.
// if there is no 'quoteId' input, we are adding a new quote.  Otherwise, editing one.
	public function edit()
	{
		if($this->session->getVar('resourceQuoteLock'))
			$this->badInput->close($this->errors->text("done", "quote"));
		$this->checkInput();
		if(!$addEdit = $this->textqp->edit('quote'))
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$this->db->formatConditions(array('resourcetimestampId' => $this->vars['resourceId']));
		$this->db->update('resource_timestamp', array('resourcetimestampTimestamp' => $this->db->formatTimestamp()));
// lock reload
		$this->session->setVar('resourceQuoteLock', TRUE);
// send back to view this resource with success message
		$this->navigate->resource($this->vars['resourceId'], $this->success->text("quotes", " " . 
			$this->messages->text("misc", $addEdit) . " "));
	}
// Check we have appropriate input.  Page and comment are optional
	private function checkInput()
	{
		if(!array_key_exists('resourceId', $this->vars) || !$this->vars['resourceId'])
			$this->badInput->close($this->errors->text("inputError", "missing"));
	}
}
?>
