<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
* URLS class
*
*/
class URLS
{
private $db;
private $vars;
private $html;
private $badInput;
private $session;
private $errors;
private $messages;
private $success;
private $url;
private $form;
private $gatekeep;
private $resourceId;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->html = FACTORY_HTML::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->url = FACTORY_URL::getInstance();
		$this->resourceId = $this->vars['resourceId'];
	}
// add, edit, delete resource attachments
	public function init()
	{
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
		$this->gatekeep->init();
		if(!array_key_exists('resourceId', $this->vars) || !array_key_exists('function', $this->vars))
			$this->badInput->close($this->errors->text("inputError", "missing"));
		$function = $this->vars['function'];
		$this->resourceId = $this->vars['resourceId'];
		$this->{$function}();
	}
// Initial editing/adding form
	private function editInit()
	{
		$this->form = FACTORY_FORM::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->session->delVar('urlLock');
		GLOBALS::setOutputHeading($this->messages->text("heading", "url", $this->messages->text('misc', 'edit')));
		$this->db->formatConditions(array('resourcetextId' => $this->resourceId));
		$recordset = $this->db->select('resource_text', array('resourcetextUrls', 'resourcetextUrlText'));
		if($this->db->numRows($recordset)) // URLs exist for this resource
			GLOBALS::buildOutputString($this->urlEditForm($recordset));
		else // add a new URL
			GLOBALS::buildOutputString($this->urlAddForm());
	}
// add a URL
	private function add()
	{
		if($this->session->getVar('urlLock'))
			$this->badInput->close($this->errors->text("done", "urlAdd"));
		$this->storeUrl();
// Lock re-uploading
		$this->session->setVar('urlLock', TRUE);
// send back to view this resource with success message
		$navigate = FACTORY_NAVIGATE::getInstance();
		$navigate->resource($this->resourceId, $this->success->text("url", " " . 
			$this->messages->text("misc", 'added') . " "));
	}
// edit urls
	private function edit()
	{
// Get primary URL if multiple URLs
		$primary = array_key_exists('urlPrimary', $this->vars) ? $this->vars['urlPrimary'] : FALSE;
// find any URLs to edit and files to delete
		foreach($this->vars as $key => $var)
		{
			$split = explode('_', $key);
			if($split[0] == 'urlEditLink')
				$editLinks[$split[1]] = $var;
			if($split[0] == 'urlEditName')
				$editNames[$split[1]] = $var;
			else if($split[0] == 'urlDelete')
				$deletes[$split[1]] = $var;
		}
		$message = FALSE;
// Edit URLs
		if(isset($editLinks))
		{
			$this->db->formatConditions(array('resourcetextId' => $this->resourceId));
			$recordset = $this->db->select('resource_text', array('resourcetextUrls', 'resourcetextUrlText'));
			$row = $this->db->fetchRow($recordset);
			$links = $this->url->getUrls($row['resourcetextUrls']);
			$urlExists = FALSE;
			if($row['resourcetextUrlText'])
			{
				$names = $this->url->getUrls($row['resourcetextUrlText']);
				$urlExists = TRUE;
			}
			else
				$names = array();
			foreach($editLinks as $number => $link)
			{
				$links[$number] = $link;
				if(trim($editNames[$number]))
				{
					$urlExists = TRUE;
					$names[$number] = $editNames[$number];
				}
				else
					$names[$number] = FALSE;
			}
			$updateArray['resourcetextUrls'] = base64_encode(serialize($links));
			if($urlExists)
				$updateArray['resourcetextUrlText'] = base64_encode(serialize($names));
			$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
			$this->db->update('resource_text', $updateArray);
			$message = $this->success->text("url", " " . $this->messages->text("misc", 'edited') . " ");
		}
// set primary URL
		$this->setPrimaryUrl($primary);
		$this->storeUrl();
		$this->session->setVar('urlLock', TRUE);
		if(isset($deletes))
			$this->deleteConfirm($deletes, $message);
// send back to view this resource with success message (deleteConfirm breaks out before this)
		$navigate = FACTORY_NAVIGATE::getInstance();
		$navigate->resource($this->resourceId, $this->success->text("url", " " . 
			$this->messages->text("misc", 'edited') . " "));
	}
// confirm delete URLs
	private function deleteConfirm($deletes, $message)
	{
		$form = FACTORY_FORM::getInstance();
		GLOBALS::setOutputHeading($this->messages->text("heading", "url", $this->messages->text('misc', 'delete')));
		$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
		$recordSet = $this->db->select(array("resource_text"), array("resourcetextUrls", "resourcetextUrlText"));
		$row = $this->db->fetchRow($recordSet);
		$links = $this->url->getUrls($row['resourcetextUrls']);
		if($row['resourcetextUrlText'])
			$names = $this->url->getUrls($row['resourcetextUrlText']);
		else
			$names = array();
		$pString = $message;
		$pString .= $form->formHeader("urls_URLS_CORE");
		$pString .= $form->hidden('function', 'delete');
		$pString .= $form->hidden('resourceId', $this->resourceId);
		$pString .= $this->messages->text('resources', 'deleteConfirmUrl') . ':' . $this->html->br();
		foreach($deletes as $index => $var)
		{
			if(!empty($names) && $names[$index])
				$label = $names[$index];
			else
				$label = $links[$index];
			$pString .= $form->checkBox(FALSE, "urlDelete_$index", TRUE) . '&nbsp;' . $label . $this->html->br();
		}
		$pString .= $this->html->p('&nbsp;' . $this->html->br() . $form->formSubmit("Confirm"));
		$pString .= $form->formEnd();
		GLOBALS::buildOutputString($pString);
		FACTORY_CLOSE::getInstance();
	}
// delete URLs
	private function delete()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "url", $this->messages->text('misc', 'delete')));
		foreach($this->vars as $key => $var)
		{
			$split = explode('_', $key);
			if($split[0] == 'urlDelete')
				$deletes[] = $split[1];
		}
		$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
		$recordSet = $this->db->select(array("resource_text"), array("resourcetextUrls", "resourcetextUrlText"));
		$row = $this->db->fetchRow($recordSet);
		$links = $this->url->getUrls($row['resourcetextUrls']);
		if($row['resourcetextUrlText'])
			$names = $this->url->getUrls($row['resourcetextUrlText']);
		else
			$names = array();
		foreach($deletes as $index)
		{
			array_splice($links, $index, 1);
			if(!empty($names))
				array_splice($names, $index, 1);
		}
		$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
		$recordset = $this->db->select('resource_text', array('resourcetextAbstract', 'resourcetextNote'));
		$row = $this->db->fetchRow($recordset);
// If all fields are null, delete the row
		if(empty($links) && !$row['resourcetextAbstract'] && !$row['resourcetextNote'])
		{
			$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
			$this->db->delete('resource_text');
		}
		else if(empty($links))
		{
			$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
			$this->db->updateNull('resource_text', array('resourcetextUrls', 'resourcetextUrlText'));
		}
		else
		{
			$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
			$updateArray['resourcetextUrls'] = base64_encode(serialize($links));
			if(empty($names) || ((sizeof($names) == 1) && !$names[0])) // set to NULL
			{
				$this->db->updateNull('resource_text', 'resourcetextUrlText');
				$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
			}
			else
				$updateArray['resourcetextUrlText'] = base64_encode(serialize($names));
			$this->db->update('resource_text', $updateArray);
		}
// send back to view this resource with success message
		$navigate = FACTORY_NAVIGATE::getInstance();
		$navigate->resource($this->resourceId, $this->success->text("url", " " . 
			$this->messages->text("misc", 'deleted') . " "));
	}
// set primary URL
	private function setPrimaryUrl($primary)
	{
		if(!$primary) // already primary so do nothing
			return;
		$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
		$recordSet = $this->db->select(array("resource_text"), array("resourcetextUrls", "resourcetextUrlText"));
		$row = $this->db->fetchRow($recordSet);
		$links = $this->url->getUrls($row['resourcetextUrls']);
		if($row['resourcetextUrlText'])
			$names = $this->url->getUrls($row['resourcetextUrlText']);
		else
			$names = array();
		$link = $links[$primary];
		unset($links[$primary]);
		array_unshift($links, $link);
		$updateArray['resourcetextUrls'] = base64_encode(serialize($links));
		if(!empty($names))
		{
			$name = $names[$primary];
			unset($names[$primary]);
			array_unshift($names, $name);
			$updateArray['resourcetextUrlText'] = base64_encode(serialize($names));
		}
		$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
		$this->db->update('resource_text', $updateArray);
	}
// Store new URL
	private function storeUrl()
	{
		if(array_key_exists('url', $this->vars) && trim($this->vars['url']))
			$link = $this->vars['url'];
		else
			return;
		if(array_key_exists('name', $this->vars))
			$name = $this->vars['name'];
		else
			$name = FALSE;
		$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
		$recordSet = $this->db->select(array("resource_text"), array("resourcetextUrls", "resourcetextUrlText"));
		if($this->db->numRows($recordSet)) // URLs already exist for this resource so need to update rather than insert
		{
			$row = $this->db->fetchRow($recordSet);
			$links = $this->url->getUrls($row['resourcetextUrls']);
			if($row['resourcetextUrlText'])
				$names = $this->url->getUrls($row['resourcetextUrlText']);
			else
				$names = array();
			$links[] = $link;
			if(empty($names))
				$names[] = FALSE;
			$names[] = $name;
			$updateArray['resourcetextUrls'] = base64_encode(serialize($links));
			$updateArray['resourcetextUrlText'] = base64_encode(serialize($names));
			$this->db->formatConditions(array("resourcetextId" => $this->resourceId));
			$this->db->update('resource_text', $updateArray);
		}
		else	// insert
		{
			$links[] = $link;
			if($name) // else leave as default NULL
			{
				$names[] = $name;
				$fields[] = 'resourcetextUrlText';
				$values[] = base64_encode(serialize($names));
			}
			$fields[] = 'resourcetextId';
			$values[] = $this->resourceId;
			$fields[] = 'resourcetextUrls';
			$values[] = base64_encode(serialize($links));
			$this->db->insert('resource_text', $fields, $values);
		}
	}
// Form for editing urls
	private function urlEditForm($recordset)
	{
		$names = array();
		while($row = $this->db->fetchRow($recordset))
		{
			$links = $this->url->getUrls($row['resourcetextUrls']);
			if($row['resourcetextUrlText'])
				$names = $this->url->getUrls($row['resourcetextUrlText']);
		}
		$pString = $this->form->formHeader("urls_URLS_CORE");
		$pString .= $this->form->hidden('function', 'edit');
		$pString .= $this->form->hidden('resourceId', $this->resourceId);
// Form elements for adding another URL
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->messages->text("resources", "url") . ":&nbsp;" .  
			$this->form->textInput(FALSE, "url", FALSE, 70), FALSE, "left", "bottom");
		$pString .= $this->table->td($this->messages->text("resources", "urlLabel") . ":&nbsp;" . 
			$this->form->textInput(FALSE, "name", FALSE, 50), FALSE, "left", "bottom");
		$pString .= $this->table->trEnd();
		$numLinks = sizeof($links);
		$index = 0;
		foreach($links as $link)
		{
			$pString .= $this->table->tableEnd();
			$pString .= $this->html->p('&nbsp;');
			$pString .= $this->table->tdEnd() . $this->table->trEnd() . $this->table->trStart() . $this->table->tdStart();
			$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
			$pString .= $this->table->trStart();
			$td = $this->messages->text('resources', 'url') . ':&nbsp;' . $this->form->textInput(FALSE, 
				"urlEditLink_$index", $link, 70);
			if(!empty($names))
				$td .= '&nbsp;&nbsp;' . $this->messages->text('resources', 'urlLabel') . ':&nbsp;' . 
					$this->form->textInput(FALSE, "urlEditName_$index", $this->html->dbToFormTidy(array_shift($names)), 50);
			else
				$td .= '&nbsp;&nbsp;' . $this->messages->text('resources', 'urlLabel') . ':&nbsp;' . 
					$this->form->textInput(FALSE, "urlEditName_$index", FALSE, 50);
			$td .= '&nbsp;&nbsp;' . $this->messages->text('misc', 'delete') . ':&nbsp;' . $this->form->checkBox(FALSE, 
				"urlDelete_$index");
			if($numLinks > 1)
			{
				$td .= '&nbsp;&nbsp;' . $this->messages->text('resources', 'primaryUrl') . ':&nbsp;';
				if($index == 0)
					$td .= $this->form->radioButton(FALSE, 'urlPrimary', $index, TRUE);
				else
					$td .= $this->form->radioButton(FALSE, 'urlPrimary', $index);
			}
			++$index;
			$pString .= $this->table->td($td);
			$pString .= $this->table->trEnd();
		}
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->trEnd() . $this->table->tdEnd() . $this->table->trStart() . $this->table->tdStart();
		$pString .= $this->html->p('&nbsp;' . $this->html->br() . $this->form->formSubmit("Save"));
		$pString .= $this->form->formEnd();
		return $pString;
	}
// Form for adding a URL
	private function urlAddForm()
	{
		$pString = $this->form->formHeader("urls_URLS_CORE");
		$pString .= $this->form->hidden('function', 'add');
		$pString .= $this->form->hidden('resourceId', $this->resourceId);
		$pString .= $this->table->tableStart(FALSE, 0, 0, 0, "left");
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->messages->text("resources", "url") . ":&nbsp;" . 
			$this->form->textInput(FALSE, "url", FALSE, 70), FALSE, "left", "bottom");
		$pString .= $this->table->td($this->messages->text("resources", "urlLabel") . ":&nbsp;" . 
			$this->form->textInput(FALSE, "name", FALSE, 50), FALSE, "left", "bottom");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->table->trEnd() . $this->table->tdEnd() . $this->table->trStart() . $this->table->tdStart();
		$pString .= $this->html->p('&nbsp;' . $this->html->br() . $this->form->formSubmit("Save"));
		$pString .= $this->form->formEnd();
		return $pString;
	}
}
?>