<?php
/**********************************************************************************
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
if not, write to the 
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2011
sirfragalot@users.sourceforge.net
**********************************************************************************/
/**
*	REGISTER class.
*
*	Registration of users
*/
class REGISTER
{
private $db;
private $vars;
private $errors;
private $messages;
private $success;
private $session;
private $form;
private $html;
private $table;
private $user;
private $template;
private $gatekeep;
private $badInput;
private $config;

	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->success = FACTORY_SUCCESS::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->user = FACTORY_USER::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->badInput = FACTORY_BADINPUT::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
	}
// display options for user registration
	public function initRegister($error = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "register"));
		$pString = $error ? $this->html->p($error, "error", "center") : FALSE;
		$recordset = $this->db->select(array('config'), 'configRegistrationModerate');
		$registrationModerate = $this->db->fetchOne($recordset);
		if($registrationModerate == 'Y')
		{
			$pString .= $this->html->p($this->messages->text('config', 'registrationRequest1'));
			$pString .= $this->form->formHeader('usersgroups_REGISTER_CORE');
			$pString .= $this->form->hidden('method', 'registerRequest');
		}
		else
		{
			$pString .= $this->form->formHeader('usersgroups_REGISTER_CORE');
			$pString .= $this->form->hidden('method', 'registerUser');
		}
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "email"), "email", FALSE, 30, 255)
			 . " " . $this->html->span('*', 'required') . 
			 ($registrationModerate == 'N' ? $this->html->br() . $this->messages->text("hint", "registerEmail") : FALSE));
		if($registrationModerate == 'Y')
			$pString .= $this->table->td('&nbsp;');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
		if($registrationModerate == 'Y')
			$pString .= $this->table->td($this->html->br() . 
			$this->form->textareaInput($this->messages->text("config", "registrationRequest2"), "registerRequest", FALSE, 75, 15));
		if(isset($this->config->WIKINDX_CAPTCHA_PUBLICKEY) && $this->config->WIKINDX_CAPTCHA_PUBLICKEY && 
			isset($this->config->WIKINDX_CAPTCHA_PRIVATEKEY) && $this->config->WIKINDX_CAPTCHA_PRIVATEKEY)
		{
			$string = $registrationModerate == 'N' ? $this->html->p('&nbsp;') : FALSE;
			require_once('recaptchalib.php');
			$pString .= $this->table->td($string . $this->messages->text('config', 'captcha1') . $this->html->br() . 
				recaptcha_get_html($this->config->WIKINDX_CAPTCHA_PUBLICKEY));
		}
		else if($registrationModerate == 'Y')
			$pString .= $this->table->td('&nbsp;');
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit("Proceed"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Write registration details and send confirmation email
	public function register()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "register"));
// check we're not reloading
		if($this->session->getVar("register_lock"))
			$this->badInput->close($this->errors->text("done", "register"), $this, 'initRegister');
		if(!$email = trim($this->vars['email']))
			$this->badInput->close($this->errors->text('inputError', 'missing'), $this, 'initRegister');
		if(!$this->captchaCheck())
			$this->badInput->close($this->errors->text('inputError', 'captcha'), $this, "initRegister");
// time() should be unique enough
		$hashKey = md5(time());
		include_once("core/modules/email/EMAIL.php");
		$emailClass = new EMAIL();
		if(!$emailClass->register($hashKey, $email))
			$this->badInput->close($this->errors->text("inputError", "mail"), $this, 'initRegister');
// END email
// Now write details to WKX_user_register database
		$fields[] = 'userregisterHashKey';
		$values[] = $hashKey;
		$fields[] = 'userregisterEmail';
		$values[] = $email;
		$fields[] = 'userregisterTimestamp';
		$values[] = $this->db->formatTimestamp();
		$this->db->insert("user_register", $fields, $values);
// do some housekeeping by removing dead registrations older than 20 days
		$this->db->formatConditions($this->db->dateIntervalCondition(20) . $this->db->greater . 
			$this->db->formatFields('userregisterTimestamp'));
		$this->db->delete('user_register');
		$this->db->optimize('user_register');
// Lock to prevent re-registration
		$this->session->setVar("register_lock", TRUE);
		GLOBALS::buildOutputString($this->success->text("registerEmail"));
	}
// user has entered hashkey for confirmation
	public function registerConfirm($error = FALSE)
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "register"));
		if(!array_key_exists('hashKey', $this->vars))
			$this->badInput->close($this->errors->text('inputError', 'missing'));
// $this->vars['hashKey'] -> check it exists in user_register
		$this->db->formatConditions(array('userregisterHashkey' => $this->vars['hashKey']));
		$recordset = $this->db->select('user_register', array('userregisterId', 'userregisterEmail'));
		if(!$this->db->numRows($recordset))
			$this->badInput->close($this->errors->text('inputError', 'noHashKey'));
		$row = $this->db->fetchRow($recordset);
		$id = $row['userregisterId'];
		$email = $row['userregisterEmail'];
		$pString = $error ? $this->html->p($error, "error", "center") : FALSE;
		$pString .= $this->form->formHeader('usersgroups_REGISTER_CORE');
		$pString .= $this->form->hidden('method', 'registerUserAdd');
		$pString .= $this->form->hidden('id', $id);
		$pString .= $this->form->hidden('hashKey', $this->vars['hashKey']);
		$pString .= $this->form->hidden('email', $email);
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "username"), "username", FALSE, 20, 255)
			 . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->passwordInput($this->messages->text("user", "password"), "password", FALSE, 15, 255)
			 . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->passwordInput($this->messages->text("user", "passwordConfirm"), 
			"passwordConfirm", FALSE, 15, 255) . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "fullname"), "fullname", FALSE, 30, 255));
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		if(isset($this->config->WIKINDX_CAPTCHA_PUBLICKEY) && $this->config->WIKINDX_CAPTCHA_PUBLICKEY && 
			isset($this->config->WIKINDX_CAPTCHA_PRIVATEKEY) && $this->config->WIKINDX_CAPTCHA_PRIVATEKEY)
		{
			require_once('recaptchalib.php');
			$pString .= $this->html->p($this->messages->text('config', 'captcha1') . $this->html->br() . 
				recaptcha_get_html($this->config->WIKINDX_CAPTCHA_PUBLICKEY));
		}
		$pString .= $this->html->p($this->form->formSubmit("Add"), FALSE, "right");
		$pString .= $this->form->formEnd();
		GLOBALS::buildOutputString($pString);
	}
// Add a user (self registration)
	public function registerUserAdd()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "register"));
		if(!trim($this->vars['username']) || !trim($this->vars['password']) || 
			!trim($this->vars['passwordConfirm']) || !$this->vars['email'] || !$this->vars['hashKey'])
			$this->badInput->close($this->errors->text('inputError', 'missing'), $this, 'registerConfirm');
		if(trim($this->vars['password']) != trim($this->vars['passwordConfirm']))
			$this->badInput->close($this->errors->text('inputError', 'missing'), $this, 'registerConfirm');
		if(!$this->captchaCheck())
			$this->badInput->close($this->errors->text('inputError', 'captcha'), $this, "registerConfirm");
// NB - writeUser returns FALSE on success!
		if($this->user->writeUser(TRUE, 0))
			$this->badInput->close($this->errors->text('inputError', 'userExists'), $this, 'registerConfirm');
// remove id from user_register
		$this->db->formatConditions(array('userregisterId' => $this->vars['id']));
		$this->db->delete('user_register');
		$this->db->optimize('user_register');
// grab id and write initial session details to grant write access
		$this->db->formatConditions(array('usersUsername' => $this->vars['username']));
		$recordset = $this->db->select('users', 'usersId');
		$userId = $this->db->fetchOne($recordset);
		$this->session->setVar("setup_UserId", $userId);
		$this->session->setVar("setup_Write", TRUE);
// Write default preferences (TRUE == insert)
		$this->user->writePreferences($userId);
		include_once("core/modules/email/EMAIL.php");
		$emailClass = new EMAIL();
		if(!$emailClass->registerUserAdd())
			$this->badInput->close($this->errors->text("inputError", "mail"), $this, 'registerConfirm');
		GLOBALS::buildOutputString($this->success->text("user", " " . $this->messages->text("misc", "added") . " "));
	}
// Store request for registration
	public function registerRequest()
	{
		GLOBALS::setOutputHeading($this->messages->text("heading", "register"));
// check we're not reloading
		if($this->session->getVar("register_lock"))
			$this->badInput->close($this->errors->text("done", "register"), $this, 'initRegister');
		if((!$email = trim($this->vars['email'])) || !$request = trim($this->vars['registerRequest']))
			$this->badInput->close($this->errors->text('inputError', 'missing'), $this, 'initRegister');
		if(!$this->captchaCheck())
			$this->badInput->close($this->errors->text('inputError', 'captcha'), $this, "initRegister");
		include_once("core/modules/email/EMAIL.php");
		$emailClass = new EMAIL();
		if(!$emailClass->registerRequest($email))
			$this->badInput->close($this->errors->text("inputError", "mail"), $this, 'initRegister');
// Now write details to WKX_user_register database
		$fields[] = 'userregisterHashKey';
		$values[] = md5(time());
		$fields[] = 'userregisterEmail';
		$values[] = trim($this->html->removeNl($email));
		$fields[] = 'userregisterRequest';
		$values[] = trim($this->html->removeNl($request));
		$fields[] = 'userregisterTimestamp';
		$values[] = $this->db->formatTimestamp();
		$this->db->insert('user_register', $fields, $values);
// Lock to prevent re-registration
		$this->session->setVar("register_lock", TRUE);
		GLOBALS::buildOutputString($this->success->text("registerRequest"));
	}
// Check for CAPTCHA input
	private function captchaCheck()
	{
		if(isset($this->config->WIKINDX_CAPTCHA_PUBLICKEY) && $this->config->WIKINDX_CAPTCHA_PUBLICKEY && 
			isset($this->config->WIKINDX_CAPTCHA_PRIVATEKEY) && $this->config->WIKINDX_CAPTCHA_PRIVATEKEY)
		{
			require_once('recaptchalib.php');
			$resp = recaptcha_check_answer ($this->config->WIKINDX_CAPTCHA_PRIVATEKEY,
				$_SERVER["REMOTE_ADDR"], $_POST["recaptcha_challenge_field"], $_POST["recaptcha_response_field"]);
			if(!$resp->is_valid)
				return FALSE;
			else
				return TRUE;
		}
		else
			return TRUE;
	}
}
?>