<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* RESOURCECOMMON class
*
* Common functions for quotes and paraphrases
* 
* @version	1
*
*	@package wikindx4\core\resources
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class RESOURCECOMMON
{
/** object */
private $db;
/** object */
private $session;
/** object */
private $html;
/** object */
private $commonBib;
/** object */
private $commonBrowse;
/** object */
private $messages;
/** array */
private $highlightPatterns = array();
/** object */
private $errors;
/** object */
private $resourceMap;
/** object */
private $gatekeep; //!< there are some functions requiring write access
/** boolean */
public $withUnixTimestamp = FALSE;
/** array */
public $listFields;
/** array */
public $resourceFields;
/** array */
public $resourceMiscFields;
/** array */
public $textFields;
/** array */
public $yearFields;
/** array */
public $pageFields;
/** array */
public $summaryFields;
/** array */
public $timestampFields;
/** array */
public $unixTimestampFields;
/** array */
public $publisherFields;
/** array */
public $collectionFields;
/** array */
public $userFields;
/** array */
public $attachmentFields;
/** array */
public $categoryFields;
/** array */
public $keywordFields;
/** array */
public $musingFields;
/** array */
public $languageFields;
/** boolean */
public $groupByResourceId = TRUE;

/**
* RESOURCECOMMON
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->resourceMap = FACTORY_RESOURCEMAP::getInstance();
		include_once("core/browse/BROWSECOMMON.php");
		$this->commonBrowse = new BROWSECOMMON();
		$this->gatekeep = FACTORY_GATEKEEP::getInstance();
	}
/**
* Set up highlighting of text if required (after a search operation).
*
* Creates array of regexp expressions for search terms
*/
	public function setHighlightPatterns()
	{
		if($this->session->getVar('search_Highlight') && empty($this->highlightPatterns))
		{
			$searchTerms = explode(",", $this->session->getVar('search_Highlight'));
			foreach($searchTerms as $term)
//				$this->highlightPatterns[] = "/($term)(?!\S*\" \S*>)/i";
				$this->highlightPatterns[] = "/($term)(?=[^>]*(<|$))/i";
		}
	}
/**
* Do pattern highlighting after a SEARCH operation
*
* @param string $text
* @return string
*/
	public function doHighlight($text)
	{
		if($this->session->getVar('search_Highlight'))
		{
			if(empty($this->highlightPatterns))
				$this->setHighlightPatterns();
/**
* Temporarily replace any URL - works for just one URL in the output string.
*/
			$url = FALSE;
			if(preg_match("/(<a.*>)/Ui", $text, $match))
			{
				$url = preg_quote($match[1], '/');
				$text = preg_replace("/$url/", "OSBIB__URL__OSBIB", $text);
			}
// Recover any URL
			if($url)
				$text = str_replace("OSBIB__URL__OSBIB", $match[1], $text);
// This double replace stops stops the search term 'ass' being used to replace the 'ass' in 'class'
//			$text = preg_replace($this->highlightPatterns, "W!I!K!I!N!D!X!$1W!I!K!I!N!D!X!", $text);
//			$text = preg_replace("/W!I!K!I!N!D!X!(.*)W!I!K!I!N!D!X!/U", $this->html->span("$1", "highlight"), $text);
			$text = preg_replace($this->highlightPatterns, $this->html->span("$1", "highlight"), $text);
		}
		return $text;
	}
/**
* show citations about this resource
*
* @param int $resourceId
* @param boolean $countOnly Default is FALSE
*/
	public function showCitations($resourceId, $countOnly = FALSE)
	{
		$ids = array();
		$search = "\\[cite\\]$resourceId";
// Abstract and note
		$this->commonBrowse->userBibCondition('resourcetextId');
		$this->db->formatConditions($this->db->formatFields('resourcetextAbstract') . $this->db->like('%', $search, '%') . 
			$this->db->or . $this->db->formatFields('resourcetextNote') . $this->db->like('%', $search, '%'));
		$this->db->formatConditions(array('resourcetextId' => $resourceId), TRUE);
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_text', array(array('resourcetextId' => 'rId'))));
// quotes
		$this->commonBrowse->userBibCondition('resourcequoteResourceId');
		$this->db->formatConditions($this->db->formatFields('resourcequotetextText') . $this->db->like('%', $search, '%'));
		$this->db->formatConditions(array('resourcequoteResourceId' => $resourceId), TRUE);
		$this->db->leftJoin('resource_quote_text', 'resourcequotetextId', 'resourcequoteId');
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
			array(array('resourcequoteResourceId' => 'rId'))));
// quote comments
		$this->commonBrowse->userBibCondition('resourcequoteResourceId');
		$this->db->formatConditions($this->db->formatFields('resourcequotecommentComment') . $this->db->like('%', $search, '%'));
		$this->db->formatConditions(array('resourcequoteResourceId' => $resourceId), TRUE);
		$this->db->leftJoin('resource_quote_comment', 'resourcequotecommentQuoteId', 'resourcequoteId');
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_quote', 
			array(array('resourcequoteResourceId' => 'rId'))));
// paraphrases
		$this->commonBrowse->userBibCondition('resourceparaphraseResourceId');
		$this->db->formatConditions($this->db->formatFields('resourceparaphrasetextText') . $this->db->like('%', $search, '%'));
		$this->db->formatConditions(array('resourceparaphraseResourceId' => $resourceId), TRUE);
		$this->db->leftJoin('resource_paraphrase_text', 'resourceparaphrasetextId', 'resourceparaphraseId');
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
			array(array('resourceparaphraseResourceId' => 'rId'))));
// paraphrase comments
		$this->commonBrowse->userBibCondition('resourceparaphraseResourceId');
		$this->db->formatConditions($this->db->formatFields('resourceparaphrasecommentComment') . $this->db->like('%', $search, '%'));
		$this->db->formatConditions(array('resourceparaphraseResourceId' => $resourceId), TRUE);
		$this->db->leftJoin('resource_paraphrase_comment', 'resourceparaphrasecommentParaphraseId', 'resourceparaphraseId');
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_paraphrase', 
			array(array('resourceparaphraseResourceId' => 'rId'))));
// musings
		$this->commonBrowse->userBibCondition('resourcemusingResourceId');
		$this->db->formatConditions($this->db->formatFields('resourcemusingtextText') . $this->db->like('%', $search, '%'));
		$this->db->formatConditions(array('resourcemusingResourceId' => $resourceId), TRUE);
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$unions[] = $this->db->queryNoExecute($this->db->selectNoExecute('resource_musing', 
			array(array('resourcemusingResourceId' => 'rId'))));
		if($countOnly)
		{
			$resultSet = $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 'rId', 
				$this->db->subQuery($this->db->union($unions), 'u')));
			if($this->db->numRows($resultSet))
				return $this->html->a('link', $this->messages->text("resources", "citedResources", ' ' . 
					$this->html->dbToHtmlTidy($this->commonBib->displayBib())), 
					'index.php?action=list_LISTSOMERESOURCES_CORE&method=citeProcess&id=' . $resourceId);
			else
				return FALSE;
		}
		else
			return $this->db->subQuery($this->db->union($unions), 'u');
	}
/** 
* Insert new resource into database.
*
 * @todo Needs code-review!  Is this even where this function should be?
 * @param array   $v    POST/GET variables needed to create resource (the actual required and optional keys should be specified)
 * @param boolean $alreadyExisted Indicates if the resource already exists. Reference, default is FALSE
 * @param array	$errs Array to push any errors that occur. Reference, default is empty array
 * @return      returns the id of the (new or already existing) resource or returns FALSE on failure.
 * @return mixed FALSE|Resource ID
 */
	public function insert( $v, &$alreadyExisted=FALSE, &$errs=array() )
	{
		$this->gatekeep->init();

		if( trim($v['type'] )) {
			$type = $v['type'];
		} else {
			array_push($errs, $this->error->text('inputError','missing','type'));
		}

		$id = false;
		$tables = $this->resourceMap->getTables($type); // resource is always first value

		foreach( $tables as $tableName ) 
		{
			$fieldPrefix = str_replace('_','',$tableName);

			$fields=array();
			if( $tableName == "resource" ) {
				$fields[ $fieldPrefix . 'Type' ] = $type;
			}
			if( $id !== false ) {
				$fields[ $fieldPrefix . 'Id' ] = $id;
			}

			$keys = $this->db->listFields($tableName);
			foreach( $keys as $key ) {
				$base = substr($key,strlen($fieldPrefix));
				$base[0] = strtolower($base[0]);
				//echo "<p>Looking for $base in \$v.</p>\n";

				if( array_key_exists($base,$v) && trim( $v[$base] )) {
					$fields[ $key ] = $v[$base];
				}
			}
			//print_r($fields);

			$msgKeys = $this->resourceMap->getMsgKeys($type);
			foreach( $msgKeys as $msgKey ) {
				if( array_key_exists($msgKey,$v) && trim( $v[$msgKey] )) {
					addField( $fields, $fieldPrefix, 
						$this->resourceMap->lookupDBfield($type,$msgKey), $v[$msgKey] );
				}
			}
			// don't add table if there is no information or if there are errors
			if( !empty($fields) && empty($errs)) {
				$this->db->insert($tableName,array_keys($fields), array_values($fields));

				// @todo I currently assume that db->insert doesn't return any errors!!!
// @mark -- insert, delete, select, update use SQL::query() which places db error messages in the public SQL->error string variable
				if( $id === FALSE && $tableName == 'resource') {
					$id = $this->db->lastAutoId('resource');
				} else if( $id == FALSE ) {
					// @todo need to push the error onto $errors before returning FALSE
					return FALSE;
				}
			}
		}
		return $id;
	}
/**
* Format field name
*
* @param string $prefix
* @param string $base
* @return string
*/
	private function fieldName( $prefix, $base )
	{
		$base[0] = strtoupper($base[0]);
		return $prefix . $base;
	}
/**
* Add field to fields array
*
* @param array $fields Reference
* @param string $prefix
* @param string $name
* @param string $value
*/
	private function addField( &$fields, $prefix, $name, $value )
	{
		$fields[ fieldName($fieldPrefix,$name) ] = $value;
	}
/**
* Return resultset for one or more resources.
*
* NB -- resource ID conditions can be set elsewhere
* $orderX is for footnote ordering in CITESTYLE/CITEFORMAT or $order1 is for the ordering of CMS output.
*
* @param array $ids Resource IDs, default is FALSE
* @param string $order1 Default is FALSE
* @param string $order2 Default is FALSE
* @param string $order3 Default is FALSE
* @param string $subQuery Can come from elsewhere such as core/cms/CMS.php. Default is FALSE
* @param boolean $sqlOnly Don't execute, just return SQL statement. Default is FALSE
* @return mixed SQL statement|SQL resource object
*/
	public function getResource($ids = FALSE, $order1 = FALSE, $order2 = FALSE, $order3 = FALSE, $subQuery = FALSE, $sqlOnly = FALSE)
	{
		$this->tableSetup();
		if(is_array($ids))
			$this->db->formatConditionsOneField($ids, 'resourceId');
		else if($ids)
			$this->db->formatConditions(array('resourceId' => $ids));
		if(!$subQuery)
		{
			$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
			if($this->withUnixTimestamp)
				$subQuery = $this->db->subQuery($this->db->queryNoExecute($this->db->selectNoExecuteWithExceptions('resource', 
					array_merge($this->resourceFields, $this->unixTimestampFields, 
					array(array('resourcecreatorCreatorSurname' => 'creatorSurname'), 'resourcecreatorResourceId')), TRUE)), 't1');
			else
				$subQuery = $this->db->subQuery($this->db->queryNoExecute($this->db->selectNoExecute('resource', 
					array_merge($this->resourceFields, 
					array(array('resourcecreatorCreatorSurname' => 'creatorSurname'), 'resourcecreatorResourceId')), TRUE)), 't1');
		}
		$this->db->leftJoin('resource_text', "resourcetextId", 'resourceId');
		$this->db->leftJoin('resource_misc', "resourcemiscId", 'resourceId');
		$this->db->leftJoin('resource_year', "resourceyearId", 'resourceId');
		$this->db->leftJoin('resource_page', "resourcepageId", 'resourceId');
		$this->db->leftJoin('resource_timestamp', "resourcetimestampId", 'resourceId');
		$this->db->leftJoin('resource_summary', 'resourcesummaryId', 'resourceId');
		$this->db->leftJoin('publisher', "publisherId", 'resourcemiscPublisher');
		$this->db->leftJoin('collection', "collectionId", 'resourcemiscCollection');
		$this->db->leftJoin('users', $this->db->formatFields('usersId'), $this->db->caseWhen('resourcemiscEditUserIdResource', 'IS NOT NULL', 
			'resourcemiscEditUserIdResource', 'resourcemiscAddUserIdResource'), FALSE);
		if($order1)
			$this->db->orderBy($order1, FALSE);
		if($order2)
			$this->db->orderBy($order2, FALSE);
		if($order3)
			$this->db->orderBy($order3, FALSE);
//		if($this->groupByResourceId)
//			$this->db->groupBy('resourceId');
		if($this->withUnixTimestamp)
		{
			if($sqlOnly)
			{
				$fields = array_merge($this->resourceFields, $this->pageFields, 
					$this->yearFields, $this->resourceMiscFields, $this->summaryFields, 
					$this->textFields, $this->timestampFields, $this->unixTimestampFields, $this->userFields, 
					$this->collectionFields, $this->publisherFields);
//				if($this->groupByResourceId)
//					$this->db->group .= ', ' . $this->db->formatFields($fields);
				return $this->db->selectNoExecuteFromSubQuery(FALSE, 
					$this->db->formatFields($fields, TRUE), $subQuery, FALSE, FALSE);
			}
			$fields = array_merge($this->resourceFields, $this->pageFields, 
				$this->yearFields, $this->resourceMiscFields, $this->summaryFields, 
				$this->textFields, $this->timestampFields, $this->unixTimestampFields, $this->userFields, 
				$this->collectionFields, $this->publisherFields);
//			if($this->groupByResourceId)
//				$this->db->group .= ', ' . $this->db->formatFields($fields);
			return $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 
				$this->db->formatFields($fields, TRUE), $subQuery, FALSE, FALSE));
		}
		else
		{
			if($sqlOnly)
			{
				$fields = array_merge($this->resourceFields, $this->pageFields, 
					$this->yearFields, $this->resourceMiscFields, $this->summaryFields, 
					$this->textFields, $this->timestampFields, $this->userFields, 
					$this->collectionFields, $this->publisherFields);
//				if($this->groupByResourceId)
//					$this->db->group .= ', ' . $this->db->formatFields($fields);
				return $this->db->selectNoExecuteFromSubQuery(FALSE, 
					$this->db->formatFields($fields), $subQuery, FALSE, FALSE);
			}
			$fields = array_merge($this->resourceFields, $this->pageFields, 
				$this->yearFields, $this->resourceMiscFields, $this->summaryFields, 
				$this->textFields, $this->timestampFields, $this->userFields, 
				$this->collectionFields, $this->publisherFields);
//			if($this->groupByResourceId)
//				$this->db->group .= ', ' . $this->db->formatFields($fields);
			return $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 
				$this->db->formatFields($fields), $subQuery, FALSE, FALSE));
		}
	}
/**
* set DB stuff including list operation fields
*/
	public function tableSetup()
	{
// table fields used when listing resources
		$this->listFields = array('resourceId', 'creatorSurname', 'resourceType', 'resourceTitle', 'resourceSubtitle', 'resourceShortTitle', 'resourceTransTitle', 'resourceTransSubtitle', 'resourceTransShortTitle', 'resourceField1', 'resourceField2', 'resourceField3', 'resourceField4', 'resourceField5', 'resourceField6', 'resourceField7', 'resourceField8', 'resourceField9', 'resourceNoSort', 'resourceTransNoSort', 'resourceIsbn', 'resourceBibtexKey', 'resourceDoi', 
		'resourcetextId', 'resourcetextNote', 'resourcetextAbstract', 'resourcetextUrls', 'resourcetextUrlText', 'resourcetextEditUserIdNote', 'resourcetextAddUserIdNote', 'resourcetextEditUserIdAbstract', 'resourcetextAddUserIdAbstract', 'resourceyearId', 'resourceyearYear1', 'resourceyearYear2', 'resourceyearYear3', 'resourceyearYear4', 'resourcepageId', 'resourcepagePageStart', 'resourcepagePageEnd', 'resourcesummaryId', 'resourcesummaryQuotes', 'resourcesummaryParaphrases', 'resourcesummaryMusings', 'resourcetimestampId', 'resourcetimestampTimestamp', 'resourcetimestampTimestampAdd', 'publisherId', 'publisherName', 'publisherLocation', 'publisherType', 'collectionId', 'collectionTitle', 'collectionTitleShort', 'collectionType', 'usersId', 'usersUsername', 'usersFullname', 'resourcemiscId', 'resourcemiscCollection', 'resourcemiscPublisher', 'resourcemiscField1', 'resourcemiscField2', 'resourcemiscField3', 'resourcemiscField4', 'resourcemiscField5', 'resourcemiscField6', 'resourcemiscTag', 'resourcemiscAddUserIdResource', 'resourcemiscEditUserIdResource', 'resourcemiscAccesses', 'resourcemiscMaturityIndex', 'resourcemiscPeerReviewed', 'resourcemiscQuarantine', 'resourcemusingtextPrivate', 'resourcemusingtextAddUserIdMusing', 'resourcemiscAccessesPeriod');
		
		$this->resourceFields = array('resourceId', 'resourceType', 'resourceTitle', 'resourceSubtitle', 'resourceShortTitle', 'resourceTitleSort', 'resourceTransTitle', 'resourceTransSubtitle', 'resourceTransShortTitle', 'resourceField1', 'resourceField2', 'resourceField3', 'resourceField4', 'resourceField5', 'resourceField6', 'resourceField7', 'resourceField8', 'resourceField9', 'resourceNoSort', 'resourceTransNoSort', 'resourceIsbn', 'resourceBibtexKey', 'resourceDoi');
		
		$this->resourceMiscFields = array('resourcemiscId', 'resourcemiscCollection', 'resourcemiscPublisher', 'resourcemiscField1', 'resourcemiscField2', 'resourcemiscField3', 'resourcemiscField4', 'resourcemiscField5', 'resourcemiscField6', 'resourcemiscTag', 'resourcemiscAddUserIdResource', 'resourcemiscEditUserIdResource', 'resourcemiscAccesses', 'resourcemiscMaturityIndex', 'resourcemiscPeerReviewed', 'resourcemiscQuarantine', 'resourcemiscAccessesPeriod');
		
		$this->textFields = array('resourcetextId', 'resourcetextNote', 'resourcetextAbstract', 'resourcetextUrls', 'resourcetextUrlText', 'resourcetextEditUserIdNote', 'resourcetextAddUserIdNote', 'resourcetextEditUserIdAbstract', 'resourcetextAddUserIdAbstract');
		
		$this->yearFields = array('resourceyearId', 'resourceyearYear1', 'resourceyearYear2', 'resourceyearYear3', 'resourceyearYear4');
		
		$this->pageFields = array('resourcepageId', 'resourcepagePageStart', 'resourcepagePageEnd');
		
		$this->summaryFields = array('resourcesummaryId', 'resourcesummaryQuotes', 'resourcesummaryParaphrases', 'resourcesummaryMusings');
		
		$this->timestampFields = array('resourcetimestampId', 'resourcetimestampTimestamp', 'resourcetimestampTimestampAdd');
			
		$this->unixTimestampFields = array(array("UNIX_TIMESTAMP(resourcetimestampTimestampAdd)" => 'addUnixTimestamp'), 
			array("UNIX_TIMESTAMP(resourcetimestampTimestamp)" => 'editUnixTimestamp'));
		
		$this->publisherFields = array('publisherId', 'publisherName', 'publisherLocation', 'publisherType');
		
		$this->collectionFields = array('collectionId', 'collectionTitle', 'collectionTitleShort', 'collectionType');
		
		$this->userFields = array('usersId', 'usersUsername', 'usersFullname');
		
		$this->attachmentFields = array('resourceattachmentsId', 'resourceattachmentsDownloads', 'resourceattachmentsTimestamp', 
			'resourceattachmentsResourceId', 'resourceattachmentsFileName', 'resourceattachmentsHashFilename');
		
		$this->categoryFields = array('resourcecategoryId', 'resourcecategoryCategories', 'resourcecategorySubcategories');
		
		$this->keywordFields = array('resourcekeywordId', 'resourcekeywordResourceId', 'resourcekeywordKeywordId');
		
		$this->languageFields = array('resourcelanguageId', 'resourcelanguageResourceId', 'resourcelanguageLanguageId');
		
		$this->musingFields = array('resourcemusingtextPrivate', 'resourcemusingtextAddUserIdMusing');
	}
}
?>
