<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* SMTP
*
* Common smtp/email functions
* 
* @version	1
*
*	@package wikindx4\core\smtp
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class SMTP
{
/** object */
private $messages;
/** object */
private $db;
/** object */
private $config;
/** object */
private $session;
/** string */
private $headers = '';

/**
* SMTP
*/
	public function __construct()
	{
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->db = FACTORY_DB::getInstance();
		if($this->config->WIKINDX_PEARMAIL_USE)
		{
			include_once('Mail.php');
			if($this->config->WIKINDX_PEARMAIL_BACKEND == 'smtp')
			{
				$SMTP = array(
						 'host'     => $this->config->WIKINDX_PEARMAIL_SMTPSERVER,
						 'port'     => $this->config->WIKINDX_PEARMAIL_SMTPPORT,
						 'auth'     => $this->config->WIKINDX_PEARMAIL_SMTPAUTH,
						 'persist'     => $this->config->WIKINDX_PEARMAIL_SMTPPERSIST,
					);
				if($this->config->WIKINDX_PEARMAIL_SMTPAUTH)
				{
					$SMTP['username'] = $this->config->WIKINDX_PEARMAIL_SMTPUSERNAME;
					$SMTP['password'] = $this->config->WIKINDX_PEARMAIL_SMTPPASSWORD;
				}
			}
			else if($this->config->WIKINDX_PEARMAIL_BACKEND == 'sendmail')
			{
				$SMTP = array(
						 'sendmail_path'     => $this->config->WIKINDX_PEARMAIL_SMPATH,
						 'sendmail_args'   => $this->config->WIKINDX_PEARMAIL_SMARGS,
					);
			}
			$this->mail = Mail::factory($this->config->WIKINDX_PEARMAIL_BACKEND, $SMTP);
		}
	}
/**
* Create email header
*/
	private function header()
	{
		if($this->config->WIKINDX_MAIL_FROM)
			$this->headers['From'] = $this->config->WIKINDX_MAIL_FROM;
		else
		{
			$recordset = $this->db->select("config", "configTitle");
			$this->title = $this->db->fetchOne($recordset);
			if(!$this->title)
				$this->title = "WIKINDX";
			$this->headers['From'] = $this->title . '@' . $_SERVER['HTTP_HOST'];
		}
		$this->headers['Content-type'] = 'text/plain; charset=UTF-8';
		if($this->config->WIKINDX_MAIL_REPLYTO)
			$this->headers['Reply-to'] = $this->config->WIKINDX_MAIL_REPLYTO;
		else
			$this->headers['Reply-to'] = 'No-Reply';
		if($this->config->WIKINDX_MAIL_RETURN_PATH)
			$this->headers['Return-path'] = $this->config->WIKINDX_MAIL_RETURN_PATH;
	}
/**
* Send the email
*
* @param mixed $addresses Either array of addresses or a single address (which might be ,|; delimited string of addresses)
* @param string $subject the email subject
* @param string $message the email message
* @return boolean
*/
	public function sendEmail($addresses, $subject, $message)
	{
		if(!$this->config->WIKINDX_MAIL_SERVER) // Email subsystem turned off
			return TRUE;
		if(!is_array($addresses))
		{
			$array = preg_split('/,|;/', $addresses);
			$addresses = array();
			foreach($array as $address)
				$addresses[] = trim($address);
		}
		$this->header();
		if(!$this->expedite($addresses, $subject, $message))
			return FALSE;
		return TRUE;
	}
/**
* Actually send mail using either mail() or PEAR::Mail
*
* @param array $addresses
* @param string $subject the email subject
* @param string $message the email message
* @return boolean
*/
	private function expedite($addresses, $subject, $message)
	{
		if($this->config->WIKINDX_PEARMAIL_USE)
		{
			$this->headers['Subject'] = $subject;
			$this->headers['To'] = 'recipients@' . $_SERVER['HTTP_HOST'];
			$send = $this->mail->send($addresses, $this->headers, $message);
			if(PEAR::isError($send))
			{
				print 'PEAR::Mail error: ' . $send->getMessage();
				return FALSE;
			}
		}
		else // PHP's mail()
		{
			$headers = '';
			foreach($this->headers as $key => $value)
				$headers .= "$key: $value\r\n";
			foreach($addresses as $address)
			{
				$headers .= "To: $address\r\n";
				if(!mail($address, $subject, $message, $headers))
					return FALSE;
			}
		}
		return TRUE;
	}
/**
* Get scriptPath
*
* @return string
*/
	public function scriptPath()
	{
		return 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['SCRIPT_NAME'];
	}
}
?>