<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/**
* SQL
*
* SQL abstraction layer for using MySQL.
* This class is provided where the PHP version is problematic with ADOdb and its methods emulate those found in 
* ADODBSQL.php.
* NB As of v3.1.1, adoDB is used solely to upgrade the database structure.
* 
* @version	1
*
*	@package wikindx4\core\sql
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class SQL
{
/** object */
private $errors;
/** string */
public $error = FALSE;
/** object */
public $handle = NULL;
// NB $dbaseType hard-wired to 'mysql' until other databases accounted for
/** string */
public $dbaseType = 'mysql';
/** string */
public $escapeString = FALSE;
/** boolean */
public $unbuffered = FALSE; // default buffered queries allow use of mysql_num_rows etc. but helps reduce memory
/** object */
private $config;
/** object */
private $utf8;
/** array */
private $vars;
/** array */
public $restoreArray = array();
/** string */
public $conditionSeparator;
/** string */
public $multiConditionSeparator;
/** string */
public $or;
/** string */
public $and;
/** string */
public $ascDesc;
/** string */
public $asc;
/** string */
public $desc;
/** string */
public $equal;
/** string */
public $notEqual;
/** string */
public $greater;
/** string */
public $less;
/** string */
public $plus;
/** string */
public $minus;
/** string */
public $alias;
/** string */
public $from;
/** array */
public $condition = array();
/** array */
public $join = array();
/** array */
public $order = array();
/** string */
public $group = FALSE;
/** string */
public $limit = FALSE;
/** boolean */
public $reset = TRUE;
/** string */
public $sqlstring = FALSE;
/** array */
private $joinUpdate = array();

/**
* SQL
*/
	public function __construct()
	{
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->config = FACTORY_CONFIG::getInstance();
		$this->utf8 = FACTORY_UTF8::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		if($this->dbaseType == 'mysql')
		{
			if(!$this->openMySQL())
				die($this->errors->text("dbError", "open"));
		}
		if($this->dbaseType == 'mysql')
		{
			$this->or = ' OR ';
			$this->and = ' AND ';
			$this->asc = ' ASC';
			$this->desc = ' DESC';
			$this->equal = ' = ';
			$this->notEqual = ' != ';
			$this->greater = ' > ';
			$this->less = ' < ';
			$this->plus = ' + ';
			$this->minus = ' - ';
			$this->alias = ' AS ';
			$this->from = ' FROM ';
		}
		$this->conditionSeparator = $this->multiConditionSeparator = $this->and;
		$this->ascDesc = $this->asc;
	}
/**
* Create SQL tables
*
* @author Mark Grimshaw
* @version 1
* @param string $xmlFile XML file (used for extra tables used in plug-in modules). Default is FALSE
*/
	public function createSQL($xmlFile = FALSE)
	{
// Close database handle opened by PHP as simultaneous database connections can be problematic with database persistence.
		mysqli_close($this->handle);
		include_once("/usr/share/php/adodb/adodb.inc.php");
		include_once("/usr/share/php/adodb/adodb-xmlschema.inc.php");
		$db =& NewADOConnection($this->dbaseType);
		$db->Connect($this->config->WIKINDX_DB_HOST, $this->config->WIKINDX_DB_USER, 
			$this->config->WIKINDX_DB_PASSWORD, $this->config->WIKINDX_DB);
//$db->debug = true; 
		if($xmlFile)
			$schemaFile = $xmlFile;
		else
			$schemaFile = 'sqlschema/createSQL.xml';
		$schema = new adoSchema($db);
		$schema->ContinueOnError(TRUE);
		$schema->setPrefix($this->config->WIKINDX_DB_TABLEPREFIX);
		$sql = $schema->ParseSchema($schemaFile);
		$result = $schema->ExecuteSchema();
		$db->Close();
// Re-open database with PHP methods.
		if($this->dbaseType == 'mysql')
		{
			if(!$this->openMySQL())
				die($this->errors->text("dbError", "open") . ' <P>--> (' . mysqli_error($this->handle) . ')');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_creator` ADD INDEX(`resourcecreatorCreatorSurname`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_creator` ADD INDEX(`resourcecreatorCreatorId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_creator` ADD INDEX(`resourcecreatorResourceId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_keyword` ADD INDEX(`resourcekeywordResourceId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_keyword` ADD INDEX(`resourcekeywordKeywordId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_category` ADD INDEX(`resourcecategoryCategoryId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_category` ADD INDEX(`resourcecategoryResourceId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_year` ADD INDEX(`resourceyearYear1`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'publisher` ADD INDEX(`publisherName`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource` ADD INDEX(`resourceType`)');
// removed because resourceTitleSort is now type TEXT which cannot have an index in MySQL
//			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
//				'resource` ADD INDEX(`resourceTitleSort`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_timestamp` ADD INDEX(`resourcetimestampTimestampAdd`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_timestamp` ADD INDEX(`resourcetimestampTimestamp`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'collection` ADD INDEX(`collectionTitle`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'creator` ADD INDEX(`creatorSurname`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'creator` ADD INDEX(`creatorSameAs`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'keyword` ADD INDEX(`keywordKeyword`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'category` ADD INDEX(`categoryCategory`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'user_bibliography` ADD INDEX(`userbibliographyTitle`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'user_bibliography_resource` ADD INDEX(`userbibliographyresourceResourceId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_attachments` ADD INDEX(`resourceattachmentsResourceId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_misc` ADD INDEX(`resourcemiscCollection`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_misc` ADD INDEX(`resourcemiscPublisher`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_custom` ADD INDEX(`resourcecustomCustomId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'statistics` ADD INDEX(`statisticsResourceId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'statistics` ADD INDEX(`statisticsAttachmentId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_language` ADD INDEX(`resourcelanguageresourceId`)');
			$this->query('ALTER TABLE `' . $this->config->WIKINDX_DB_TABLEPREFIX . 
				'resource_language` ADD INDEX(`resourcelanguageLanguageId`)');
		}
	}
/**
* Close SQL database
*
* @author Mark Grimshaw
* @version 1
*/
	public function closeMySQL()
	{
		mysqli_close($this->handle);
	}
/**
* Open SQL database
*
* @author Mark Grimshaw
* @version 1
* @return boolean
*/
	private function openMySQL()
	{
		$host = $this->config->WIKINDX_DB_PERSISTENT === TRUE ? 'p:' . $this->config->WIKINDX_DB_HOST : $this->config->WIKINDX_DB_HOST;
		if(!$this->handle = mysqli_connect($host, $this->config->WIKINDX_DB_USER, $this->config->WIKINDX_DB_PASSWORD, $this->config->WIKINDX_DB))
			return FALSE;
// v4.1 and up offer UTF8 support and v5.0 offers stored functions -- we require at least v4
		$mysqlVersion = mysqli_get_server_info($this->handle);
		$split = explode('\.', $mysqlVersion);
		if(($split[0] < 4) && ($split[1] < 1))
			die("In order to support UTF-8 character sets, WIKINDX requires MySQL 4.1.1 or greater.  
				Your MySQL version is {$mysqlVersion}. Please upgrade MySQL or use WIKINDX v3.8.2 which 
				supports MySQL v3.23 and above.");
// Set for UTF8
		$this->query('SET NAMES utf8');
		$this->query('SET CHARACTER SET utf8');
// OR
/* $db_charset = mysql_query( "SHOW VARIABLES LIKE 'character_set_database'" );
$charset_row = mysql_fetch_assoc( $db_charset );
print_r($charset_row);
mysql_query( "SET NAMES '" . $charset_row['Value'] . "'" );
unset( $db_charset, $charset_row );
*/
// Avoid grouping errors such as:
// (Mixing of GROUP columns (MIN(),MAX(),COUNT(),...) with no GROUP columns is illegal if there is no GROUP BY clause)
//		$this->query("SET sql_mode='TRADITIONAL'");
// The reverse of this is:
		$this->query( "SET sql_mode='ONLY_FULL_GROUP_BY'");
		return TRUE;
	}
/**
* execute queries and return recordset
*
* @author Mark Grimshaw
* @version 1
* @param string $querystring
* @param boolean $saveSession Default is FALSE
* @return object
*/
	public function query($querystring, $saveSession = FALSE)
	{
		$querystring .= $this->subClause();
//print $querystring . '<p>';
		if($this->config->WIKINDX_PRINT_SQL)
			print "<hr><strong>[EXEC]</strong> " . $this->beautify($querystring);
		$this->error .= $querystring;
		if($this->dbaseType == 'mysql')
		{
			if($this->unbuffered)
			{
				if(!$recordset = mysql_unbuffered_query($querystring, $this->handle))
					die($this->error . ' <P>--> (' . mysql_error($this->handle) . ')');
			}
			else
			{
				if(!$recordset = mysqli_query($this->handle, $querystring))
					die($this->error . ' <P>--> (' . mysqli_error($this->handle) . ')');
			}
		}
		GLOBALS::incrementDbQueries();
		$this->restore();
		if($this->reset)
			$this->resetSubs();
		if($saveSession)
			$this->session->setVar('sql_Stmt', base64_encode($querystring));
		$this->sqlstring = $querystring;
		return $recordset;
	}
/**
* create the entire querystring but do not execute
*
* @author Mark Grimshaw
* @version 1
* @param string $querystring
* @param boolean $saveSession Default is FALSE
* @return string
*/
	public function queryNoExecute($querystring, $saveSession = FALSE)
	{
		$querystring .= $this->subClause();
		if($this->config->WIKINDX_PRINT_SQL)
			print "<hr><strong>[NOEXEC]</strong> " . $this->beautify($querystring);
		$this->restore();
		if($this->reset)
			$this->resetSubs();
		if($saveSession)
			$this->session->setVar('sql_Stmt', base64_encode($querystring));
		return $querystring;
	}
/**
* execute queries and return recordset
*
* Ignore error warnings (used with SYSTEMCHECK.php)
*
* @author Mark Grimshaw
* @version 1
* @param string query
* @return object
*/
	public function queryNoError($querystring)
	{
		$querystring .= $this->subClause();
		if($this->dbaseType == 'mysql')
			$recordset = mysqli_query($this->handle, $querystring);
		GLOBALS::incrementDbQueries();
		$this->restore();
		if($this->reset)
			$this->resetSubs();
		return $recordset;
	}
/**
* Free mysqli result set
*
* @param object $resultset
*/
	public function freeSet($resultset)
	{
		mysqli_free_result($resultset);
	}
/**
* Formulate subclause after main query
*
* @return string
*/
	private function subClause()
	{
		$clause = '';
		if($this->dbaseType == 'mysql')
		{
			if(!empty($this->join))
				$clause .= ' ' . join(' ', $this->join);
			if(!empty($this->condition))
				$clause .= ' WHERE ' . join($this->multiConditionSeparator, $this->condition);
			if($this->group)
				$clause .= ' ' . $this->group;
			if(!empty($this->order))
				$clause .= ' ORDER BY ' . join(', ', $this->order);
			if($this->limit)
				$clause .= ' ' . $this->limit;
		}
		if(!empty($this->joinUpdate)) // To allow for restore() in case $this->join has been used in update()
			$this->join = $this->joinUpdate;
		return $clause;
	}
/**
* reset various strings and arrays used in subclauses
*/
	public function resetSubs()
	{
		$this->conditionSeparator = $this->multiConditionSeparator = $this->and;
		$this->ascDesc = $this->asc;
		$this->order = $this->join = $this->condition = array();
		$this->group = $this->limit = FALSE;
	}
/**
* Store and restore various and arrays used in subclauses
*
* @param boolean $restore Default is FALSE
*/
	public function restore($restore = FALSE)
	{
		if(!$restore) // i.e. do the initial store
		{
			$this->restoreArray = array();
			$this->restoreArray[] = $this->conditionSeparator;
			$this->restoreArray[] = $this->multiConditionSeparator;
			$this->restoreArray[] = $this->ascDesc;
			$this->restoreArray[] = $this->order;
			$this->restoreArray[] = $this->join;
			$this->restoreArray[] = $this->condition;
			$this->restoreArray[] = $this->group;
			$this->restoreArray[] = $this->limit;
		}
		else
		{
			$this->conditionSeparator = array_shift($this->restoreArray);
			$this->multiConditionSeparator = array_shift($this->restoreArray);
			$this->ascDesc = array_shift($this->restoreArray);
			$this->order = array_shift($this->restoreArray);
			$this->join = array_shift($this->restoreArray);
			$this->condition = array_shift($this->restoreArray);
			$this->group = array_shift($this->restoreArray);
			$this->limit = array_shift($this->restoreArray);
		}
	}
/**
* List fields in a database table
*
* @author Mark Grimshaw
* @version 1
* @see SQL::listFields()
* @param string $table
* @return array
*/
	public function listFields($table)
	{
		$fields = array();
		$table = $this->formatTables($table, FALSE);
		if($this->dbaseType == 'mysql')
			$recordset = mysqli_query($this->handle, "SHOW COLUMNS FROM $table");
		while($row = $this->fetchRow($recordset))
			$fields[] = $row['Field'];
		return $fields;
	}
/**
* show all tables in db
*
* @author Mark Grimshaw
* @version 2
* @return array
*/
	public function listTables()
	{
		$tables = array();
		$db = $this->config->WIKINDX_DB;
		if($this->dbaseType == 'mysql')
			$recordset = mysqli_query($this->handle, "SHOW TABLES FROM `$db`");
		while($row = $this->fetchRow($recordset))
			$tables[] = $row["Tables_in_" . $this->config->WIKINDX_DB];
		return $tables;
	}
/**
* Create a table
*
* @param string $newTable
* @param array $fieldsArray
*/
	public function createTable($newTable, $fieldsArray)
	{
		$newTable = $this->config->WIKINDX_DB_TABLEPREFIX . $newTable;
		if($this->dbaseType == 'mysql')
		{
			$sql = '(' . join(', ', $fieldsArray) . ')' . ' ENGINE=MyISAM CHARSET=utf8';
			$this->query("CREATE TABLE `$newTable` $sql");
		}
	}
/**
* Rename a table
*
* @param string $oldTable
* @param string $newTable
*/
	public function renameTable($oldTable, $newTable)
	{
		$oldTable = $this->config->WIKINDX_DB_TABLEPREFIX . $oldTable;
		$newTable = $this->config->WIKINDX_DB_TABLEPREFIX . $newTable;
		if($this->dbaseType == 'mysql')
			$this->query("ALTER TABLE `$oldTable` RENAME `$newTable`");
	}
/**
* Drop a table
*
* @param string $table
*/
	public function dropTable($table)
	{
		$table = $this->config->WIKINDX_DB_TABLEPREFIX . $table;
		if($this->dbaseType == 'mysql')
			$this->query("DROP TABLE `$table`");
	}
/**
* return numRows from recordset
*
* @author Mark Grimshaw
* @version 1
* @param object $recordset
* @return int
*/
	public function numRows($recordset)
	{
		if($this->dbaseType == 'mysql')
			return mysqli_num_rows($recordset);
	}
/**
* Fetch a row from the database
*
* @author Mark Grimshaw
* @version 2
* @param object $recordset
* @return array
*/
	public function fetchRow($recordset)
	{
		if($this->dbaseType == 'mysql')
			$row = mysqli_fetch_assoc($recordset);
		if(!is_array($row))
			return $row;
		$row = array_map(array($this, "sqlAddSlashes"), $row);
		return $row;
	}
/**
* Fetch one field value from the database
*
* @author Mark Grimshaw
* @version 2
* @param object $recordset
* @return string
*/
	public function fetchOne($recordset)
	{
		if($this->dbaseType == 'mysql')
			$row = mysqli_fetch_row($recordset);
		return addslashes($row[0]);
	}
/**
* Add slashes to string
*
* @param string $element
* @return string
*/
	private function sqlAddSlashes($element)
	{
		return addslashes($element);
	}
/**
* Create a UNION sub query -- MySQL 4.1 and above.
*
* @param mixed $stmt string or array select statement(s) to be unionized
* @param boolean $all Default FALSE.  Set to TRUE to have 'UNION ALL'
* @return string
*/
	public function union($stmt, $all = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			if(is_array($stmt))
			{
				if($all)
					return join(' UNION ALL ', $stmt);
				else
					return join(' UNION ', $stmt);
			}
			else
			{
				if($all)
					return "UNION ALL $stmt";
				else
					return " UNION $stmt";
			}
		}
	}
/**
* Create a subquery from a SQL statement
*
* @author Mark Grimshaw
* @version 1
* @param string $stmt Pre-defined SQL stmt (which may be a subquery itself)
* @param boolean $alias Boolean table alias sometimes required for subquery SELECT statements.  Default is FALSE
* @param boolean $from TRUE If FALSE, don't add the initial 'FROM'
* @param boolean $clause Default is FALSE. If TRUE, add all conditions, joins, groupBy, orderBy etc. clauses
* @return string
*/
	public function subQuery($stmt, $alias = FALSE, $from = TRUE, $clause = FALSE)
	{
		if(!$stmt)
			$this->error = $this->errors->text("dbError", "subQuery");
		if($clause)
		{
			$stmt .= $this->subClause();
			$this->restore();
			if($this->reset)
				$this->resetSubs();
		}
		if($this->dbaseType == 'mysql')
		{
			$from ? $from = 'FROM ' : '';
			if($alias)
				$return = "$from($stmt) AS " . $this->formatTables($alias);
			else
				$return = "$from($stmt)";
		}
		return $return;
	}
/**
* Create a subquery from SQL fields
*
* If $alias is FALSE: 'FROM (SELECT $distinct $field $subquery $clause)'
* If $alias is TRUE: 'FROM (SELECT $distinct $field $subquery $clause) AS '
*
* @author Mark Grimshaw
* @version 1
* @param array $fields
* @param string $subquery Formatted subquery string
* @param boolean $alias Boolean table alias sometimes required for subquery SELECT statements.  Default is FALSE
* @param boolean $clause Default is FALSE. If TRUE, add all conditions, joins, groupBy, orderBy etc. clauses
* @param boolean $distinct Default is FALSE
* @param boolean $tidy Format fields for SQL queries. Default is TRUE
* @return string
*/
	public function subQueryFields($fields, $subquery, $alias = FALSE, $clause = FALSE, $distinct = FALSE, $tidy = TRUE)
	{
		if($clause)
		{
			$clause = $this->subClause();
			$this->restore();
			if($this->reset)
				$this->resetSubs();
		}
		if($distinct)
			$distinct = ' DISTINCT ';
		if($tidy)
			$field = $this->formatFields($fields);
		else
			$field = $fields;
		if($this->dbaseType == 'mysql')
		{
			if($alias)
				$return = "FROM (SELECT $distinct $field $subquery $clause) AS " . $this->formatTables($alias);
			else
				$return = "FROM (SELECT $distinct $field $subquery $clause)";
		}
		return $return;
	}
/**
* Execute SELECT statement
*
* @author Mark Grimshaw
* @version 1
* @param array $tables
* @param mixed $fields Array of fields or can be '*'
* @param boolean $distinct Default is FALSE
* @param boolean $tidyFields Format fields for SQL. Default is TRUE
* @param boolean $alias Default is FALSE
* @return object recordset
*/
	public function select($tables, $fields, $distinct = FALSE, $tidyFields = TRUE, $alias = FALSE)
	{
		$table = $this->formatTables($tables);
		if($tidyFields)
		{
			if(!is_array($fields) && $fields == '*')
				$field = '*';
			else
				$field = $this->formatFields($fields);
		}
		else
			$field = $fields;
		$this->error = $this->errors->text("dbError", "read");
		if($alias)
			$alias = ' AS ' . $this->formatTables($alias);
		if($this->dbaseType == 'mysql')
		{
			if($distinct)
				return $this->query("SELECT DISTINCT $field FROM $table $alias");
			else
				return $this->query("SELECT $field FROM $table $alias");
		}
	}
/**
* Execute SELECT MAX() statement
*
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $maxField
* @param boolean $alias Default is FALSE
* @param array $otherFields Other fields to add to the query. Default is FALSE
* @param string $subQuery Default is FALSE
* @return object recordset
*/
	public function selectMax($table, $maxField, $alias = FALSE, $otherFields = FALSE, $subQuery = FALSE)
	{
		if($table)
			$table = 'FROM ' . $this->formatTables($table);
		if(!$alias)
			$alias = $this->formatFields($maxField);
		else
			$alias = $this->formatFields($alias);
		if($otherFields)
			$otherFields = ', ' . $this->formatFields($otherFields);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
			return $this->query('SELECT MAX(' . $this->formatFields($maxField) . ") AS $alias $otherFields $table $subQuery");
	}
/**
* Execute a SELECT MIN() statement
*
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $minField
* @return object recordset
*/
	public function selectMin($table, $minField)
	{
		$table = $this->formatTables($table);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
			return $this->query('SELECT MIN(' . $this->formatFields($minField) . ") AS $minField FROM $table");
	}
/**
* Execute a "SELECT FROM_UNIXTIME(AVG(UNIX_TIMESTAMP($field))) AS $field FROM $table" statement
*
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $field
* @return object recordset
*/
	public function selectAverageDate($table, $field)
	{
		$table = $this->formatTables($table);
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
			return $this->query("SELECT FROM_UNIXTIME(AVG(UNIX_TIMESTAMP($field))) AS $field FROM $table");
	}
/**
* Execute a "SELECT COUNT(*) AS `count`, $field FROM $table" statement
*
* NB 'count' field in recordset
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $field
* @return object recordset
*/
	public function selectCount($table, $field)
	{
		$table = $this->formatTables($table);
		$this->groupBy($field);
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
			return $this->query("SELECT COUNT(*) AS `count`, $field FROM $table");
	}
/**
* Execute a "SELECT COUNT(*) AS `count`, $field FROM $table" statement
*
* NB 'count' field in recordset.
* MAX is achieved by grouping and ordering on $field
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $field
* @return object recordset
*/
	public function selectCountMax($table, $field)
	{
		$table = $this->formatTables($table);
		$this->groupBy($field);
		$field = $this->formatFields($field);
		$this->ascDesc = $this->desc;
		$this->orderBy('count', FALSE);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
			return $this->query("SELECT COUNT(*) AS `count`, $field FROM $table");
	}
/**
* Execute a "SELECT $field, COUNT($field) AS `count` $otherFields $table $subQuery" statement
*
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $field
* @param array $otherFields Other fields to add to the query. Default is FALSE
* @param string $subQuery Default is FALSE
* @param boolean $group Default is TRUE
* @return object recordset
*/
	public function selectCounts($table, $field, $otherFields = FALSE, $subQuery = FALSE, $group = TRUE)
	{
// NB NULL value rows are not gathered
		if($table)
			$table = 'FROM ' . $this->formatTables($table);
		if($subQuery && strpos(ltrim($subQuery), 'FROM') !== 0)
			$subQuery = 'FROM ' . $subQuery;
		if($otherFields)
			$otherFields = ', ' . $this->formatFields($otherFields);
		if($group)
		{
			$this->groupBy($field);
			$this->group .= $otherFields;
		}
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
		{
			$subQuery ? "$subQuery" : '';
			return $this->query("SELECT $field, COUNT($field) AS `count` $otherFields $table $subQuery");
		}
	}
/**
* Create a "SELECT $field, COUNT($field) AS `count` $otherFields $table $subQuery $clause" statement without executing
*
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $field
* @param array $otherFields Other fields to add to the query. Default is FALSE
* @param string $subQuery Default is FALSE
* @param boolean $group Default is TRUE
* @param string $clause Default is FALSE
* @return string
*/
	public function selectCountsNoExecute($table, $field, $otherFields = FALSE, $subQuery = FALSE, $group = TRUE, $clause = FALSE)
	{
// NB NULL value rows are not gathered
		if($table)
			$table = 'FROM ' . $this->formatTables($table);
		if($subQuery && strpos(ltrim($subQuery), 'FROM') !== 0)
			$subQuery = 'FROM ' . $subQuery;
		if($otherFields)
			$otherFields = ', ' . $this->formatFields($otherFields);
		if($group)
		{
			$this->groupBy($field);
			$this->group .= $otherFields;
		}
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($clause)
		{
			$clause = $this->subClause();
			$this->restore();
			if($this->reset)
				$this->resetSubs();
		}
		if($this->dbaseType == 'mysql')
		{
			$subQuery ? "$subQuery" : '';
			return "SELECT $field, COUNT($field) AS `count` $otherFields $table $subQuery $clause";
		}
	}
/**
* Execute a "SELECT COUNT(DISTINCT $field) AS `count` $table $subQuery" statement
*
* NB 'count' field in the recordset
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $field
* @param string $subQuery Default is FALSE
* @return object recordset
*/
	public function selectCountDistinctField($table, $field, $subQuery = FALSE)
	{
		if($table)
			$table = 'FROM ' . $this->formatTables($table);
		if($subQuery && strpos(ltrim($subQuery), 'FROM') !== 0)
			$subQuery = 'FROM ' . $subQuery;
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
			return $this->query("SELECT COUNT(DISTINCT $field) AS `count` $table $subQuery");
	}
/**
* Create a "SELECT COUNT(DISTINCT $field) AS `count` $table $subQuery $clause" statement without executing
*
* NB 'count' field in the recordset
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $field
* @param string $subQuery Default is FALSE
* @param string $clause Default is FALSE
* @return string
*/
	public function selectCountDistinctFieldNoExecute($table, $field, $subQuery = FALSE, $clause = FALSE)
	{
		if($table)
			$table = 'FROM ' . $this->formatTables($table);
		if($subQuery && strpos(ltrim($subQuery), 'FROM') !== 0)
			$subQuery = 'FROM ' . $subQuery;
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($clause)
		{
			$clause = $this->subClause();
			$this->restore();
			if($this->reset)
				$this->resetSubs();
		}
		if($this->dbaseType == 'mysql')
			return "SELECT COUNT(DISTINCT $field) AS `count` $table $subQuery $clause";
	}
/**
* Execute a "SELECT COUNT(DISTINCT $field) AS `count` $subQuery" statement
*
* @author Mark Grimshaw
* @param string $field
* @param string $subQuery
* @return object recordset
*/
	public function selectCountFromSubquery($field, $subQuery)
	{
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
			return $this->query("SELECT COUNT(DISTINCT $field) AS `count` $subQuery");
	}
/**
* Create a "SELECT COUNT(DISTINCT $field) AS `count` $subQuery $clause" statment without executing 
* @author Mark Grimshaw
* @param string $field
* @param string $subQuery
* @param string $clause Default is FALSE
* @return string
*/
	public function selectCountFromSubqueryNoExecute($field, $subQuery, $clause = FALSE)
	{
		$field = $this->formatFields($field);
		$this->error = $this->errors->text("dbError", "read");
		if($clause)
		{
			$clause = $this->subClause();
			$this->restore();
			if($this->reset)
				$this->resetSubs();
		}
		if($this->dbaseType == 'mysql')
			return "SELECT COUNT(DISTINCT $field) AS `count` $subQuery $clause";
	}
/**
* Execute a SELECT statement
*
* @author Mark Grimshaw
* @version 1
* @see SQL::select()
* @param array $tables
* @param mixed $fields Array of fields or '*'
* @param boolean $distinct Default is FALSE
* @return object recordset
*/
	public function selectWithExceptions($tables, $fields, $distinct = FALSE)
	{
		if(!is_array($fields) && $fields == '*')
			$field = '*';
		else
			$field = $this->formatFields($fields, TRUE);
		$table = $this->formatTables($tables);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
		{
			if($distinct)
				return $this->query("SELECT DISTINCT $field FROM $table");
			else
				return $this->query("SELECT $field FROM $table");
		}
	}
/**
* Create a SELECT statement without executing
*
* Either: "SELECT DISTINCT $field FROM $table $alias $clause" or "SELECT $field FROM $table $alias $clause"
*
* @author Mark Grimshaw
* @version 1
* @param array $tables
* @param mixed $fields Array of fields or '*'
* @param boolean $distinct Default is FALSE
* @param boolean $tidyFields Format fields. Default is TRUE
* @param string $clause Default is FALSE
* @param boolean $alias Default is FALSE
* @return string
*/
	public function selectNoExecute($tables, $fields, $distinct = FALSE, $tidyFields = TRUE, $clause = FALSE, $alias = FALSE)
	{
		if($tidyFields)
		{
			if(!is_array($fields) && $fields == '*')
				$field = '*';
			else
				$field = $this->formatFields($fields);
		}
		else
			$field = $fields;
		$table = $this->formatTables($tables);
		$this->error = $this->errors->text("dbError", "read");
		if($clause)
		{
			$clause = $this->subClause();
			$this->restore();
			if($this->reset)
				$this->resetSubs();
		}
		if($alias)
			$alias = ' AS ' . $this->formatTables($alias);
		if($this->dbaseType == 'mysql')
		{
			if($distinct)
				return "SELECT DISTINCT $field FROM $table $alias $clause";
			else
				return "SELECT $field FROM $table $alias $clause";
		}
	}
/**
* Execute a SELECT statement with a subquery
*
* If $tables is FALSE, statement is "SELECT $field $subQuery" else it is "SELECT $field $subQuery $tables"
* @author Mark Grimshaw
* @version 1
* @param array $tables
* @param mixed $fields Array of fields or '*'
* @param string $subQuery
* @param boolean $distinct Default is FALSE
* @param boolean $tidy Format fields. Default is TRUE
* @return object recordset
*/
	public function selectFromSubQuery($tables, $fields, $subQuery, $distinct = FALSE, $tidy = TRUE)
	{
		if(!is_array($fields) && $fields == '*')
			$field = '*';
		else
		{
			if($tidy)
				$field = $this->formatFields($fields);
			else
				$field = $fields;
		}
		if($tables)
			$table = ', ' . $this->formatTables($tables);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
		{
			if($tables)
			{
				if($distinct)
					return $this->query("SELECT DISTINCT $field $subQuery $table"); // 'FROM' is already part of subqQuery -- FROM($subQuery) 
				else
					return $this->query("SELECT $field $subQuery $table"); // 'FROM' is already part of subqQuery -- FROM($subQuery) 
			}
			else
			{
				if($distinct)
					return $this->query("SELECT DISTINCT $field $subQuery");
				else
					return $this->query("SELECT $field $subQuery");
			}
		}
	}
/**
* Create a SELECT statement with a subquery without executing
*
* If $tables is FALSE, statement is "SELECT $field $subQuery $clause" else it is "SELECT $field $subQuery $tables $clause"
* @author Mark Grimshaw
* @version 1
* @param array $tables
* @param mixed $fields Array of fields or '*'
* @param string $subQuery
* @param boolean $distinct Default is FALSE
* @param boolean $tidy Format fields. Default is TRUE
* @param boolean $clause Default is FALSE
* @return object recordset
*/
	public function selectNoExecuteFromSubQuery($tables, $fields, $subQuery, $distinct = FALSE, $tidy = TRUE, $clause = FALSE)
	{
		if(!is_array($fields) && $fields == '*')
			$field = '*';
		else
		{
			if($tidy)
				$field = $this->formatFields($fields);
			else
				$field = $fields;
		}
		if($tables)
			$table = ', ' . $this->formatTables($tables);
		$this->error = $this->errors->text("dbError", "read");
		if($clause)
		{
			$clause = $this->subClause();
			$this->restore();
			if($this->reset)
				$this->resetSubs();
		}
		if($this->dbaseType == 'mysql')
		{
			if($tables)
			{
				if($distinct)
					return "SELECT DISTINCT $field $subQuery $table $clause"; // 'FROM' is already part of subqQuery -- FROM($subQuery) 
				else
					return "SELECT $field $subQuery $table $clause"; // 'FROM' is already part of subqQuery -- FROM($subQuery) 
			}
			else
			{
				if($distinct)
					return "SELECT DISTINCT $field $subQuery $clause";
				else
					return "SELECT $field $subQuery $clause";
			}
		}
	}
/**
* Create a SELECT statement without executing
*
* @author Mark Grimshaw
* @version 1
* @see SQL::select()
* @param array $tables
* @param mixed $fields Array of fields or '*'
* @param boolean $distinct Default is FALSE
* @return string
*/
	public function selectNoExecuteWithExceptions($tables, $fields, $distinct = FALSE)
	{
		if(!is_array($fields) && $fields == '*')
			$field = '*';
		else
			$field = $this->formatFields($fields, TRUE);
		$table = $this->formatTables($tables);
		$this->error = $this->errors->text("dbError", "read");
		if($this->dbaseType == 'mysql')
		{
			if($distinct)
				return "SELECT DISTINCT $field FROM $table";
			else
				return "SELECT $field FROM $table";
		}
	}
/**
* Execute an INSERT statement
*
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param array $fields
* @param array $values
*/
	public function insert($table, $fields, $values)
	{
		$field = $this->formatFields($fields);
		$table = $this->formatTables($table);
		$value = is_array($values) ? $this->formatValues($values) : $this->tidyInput($values);
		$this->error = $this->errors->text("dbError", "write");
		if($this->dbaseType == 'mysql')
			$this->query("INSERT INTO $table ($field) VALUES($value)");
	}
/**
* Execute an UPDATE statement for an array of fields
*
* @author Mark Grimshaw
* @version 1
* @see SQL::update()
* @param string $table
* @param array $updateArray
*/
	public function update($table, $updateArray)
	{
		$set = $this->formatUpdate($updateArray);
		$table = $this->formatTables($table);
		$this->error = $this->errors->text("dbError", "write");
		$join = FALSE;
		if(!empty($this->join))
		{
			$join = join(' ', $this->join);
			$this->joinUpdate = $this->join;
			$this->join = array();
		}
		if($this->dbaseType == 'mysql')
			$this->query("UPDATE $table $join $set");
	}
/**
* Execute an UPDATE statement for an array of fields, setting the timestamp of a field
*
* If there is no value for a $updateArray key, the timestamp is set to NOW()
* @author Mark Grimshaw
* @version 1
* @see SQL::updateTimestamp
* @param string $table
* @param array $updateArray
*/
	public function updateTimestamp($table, $updateArray)
	{
		if($this->dbaseType == 'mysql')
		{
			foreach($updateArray as $field => $value)
			{
				if(!$value)
					$value = 'NOW()';
				$fieldArray[] = "`$field` = $value";
			}
			$set = "SET " . implode(", ", $fieldArray);
		}
		$table = $this->formatTables($table);
		$this->error = $this->errors->text("dbError", "write");
		if($this->dbaseType == 'mysql')
			$this->query("UPDATE $table $set");
	}
/**
* Execute an UPDATE statement for a single field
*
* @author Mark Grimshaw
* @version 1
* @param string $table
* @param string $set Set statement
*/
	public function updateSingle($table, $set)
	{
		$table = $this->formatTables($table);
		$this->error = $this->errors->text("dbError", "write");
		if($this->dbaseType == 'mysql')
			$this->query("UPDATE $table SET $set");
	}
/**
* Execute an UPDATE statement setting the fields to NULL
*
* @author Mark Grimshaw
* @version 1
* @see SQL::updateNull()
* @param string $table
* @param array $nulls Array of fields to set to NULL
*/
	public function updateNull($table, $nulls)
	{
		$table = $this->formatTables($table);
		$this->error = $this->errors->text("dbError", "write");
		if($this->dbaseType == 'mysql')
		{
			if(is_array($nulls))
			{
				foreach($nulls as $null)
					$sqlArray[] = "`$null`=NULL";
				$set = implode(", ", $sqlArray);
			}
			else
				$set = "`$nulls`=NULL";
			$this->query("UPDATE $table SET $set");	
		}
	}
/**
* Execute a DELETE statement
* 
* NB Unless you want to delete all rows from a table, set the condition first!
* @author Mark Grimshaw
* @version 1
* @param string $table
*/
	public function delete($table)
	{
		$table = $this->formatTables($table);
		$this->error = $this->errors->text("dbError", "write");
		if($this->dbaseType == 'mysql')
			$this->query("DELETE FROM $table");
	}
/**
* Return last auto_increment ID
*
* @author Mark Grimshaw
* @version 1
* @return int
*/
	public function lastAutoID()
	{
		if($this->dbaseType == 'mysql')
			return mysqli_insert_id($this->handle);
	}
/**
* Format fields for database type
*
* @author Mark Grimshaw
* @version 1
* @param array $fields
* @param boolean $withExceptions Default is FALSE
* @return string
*/
	public function formatFields($fields, $withExceptions = FALSE)
	{
		if(!is_array($fields))
		{
			if(sizeof($split = explode('.', $fields)) > 1)
				return $this->formatTables($split[0]) . ".`$split[1]`";
			return "`$fields`";
		}
		if(empty($fields))
			return NULL;
		foreach($fields as $field)
		{
			if(is_array($field))
			{
				if($withExceptions)
					$array[] = $this->formatAliasWithExceptions($field);
				else
					$array[] = $this->formatAlias($field);
			}
			else
			{
				if(sizeof($split = explode('.', $field)) > 1)
					$array[] = $this->formatTables($split[0]) . ".`$split[1]`";
				else
					$array[] = "`$field`";
			}
		}
		return implode(", ", $array);
	}
/**
* Add delimiters to statements
*
* @author Mark Grimshaw
* @version 1
* @param string $stmt
* @param string $delimiter 'backtick', 'singleQuote', 'doubleQuote' etc. based on MySQL
* @return string delimited statement
*/
	public function delimit($stmt, $delimiter)
	{
		if($delimiter == 'backtick')
		{
			if($this->dbaseType == 'mysql')
				return '`' . $stmt . '`';
		}
		else if ($delimiter == 'singleQuote')
		{
			if($this->dbaseType == 'mysql')
			return "'" . $stmt . "'";
		}
		else if ($delimiter == 'doubleQuote')
		{
			if($this->dbaseType == 'mysql')
			return '"' . $stmt . '"';
		}
		else if ($delimiter == 'parentheses')
		{
			if($this->dbaseType == 'mysql')
			return '(' . $stmt . ')';
		}
		else if ($delimiter == 'brackets')
		{
			if($this->dbaseType == 'mysql')
			return '[' . $stmt . ']';
		}
	}
/**
* Format tables for database type
*
* @param mixed $tables Array of tables or single table
* @param boolean $brackets Default is FALSE
* @return string
*/
	public function formatTables($tables, $brackets = FALSE)
	{
		if(!$tables)
			return FALSE;
		if(!is_array($tables))
			return "`" . $this->config->WIKINDX_DB_TABLEPREFIX . $tables . "`";
		foreach($tables as $table)
		{
			if(is_array($table))
				$array[] = $this->formatAlias($table, TRUE);
			else
				$array[] = "`" . $this->config->WIKINDX_DB_TABLEPREFIX . $table . "`";
		}
		if($brackets)
			return '(' . implode(", ", $array) . ')';
		else
			return implode(", ", $array);
	}
/**
* Format field values for database type
*
* @param mixed $values Array of values or single value
* @return string
*/
	private function formatValues($values)
	{
		if(empty($values)) {
			return NULL;
		}
		foreach($values as $value)
		{
			if(is_array($value))
				$array[] = $this->formatAlias($value);
			else
				$array[] = $this->tidyInput($value);
		}
		return implode(", ", $array);
	}
/**
* Create a WHERE() statement
*
* @param array $conditions
* @param string $join Default is ''
* @return string
*/
	public function whereStmt($conditions, $join = '')
	{
		$array = array();
		foreach($conditions as $field => $value)
			$array[] = $field . $value;
		return 'WHERE (' . join($join, $array) . ')';
	}
/**
* Set up the SQL conditions for the next query.
*
* Conditions should be set before almost every SQL query. After the query is executed, the conditions are reset automatically.
* Multiple conditions are joined with $this->conditionSeparator which by default is set to $this->and (it could be $this->or).
* $this->conditionSeparator is reset automatically after each query.
*
* @param mixed $condition Array of field => condition conditions or formatted condition string
* @param boolean $notEqual Default is FALSE
* @param boolean $returnString Default is FALSE. If TRUE, don't set the condition but return a formatted condition string instead
* @return string Optional return
*/
	public function formatConditions($condition, $notEqual = FALSE, $returnString = FALSE)
	{
		if(!is_array($condition))
		{
			$this->condition[] = '(' . $condition . ')';
			return;
		}
		if($notEqual)
		{
			if($notEqual === ">")
				$equal = $this->greater;
			else if($notEqual === "<")
				$equal = $this->less;
			else
				$equal = $this->notEqual;
		}
		else
			$equal = $this->equal;
		foreach($condition as $field => $value)
		{
/**
* Check for conditions such as 'IS NULL' or 'IS NOT NULL'
*/
			if(stripos(trim($value), 'IS NULL') === 0 || stripos(trim($value), 'IS NOT NULL') === 0)
				$array[] = $this->formatFields($field) . $value;
			else
				$array[] = $this->formatFields($field) . $equal . $this->tidyInput($value);
		}
		if($returnString)
			return implode($this->conditionSeparator, $array);
		if($this->dbaseType == 'mysql')
			$this->condition[] = '('. implode($this->conditionSeparator, $array) . ')';
	}
/**
* Format multiple conditions for one field using ' OR '
*
* Conditions should be set before almost every SQL query. After the query is executed, the conditions are reset automatically.
* Multiple conditions are joined with $this->conditionSeparator which by default is set to $this->and (it could be $this->or).
* $this->conditionSeparator is reset automatically after each query.
*
* @param mixed $condition Array of field => condition conditions or formatted condition string
* @param string $field
* @param boolean $notEqual Default is FALSE
* @param boolean $tidy Format the field for the database type. Default is TRUE.
*/
	public function formatConditionsOneField($condition, $field, $notEqual = FALSE, $tidy = TRUE)
	{
		if($notEqual)
		{
			if($notEqual === ">")
				$equal = $this->greater;
			else if($notEqual === "<")
				$equal = $this->less;
			else
				$equal = $this->notEqual;
		}
		else
			$equal = $this->equal;
		if(!is_array($condition))
		{
			if((stripos(trim($condition), 'IS NULL') === 0) || (stripos(trim($condition), 'IS NOT NULL') === 0) || !$tidy)
				$this->condition[] = '(' . $this->formatFields($field) . $condition . ')';
			else
				$this->condition[] = '(' . $this->formatFields($field) . $equal . $condition . ')';
			return;
		}
		foreach($condition as $value)
		{
/**
* Check for conditions such as 'IS NULL' or 'IS NOT NULL'
*/
			if((stripos(trim($value), 'IS NULL') === 0) || (stripos(trim($value), 'IS NOT NULL') === 0) || !$tidy)
				$array[] = $this->formatFields($field) . $value;
			else
				$array[] = $this->formatFields($field) . $equal . $this->tidyInput($value);
		}
		if(!isset($array))
			return;
		if($this->dbaseType == 'mysql')
			$this->condition[] = '('. implode($this->or, $array) . ')';
	}
/**
* Format fields for an UPDATE statement
*
* @author Mark Grimshaw
* @version 1
* @see ADOSQL::update
* @param array $array
* @return string
*/
	private function formatUpdate($array)
	{
		if($this->dbaseType == 'mysql')
		{
			foreach($array as $field => $value)
				$fieldArray[] = "`$field`=" . $this->tidyInput($value);
			return "SET " . implode(", ", $fieldArray);
		}
	}
/**
* Format fields as aliases
*
* @author Mark Grimshaw
* @version 1
* @param array $array Field => Alias
* @param string $table Default is FALSE. If TRUE, prepend table name to alias
* @return string
*/
	private function formatAlias($array, $table = FALSE)
	{
		list($key, $value) = each($array);
		if($this->dbaseType == 'mysql')
		{
			if($table)
				return '`' . $this->config->WIKINDX_DB_TABLEPREFIX .  "$key` AS `" . $this->config->WIKINDX_DB_TABLEPREFIX . "$value`";
			if(sizeof($split = explode('.', $key)) > 1)
				return $this->formatTables($split[0]) . ".`$split[1]` AS `$value`";
			else
				return "`$key` AS `$value`";
		}
	}
/**
* Format fields as aliases
*
* Array keys are checked for UNIX_TIMESTAMP or DATE_FORMAT
* @author Mark Grimshaw
* @version 1
* @param array $array Field => Alias
* @return string
*/
	private function formatAliasWithExceptions($array)
	{
		list($key, $value) = each($array);
/**
* For something like DATE_FORMAT(timestamp,'%d/%b/%Y'), we don't want backticks.
* Add other exceptions here...
*/
		if($this->dbaseType == 'mysql')
		{
			if(preg_match("/^DATE_FORMAT/", $key))
				return $key . " AS `$value`";
			if(preg_match("/^UNIX_TIMESTAMP/", $key))
				return $key . " AS `$value`";
			if(sizeof($split = explode('.', $key)) > 1)
				return $this->formatTables($split[0]) . ".`$split[1]` AS `$value`";
			else
				return "`$key` AS `$value`";
		}
	}
/**
* Format a timestamp value as "Y-m-d H:i:s"
*
* @author Mark Grimshaw
* @version 1
* @param string $time UNIX epoch time. Default is FALSE (in which case time() is used)
* @return string
*/
	public function formatTimestamp($time = FALSE)
	{
		if(!$time)
			$time = time();
		if($this->dbaseType == 'mysql')
			return date("Y-m-d H:i:s", $time);
	}
/**
* Optimize a table
*
* @author Mark Grimshaw
* @version 1
* @param string $table
*/
	public function optimize($table)
	{
		$table = $this->formatTables($table);
		if($this->dbaseType == 'mysql')
			$this->query("OPTIMIZE TABLE $table");
	}
/**
* Format field values for database type
*
* Fields are trimmed
* @author Mark Grimshaw
* @version 1
* @param string $string
* @return string
*/
	public function tidyInput($string)
	{
		if(array_key_exists('utf8CharTest', $this->vars)) // input comes from form
			$string = $this->utf8->testFormInput(trim($string));
		else
			$string = trim($string);
		if($this->dbaseType == 'mysql')
		{
			if($this->escapeString)
				return "'" . mysqli_real_escape_string($this->handle, $string) . "'";
			else
				return "'" . $string . "'";
		}
	}
/**
* Format field values for database type
*
* Fields are not trimmed
* @author Mark Grimshaw
* @version 1
* @param string $string
* @return string
*/
	public function tidyInputNoTrim($string)
	{
		if(array_key_exists('', $this->vars)) // input comes from form
			$string = $this->utf8->testFormInput($string);
		if($this->dbaseType == 'mysql')
		{
			if($this->escapeString)
				return "'" . mysqli_real_escape_string($this->handle, $string) . "'";
			else
				return "'" . $string . "'";
		}
	}
/**
* Format field values for database type
*
* For use with ORDER or GROUP by clauses
* @author Mark Grimshaw
* @version 1
* @param string $string
* @return string
*/
	public function tidyInputClause($string)
	{
		if($this->dbaseType == 'mysql')
		{
			if($this->escapeString)
				return "`" . mysqli_real_escape_string($this->handle, $string) . "`";
			else
				return "`$string`";
		}
	}
/**
* Write a WIKINDX database cache
*
* @author Mark Grimshaw
* @version 1
* @param string $field
* @param array $array
*/
	public function writeCache($field, $array)
	{
		if($this->dbaseType == 'mysql')
			$this->updateSingle('cache', 
				$this->formatFields($field) ."=" . $this->tidyInput(base64_encode(serialize($array))));
	}
/**
* Read a WIKINDX database cache
*
* @author Mark Grimshaw
* @version 1
* @param string $field
* @return array
*/
	public function readCache($field)
	{
		$resultset = $this->select('cache', $field);
		if(!$this->numRows($resultset))
			return FALSE;
		return unserialize(base64_decode($this->fetchOne($resultset)));
	}
/**
* Delete a WIKINDX database cache
*
* @author Mark Grimshaw
* @version 1
* @param string $field
*/
	public function deleteCache($field)
	{
		$this->updateNull('cache', $field);
	}
/**
* prepend the configured table name to the field names
*
* @param string $table
* @param mixed $fields Array of field names or single field name
* @return mixed
*/
	public function prependTableToField($table, $fields)
	{
		$table = str_replace('_', '', $table);
		if(is_array($fields))
		{
			foreach($fields as $field)
				$changed[] = $table . $field;
		}
		else
			$changed = $table . $fields;
		return $changed;
	}
/**
* Return a ratio alias of $field / number days since e.g. resource added.
*
* @param string $field (e.g. 'resourcemiscAccesses', or 'resourceattachmentDownloads')
* @param string $denominator (e.g. 'resourcetimestampTimestampAdd')
* @param boolean $alias Default is FALSE
* @param boolean $avg Default is FALSE. If TRUE, calculate an average of multiple $fields
* @param int $round Default is 3
* @param mixed $otherFields FALSE (default) or comma-delimited list of database fields to include in the GROUP BY
* @param boolean $group FALSE (default) or GROUP BY $field and $otherFields
* @return int
*/
	public function dateDiffRatio($field, $denominator, $alias = FALSE, $avg = FALSE, $round = 3, $otherFields = FALSE, $group = FALSE)
	{
		if($otherFields)
			$otherFields = ', ' . $this->formatFields($otherFields);
		if($group)
		{
			$this->groupBy($field);
			$this->group .= $otherFields;
		}
		if($this->dbaseType == 'mysql')
		{
			if($alias)
				$alias = ' AS ' . $this->formatFields($alias);
			if($avg)
				return 'ROUND(AVG(' . $this->formatFields($field) . ' / DATEDIFF(CURDATE(), ' . $this->formatFields($denominator) . 
					")), $round)$alias";
			return 'ROUND(' . $this->formatFields($field) . ' / DATEDIFF(CURDATE(), ' . $this->formatFields($denominator) . 
				"), $round)$alias";
		}
	}
/**
* Return number months difference between two database timestamps
*
* @param string $date1 Timestamp value from database
* @param string $date2 Default is FALSE. If FALSE, NOW() is assumed
* @return int
*/
	public function monthDiff($date1, $date2 = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($date2)
				return $this->fetchOne($this->query("SELECT 
					PERIOD_DIFF(DATE_FORMAT('$date2', '%Y%m'), DATE_FORMAT('$date1', '%Y%m'))"));
			else
				return $this->fetchOne($this->query("SELECT 
					PERIOD_DIFF(DATE_FORMAT(NOW(), '%Y%m'), DATE_FORMAT('$date1', '%Y%m'))"));
		}
		return FALSE;
	}
/**
* Create a ROUND() clause
*
* @param string $clause
* @param boolean $alias Default is FALSE
* @param int $round Default is 3
* @return string
*/
	public function round($clause, $alias = FALSE, $round = 3)
	{
		if($this->dbaseType == 'mysql')
		{
			if($alias)
				$alias = ' AS ' . $this->formatFields($alias);
			return "ROUND($clause, $round)$alias";
		}
	}
/**
* Return a AVG() clause
*
* @param string $clause
* @return string
*/
	public function avg($clause)
	{
		if($this->dbaseType == 'mysql')
			return "AVG($clause)";
	}
/**
* Create a condition clause for a time interval: "DATE_SUB($fromTime, INTERVAL $limit $timescale)"
*
* @param string $limit
* @param string $timescale Default is 'day'
* @param string $fromTime Default is 'now'
* @return string
*/
	public function dateIntervalCondition($limit, $timescale = 'day', $fromTime = 'now')
	{
		if($this->dbaseType == 'mysql')
		{
			if($fromTime == 'now')
				$fromTime = 'CURDATE()';
			if($timescale == 'day')
				$timescale = 'DAY';
			return "DATE_SUB($fromTime, INTERVAL $limit $timescale)";
		}
	}
/**
* Create a CONCAT clause
*
* @param array $array
* @param string $separator Default is FALSE. If TRUE, CONCAT_WS() is used, else CONCAT().
* @return string
*/
	public function concat($array, $separator = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($separator !== FALSE)
				return "CONCAT_WS('$separator', " . implode(", ", $array) . ")";
			else
				return "CONCAT(" . implode(", ", $array) . ")";
		}
	}
/**
* Create a CASE WHEN() THEN clause
*
* @param string $subject
* @param string $test
* @param string $result
* @param string $default Default is FALSE
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
* @return string
*/
	public function caseWhen($subject, $test, $result, $default = FALSE, $tidy = TRUE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($test)
				$test = ' ' . $test;
			if($tidy)
			{
				if($default)
					$default = ' ELSE (' . $this->formatFields($default) . ')';
				return 'CASE WHEN (' . $this->formatFields($subject) . $test . ') THEN (' . 
					$this->formatFields($result) . ')' . $default . ' END';
			}
			else
			{
				if($default)
					$default = ' ELSE ' . $default;
				return 'CASE WHEN (' . $subject . ' ' . $test . ') THEN (' . 
					$result . ')' . $default . ' END';
			}
		}
	}
/** 
* Create an IF clause: "IF($field $test, $result, $default)"
*
* @param string $field
* @param string $test
* @param string $result
* @param string $default
* @return string
*/
	public function ifClause($field, $test, $result, $default)
	{
		if($this->dbaseType == 'mysql')
			return "IF($field $test, $result, $default)";
	}
/** 
* Create a LEFT JOIN clause.
*
* Clauses are stored in $this->join array for use at the next query after which the array is emptied.
* You should set up your join statements before each query.
* @param string $table
* @param string $left
* @param string $right Default is FALSE
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
*/
	public function leftJoin($table, $left, $right = FALSE, $tidy = TRUE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($tidy && $right)
			{
					$left = $this->formatFields($left);
					$right = $this->equal . $this->formatFields($right);
			}
			else if($right)
				$right = $this->equal . $right;
			$this->join[] = 'LEFT JOIN ' . $this->formatTables($table, FALSE) . ' ON ' . $left . $right;
		}
	}
/** 
* Create a LEFT JOIN clause on a subquery
*
* Clauses are stored in $this->join array for use at the next query after which the array is emptied.
* You should set up your join statements before each query.
* @param string $subQuery
* @param string $left
* @param string $right Default is FALSE
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
*/
	public function leftJoinSubQuery($subQuery, $left, $right = FALSE, $tidy = TRUE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($tidy && $right)
			{
					$left = $this->formatFields($left);
					$right = $this->equal . $this->formatFields($right);
			}
			else if($right)
				$right = $this->equal . $right;
			$this->join[] = 'LEFT JOIN ' . $subQuery . ' ON ' . $left . $right;
		}
	}
/** 
* Create an ORDER BY clause
*
* Clauses are stored in $this->order array for use at the next query after which the array is emptied.
* You should set up your order statements before each query.
* @param string $field
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
*/
	public function orderBy($field, $tidy = TRUE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($tidy)
				$this->order[] = $this->formatFields($field) . ' '. $this->ascDesc;
			else
				$this->order[] = $field . ' ' . $this->ascDesc;
		}
		$this->collateSet = FALSE; // reset
	}
/** 
* Create an ORDER BY RAND() clause
*
* Clauses are stored in $this->order array for use at the next query after which the array is emptied.
* You should set up your order statements before each query.
*/
	public function orderByRandom()
	{
		if($this->dbaseType == 'mysql')
			$this->order[] = ' RAND()';
		$this->collateSet = FALSE; // reset
	}
/** 
* Create an ORDER BY clause with additional COLLATION for UTF8
*
* Clauses are stored in $this->order array for use at the next query after which the array is emptied.
* You should set up your order statements before each query.
* @param string $field
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
*/
	public function orderByCollate($field, $tidy = TRUE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($tidy)
				$this->order[] = $this->formatFields($field) . ' COLLATE utf8_unicode_ci' . $this->ascDesc;
			else
				$this->order[] = $field . ' COLLATE utf8_unicode_ci' . $this->ascDesc;
		}
		$this->collateSet = FALSE; // reset
	}
/** 
* Create a GROUP BY clause
*
* Clauses are stored in $this->group array for use at the next query after which the array is emptied.
* You should set up your group statements before each query.
* @param string $field
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
* @param string $having Default is FALSE. If TRUE, group by clause has ' HAVING $having' appended to it.
*/
	public function groupBy($field, $tidy = TRUE, $having = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($tidy)
				$this->group = 'GROUP BY ' . $this->formatFields($field);
			else
				$this->group = 'GROUP BY ' . $field;
			if($having)
				$this->group .= " HAVING $having";
		}
	}
/** 
* Create a LIMIT clause
*
* Clauses are stored in the $this->limit string for use at the next query after which the string is reset.
* You should set up your limit statement before each query.
* @param int $limit
* @param int $offset
*/
	public function limit($limit, $offset)
	{
		if($limit < 1)
			return; // if limit is set to -1, we don't want a limit
		if($this->dbaseType == 'mysql')
			$this->limit = " LIMIT $offset, $limit";
	}
/** 
* Create a REPLACE clause: "REPLACE(' . $field . ", '$find', '$replace')"
*
* @param string $field
* @param string $find
* @param string $replace
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
* @return string
*/
	public function replace($field, $find, $replace, $tidy = TRUE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($tidy)
				return ' REPLACE(' . $this->formatFields($field) . ", '$find', '$replace')";
			else
				return ' REPLACE(' . $field . ", '$find', '$replace')";
		}
	}
/** 
* Create a COALESCE clause: "COALESCE($fields) $alias"
*
* @param mixed $fields Array of field names or single field name
* @param string $alias
* @return string
*/
	public function coalesce($fields, $alias = FALSE)
	{
		$coalesce = array();
		if($this->dbaseType == 'mysql')
		{
			if($alias)
				$alias = ' AS ' . $this->formatFields($alias);
			if(is_array($fields))
			{
				foreach($fields as $field)
					$coalesce[] = $this->formatFields($field);
				return ' COALESCE(' . join(', ', $coalesce) . ')' . $alias;
			}
			else
				return " COALESCE($fields) $alias";
		}
	}
/** 
* Create an UPPER clause
*
* @param string $field
* @param boolean $tidy Default is TRUE. If TRUE, format fields for database type
* @return string
*/
	public function upper($field, $tidy = TRUE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($tidy)
				return ' UPPER(' . $this->formatFields($field) . ')';
			else
				return " UPPER($field)";
		}
	}
/** 
* Create a REGEXP clause
*
* @param string $first
* @param string $test
* @param string $last
* @param boolean $not Default is FALSE
* @return string
*/
	public function regexp($first, $test, $last, $not = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			$not = $not ? ' NOT' : FALSE;
			return "$not REGEXP " . $this->tidyInput($first . $test . $last);
		}
	}
/** 
* Create a LIKE clause
*
* @param string $first
* @param string $test
* @param string $last
* @param boolean $not Default is FALSE
* @return string
*/
	public function like($first, $test, $last, $not = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			$not = $not ? ' NOT' : FALSE;
			return "$not LIKE " . $this->tidyInput($first . $test . $last);
		}
	}
/**
* Create a COUNT() clause
*
* @param string $field Database field to count
* @param string $operator Optional =, !=, >, <. Default is FALSE
* @param string $comparison Comparison following $operator. Default is FALSE
* @param boolean $distinct TRUE/FALSE (default).  COUNT(DISTINCT `field`)
* @param string $alias COUNT(`field`) AS $alias. Default is FALSE
* @return string
*/
	public function count($field, $operator = FALSE, $comparison = FALSE, $distinct = FALSE, $alias = FALSE)
	{
		if($operator)
		{
			if($operator === ">")
				$comparison = ' ' . $this->greater . ' ' . $this->tidyInput($comparison);
			else if($operator === "<")
				$comparison = ' ' . $this->less . ' ' . $this->tidyInput($comparison);
			else if($operator === '!=')
				$comparison = ' ' . $this->notEqual . ' ' . $this->tidyInput($comparison);
			else
				$comparison = ' ' . $this->equal . ' ' . $this->tidyInput($comparison);
		}
		if($distinct)
			$distinct = 'DISTINCT ';
		if($alias)
			$alias = ' AS ' . $this->formatFields($alias);
		return "COUNT($distinct" . $this->formatFields($field) . ")$alias" . $comparison;
	}
/**
* Clause for using an index in a SQL query
*
* @param string $field
* @param string $type Default is 'FORCE'
*/
	public function indexHint($field, $type = 'FORCE')
	{
		if($this->dbaseType == 'mysql')
		{
			if($type == 'FORCE')
				return ' FORCE INDEX(' . $this->formatFields($field) . ') ';
			else if($type == 'IGNORE')
				return ' IGNORE INDEX(' . $this->formatFields($field) . ') ';
			else if($type == 'USE')
				return ' USE INDEX(' . $this->formatFields($field) . ') ';
		}
	}
/**
* Create an IN() clause
*
* @param string $stmt IN ($stmt)
* @param boolean $not Default is FALSE
* @return string
*/
	public function inClause($stmt, $not = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($not)
				return " NOT IN ($stmt)";
			else
				return " IN ($stmt)";
		}
	}
/**
* Create an EXISTS() clause
*
* @param string $stmt IN ($stmt)
* @param boolean $not Default is FALSE
* @return string
*/
	public function existsClause($stmt, $not = FALSE)
	{
		if($this->dbaseType == 'mysql')
		{
			if($not)
				return " NOT EXISTS ($stmt)";
			else
				return " EXISTS ($stmt)";
		}
	}
/**
* Create a SUM() statement
*
* @param string $field
* @param string $alias Default is FALSE
* @return string
*/
	public function sum($field, $alias = FALSE)
	{
		if($alias)
			$alias = ' AS' . $this->formatFields($alias);
		if($this->dbaseType == 'mysql')
			return 'SUM(' . $this->formatFields($field) . ")$alias";
	}
/**
* SQL stored function to strip HTML from field value (cf WIKINDX Search functions).
*
* Taken from:  http://www.artfulsoftware.com/infotree/queries.php#567
*
* However, as MySQL 5.x requires SUPER privileges or a server variable to be set, this cannot be used out of the box.  Kept 
* here in case future MySQL allows it 'out of the box' again.
*/
	public function fnStripHtml()
	{
		if($this->dbaseType == 'mysql')
		{
//			$this->query('SET GLOBAL log_bin_trust_function_creators=1');
/*			$this->query('DROP FUNCTION IF EXISTS fnStripHtml');
			if(!mysql_query(
				"CREATE FUNCTION fnStripHtml(Dirty varchar(4000)) 
				RETURNS varchar(4000) 
				DETERMINISTIC 
				NO SQL 
				BEGIN 
				  DECLARE iStart, iEnd, iLength int;
				  WHILE Locate( '<', Dirty ) > 0 And Locate( '>', Dirty, Locate( '<', Dirty )) > 0 DO 
					BEGIN 
					  SET iStart = Locate( '<', Dirty ), iEnd = Locate( '>', Dirty, Locate('<', Dirty ));
					  SET iLength = ( iEnd - iStart) + 1;
					  IF iLength > 0 THEN 
						BEGIN 
							SET Dirty = Insert( Dirty, iStart, iLength, '');
							SET Dirty = Replace(Dirty,'&nbsp;','');
							SET Dirty = Replace(Dirty,'\r','');
							SET Dirty = Replace(Dirty,'\n','');
						END;
					  END IF;
					END;
				  END WHILE;
				RETURN Dirty;
				END", $this->handle))
				die($this->error . ' <P>--> (' . mysql_error() . ')');
*/
			$this->query('DROP PROCEDURE IF EXISTS fnStripHtml');
			if(!mysql_query(
				"CREATE PROCEDURE fnStripHtml(IN Dirty varchar(4000), OUT Clean varchar(4000)) 
				DETERMINISTIC 
				NO SQL 
				BEGIN 
				  DECLARE iStart, iEnd, iLength int;
				  WHILE Locate( '<', Dirty ) > 0 And Locate( '>', Dirty, Locate( '<', Dirty )) > 0 DO 
					BEGIN 
					  SET iStart = Locate( '<', Dirty ), iEnd = Locate( '>', Dirty, Locate('<', Dirty ));
					  SET iLength = ( iEnd - iStart) + 1;
					  IF iLength > 0 THEN 
						BEGIN 
							SET Dirty = Insert( Dirty, iStart, iLength, '');
							SET Dirty = Replace(Dirty,'&nbsp;','');
							SET Dirty = Replace(Dirty,'\r','');
							SET Dirty = Replace(Dirty,'\n','');
						END;
					  END IF;
					SET Clean = Dirty;
					END;
				  END WHILE;
				END", $this->handle))
				die($this->error . ' <P>--> (' . mysqli_error($this->handle) . ')');
		}
	}
/**
* Beautify very briefly a SQL statement to facilitate debugging.
*
* Return Sql instruction packaged in a nice HTML
*
* @author Stephane Aulery
* @param string $sqlStatement Default is ""
* @return string
*/
	public function beautify($sqlStatement = "")
	{
		$keyWords = array(
			"SELECT ",
			"DISTINCT ",
			"UPDATE ",
			"EXECUTE ",
			"INSERT ",
			"SET ",
			"UNION ",
			
			"FROM ",
			"LEFT JOIN ",
			"WHERE ",
			"GROUP BY ",
			"ORDER BY ", 
			"CASE ", 
			"END",
			
			" ON ",
			" AND ",
			" OR ",
			" WHEN ",
			" THEN ",
			" ELSE ",
			" AS ",
		);
		$prettyKeyWords = array(
			"<span style=\"font-weight:bold;color:red;\">SELECT </span>",
			"<span style=\"font-weight:bold;color:red;\">DISTINCT </span>",
			"<span style=\"font-weight:bold;color:red;\">UPDATE </span>",
			"<span style=\"font-weight:bold;color:red;\">EXECUTE </span>",
			"<span style=\"font-weight:bold;color:red;\">INSERT </span>",
			"<span style=\"font-weight:bold;color:red;\">SET </span>",
			"<span style=\"font-weight:bold;color:red;\">UNION </span>",
			"<span style=\"font-weight:bold;color:blue;\">FROM </span>",
			"<span style=\"font-weight:bold;color:blue;\">LEFT JOIN </span>",
			"<span style=\"font-weight:bold;color:blue;\">WHERE </span>",
			"<span style=\"font-weight:bold;color:blue;\">GROUP BY </span>",
			"<span style=\"font-weight:bold;color:blue;\">ORDER BY </span>",
			"<span style=\"font-weight:bold;color:blue;\">CASE </span>",
			"<span style=\"font-weight:bold;color:blue;\">END</span>",
			"<span style=\"font-weight:bold;color:green;\"> ON </span>",
			"<span style=\"font-weight:bold;color:green;\"> AND </span>",
			"<span style=\"font-weight:bold;color:green;\"> OR </span>",
			"<span style=\"font-weight:bold;color:green;\"> WHEN </span>",
			"<span style=\"font-weight:bold;color:green;\"> THEN </span>",
			"<span style=\"font-weight:bold;color:green;\"> ELSE </span>",
			"<span style=\"font-weight:bold;color:green;\"> AS </span>",
		);
		
		return str_replace($keyWords, $prettyKeyWords, $sqlStatement);
	}
}
?>
