<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2012
sirfragalot@users.sourceforge.net
*/
/**
* SQLSTATEMENTS
*
* Some default SQL statements/clauses.
* 
* @version	1
*
*	@package wikindx4\core\sql
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class SQLSTATEMENTS
{
/** object */
private $db;
/** object */
private $session;
/** object */
private $common;
/** object */
private $commonBib;
/** object */
private $stats;
/** string */
private $totalResourceSubquery;
/** string */
public $unions = FALSE;
/** array */
public $doneJoins = array();
/** array */
public $subQueryFields = array();
/** string */
private $countQuery = FALSE;
/** string */
private $countAlphaQuery = FALSE;
/** string */
public $listMethodAscDesc = 'list_AscDesc';
/** string */
public $listType = 'list';
/** array */
public $execCond = array();
/** array */
public $execCondOneField = array();
/** array */
public $execJoin = array();
/** array */
public $orderedJoins = array();
/** string */
public $listQuarantined = FALSE;
/** string */
private $metadataSubquery = FALSE;
/** array */
private $listFields;
/** array */
private $resourceFields;
/** array */
private $resourceMiscFields;
/** array */
private $textFields;
/** array */
private $yearFields;
/** array */
private $pageFields;
/** array */
private $summaryFields;
/** array */
private $timestampFields;
/** array */
private $publisherFields;
/** array */
private $collectionFields;
/** array */
private $userFields;
/** array */
private $attachmentFields;
/** array */
private $categoryFields;
/** array */
private $keywordFields;
/** array */
private $musingFields;
/** array */
private $languageFields;
/** boolean */
public $metadataPaging = FALSE;

/**
* SQLSTATEMENTS
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->common = FACTORY_LISTCOMMON::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		$this->stats = FACTORY_STATISTICS::getInstance();
		$res = FACTORY_RESOURCECOMMON::getInstance();
		$res->tableSetup();
		$this->listFields = $res->listFields;
		$this->resourceFields = $res->resourceFields;
		$this->resourceMiscFields = $res->resourceMiscFields;
		$this->textFields = $res->textFields;
		$this->yearFields = $res->yearFields;
		$this->pageFields = $res->pageFields;
		$this->summaryFields = $res->summaryFields;
		$this->timestampFields = $res->timestampFields;
		$this->publisherFields = $res->publisherFields;
		$this->collectionFields = $res->collectionFields;
		$this->userFields = $res->userFields;
		$this->attachmentFields = $res->attachmentFields;
		$this->categoryFields = $res->categoryFields;
		$this->keywordFields = $res->keywordFields;
		$this->musingFields = $res->musingFields;
		$this->languageFields = $res->languageFields;
	}
/**
* Select front page resources according to $limit resources from last $days
*
* @param int $days
* @param int $limit
* @return string SQL statement ready to be executed
*/
	public function frontSetDays($days, $limit)
	{
		$this->quarantineCond(TRUE);
		$this->db->formatConditions("DATE_SUB(CURDATE(), INTERVAL $days DAY)<=" . 
			$this->db->formatFields('resourcetimestampTimestamp'));
		$this->db->limit($limit, 0);
		$this->db->ascDesc = $this->db->desc;
		$this->db->orderBy($this->db->formatFields('resourcetimestampTimestamp'), FALSE);
		$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourcetimestampId');
		$subQuery = $this->db->subQuery($this->db->queryNoExecute(
			$this->db->selectNoExecute('resource_timestamp', $this->db->formatFields(array_merge($this->resourceMiscFields, 
			$this->timestampFields)), TRUE, FALSE)), 't');
		$split = explode('FROM (', $subQuery);
		$split = explode(') AS ', $split[1]);
		$this->session->setVar('list_SubQuery', base64_encode($split[0]));
		$this->db->ascDesc = $this->db->desc;
		$this->db->orderBy($this->db->formatFields('resourcetimestampTimestamp'), FALSE);
		$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
			$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
		$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
			$this->db->tidyInput('book') . $this->db->or . 
			$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
			$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
			$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
		$this->db->orderBy('resourceTitleSort', FALSE);
		$this->db->leftJoin('resource', 'resourceId', 'resourcetimestampId');
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcetimestampId'); 
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId'); 
		$this->db->leftJoin('resource_year', 'resourceyearId', 'resourcetimestampId');
		$this->db->leftJoin('resource_text', 'resourcetextId', 'resourcetimestampId');
		$this->db->leftJoin('resource_page', 'resourcepageId', 'resourcetimestampId');
		$this->db->leftJoin('resource_summary', 'resourcesummaryId', 'resourcetimestampId');
		$this->db->leftJoin('resource_musing', 'resourcemusingresourceId', 'resourcetimestampId');
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$this->db->leftJoin('publisher', 'resourcemiscPublisher', 'publisherId');
		$this->db->leftJoin('collection', 'resourcemiscCollection', 'collectionId');
		$this->db->leftJoin('users', 'usersId', $this->db->caseWhen('resourcemiscEditUserIdResource', 'IS NOT NULL', 
			'resourcemiscEditUserIdResource', 'resourcemiscAddUserIdResource'), FALSE);
		$listQuery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, $this->db->formatFields($this->listFields), 
			$subQuery, FALSE, FALSE));
		$split = explode('FROM (', $subQuery);
		$split2 = explode(') AS ', $split[1]);
		$query = preg_replace('/LIMIT \d+, \d+/', '', $split2[0]);
		$this->session->setVar('list_SubQuery', base64_encode($query));
		$this->session->delVar('list_NextPreviousIds');
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$this->session->delVar('setup_PagingTotal');
		return $listQuery;
	}
/**
* Select front page resources according to latest $limit resources
*
* @param int $limit
* @return string SQL statement ready to be executed
*/
	public function frontSetNumber($limit)
	{
		$this->quarantineCond(TRUE);
		$this->db->limit($limit, 0);
		$this->db->ascDesc = $this->db->desc;
		$this->db->orderBy($this->db->formatFields('resourcetimestampTimestamp'), FALSE);
		$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourcetimestampId');
		$subQuery = $this->db->subQuery($this->db->queryNoExecute(
			$this->db->selectNoExecute('resource_timestamp', $this->db->formatFields(array_merge($this->resourceMiscFields, 
			$this->timestampFields)), TRUE, FALSE)), 't');
		$split = explode('(', $subQuery, 2);
		$split = explode(')', $split[1]);
		$this->session->setVar('list_SubQuery', base64_encode($split[0]));
		$this->db->ascDesc = $this->db->desc;
		$this->db->orderBy($this->db->formatFields('resourcetimestampTimestamp'), FALSE);
		$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
			$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
		$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
			$this->db->tidyInput('book') . $this->db->or . 
			$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
			$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
			$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
		$this->db->orderBy('resourceTitleSort');
		$this->db->leftJoin('resource', 'resourceId', 'resourcetimestampId');
		$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcetimestampId'); 
		$this->db->leftJoin('creator', 'creatorId', 'resourcecreatorCreatorId');
		$this->db->leftJoin('resource_year', 'resourceyearId', 'resourcetimestampId');
		$this->db->leftJoin('resource_text', 'resourcetextId', 'resourcetimestampId');
		$this->db->leftJoin('resource_page', 'resourcepageId', 'resourcetimestampId');
		$this->db->leftJoin('resource_summary', 'resourcesummaryId', 'resourcetimestampId');
		$this->db->leftJoin('resource_musing', 'resourcemusingresourceId', 'resourcetimestampId');
		$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		$this->db->leftJoin('publisher', 'resourcemiscPublisher', 'publisherId');
		$this->db->leftJoin('collection', 'resourcemiscCollection', 'collectionId');
		$this->db->leftJoin('users', 'usersId', $this->db->caseWhen('resourcemiscEditUserIdResource', 'IS NOT NULL', 
			'resourcemiscEditUserIdResource', 'resourcemiscAddUserIdResource'), FALSE);
		$listQuery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, $this->db->formatFields($this->listFields), 
			$subQuery, FALSE, FALSE));
		$split = explode('FROM (', $subQuery);
		$split2 = explode(') AS ', $split[1]);
		$query = preg_replace('/LIMIT \d+, \d+/', '', $split2[0]);
		$this->session->setVar('list_SubQuery', base64_encode($query));
		$this->session->delVar('list_NextPreviousIds');
		$this->session->delVar('mywikindx_PagingStart');
		$this->session->delVar('mywikindx_PagingStartAlpha');
		$this->session->delVar('setup_PagingTotal');
		return $listQuery;
	}
/**
* Create LIST statement
*
* @param string $addEditType
* @param string $addEditValue
* @param string $order Default is 'creator'
* @return string SQL statement ready to be executed
*/
	public function listList($addEditType, $addEditValue, $order = 'creator')
	{
		$this->session->setVar('sql_TotalSql', base64_encode($this->totalResourceSubquery));
		$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
		$np = array();
		if($addEditType)
		{
			if(($this->session->getVar('setup_PagingStyle') != 'A') || (($this->session->getVar('setup_PagingStyle') == 'A') && 
				($order != 'title') && ($order != 'creator') && ($order != 'attachments')))
				$this->db->limit($this->session->getVar('setup_Paging'), $this->common->pagingObject->start);// "LIMIT $limitStart, $limit";
			$this->db->orderBy('resourcecreatorCreatorSurname');
			$this->db->orderBy('resourceTitleSort');
			$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
			if(!array_key_exists('resource_year', $this->doneJoins))
				$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
			$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
			$this->db->orderBy('creatorSurname');
			$this->db->orderBy('resourceTitleSort');
			$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
				$this->db->tidyInput('book') . $this->db->or . 
				$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
				$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
				$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
			$subQuery = $this->db->subQueryFields(
				array_unique(array_merge($this->resourceFields, $this->resourceMiscFields, array('creatorSurname'), 
				$this->yearFields)), $totalSubQuery, 't2', TRUE, FALSE);
			$this->doneJoins['resource'] = FALSE;
			$this->doneJoins['resource_misc'] = FALSE;
			$this->doneJoins['resource_year'] = FALSE;
			$this->doneJoins['resource_creator'] = FALSE;
		}
		else // list all and order by...
		{
			if(($this->session->getVar('setup_PagingStyle') != 'A') || (($this->session->getVar('setup_PagingStyle') == 'A') && 
				($order != 'title') && ($order != 'creator') && ($order != 'attachments')))
				$this->db->limit($this->session->getVar('setup_Paging'), $this->common->pagingObject->start); // "LIMIT $limitStart, $limit";
			if($order == 'creator')
			{
				$this->db->orderBy('c.resourcecreatorCreatorSurname');
				$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
				if(!array_key_exists('resource', $this->doneJoins))
					$this->db->leftJoin('resource', 'resourceId', 'resourcemiscId');
				if(!array_key_exists('resource_year', $this->doneJoins))
					$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
				$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
				$this->db->orderBy('creatorSurname');
				$this->db->orderBy('resourceTitleSort');
				$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
					$this->db->tidyInput('book') . $this->db->or . 
					$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
					$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
					$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
				if(strpos($this->totalResourceSubquery, 'WHERE ') === FALSE)
				{
					$np['sub'] = $totalSubQuery;
					$np['orders'] = $this->db->order;
					$np['joins']['resource_year']['intField'] = 'resourceyearId';
					$np['joins']['resource_year']['extField'] = 'resourcemiscId';
					$np['joins']['resource']['intField'] = 'resourceId';
					$np['joins']['resource']['extField'] = 'resourcemiscId';
					$np['fields'][] = 'creatorSurname';
				}
				$subQuery = $this->db->subQueryFields(
					array_unique(array_merge($this->resourceFields, $this->subQueryFields, $this->resourceMiscFields, 
					array('creatorSurname'), $this->yearFields)), $totalSubQuery, 't2', TRUE, FALSE);
				$this->doneJoins['resource'] = FALSE;
				$this->doneJoins['resource_misc'] = FALSE;
				$this->doneJoins['resource_year'] = FALSE;
				$this->doneJoins['resource_creator'] = FALSE;
			}
			else if($order == 'title')
			{
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->db->orderBy('resourceTitleSort');
					$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
						$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
					$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
						$this->db->tidyInput('book') . $this->db->or . 
						$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
						$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
						$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
					$subQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
				}
				else
				{
					$this->db->orderBy('resourceTitleSort');
					$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
					if(!array_key_exists('resource_year', $this->doneJoins))
						$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
					$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
					$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
					$this->db->orderBy('resourceTitleSort');
					$this->db->orderBy('creatorSurname');
					$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
						$this->db->tidyInput('book') . $this->db->or . 
						$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
						$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
						$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
					if(strpos($this->totalResourceSubquery, 'WHERE ') === FALSE)
					{
						$np['sub'] = $totalSubQuery;
						$np['orders'] = $this->db->order;
						$np['joins']['resource_year']['intField'] = 'resourceyearId';
						$np['joins']['resource_year']['extField'] = 'resourcemiscId';
						$np['joins']['resource_creator']['intField'] = 'resourcecreatorResourceId';
						$np['joins']['resource_creator']['extField'] = 'resourcemiscId';
						$np['fields'][] = array('resourcecreatorCreatorSurname' => 'creatorSurname');
					}
					$subQuery = $this->db->subQueryFields(
						array_unique(array_merge($this->resourceFields, $this->subQueryFields, 
						array(array('resourcecreatorCreatorSurname' => 'creatorSurname')), $this->yearFields)), 
						$totalSubQuery, 't2', TRUE, TRUE);
				}
				$this->doneJoins['resource'] = FALSE;
				$this->doneJoins['resource_misc'] = FALSE;
				$this->doneJoins['resource_year'] = FALSE;
				$this->doneJoins['resource_creator'] = FALSE;
			}
			else if($order == 'publisher')
			{
				$this->db->orderBy('publisherName');
				$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
				if(!array_key_exists('resource', $this->doneJoins))
					$this->db->leftJoin('resource', 'resourceId', 'resourcemiscId');
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
				if(!array_key_exists('resource_year', $this->doneJoins))
					$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
				$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
				$this->db->orderBy('publisherName');
				$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
					$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
				$this->db->orderBy('resourceTitleSort');
				$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
					$this->db->tidyInput('book') . $this->db->or . 
					$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
					$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
					$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
				if(strpos($this->totalResourceSubquery, 'WHERE ') === FALSE)
				{
					$np['sub'] = $totalSubQuery;
					$np['orders'] = $this->db->order;
					$np['joins']['resource_year']['intField'] = 'resourceyearId';
					$np['joins']['resource_year']['extField'] = 'resourcemiscId';
					$np['joins']['resource_creator']['intField'] = 'resourcecreatorResourceId';
					$np['joins']['resource_creator']['extField'] = 'resourcemiscId';
					$np['joins']['resource']['intField'] = 'resourceId';
					$np['joins']['resource']['extField'] = 'resourcemiscId';
					$np['fields'][] = array('resourcecreatorCreatorSurname' => 'creatorSurname');
					$np['fields'][] = 'publisherName';
				}
				$subQuery = $this->db->subQueryFields(
					array_unique(array_merge($this->resourceFields, $this->subQueryFields, $this->resourceMiscFields, 
					array(array('resourcecreatorCreatorSurname' => 'creatorSurname')), $this->publisherFields, 
					$this->yearFields)), 
					$totalSubQuery, 't2', TRUE, TRUE);
				$this->doneJoins['resource'] = FALSE;
				$this->doneJoins['resource_misc'] = FALSE;
				$this->doneJoins['publisher'] = FALSE;
				$this->doneJoins['resource_year'] = FALSE;
				$this->doneJoins['resource_creator'] = FALSE;
			}
			else if($order == 'year')
			{
				$this->db->orderBy($this->db->coalesce(array('resourceyearYear1', 'resourceyearYear2', 'resourceyearYear3')), FALSE);
				$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
				if(!array_key_exists('resource', $this->doneJoins))
					$this->db->leftJoin('resource', 'resourceId', 'resourceyearId');
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
				$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
				$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
					$this->db->tidyInput('book') . $this->db->or . 
					$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
					$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
					$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
				$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
					$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
				$this->db->orderBy('resourceTitleSort');
				if(strpos($this->totalResourceSubquery, 'WHERE ') === FALSE)
				{
					$np['sub'] = $totalSubQuery;
					$np['orders'] = $this->db->order;
					$np['joins']['resource_creator']['intField'] = 'resourcecreatorResourceId';
					$np['joins']['resource_creator']['extField'] = 'resourceId';
					$np['joins']['resource']['intField'] = 'resourceId';
					$np['joins']['resource']['extField'] = 'resourcemiscId';
					$np['fields'][] = array('resourcecreatorCreatorSurname' => 'creatorSurname');
					$np['fields'][] = 'resourceyearYear1';
					$np['fields'][] = 'resourceyearYear2';
				}
				$subQuery = $this->db->subQueryFields(
					array_unique(array_merge($this->resourceFields, array(array('resourcecreatorCreatorSurname' => 'creatorSurname')), 
					$this->subQueryFields, $this->yearFields)), $totalSubQuery, 't2', TRUE, TRUE);
				$this->doneJoins['resource'] = FALSE;
				$this->doneJoins['resource_misc'] = FALSE;
				$this->doneJoins['resource_year'] = FALSE;
				$this->doneJoins['resource_creator'] = FALSE;
			}
			else if($order == 'timestamp')
			{
				$this->db->orderBy('resourcetimestampTimestamp');
				$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
				if(!array_key_exists('resource', $this->doneJoins))
					$this->db->leftJoin('resource', 'resourceId', 'resourcetimestampId');
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
				if(!array_key_exists('resource_year', $this->doneJoins))
					$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
				$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
				$this->db->orderBy('resourcetimestampTimestamp');
				$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
					$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
				$this->db->orderBy('resourceTitleSort');
				$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
					$this->db->tidyInput('book') . $this->db->or . 
					$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
					$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
					$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
				if(strpos($this->totalResourceSubquery, 'WHERE ') === FALSE)
				{
					$np['sub'] = $totalSubQuery;
					$np['orders'] = $this->db->order;
					$np['joins']['resource']['intField'] = 'resourceId';
					$np['joins']['resource']['extField'] = 'resourcemiscId';
					$np['joins']['resource_year']['intField'] = 'resourceyearId';
					$np['joins']['resource_year']['extField'] = 'resourcemiscId';
					$np['joins']['resource_creator']['intField'] = 'resourcecreatorResourceId';
					$np['joins']['resource_creator']['extField'] = 'resourcemiscId';
					$np['fields'][] = array('resourcecreatorCreatorSurname' => 'creatorSurname');
					$np['fields'][] = 'resourcetimestampTimestamp';
				}
				$subQuery = $this->db->subQueryFields(
					array_unique(array_merge($this->resourceFields, $this->resourceMiscFields, $this->subQueryFields, 
					array(array('resourcecreatorCreatorSurname' => 'creatorSurname')), $this->yearFields, $this->timestampFields)), 
					$totalSubQuery, 't2', TRUE, TRUE);
				$this->doneJoins['resource'] = FALSE;
				$this->doneJoins['resource_misc'] = FALSE;
				$this->doneJoins['resource_timestamp'] = FALSE;
				$this->doneJoins['resource_year'] = FALSE;
				$this->doneJoins['resource_creator'] = FALSE;
			}
			else if(($order == 'viewsIndex') || ($order == 'downloadsIndex') || ($order == 'popularityIndex'))
			{
				$this->db->orderBy('index');
				$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
				if(!array_key_exists('resource', $this->doneJoins))
					$this->db->leftJoin('resource', 'resourceId', 'resourcemiscId');
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
				if(!array_key_exists('resource_year', $this->doneJoins))
					$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
				$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
				$this->db->orderBy('index');
				$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
					$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
				$this->db->orderBy('resourceTitleSort');
				$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
					$this->db->tidyInput('book') . $this->db->or . 
					$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
					$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
					$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
				if(($order == 'viewsIndex') || ($order == 'popularityIndex'))
					$subQuery = $this->db->subQueryFields(
						array_merge($this->resourceFields, $this->resourceMiscFields, $this->timestampFields, 
						array(array('resourcecreatorCreatorSurname' => 'creatorSurname'), 'resourceyearId', 'resourceyearYear1', 
						'resourceyearYear2', 'resourceyearYear3', 'resourceyearYear4', 'index')), 
						$totalSubQuery, 't2', TRUE, TRUE);
				else if($order == 'downloadsIndex')
					$subQuery = $this->db->subQueryFields(
						array_merge($this->resourceFields, $this->resourceMiscFields, 
						array(array('resourcecreatorCreatorSurname' => 'creatorSurname'), 'resourceyearId', 'resourceyearYear1', 
						'resourceyearYear2', 'resourceyearYear3', 'resourceyearYear4', 'index')), 
						$totalSubQuery, 't2', TRUE, TRUE);
				$this->doneJoins['resource'] = FALSE;
				$this->doneJoins['resource_misc'] = FALSE;
				$this->doneJoins['resource_year'] = FALSE;
				$this->doneJoins['resource_creator'] = FALSE;
			}
			else if($order == 'maturityIndex')
			{
				$this->db->orderBy('resourcemiscMaturityIndex');
				$totalSubQuery = $this->db->subQuery($this->totalResourceSubquery, 't1', TRUE, TRUE);
				if(!array_key_exists('resource', $this->doneJoins))
					$this->db->leftJoin('resource', 'resourceId', 'resourcemiscId');
				$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourceId');
				if(!array_key_exists('resource_year', $this->doneJoins))
					$this->db->leftJoin('resource_year', 'resourceyearId', 'resourceId');
				$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
				$this->db->orderBy('resourcemiscMaturityIndex');
				$this->db->orderBy($this->db->ifClause($this->db->formatFields('creatorSurname'), 'IS NOT NULL', 
					$this->db->formatFields('creatorSurname'), $this->db->formatFields('resourceTitleSort')), FALSE);
				$this->db->orderBy('resourceTitleSort');
				$this->db->orderBy($this->db->caseWhen('(' . $this->db->formatFields('resourceType'), $this->db->equal . 
					$this->db->tidyInput('book') . $this->db->or . 
					$this->db->formatFields('resourceType') . $this->db->equal . $this->db->tidyInput('book_article') . ')' . 
					$this->db->and . $this->db->formatFields('resourceyearYear2') . ' IS NOT NULL ', 
					$this->db->formatFields('resourceyearYear2'), $this->db->formatFields('resourceyearYear1'), FALSE), FALSE);
				if(strpos($this->totalResourceSubquery, 'WHERE ') === FALSE)
				{
					$np['sub'] = $totalSubQuery;
					$np['orders'] = $this->db->order;
					$np['joins']['resource']['intField'] = 'resourceId';
					$np['joins']['resource']['extField'] = 'resourcemiscId';
					$np['joins']['resource_year']['intField'] = 'resourceyearId';
					$np['joins']['resource_year']['extField'] = 'resourcemiscId';
					$np['joins']['resource_creator']['intField'] = 'resourcecreatorResourceId';
					$np['joins']['resource_creator']['extField'] = 'resourcemiscId';
					$np['fields'][] = array('resourcecreatorCreatorSurname' => 'creatorSurname');
				}
				$subQuery = $this->db->subQueryFields(
					array_merge($this->resourceFields, $this->resourceMiscFields, array(array('resourcecreatorCreatorSurname' => 'creatorSurname'), 
					'resourceyearId', 'resourceyearYear1', 'resourceyearYear2', 'resourceyearYear3', 'resourceyearYear4')), 
					$totalSubQuery, 't2', TRUE, TRUE);
				$this->doneJoins['resource'] = FALSE;
				$this->doneJoins['resource_misc'] = FALSE;
				$this->doneJoins['resource_year'] = FALSE;
				$this->doneJoins['resource_creator'] = FALSE;
			}
		}
		if(!array_key_exists('resource_misc', $this->doneJoins))
			$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourceId');
		if(!array_key_exists('resource', $this->doneJoins))
			$this->db->leftJoin('resource', 'resourceId', 'resourcemiscId');
		if(!array_key_exists('resource_creator', $this->doneJoins))
			$this->db->leftJoin('resource_creator', 'resourceId', 'resourcemiscId');
		if(!array_key_exists('resource_year', $this->doneJoins))
			$this->db->leftJoin('resource_year', 'resourceyearId', 'resourcemiscId');
		if(!array_key_exists('resource_timestamp', $this->doneJoins))
			$this->db->leftJoin('resource_timestamp', 'resourcetimestampId', 'resourcemiscId');
		if(!array_key_exists('resource_text', $this->doneJoins))
			$this->db->leftJoin('resource_text', 'resourcetextId', 'resourcemiscId');
		if(!array_key_exists('resource_page', $this->doneJoins))
			$this->db->leftJoin('resource_page', 'resourcepageId', 'resourcemiscId');
		if(!array_key_exists('resource_summary', $this->doneJoins))
			$this->db->leftJoin('resource_summary', 'resourcesummaryId', 'resourcemiscId');
		if(!array_key_exists('resource_musing', $this->doneJoins))
			$this->db->leftJoin('resource_musing', 'resourcemusingResourceId', 'resourcemiscId');
		if(!array_key_exists('resource_musing_text', $this->doneJoins))
			$this->db->leftJoin('resource_musing_text', 'resourcemusingtextId', 'resourcemusingId');
		if(!array_key_exists('publisher', $this->doneJoins))
			$this->db->leftJoin('publisher', 'publisherId', 'resourcemiscPublisher');
		if(!array_key_exists('collection', $this->doneJoins))
			$this->db->leftJoin('collection', 'collectionId', 'resourcemiscCollection');
		if(!array_key_exists('users', $this->doneJoins))
			$this->db->leftJoin('users', $this->db->formatFields('usersId'), 
			$this->db->caseWhen('resourcemiscEditUserIdResource', 'IS NOT NULL', 
			'resourcemiscEditUserIdResource', 'resourcemiscAddUserIdResource'), FALSE);
		$listQuery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
			$this->db->formatFields($this->listFields), $subQuery, TRUE, FALSE));
		$this->saveSubQuery($subQuery, $np, $order);
		$this->doneJoins = array();
		return $listQuery;
	}
/**
* Save the subquery for use in RESOURCEVIEW and setting next/previous links
*
* @param string $subQuery
* @param array
*/
	private function saveSubQuery($subQuery, $np, $order)
	{
// If not alpha paging, save subquery so that next and previous links can be created when viewing single resources.
		if(($this->session->getVar('setup_PagingStyle') != 'A') || (($this->session->getVar('setup_PagingStyle') == 'A') && 
			($order != 'title') && ($order != 'creator') && ($order != 'attachments')))
		{
			if($this->common->pagingObject->start == 0)
				$this->session->delVar('list_NextPreviousIds');
			if(!empty($np)) // List All -- no conditions set
			{
				$this->db->order = $np['orders'];
				$np['fields'][] = 'resourcemiscId';
				$tables = array('resource_misc', 'resource_attachments', 'resource', 'resource_creator', 'creator', 
					'resource_text', 'resource_category', 'resource_keyword', 'keyword', 'resource_timestamp', 'resource_year', 
					'resource_quote', 'resource_quote_text', 'resource_paraphrase', 'resource_paraphrase_text', 
					'resource_musing', 'resource_musing_text', 'resource_user_tags', 'user_tags', 'resource_custom', 
					'user_bibliography_resource', 'publisher', 'resource_language', 'resource_quote_comment', 'resource_paraphrase_comment');
				foreach($tables as $tableOrder)
				{
					foreach($np['joins'] as $table => $array)
					{
						$tSplit = explode('.', $table);
						if((sizeof($tSplit) == 2) && ($tSplit[1] == $tableOrder))
						{
							$array['alias'] = $tSplit[0];
							$array['table'] = $tSplit[1];
							$this->db->leftJoin(array(array($array['table'] => $array['alias'])), $array['intField'], $array['extField']);
						}
						else if($table == $tableOrder)
							$this->db->leftJoin($table, $array['intField'], $array['extField']);
					}
				}
				$query = $this->db->selectNoExecuteFromSubQuery(FALSE, $np['fields'], $np['sub'], TRUE, TRUE, TRUE);
			}
			else // conditions set
			{
				$split = explode('FROM (', $subQuery, 2);
				$split = explode('FROM (', $split[1], 2);
				$query = $split[0] . ' FROM (' . $split[1];
				$split = explode(') AS ', $query, -1);
				$query = join(') AS ', $split);
				$query = preg_replace('/LIMIT \d+, \d+/', '', $query);
			}
			$query = base64_encode($query);
			$this->session->setVar('list_SubQueryMulti', $query);
			$this->session->setVar('list_SubQuery', $query);
		}
		else if(($this->session->getVar('setup_PagingStyle') == 'A') && 
			(($order == 'title') || ($order == 'creator') || ($order == 'attachments')))
			$this->session->delVar('list_SubQuery');
	}
/**
* Create subquery initial order SELECT statement with minimal fields
*
* @param string $addEditType
* @param string $addEditValue
* @param string $order Default is FALSE
* @param boolean $distinct Default is FALSE
* @param string $queryString Default is FALSE
* @param string $totalSubQuery Default is FALSE
*/
	public function createTotalResourceSubqueryList($addEditType, $addEditValue, $order = FALSE, $distinct = FALSE, $queryString = FALSE, 
		$totalSubQuery = FALSE)
	{
		if($this->metadataPaging)
			$this->common->metadataPaging = TRUE;
		$this->quarantineCond();
		if($addEditType) // $order should be FALSE but we order by creator then title
		{
			if($this->unions)
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array(array('resourcemiscId' => 'miscId'), 'creatorSurname', 'resourceTitle'), $this->db->subQuery($this->unions)));
			else
			{
				$this->commonBib->userBibCondition('resourcecreatorResourceId', TRUE);
				$this->db->leftJoin('resource', 'resourceId', 'resourcecreatorResourceId' );
				$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourceId');
				$this->db->formatConditions(array($addEditType => $addEditValue));
				$this->db->reset = FALSE;
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->countAlphaQuery = $this->db->select('resource_creator', 
					array(array('resourcecreatorCreatorSurname' => 'creatorSurname')), TRUE);
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->countQuery = $this->db->selectCountDistinctField('resource_creator', 'resourcecreatorResourceId');
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, 'creator', $queryString, $this->countAlphaQuery);
				$this->db->reset = TRUE;
				$this->pagingAlphaCondition('creator');
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_creator', 
					array_merge($this->resourceFields, $this->resourceMiscFields, 
					array(array('resourcecreatorCreatorSurname' => 'creatorSurname'), 'resourcecreatorResourceId')), TRUE) . 
					$this->db->indexHint('resourcecreatorCreatorSurname'));
			}
		}
// sometimes (e.g. LISTSOMERESOURCES::citeProcess()), the totalSubQuery is derived elsewhere.  Default $order is 'creator'
		else if($totalSubQuery) 
		{
			$subQueryFields = array();
			if(!$order)
				$order = 'creator';
			if(!array_key_exists('resource_misc', $this->doneJoins))
			{
				$this->execJoin['resource_misc']['intField'] = 'resourcemiscId';
				$this->execJoin['resource_misc']['extField'] = 'rId';
			}
			$this->commonBib->userBibCondition('resourcemiscId', TRUE);
			if($order == 'creator')
			{
				if(!array_key_exists('c', $this->doneJoins))
					$this->db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'rId');
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->executeCondJoin();
					$this->countAlphaQuery = $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 
						array(array('resourcecreatorCreatorSurname' => 'creatorSurname'), 'resourcecreatorResourceId'), $totalSubQuery, TRUE));
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields);
				$subQueryFields[] = array('c.resourcecreatorCreatorSurname' => 'creatorSurname');
			}
			else if($order == 'title')
			{
				$this->execJoin['resource']['intField'] = 'resourceId';
				$this->execJoin['resource']['extField'] = 'resourcemiscId';
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->executeCondJoin();
					$this->countAlphaQuery = $this->db->query($this->db->selectNoExecuteFromSubQuery(FALSE, 
						'resourceTitleSort', $totalSubQuery, TRUE));
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->resourceFields);
			}
			else if($order == 'publisher')
			{
				$this->execJoin['publisher']['intField'] = 'publisherId';
				$this->execJoin['publisher']['extField'] = 'resourcemiscPublisher';
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->publisherFields);
			}
			else if($order == 'year')
			{
				$this->execJoin['resource_year']['intField'] = 'resourceyearId';
				$this->execJoin['resource_year']['extField'] = 'resourcemiscId';
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->yearFields);
			}
			else if($order == 'timestamp')
			{
				$this->execJoin['resource_timestamp']['intField'] = 'resourcetimestampId';
				$this->execJoin['resource_timestamp']['extField'] = 'resourcemiscId';
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->timestampFields);
			}
			$this->executeCondJoin();
			$this->countQuery = $this->db->selectCountFromSubquery('rId', $totalSubQuery);
			$cq = $this->db->sqlstring;
			$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
			$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
			$this->pagingAlphaCondition($order);
			if($this->session->getVar('setup_PagingStyle') == 'A')
			{
				$this->execJoin['resource_year']['intField'] = 'resourceyearId';
				$this->execJoin['resource_year']['extField'] = 'resourcemiscId';
				$this->executeCondJoin();
				if(!array_key_exists('c', $this->doneJoins))
					$this->db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'resourcemiscId');
				$this->subQueryFields = array_merge($this->subQueryFields, $this->yearFields);
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubquery(FALSE, 
					array_merge(array_unique($this->subQueryFields), array(array('c.resourcecreatorCreatorSurname' => 'creatorSurname'))), 
					$totalSubQuery, TRUE));
			}
			else
			{
				$this->executeCondJoin();
				if(($order == 'creator') && (!array_key_exists('c', $this->doneJoins)))
					$this->db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'resourcemiscId');
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubquery(FALSE, 
					array_merge(array_unique($this->subQueryFields), $subQueryFields), $totalSubQuery, TRUE));
			}
		}
		else if($order == 'title')
		{
			if($this->unions)
			{
				$this->commonBib->userBibCondition('id', TRUE);
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{		
					$this->db->leftJoin('resource', 'resourceId', 'id');
					$this->executeCondJoin();
					$this->countAlphaQuery = $this->db->selectFromSubQuery(FALSE, 'resourceTitleSort', 
						$this->db->subQuery($this->unions, 'u'), TRUE);
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField(FALSE, 'id', $this->db->subQuery($this->unions, 'u'));
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->pagingAlphaCondition($order);
				$this->executeCondJoin();
				if(!array_key_exists('resource_misc', $this->doneJoins))
					$this->db->leftJoin('resource_misc', 'resourcemiscId', 'id');
				if(!array_key_exists('resource', $this->doneJoins))
					$this->db->leftJoin('resource', 'resourceId', 'id');
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->resourceFields);
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array_unique($this->subQueryFields), $this->db->subQuery($this->unions, 'u')));
			}
			else
			{
				if(!array_key_exists('resource', $this->doneJoins))
				{
					$this->execJoin['resource']['intField'] = 'resourceId';
					$this->execJoin['resource']['extField'] = 'resourcemiscId';
				}
				$this->commonBib->userBibCondition('resourcemiscId', TRUE);
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->executeCondJoin();
					$this->countAlphaQuery = $this->db->select('resource_misc', 'resourceTitleSort');
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->pagingAlphaCondition($order);
				if(!array_key_exists('resource_creator', $this->doneJoins))
					$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
				$this->executeCondJoin();
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->resourceFields);
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->subQueryFields = array_merge($this->subQueryFields, $this->yearFields);
					if(!array_key_exists('c', $this->doneJoins))
						$this->db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'resourcemiscId');
					if(!array_key_exists('resource_year', $this->doneJoins))
						$this->db->leftJoin('resource_year', 'resourceyearId', 'resourcemiscId');
					$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
						array_merge(array_unique($this->subQueryFields), array(array('c.resourcecreatorCreatorSurname' => 'creatorSurname'))), 
						$distinct));
				}
				else
					$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
						array_unique($this->subQueryFields), $distinct));
			}
		}
		else if($order == 'publisher')
		{
			if($this->unions)
			{
				$this->commonBib->userBibCondition('id', TRUE);
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField(FALSE, 'id', $this->db->subQuery($this->unions, 'u'));
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->executeCondJoin();
				if(!array_key_exists('resource_misc', $this->doneJoins))
					$this->db->leftJoin('resource_misc', 'resourcemiscId', 'id');
				if(!array_key_exists('publisher', $this->doneJoins))
					$this->db->leftJoin('publisher', 'publisherId', 'resourcemiscPublisher');
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array_unique(array_merge($this->resourceMiscFields, $this->publisherFields, $this->subQueryFields)), 
					$this->db->subQuery($this->unions, 'u')));
			}
			else
			{
				$this->commonBib->userBibCondition('resourcemiscId', TRUE);
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				if(!array_key_exists('resource_creator', $this->doneJoins))
					$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
				$this->executeCondJoin();
				if(!array_key_exists('publisher', $this->doneJoins))
					$this->db->leftJoin('publisher', 'publisherId', 'resourcemiscPublisher');
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
					array_unique(array_merge($this->resourceMiscFields, $this->publisherFields, $this->subQueryFields)), $distinct));
			}
		}
		else if($order == 'year')
		{
			if($this->unions)
			{
				$this->commonBib->userBibCondition('id', TRUE);
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField(FALSE, 'id', $this->db->subQuery($this->unions, 'u'));
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->executeCondJoin();
				if(!array_key_exists('resource_misc', $this->doneJoins))
					$this->db->leftJoin('resource_misc', 'resourcemiscId', 'id');
				if(!array_key_exists('resource_year', $this->doneJoins))
					$this->db->leftJoin('resource_year', 'resourceyearId', 'resourcemiscId');
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->yearFields);
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array_unique($this->subQueryFields), $this->db->subQuery($this->unions, 'u')));
			}
			else
			{
				$this->commonBib->userBibCondition('resourcemiscId', TRUE);
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
				if(!array_key_exists('resource_year', $this->doneJoins))
					$this->db->leftJoin('resource_year', 'resourceyearId', 'resourcemiscId');
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				if(!array_key_exists('resource_creator', $this->doneJoins))
					$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
				$this->executeCondJoin();
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields, $this->yearFields);
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
					array_unique($this->subQueryFields), $distinct));
			}
		}
		else if($order == 'timestamp')
		{
			if($this->unions)
			{
				$this->commonBib->userBibCondition('id', TRUE);
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField(FALSE, 'id', $this->db->subQuery($this->unions, 'u'));
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				if(!array_key_exists('resource_misc', $this->doneJoins))
					$this->db->leftJoin('resource_misc', 'resourcemiscId', 'id');
				if(!array_key_exists('resource_timestamp', $this->doneJoins))
					$this->db->leftJoin('resource_timestamp', 'resourcetimestampId', 'resourcemiscId');
				$this->executeCondJoin();
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array_unique(array_merge($this->resourceMiscFields, $this->timestampFields, $this->subQueryFields)), 
					$this->db->subQuery($this->unions, 'u')));
			}
			else
			{
				$this->commonBib->userBibCondition('resourcemiscId', TRUE);
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
				if(!array_key_exists('resource_timestamp', $this->doneJoins))
					$this->db->leftJoin('resource_timestamp', 'resourcetimestampId', 'resourcemiscId');
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				if(!array_key_exists('resource_creator', $this->doneJoins))
					$this->db->leftJoin('resource_creator', 'resourcecreatorResourceId', 'resourcemiscId');
				$this->executeCondJoin();
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
					$this->db->formatFields(array_unique(array_merge($this->resourceMiscFields, $this->timestampFields, 
					$this->subQueryFields))), $distinct, FALSE));
			}
		}
		else if($order == 'viewsIndex') // only from LIST -- no conditions elsewhere
		{
			$this->commonBib->userBibCondition('resourcetimestampId');
			$this->countQuery = $this->db->selectCountDistinctField('resource_timestamp', 'resourcetimestampId');
			$cq = $this->db->sqlstring;
			$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
			$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
			$this->commonBib->userBibCondition('resourcemiscId');
			if(!$setSum = $this->stats->getMaxAccessRatio())
				$setSum = 0.1; // Avoids divide by 0 but still produces a 0 result
			$dateDiff = $this->db->dateDiffRatio('resourcemiscAccesses', 'resourcetimestampTimestampAdd');
			$ratio = $this->db->round("$dateDiff / $setSum", 'index');
			$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourcetimestampId');
			$this->commonBib->userBibCondition('resourcetimestampId');
			$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_timestamp', 
				$this->db->formatFields(array_unique(array_merge($this->resourceMiscFields, $this->timestampFields, 
				$this->subQueryFields))) . ', ' . $ratio, $distinct, FALSE));
			$this->doneJoins['resource_timestamp'] = FALSE;
		}
		else if($order == 'downloadsIndex') // only from LIST -- no conditions elsewhere
		{
			$this->commonBib->userBibCondition('resourcemiscId');
			$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
			$cq = $this->db->sqlstring;
			$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
			$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
			$this->commonBib->userBibCondition('resourceattachmentsResourceId');
			if(!$setSum = $this->stats->getMaxDownloadRatio())
				$setSum = 0.1;
			$dateDiff = $this->db->dateDiffRatio('resourceattachmentsDownloads', 'resourceattachmentsTimestamp', FALSE, TRUE);
			$ratio = $this->db->round("$dateDiff / $setSum");
			$case = $this->db->round($this->db->caseWhen($this->db->formatFields('resourceattachmentsResourceId'), 'IS NULL', 
				'0', $ratio, FALSE), 'index');
			$this->db->leftJoin('resource_attachments', 'resourceattachmentsResourceId', 'resourcemiscId');
			$this->db->groupBy(array_merge($this->resourceMiscFields, $this->attachmentFields));
			$this->commonBib->userBibCondition('resourcemiscId');
			$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
				$this->db->formatFields(array_merge($this->resourceMiscFields, $this->attachmentFields)) . ', ' . 
				$case, $distinct, FALSE));
		}
		else if($order == 'popularityIndex') // only from LIST -- no conditions elsewhere
		{
			$this->commonBib->userBibCondition('resourcemiscId');
			$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
			$cq = $this->db->sqlstring;
			$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
			$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
			$this->commonBib->userBibCondition('resourcemiscId');
			if(!$setSumAccess = $this->stats->getMaxAccessRatio())
				$setSumAccess = 0.1;
			$this->commonBib->userBibCondition('resourceattachmentsResourceId');
			if(!$setSumDownload = $this->stats->getMaxDownloadRatio())
				$setSumDownload = 0.1;
			$dateDiffAccess = $this->db->dateDiffRatio('resourcemiscAccesses', 'resourcetimestampTimestampAdd');
			$dateDiffDownload = $this->db->dateDiffRatio('resourceattachmentsDownloads', 'resourceattachmentsTimestamp', FALSE, TRUE);
			$ratioAccess = $this->db->round("$dateDiffAccess / $setSumAccess") . ' * 0.5';
			$ratioDownload = $this->db->round("$dateDiffDownload / $setSumDownload") . ' * 1.5';
			$ratioSum = $ratioAccess . ' + ' . $ratioDownload;
			$case = $this->db->round($this->db->caseWhen($this->db->formatFields('resourceattachmentsResourceId'), 'IS NULL', 
				$ratioAccess, $ratioSum, FALSE), 'index');
			$this->db->leftJoin('resource_misc', 'resourcemiscId', 'resourcetimestampId');
			$this->db->leftJoin('resource_attachments', 'resourceattachmentsResourceId', 'resourcetimestampId');
			$this->db->groupBy(array_merge(array('resourceattachmentsResourceId'), $this->resourceMiscFields, $this->timestampFields));
			$this->commonBib->userBibCondition('resourcetimestampId');
			$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_timestamp', 
				$this->db->formatFields(array_merge($this->resourceMiscFields, $this->timestampFields)) . ', ' . 
				$case, $distinct, FALSE));
			$this->doneJoins['resource_timestamp'] = FALSE;
		}
		else if($order == 'maturityIndex') // only from LIST -- no conditions elsewhere
		{
			$this->commonBib->userBibCondition('resourcemiscId');
			$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
			$cq = $this->db->sqlstring;
			$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
			$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
			$this->pagingAlphaCondition($order);
			$this->commonBib->userBibCondition('resourcemiscId');
			$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
				$this->resourceMiscFields, $distinct));
		}
		else if($order == 'attachments') // NB use return from here as we only list file names
		{
			if(!$this->session->getVar('setup_Superadmin'))
				$this->execCond[] = array('resourceattachmentsEmbargo' => 'N'); // For non-admins, view only unembargoed attachments
			if($this->unions)
			{
				if(!array_key_exists('resource_misc', $this->doneJoins))
				{
					$this->execJoin['resource_misc']['intField'] = 'resourcemiscId';
					$this->execJoin['resource_misc']['extField'] = 'id';
				}
				$this->commonBib->userBibCondition('id', TRUE);
				$this->execJoin['resource_attachments']['intField'] = 'resourceattachmentsResourceId';
				$this->execJoin['resource_attachments']['extField'] = 'id';
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->executeCondJoin();
					$this->countAlphaQuery = $this->db->selectFromSubQuery('resource_attachments', 'resourceattachmentsFileName', 
						$this->db->subQuery($this->unions, 'u'), TRUE);
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField(FALSE, 'resourceattachmentsId', 
					$this->db->subQuery($this->unions, 'u'));
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->pagingAlphaCondition($order);
				if($this->session->getVar('setup_PagingStyle') != 'A')
				{
					$this->db->limit($this->session->getVar('setup_Paging'), $this->common->pagingObject->start);
					$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
					$this->db->orderBy('resourceattachmentsFileName');
				}
				$this->executeCondJoin();
				return $this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array_unique($this->subQueryFields), $this->db->subQuery($this->unions, 'u')));
			}
			else
			{
				if(!array_key_exists('resource_misc', $this->doneJoins))
				{
					$this->execJoin['resource_misc']['intField'] = 'resourcemiscId';
					$this->execJoin['resource_misc']['extField'] = 'resourceattachmentsResourceId';
				}
				$this->commonBib->userBibCondition('resourceattachmentsResourceId', TRUE);
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->executeCondJoin();
					$this->countAlphaQuery = $this->db->select('resource_attachments', 'resourceattachmentsFileName', TRUE);
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField('resource_attachments', 'resourceattachmentsId');
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->pagingAlphaCondition($order);
				if($this->session->getVar('setup_PagingStyle') != 'A')
				{
					$this->db->limit($this->session->getVar('setup_Paging'), $this->common->pagingObject->start);
					$this->db->ascDesc = $this->session->getVar($this->listMethodAscDesc);
					$this->db->orderBy('resourceattachmentsFileName');
				}
				$this->executeCondJoin();
				return $this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecute('resource_attachments', 
					array_unique(array_merge($this->subQueryFields, $this->attachmentFields)), $distinct));
			}
		}
		else // all other $order including 'creator'
		{
			if($this->unions)
			{
				if(!array_key_exists('resource_misc', $this->doneJoins))
				{
					$this->execJoin['resource_misc']['intField'] = 'resourcemiscId';
					$this->execJoin['resource_misc']['extField'] = 'id';
				}
				$this->commonBib->userBibCondition('id', TRUE);
				if(!array_key_exists('c', $this->doneJoins))
				{
					$this->execJoin['c.resource_creator']['intField'] = 'c.resourcecreatorResourceId';
					$this->execJoin['c.resource_creator']['extField'] = 'resourcemiscId';
				}
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->executeCondJoin();
					$this->countAlphaQuery = $this->db->selectFromSubQuery(FALSE, 
						array(array('c.resourcecreatorCreatorSurname' => 'creatorSurname')), $this->db->subQuery($this->unions, 'u'), TRUE);
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField(FALSE, 'id', $this->db->subQuery($this->unions, 'u'));
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->pagingAlphaCondition($order);
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields);
				$this->executeCondJoin();
				$this->totalResourceSubquery = $this->db->queryNoExecute($this->db->selectNoExecuteFromSubQuery(FALSE, 
					array_merge(array_unique($this->subQueryFields), array(array('c.resourcecreatorCreatorSurname' => 'creatorSurname'), 
					array('c.resourcecreatorResourceId' => 'resourcecreatorResourceId'))), 
					$this->db->subQuery($this->unions, 'u'), $distinct));
			}
			else
			{
				$this->commonBib->userBibCondition('resourcemiscId', TRUE);
				if($this->session->getVar('setup_PagingStyle') == 'A')
				{
					$this->executeCondJoin();
					if(!array_key_exists('c', $this->doneJoins))
						$this->db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'resourcemiscId');
					$this->countAlphaQuery = $this->db->select('resource_misc', 
						array(array('c.resourcecreatorCreatorSurname' => 'creatorSurname')), TRUE);
					if(!array_key_exists('c', $this->doneJoins))
						$this->db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'resourcemiscId');
					$caq = $this->db->sqlstring;
					$this->session->setVar('sql_CountAlphaStmt', base64_encode($caq));
				}
				$this->executeCondJoin();
				$this->countQuery = $this->db->selectCountDistinctField('resource_misc', 'resourcemiscId');
				$cq = $this->db->sqlstring;
				$this->session->setVar('sql_CountStmt', base64_encode($cq));
// get total for paging links and for next and previous buttons
				$this->common->pagingStyle($this->countQuery, $this->listType, $order, $queryString, $this->countAlphaQuery);
				$this->pagingAlphaCondition($order);
				if(!array_key_exists('c', $this->doneJoins))
					$this->db->leftJoin(array(array('resource_creator' => 'c')), 'c.resourcecreatorResourceId', 'resourcemiscId');
				$this->subQueryFields = array_merge($this->subQueryFields, $this->resourceMiscFields);
				$this->executeCondJoin();
				$this->totalResourceSubquery = 
					$this->db->queryNoExecute($this->db->selectNoExecute('resource_misc', 
					array_merge(array_unique($this->subQueryFields), 
					array(array('c.resourcecreatorCreatorSurname' => 'creatorSurname'), 
					array('c.resourcecreatorResourceId' => 'resourcecreatorResourceId'))), $distinct));
			}
		}
	}
/**
* Conditions for quarantined resources
*
* @param boolean $front Default is FALSE
*/
	private function quarantineCond($front = FALSE)
	{
		if($this->session->getVar('setup_Quarantine') == 'N')
			return;
		if($this->session->getVar('setup_Superadmin') && !$this->listQuarantined)
			return;
		if($this->session->getVar('setup_Superadmin') && $this->listQuarantined)
			$this->execCond[] = array('resourcemiscQuarantine' => 'Y'); // Admin listing quarantined resources
		else if($front)
			$this->db->formatConditions(array('resourcemiscQuarantine' => 'N'));
		else
			$this->execCond[] = array('resourcemiscQuarantine' => 'N');
	}
/**
* process the conditions and joins for search and select that are defined elsewhere.
*
* We can precisely control the order of joins here
*/
	private function executeCondJoin()
	{
//		if(empty($this->orderedJoins))
			$this->orderJoins();
		foreach($this->execCond as $cond)
			$this->db->formatConditions($cond);
		foreach($this->execCondOneField as $field => $array)
			$this->db->formatConditionsOneField($array, $field);
		foreach($this->orderedJoins as $table => $array)
		{
			if(array_key_exists('alias', $array))
			{
				$this->db->leftJoin(array(array($array['table'] => $array['alias'])), $array['intField'], $array['extField']);
				$this->doneJoins[$array['alias']] = FALSE;
			}
			else
			{
				$this->db->leftJoin($table, $array['intField'], $array['extField']);
				$this->doneJoins[$table] = FALSE;
			}
		}
	}
/**
* Order the joins
*/
	private function orderJoins()
	{
		$fields = array('resource' => $this->resourceFields, 'resource_misc' => $this->resourceMiscFields, 
			'resource_text' => $this->textFields, 'resource_musing_text' => $this->musingFields, 
			'resource_attachments' => $this->attachmentFields, 'publisher' => $this->publisherFields, 
			'resource_year' => $this->yearFields);
		$tables = array('resource_misc', 'resource_attachments', 'resource', 'resource_creator', 'creator', 
			'resource_text', 'resource_category', 'resource_keyword', 'keyword', 'resource_timestamp', 'resource_year', 
			'resource_quote', 'resource_quote_text', 'resource_paraphrase', 'resource_paraphrase_text', 
			'resource_musing', 'resource_musing_text', 'resource_user_tags', 'user_tags', 'resource_custom', 
			'user_bibliography_resource', 'publisher', 'resource_language', 'resource_quote_comment', 'resource_paraphrase_comment');
		foreach($tables as $tableOrder)
		{
			foreach($this->execJoin as $table => $array)
			{
				$tSplit = explode('.', $table);
				if((sizeof($tSplit) == 2) && ($tSplit[1] == $tableOrder))
				{
					$array['alias'] = $tSplit[0];
					$array['table'] = $tSplit[1];
					$this->orderedJoins[$table] = $array;
					if(array_key_exists($tSplit[1], $fields))
						$this->subQueryFields = array_merge($this->subQueryFields, $fields[$tSplit[1]]);
				}
				else if($table == $tableOrder)
				{
					$this->orderedJoins[$table] = $array;
					if(array_key_exists($table, $fields))
						$this->subQueryFields = array_merge($this->subQueryFields, $fields[$table]);
				}
			}
		}
	}
/**
* Set SQL conditions for title and creator order when using pagingAlpha
*
* @param string $order
*/
	private function pagingAlphaCondition($order)
	{
		if(($this->session->getVar('setup_PagingStyle') != 'A') || empty($this->common->pagingObject->pagingArray))
			return;
		if($order == 'title')
		{
			if($this->common->pagingObject->start >= $this->common->pagingObject->sizeOfPA)
				$this->common->pagingObject->start = 0;
			foreach($this->common->pagingObject->pagingArray[$this->common->pagingObject->start] as $letter)
			{
// '??'  force this to 26th element and display all non-alpha titles at the end
				if($letter == '??')
					$conditions[] = $this->db->formatFields('resourceTitleSort') . ' ' . $this->db->regexp(FALSE, '^[^A-Z]', FALSE);
				else
					$conditions[] = $this->db->upper('resourceTitleSort') . ' ' .  $this->db->like(FALSE, $letter, '%');
			}
		}
		else if($order == 'creator')
		{
			if($this->common->pagingObject->start >= $this->common->pagingObject->sizeOfPA)
				$this->common->pagingObject->start = 0;
			foreach($this->common->pagingObject->pagingArray[$this->common->pagingObject->start] as $letter)
			{
				if($letter == '--')
					$conditions[] = $this->db->formatFields('c.resourcecreatorCreatorSurname') . ' IS NULL ';
				else if($letter == '??')
					$conditions[] = $this->db->upper('c.resourcecreatorCreatorSurname') . ' ' . 
						$this->db->regexp(FALSE, '^[^A-Z]', FALSE);
				else
					$conditions[] = $this->db->upper('c.resourcecreatorCreatorSurname') . ' ' . $this->db->like(FALSE, $letter, '%');
			}
		}
		else if($order == 'attachments')
		{
			if($this->common->pagingObject->start >= $this->common->pagingObject->sizeOfPA)
				$this->common->pagingObject->start = 0;
			foreach($this->common->pagingObject->pagingArray[$this->common->pagingObject->start] as $letter)
			{
// '??'  force this to 26th element and display all non-alpha filenames at the end
				if($letter == '??')
					$conditions[] = $this->db->formatFields('resourceattachmentsFileName') . ' ' . $this->db->regexp(FALSE, '^[^A-Z]', FALSE);
				else
					$conditions[] = $this->db->upper('resourceattachmentsFileName') . ' ' .  $this->db->like(FALSE, $letter, '%');
			}
		}
		$condition = join($this->db->or, $conditions);
		$this->db->formatConditions($condition);
	}
}
?>
