<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/**
* USER
*
* Common methods for handling users.
* 
* @version	1
*
*	@package wikindx4\core\usergroups
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/

class USER
{
/** object */
private $db;
/** array */
private $vars;
/** object */
private $errors;
/** object */
private $messages;
/** object */
private $session;
/** object */
private $template;
/** object */
private $form;
/** object */
private $html;
/** object */
private $table;
/** object */
private $loadstyle;

/**
* USER
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->vars = GLOBALS::getVars();
		$this->errors = FACTORY_ERRORS::getInstance();
		$this->messages = FACTORY_MESSAGES::getInstance();
		$this->template = FACTORY_TEMPLATE::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->form = FACTORY_FORM::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->table = FACTORY_TABLE::getInstance();
		$this->loadStyle = FACTORY_LOADSTYLE::getInstance();
	}
/**
* Write user details to WKX_users
*
* If $admin = 0, this is a standard user.
* If $admin = 1, this is the initial configuration/reconfiguration and we need to insert superadmin details into the database.
* If $admin = 2, this is an admin editing or adding a user.
* a) the user may not change the username from this screen.
* When adding a user, we need to check there is not already $username (caseless) in the database.
* NB - returns FALSE in case of success, an error message in case of failure.
*
* @param boolean $add Default is TRUE
* @param int $admin Default is 0
* @return mixed
*/
	public function writeUser($add = TRUE, $admin = 0)
	{
		if(array_key_exists('username', $this->vars))
		{
			$username = $this->html->removeNl($this->vars['username']);
// check for existing usernames (remove current user from search if already logged in with setup_userId)
			$this->db->formatConditions(array('usersUsername' => $username));
			if($userId = $this->session->getVar('setup_UserId') && !$add)
				$this->db->formatConditions(array('usersId' => $userId), TRUE); // Not equal to
// existing user with that username found (not this user)
			$recordset = $this->db->select('users', 'usersId');
			if($this->db->numRows($recordset))
				return $this->errors->text("inputError", "userExists");
		}
		$password = $this->html->removeNl($this->vars['password']);
		if(!$add) // update
		{
			if(!$admin) // user editing own details
			{
				$userId = $this->session->getVar('setup_UserId');
				$cookie = FACTORY_COOKIE::getInstance();
				if(array_key_exists('cookie', $this->vars) && $this->vars['cookie'])
				{
					$cookie->storeCookie($this->vars['uname']);
					$update['usersCookie'] = 'Y';
				}
				else
				{
// remove any wikindx cookie that has been set
					$cookie->deleteCookie();
					$update['usersCookie'] = 'N';
				}
			}
			else if($admin == 1) // superadmin configuration - userId always 1 for superadmin
				$userId = 1;
			else if($admin == 2) // admin editing user
			{
				$userId = $this->vars['userId'];
				if(array_key_exists('admin', $this->vars) && $this->vars['admin'])
					$update['usersAdmin'] = 'Y';
				else
					$update['usersAdmin'] = 'N';
				if(array_key_exists('department', $this->vars) && ($dept = trim($this->vars['department'])))
					$update['usersDepartment'] = $dept;
				else if(array_key_exists('departmentId', $this->vars) && ($dept = $this->vars['departmentId']))
					$update['usersDepartment'] = $dept;
				else
					$nulls[] = 'usersDepartment';
				if(array_key_exists('institution', $this->vars) && ($inst = trim($this->vars['institution'])))
					$update['usersInstitution'] = $inst;
				else if(array_key_exists('institutionId', $this->vars) && ($inst = $this->vars['institutionId']))
					$update['usersInstitution'] = $inst;
				else
					$nulls[] = 'usersInstitution';
				if(array_key_exists('creatorId', $this->vars) && $this->vars['creatorId'])
					$update['usersIsCreator'] = $this->vars['creatorId'];
				else
					$nulls[] = 'usersIsCreator';
			}
			if(isset($nulls))
			{
				$this->db->formatConditions(array('usersId' => $userId));
				$this->db->updateNull('users', $nulls);
			}
			$this->db->formatConditions(array('usersId' => $userId));
			$recordset = $this->db->select('users', 'usersPassword');
// The encrypted password is displayed on the browser screen.  Need to check if it's unchanged as we don't 
// want to encrypt the encrypted password!
			if($this->db->numRows($recordset))
			{
				$this->pwdInputEncrypted = FALSE;
				$pwd = $this->db->fetchOne($recordset);
				if($password != $pwd)
					$password = crypt($password, strrev(time()));
				else
					$this->pwdInputEncrypted = TRUE;
			}
			$update['usersPassword'] = $password;
			if(array_key_exists('fullname', $this->vars) && 
				$fullname = $this->db->tidyInput($this->html->removeNl($this->vars['fullname'])))
				$update['usersFullname'] = $this->html->removeNl($this->vars['fullname']);
// Ensure it's set to NULL
			else
			{
				$this->db->formatConditions(array('usersId' => $userId));
				$this->db->updateNull('users', 'usersFullname');
			}
			if($admin == 1) // superadmin configuration
				$update['usersUsername'] = $username;
			else
				$update['usersEmail'] = $this->vars['email'];
			$this->db->formatConditions(array('usersId' => $userId));
			$this->db->update('users', $update);
		}
		else // insert new user
		{
			$password = crypt($password, strrev(time()));
			$field[] = 'usersUsername';
			$value[] = $username;
			$field[] = 'usersPassword';
			$value[] = $password;
			if(array_key_exists('email', $this->vars) && 
				$fullname = $this->html->removeNl($this->vars['email']))
			{
				$field[] = 'usersEmail';
				$value[] = $this->vars['email'];
			}
			if($admin == 1) // if == 0, default db field value is 'N'
			{
				$field[] = 'usersAdmin';
				$value[] = 'Y';
				$field[] = 'usersFullname';
				$value[] = 'superAdmin';
			}
			else if($admin == 2) // admin editing a user
			{
				if(array_key_exists('admin', $this->vars) && $this->vars['admin'])
				{
					$field[] = 'usersAdmin';
					$value[] = 'Y';
				}
				if(array_key_exists('department', $this->vars) && ($dept = trim($this->vars['department'])))
				{
					$field[] = 'usersDepartment';
					$value[] = $dept;
				}
				else if(array_key_exists('departmentId', $this->vars) && ($dept = $this->vars['departmentId']))
				{
					$field[] = 'usersDepartment';
					$value[] = $dept;
				}
				if(array_key_exists('institution', $this->vars) && ($inst = trim($this->vars['institution'])))
				{
					$field[] = 'usersInstitution';
					$value[] = $inst;
				}
				else if(array_key_exists('institutionId', $this->vars) && ($inst = $this->vars['institutionId']))
				{
					$field[] = 'usersInstitution';
					$value[] = $inst;
				}
				if(array_key_exists('creatorId', $this->vars) && $this->vars['creatorId'])
				{
					$field[] = 'usersIsCreator';
					$value[] = $this->vars['creatorId'];
				}
			}
			if(array_key_exists('fullname', $this->vars) && 
				$fullname = $this->html->removeNl($this->vars['fullname']))
			{
				$field[] = 'usersFullname';
				$value[] = $fullname;
			}
			$field[] = 'usersTimestamp';
			$value[] = '2012-01-01 01:01:01';
			$field[] = 'usersNotifyTimestamp';
			$value[] = '2012-01-01 01:01:01';
			$this->db->insert('users', $field, $value);
			$userId = $this->db->lastAutoId();
// set the users.notifyTimestamp and users.timestamp to current date
			$this->db->formatConditions(array('usersId' => $userId));
			$this->db->updateTimestamp('users', array('usersNotifyTimestamp' => FALSE, 'usersTimestamp' => FALSE));
// write userId to session if not adding a new user
			if(!$add)
				$this->session->setVar('setup_UserId', $userId);
// insert preferences to table
			$this->writePreferences($userId);
		}
		return FALSE; // success!
	}
/**
* Compare encrypted passwords.
*
* Return FALSE for password not found or password doesn't match.
* Superadmin is always id = 1
* @param string $username
* @param string $pwdInput
* @return boolean
*/
	public function checkPassword($username, $pwdInput)
	{
		$fields = $this->db->prependTableToField('users', array("Id", "Password", "Admin", "Cookie"));
		$this->db->formatConditions(array('usersUsername' => $username));
		$recordset = $this->db->select('users', $fields);
		if(!$this->db->numRows($recordset))
			return FALSE;
		$row = $this->db->fetchRow($recordset);
// only the superadmin may log on when multi user is not enabled
		if(!$this->session->getVar("setup_MultiUser") && ($row['usersId'] != 1))
			return FALSE;
		if(crypt($pwdInput, $row['usersPassword']) != $row['usersPassword'])
			return FALSE;
// Logged in, now set up environment
		$this->environment($row, $username);
		return TRUE;
	}
/**
* set up user environment on first logon
*
* @param array $row
* @param string $username Default is FALSE
*/
	public function environment($row, $username = FALSE)
	{
// First delete any pre-existing session (other than setup_) in case this user has been logging on and off as different users
		$this->session->clearSessionData();
		if($row['usersAdmin'] == 'Y')
			$this->session->setVar("setup_Superadmin", TRUE);
		$this->session->setVar("setup_UserId", $row['usersId']);
		$this->session->setVar("setup_Write", TRUE);
		$this->session->delVar("setup_ReadOnly");
		$this->writeSessionPreferences($row['usersId']);
// Reload message files
		$this->messages = FACTORY_MESSAGES::getFreshInstance();
		$this->errors = FACTORY_ERRORS::getFreshInstance();
//		$this->bib->grabBibliographies();
// If $username, we are logging in without using a cookie.  Check if we require a cookie to be set.
		if(array_key_exists('usersCookie', $row) && ($row['usersCookie'] == 'Y'))
		{
			$cookie = FACTORY_COOKIE::getInstance();
			$cookie->storeCookie($username);
		}
	}
/**
* create 'added by..., edited by... details for displaying resources
*
* @param array $row
* @param boolean $both Default is FALSE
* @param string $type Default is 'resource'
* @return array ($userAdd, $userEdit)
*/
	public function displayUserAddEdit($row, $both = FALSE, $type = 'resource')
	{
		if(!$this->session->getVar('setup_MultiUser'))
			return FALSE;
		if($type == 'resource')
		{
			$add = $row['resourcemiscAddUserIdResource'];
			$edit = $row['resourcemiscEditUserIdResource'];
		}
		else if($type == 'abstract')
		{
			$add = $row['resourcetextAddUserIdAbstract'];
			$edit = $row['resourcetextEditUserIdAbstract'];
		}
		else if($type == 'note')
		{
			$add = $row['resourcetextAddUserIdNote'];
			$edit = $row['resourcetextEditUserIdNote'];
		}
		else if($type == 'custom')
		{
			$add = $row['resourcecustomAddUserIdCustom'];
			$edit = $row['resourcecustomEditUserIdCustom'];
		}
		else if(($type == 'musing') || ($type == 'quote') || ($type == 'paraphrase') || ($type == 'comment'))
		{
// NB in this case, $row is not an array but the user ID
			$this->db->formatConditions(array('usersId' => $row));
			$resultset = $this->db->select('users', array('usersUsername', 'usersFullname'));
			if(!$this->db->numRows($resultset))
				return array($this->messages->text("user", "unknown"), FALSE);
			$row = $this->db->fetchRow($resultset);
			if($row['usersFullname'])
				$user = $this->messages->text("hint", "addedBy", stripslashes($row['usersFullname']));
			else if($row['usersUsername'])
				$user = $this->messages->text("hint", "addedBy", stripslashes($row['usersUsername']));
			else
				$user = $this->messages->text("hint", "addedBy", $this->messages->text("user", "unknown"));
			return array($user, FALSE);
		}
		if($edit)
		{
			if($row['usersFullname'])
				$userEdit = $this->messages->text("hint", "editedBy", stripslashes($row['usersFullname']));
			else if($row['usersUsername'])
				$userEdit = $this->messages->text("hint", "editedBy", stripslashes($row['usersUsername']));
			else
				$userEdit = $this->messages->text("hint", "editedBy", $this->messages->text("user", "unknown"));
			if(!$both)
				return $userEdit;
			$this->db->formatConditions(array('usersId' => $add));
			$resultset = $this->db->select('users', array('usersUsername', 'usersFullname'));
			$row = $this->db->fetchRow($resultset);
			if($row['usersFullname'])
				$userAdd = $this->messages->text("hint", "addedBy", stripslashes($row['usersFullname']));
			else if($row['usersUsername'])
				$userAdd = $this->messages->text("hint", "addedBy", stripslashes($row['usersUsername']));
			else
				$userAdd = $this->messages->text("hint", "addedBy", $this->messages->text("user", "unknown"));
			return array($userAdd, $userEdit);
		}
		if($add)
		{
			if($row['usersFullname'])
				$userAdd = $this->messages->text("hint", "addedBy", stripslashes($row['usersFullname']));
			else if($row['usersUsername'])
				$userAdd = $this->messages->text("hint", "addedBy", stripslashes($row['usersUsername']));
			else
				$userAdd = $this->messages->text("hint", "addedBy", $this->messages->text("user", "unknown"));
			if(!$both)
				return $userAdd;
			$this->db->formatConditions(array('usersId' => $edit));
			$resultset = $this->db->select('users', array('usersUsername', 'usersFullname'));
			if(!$this->db->numRows($resultset))
				return array($userAdd, FALSE);
			$row = $this->db->fetchRow($resultset);
			if($row['usersFullname'])
				$userEdit = $this->messages->text("hint", "editedBy", stripslashes($row['usersFullname']));
			else if($row['usersUsername'])
				$userEdit = $this->messages->text("hint", "editedBy", stripslashes($row['usersUsername']));
			else
				$userEdit = $this->messages->text("hint", "editedBy", $this->messages->text("user", "unknown"));
			return array($userAdd, $userEdit);
		}
	}
/**
* create username
*
* Plain version of displayUserAddEdit() used in core/modules/email/EMAIL.php for notify emails
* @param int $addId
* @return string
*/
	public function displayUserAddEditPlain($addId)
	{
		$user = $userAdd = FALSE;
		if($addId)
		{
			$this->db->formatConditions(array('usersId' => $addId));
			$recordset = $this->db->select('users', array('usersUsername', 'usersFullname'));
			$row2 = $this->db->fetchRow($recordset);
			if($row2['usersFullname'])
				$userAdd = $row2['usersFullname'];
			else if($row2['usersUsername'])
				$userAdd = $row2['usersUsername'];
			else
				$userEdit = $this->messages->text("user", "unkown");
		}
		if($userAdd)
			return $userAdd;
		return $user;
	}
/**
* Store user preferences in the session setup_ array
*
* Can come from users or config (default values for readOnly user in which case $userId = FALSE)
* @param int $userId
* @param string $table
* @param boolean $upgrade Default is FALSE
* @return boolean
*/
	public function writeSessionPreferences($userId, $table = 'users', $upgrade = FALSE)
	{
		$bib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
		if($table == 'users')
		{
			$basic = array("Paging", "PagingMaxLinks", "StringLimit", 
				"Language", "Style", "Template", "PagingStyle", "PagingTagCloud", "UseBibtexKey", 
				"UseWikindxKey", "DisplayBibtexLink", "DisplayCmsLink", "CmsTag", "Listlink", "TemplateMenu");
			$preferences = $this->db->prependTableToField($table, $basic);
		}
		else
		{
			$basic = array("Paging", "PagingMaxLinks", "StringLimit", 
				"Language", "Style", "Template", "PagingTagCloud", "Listlink");
			$preferences = $this->db->prependTableToField($table, $basic);
		}
		if($userId)
			$this->db->formatConditions(array('usersId' => $userId));
		$recordset = $this->db->select($table, $preferences);
		if(!$this->db->numRows($recordset))
			return FALSE;
		$row = $this->db->fetchRow($recordset);
/**
* Check requested style plug-in has not been deleted.  If so, return first in list so that something is safely 
* displayed when listing bibliographies.  Not required after upgrading to v4
*/
		if(!$upgrade)
		{
			$styles = $this->loadStyle->loadDir("styles/bibliography");
			if(!array_key_exists($row[$table . 'Style'], $styles))
			{
				$styleKeys = array_keys($styles);
				$row['style'] = array_shift($styleKeys);
			}
		}
		$table = str_replace('_', '', $table);
		foreach($basic as $pref)
		{
			if(($pref == 'PagingStyle') || ($pref == 'UseWikindxKey') || ($pref == 'UseBibtexKey')
				 || ($pref == 'DisplayBibtexLink') || ($pref == 'DisplayCmsLink') || ($pref == 'Listlink'))
			{
				if($row[$table . $pref] == 'N')
					$this->session->delVar("setup_$pref");
				else if($pref == 'PagingStyle')
					$this->session->setVar('setup_PagingStyle', $row[$table . $pref]);
				else
					$this->session->setVar("setup_$pref", $row[$table . $pref]);
			}
			else if($pref == $table . 'CmsTag')
			{
				if($row[$pref])
				{
					$cms = unserialize(base64_decode($row[$pref]));
					$this->session->setVar("setup_CmsTagStart", $cms[0]);
					$this->session->setVar("setup_CmsTagEnd", $cms[1]);
				}
				else
				{
					$this->session->delVar("setup_CmsTagStart");
					$this->session->delVar("setup_CmsTagEnd");
				}
			}
			else
				$this->session->setVar("setup_" . $pref, $row[$table . $pref]);
		}
// The system also requires userRegistration, notify, multiUser, maxPaste fileAttach and fileViewLoggedOnOnly etc. from 
// WKX_config
		$fields = $this->db->prependTableToField('config', array('UserRegistration', 'MultiUser', 'Notify', 
			'FileAttach', 'FileViewLoggedOnOnly', 'MaxPaste', 'LastChanges', 'LastChangesType', 'ImportBib', 
			'LastChangesDayLimit', 'Quarantine', 'Listlink', 'MetadataAllow', 'MetadataUserOnly'));
		$recordset = $this->db->select('config', $fields);
		$row = $this->db->fetchRow($recordset);
		if($row['configUserRegistration'] == 'Y')
			$this->session->setVar("setup_UserRegistration", TRUE);
		$this->session->setVar("setup_MultiUser", $row['configMultiUser']);
		if($row['configNotify'] == 'Y')
			$this->session->setVar("setup_Notify", $row['configNotify']);
		if($row['configFileAttach'] == 'Y')
			$this->session->setVar("setup_FileAttach", TRUE);
		if($row['configFileViewLoggedOnOnly'] == 'Y')
			$this->session->setVar("setup_FileViewLoggedOnOnly", TRUE);
		if($row['configImportBib'] == 'Y')
			$this->session->setVar("setup_ImportBib", TRUE);
		$this->session->setVar("setup_MaxPaste", $row['configMaxPaste']);
		$this->session->setVar("setup_LastChanges", $row['configLastChanges']);
		$this->session->setVar("setup_LastChangesDayLimit", $row['configLastChangesDayLimit']);
		$this->session->setVar("setup_LastChangesType", $row['configLastChangesType']);
		$this->session->setVar("setup_Quarantine", $row['configQuarantine']);
		$this->session->setVar("setup_MetadataAllow", $row['configMetadataAllow']);
		$this->session->setVar("setup_MetadataUserOnly", $row['configMetadataUserOnly']);
		if($userId)
			$this->session->setVar("setup_UserId", $userId);
		else if($row['configListlink'] == 'Y')
			$this->session->setVar("setup_Listlink", $row['configListlink']);
		$bibs = $bib->getUserBibs();
		if(empty($bibs))
			$bibs = $bib->getGroupBibs();
		if(!empty($bibs))
			$this->session->setVar("setup_Bibliographies", TRUE);
		return TRUE;
	}
/**
* Load user details into session mywikindx_ array
*
* @param int $id Default is FALSE
*/
	public function loadSession($id = FALSE)
	{
		if(!$id)
			$id = $this->session->getVar("setup_UserId");
		$userArray = array("usersUsername", "usersPassword", "usersEmail", "usersFullname", "usersAdmin", "usersCookie", 
			"usersDepartment", "usersInstitution", "usersIsCreator");
		for($i = 1; $i < 4; $i++)
			$userArray[] = "usersPasswordQuestion$i";
		$this->db->formatConditions(array('usersId' => $id));
		$recordset = $this->db->select('users', $userArray);
		$row = $this->db->fetchRow($recordset);
		foreach($userArray as $key)
		{
			if(($key == 'admin') || ($key == 'cookie'))
			{
				if($row[$key] == 'Y')
					$this->session->setVar("mywikindx_" . str_replace('users', '', $key), TRUE);
				else
					$this->session->delVar("mywikindx_" . str_replace('users', '', $key));
				continue;
			}
			if($row[$key])
				$this->session->setVar("mywikindx_" . str_replace('users', '', $key), $row[$key]);
		}
//		$this->bib->grabBibliographies($id);
	}
/**
* display user details from users table
*
* If $admin != FALSE, it is the user id that an admin is editing
* @param string $form
* @param string $hidden
* @param int $admin Default is FALSE
* @return string
*/
	public function displayUserDetails($form, $hidden, $admin = FALSE)
	{
		$pString = $this->form->formHeader($form);
		$pString .= $this->form->hidden('method', $hidden);
		$sessVar = $this->html->dbToFormTidy($this->session->getVar("mywikindx_Username"));
		$pString .= $this->form->hidden("uname", $sessVar);
		$pString .= $this->table->tableStart();
		$pString .= $this->table->trStart();
		$pString .= $this->table->td($this->html->b($this->messages->text("user", "username")) . ":&nbsp;&nbsp;$sessVar" . 
			$this->html->br() . "&nbsp;");
		$pString .= $this->table->trEnd();
		$pString .= $this->table->trStart();
// Firefox seems to have a bug in that $sessVar in the two password form inputs below are not the same.
// Konqueror, for example, does not have this feature.
		$sessVar = $this->html->dbToFormTidy($this->session->getVar("mywikindx_Password"));
		$pString .= $this->table->td($this->form->passwordInput($this->messages->text("user", "password"), 
			"password", $sessVar, 15, 255)
			 . " " . $this->html->span('*', 'required'));
		$pString .= $this->table->td($this->form->passwordInput($this->messages->text("user", "passwordConfirm"), 
		"passwordConfirm", $sessVar, 15, 255) . " " . $this->html->span('*', 'required'));
		$sessVar = $this->html->dbToFormTidy($this->session->getVar("mywikindx_Email"));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "email"), "email", $sessVar, 20, 255)
			 . " " . $this->html->span('*', 'required'));
		$sessVar = $this->html->dbToFormTidy($this->session->getVar("mywikindx_Fullname"));
		$pString .= $this->table->td($this->form->textInput($this->messages->text("user", "fullname"), 
			"fullname", $sessVar, 20, 255));
		if($admin)
		{
			$pString .= $this->table->trEnd();
			$pString .= $this->table->tableEnd();
			$pString .= $this->form->hidden("userId", $admin);
			$pString .= $this->html->br() . "&nbsp;" . $this->html->br();
			$pString .= $this->table->tableStart();
			$pString .= $this->table->trStart();
// Department
			$td = $this->form->textInput($this->messages->text("user", "department"), "department", FALSE, 30, 255);
			$resultset = $this->db->select('users', 'usersDepartment', TRUE);
			$initial = FALSE;
			while($row = $this->db->fetchRow($resultset))
			{
				$department = $this->html->dbToFormTidy($row['usersDepartment']);
				if(!$department)
					continue;
				$departments[$department] = $department;
			}
			if(isset($departments))
			{
// add 0 => IGNORE to departments array
				$temp[0] = $this->messages->text("misc", "ignore");
				foreach($departments as $key => $value)
					$temp[$key] = $value;
				$departments = $temp;
				unset($temp);
				$sessVar = $this->html->dbToFormTidy($this->session->getVar("mywikindx_Department"));
				$td .= $this->html->br() . $this->form->selectedBoxValue('&nbsp;', "departmentId", $departments, $sessVar, 1);
			}
			$pString .= $this->table->td($td);
// Institution
			$td = $this->form->textInput($this->messages->text("user", "institution"), "institution", FALSE, 30, 255);
			$resultset = $this->db->select('users', 'usersInstitution', TRUE);
			$initial = FALSE;
			while($row = $this->db->fetchRow($resultset))
			{
				$institution = $this->html->dbToFormTidy($row['usersInstitution']);
				if(!$institution)
					continue;
				$institutions[$institution] = $institution;
			}
			if(isset($institutions))
			{
// add 0 => IGNORE to insitutions array
				$temp[0] = $this->messages->text("misc", "ignore");
				foreach($institutions as $key => $value)
					$temp[$key] = $value;
				$institutions = $temp;
				unset($temp);
				$sessVar = $this->html->dbToFormTidy($this->session->getVar("mywikindx_Institution"));
				$td .= $this->html->br() . $this->form->selectedBoxValue('&nbsp;', "institutionId", $institutions, $sessVar, 1);
			}
			$pString .= $this->table->td($td);
// User is creator
			$creator = FACTORY_CREATOR::getInstance();
			$creators = $creator->grabAll(FALSE, FALSE, FALSE, TRUE);
			if(is_array($creators))
			{
// add 0 => IGNORE to creators array
				$temp[0] = $this->messages->text("misc", "ignore");
				foreach($creators as $key => $value)
					$temp[$key] = $value;
				$creators = $temp;
				unset($temp);
				$sessVar = $this->html->dbToFormTidy($this->session->getVar("mywikindx_IsCreator"));
				$pString .= $this->table->td($this->form->selectedBoxValue($this->messages->text("user", "isCreator"), "creatorId", 
					$creators, $sessVar, 1));
			}
			$sessVar = $this->session->getVar("mywikindx_Admin") == 'Y' ? 'CHECKED' : FALSE;
			$pString .= $this->table->td($this->form->checkbox($this->messages->text("user", "admin"), "admin", $sessVar));
		}
		else
		{
			$sessVar = $this->session->getVar("mywikindx_Cookie") == 'Y' ? 'CHECKED' : FALSE;
			$pString .= $this->table->td($this->form->checkbox($this->messages->text("user", "cookie"), "cookie", $sessVar));
		}
		$pString .= $this->table->trEnd();
		$pString .= $this->table->tableEnd();
		$pString .= $this->html->p($this->form->formSubmit("Edit"), FALSE, "left");
		$pString .= $this->form->formEnd();
		return $pString;
	}
/**
* insert/update user preferences
*
* @param id $userId
*/
	public function writePreferences($userId)
	{
// Set paging_start back to 0
		$this->session->setVar('mywikindx_PagingStart', 0);
		$preferences = array("Paging", "PagingMaxLinks", "StringLimit", "Language", "Style", "Template", 'PagingStyle', 
			"PagingTagCloud", "UseBibtexKey", "UseWikindxKey", "DisplayBibtexLink", "DisplayCmsLink", "Listlink", "TemplateMenu");
		foreach($preferences as $pref)
		{
			if($pref == 'PagingStyle') // checkbox
			{
				if($this->session->getVar('setup_PagingStyle') == 'A')
					$updateArray["usersPagingStyle"] = 'A';
				else
					$updateArray["usersPagingStyle"] = 'N';
			}
			else if(($pref == "UseWikindxKey") || ($pref == "UseBibtexKey") || 
			($pref == "DisplayBibtexLink") || ($pref == "DisplayCmsLink") || ($pref == "Listlink"))
			{
				if($this->session->getVar("setup_$pref"))
					$updateArray['users' . $pref] = 'Y';
				else
					$updateArray['users' . $pref] = 'N';
			}
			else if(($pref == 'TemplateMenu') && !$this->session->getVar('setup_TemplateMenu')) // no level reduction
				$updateArray['users' . $pref] = 0;
			else
				$updateArray['users' . $pref] = $this->session->getVar("setup_" . $pref);
		}
		$this->db->formatConditions(array('usersId' => $userId));
		$this->db->update('users', $updateArray);
	}
/**
* List user's user groups
*
* @return mixed FALSE|array
*/
	public function listUserGroups()
	{
		$this->db->formatConditions(array('usergroupsAdminId' => $this->session->getVar('setup_UserId')));
		$this->db->orderBy('usergroupsTitle');
		$recordset = $this->db->select(array('user_groups'), array('usergroupsId', 'usergroupsTitle'));
		if(!$this->db->numRows($recordset))
			return FALSE;
		else
		{
			while($row = $this->db->fetchRow($recordset))
				$groups[$row['usergroupsId']] = $row['usergroupsTitle'];
			return $groups;
		}
	}
/**
* Get users from users.
*
* Return associative array of id => group of FALSE if no users.
* @param boolean $full Default is FALSE
* @param boolean $useBib Default is FALSE
* @param string $addEdit Default is 'add'
* @return mixed FALSE|array
*/
	public function grabAll($full = FALSE, $useBib = FALSE, $addEdit = 'add')
	{
		if($useBib)
		{
			if($addEdit == 'add')
				$field = 'resourcemiscAddUserIdResource';
			else
				$field = 'resourcemiscEditUserIdResource';
			$this->db->formatConditions(array('userbibliographyresourceBibliographyId' => $useBib));
			$this->db->leftJoin('resource_misc', 'resourcemiscId', 'userbibliographyresourceResourceId');
			$sq = $this->db->subQuery($this->db->selectNoExecute('user_bibliography_resource', $field, TRUE), 't', TRUE, TRUE);
			$this->db->formatConditions($this->db->formatFields('usersId') . $this->db->equal . 
				$this->db->formatFields($field));
		}
		$this->db->orderBy('usersUsername');
		if($full)
		{
			if($useBib)
				$recordset = $this->db->selectFromSubQuery('users', array("usersId", "usersUsername", "usersFullname", "usersAdmin"), $sq);
			else
				$recordset = $this->db->select('users', array("usersId", "usersUsername", "usersFullname", "usersAdmin"), TRUE);
		}
		else
		{
			if($useBib)
				$recordset = $this->db->selectFromSubQuery('users', array("usersId", "usersUsername"), $sq);
			else
				$recordset = $this->db->select('users', array("usersId", "usersUsername"), TRUE);
		}
		while($row = $this->db->fetchRow($recordset))
		{
			$users[$row['usersId']] = $this->html->dbToFormTidy($row['usersUsername']);
			if($full && $row['usersFullname'])
				$users[$row['usersId']] .= " (" . $this->html->dbToFormTidy($row['usersFullname']) . ")";
			if($full && ($row['usersAdmin'] == 'Y'))
				$users[$row['usersId']] .= " ADMIN";
		}
		if(isset($users))
			return $users;
		else
			return FALSE; // shouldn't ever happen but just in case....
	}
/**
* Delete user(s) from user table and handle users' metadata
*
* @param array $input Array of ids in WKX_users.
* @return boolean
*/
	public function deleteSql($input)
	{
// Delete from users
		$this->db->formatConditionsOneField($input, 'usersId');
		$this->db->delete('users');
// Delete from user_bibliography
		$this->db->formatConditionsOneField($input, 'userbibliographyUserId');
		$this->db->delete('user_bibliography');
// Delete from user_groups_users
		$this->db->formatConditionsOneField($input, 'usergroupsusersUserId');
		$this->db->delete('user_groups_users');
		$this->db->optimize('users');
		$this->db->optimize('user_bibliography');
		$this->db->optimize('user_groups_users');
// Manage deleted user's metadata
// $this->vars['userMetadata']:
// 0 -- do nothing
// 1 -- transfer to superadmin
// 2 -- delete
		if(!array_key_exists('userMetadata', $this->vars) || !$this->vars['userMetadata'])
			return TRUE;
		if($this->vars['userMetadata'] == 1) // to superadmin
		{
			$updateArray = array('resourcequotetextAddUserIdQuote' => 1);
			$this->db->formatConditionsOneField($input, 'resourcequotetextAddUserIdQuote');
			$this->db->update('resource_quote_text', $updateArray);
			$updateArray = array('resourcequotecommentAddUserIdQuote' => 1, 'resourcequotecommentPrivate' => 'N');
			$this->db->formatConditionsOneField($input, 'resourcequotecommentAddUserIdQuote');
			$this->db->update('resource_quote_comment', $updateArray);
			$updateArray = array('resourceparaphrasetextAddUserIdParaphrase' => 1);
			$this->db->formatConditionsOneField($input, 'resourceparaphrasetextAddUserIdParaphrase');
			$this->db->update('resource_paraphrase_text', $updateArray);
			$updateArray = array('resourceparaphrasecommentAddUserIdParaphrase' => 1, 'resourceparaphrasecommentPrivate' => 'N');
			$this->db->formatConditionsOneField($input, 'resourceparaphrasecommentAddUserIdParaphrase');
			$this->db->update('resource_paraphrase_comment', $updateArray);
			$updateArray = array('resourcemusingtextAddUserIdMusing' => 1, 'resourcemusingtextPrivate' => 'N');
			$this->db->formatConditionsOneField($input, 'resourcemusingtextAddUserIdMusing');
			$this->db->update('resource_musing_text', $updateArray);
		}
		else if($this->vars['userMetadata'] == 2) // delete
		{
			foreach($input as $userId)
			{
				$this->db->formatConditions(array('resourcequotetextAddUserIdQuote' => $userId));
				$resultset = $this->db->select('resource_quote_text', 'resourcequotetextId');
				while($row = $this->db->fetchRow($resultset))
				{
// Remove this user's quotes from resource_quote
					$this->db->formatConditions(array('resourcequoteId' => $row['resourcequotetextId']));
					$this->db->delete('resource_quote');
// remove all comments relating to this quote
					$this->db->formatConditions(array('resourcequotecommentQuoteId' => $row['resourcequotetextId']));
					$this->db->delete('resource_quote_comment');
				}
				$this->db->formatConditions(array('resourceparaphrasetextAddUserIdParaphrase' => $userId));
				$resultset = $this->db->select('resource_paraphrase_text', 'resourceparaphrasetextId');
				while($row = $this->db->fetchRow($resultset))
				{
// Remove this user's paraphrases from resource_paraphrase
					$this->db->formatConditions(array('resourceparaphraseId' => $row['resourceparaphrasetextId']));
					$this->db->delete('resource_paraphrase');
// remove all comments relating to this paraphrase
					$this->db->formatConditions(array('resourceparaphrasecommentParaphraseId' => $row['resourceparaphrasetextId']));
					$this->db->delete('resource_paraphrase_comment');
				}
			}
// remove users' quotes
			$this->db->formatConditionsOneField($input, 'resourcequotetextAddUserIdQuote');
			$this->db->delete('resource_quote_text');
// remove users' comments for other quotes
			$this->db->formatConditionsOneField($input, 'resourcequotecommentAddUserIdQuote');
			$this->db->delete('resource_quote_comment');
// remove users' paraphrases
			$this->db->formatConditionsOneField($input, 'resourceparaphrasetextAddUserIdParaphrase');
			$this->db->delete('resource_paraphrase_text');
// remove users' comments for other paraphrases
			$this->db->formatConditionsOneField($input, 'resourceparaphrasecommentAddUserIdParaphrase');
			$this->db->delete('resource_paraphrase_comment');
// remove users' musings
			$this->db->formatConditionsOneField($input, 'resourcemusingtextAddUserIdMusing');
			$this->db->delete('resource_musing_text');
		}
		return TRUE;
	}
}
?>
