<?php
/**
WIKINDX: Bibliographic Management system.
Copyright (C)

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

The WIKINDX Team 2013
sirfragalot@users.sourceforge.net
*/
/**
* USERTAGS
*
* Common methods for handling user tags.
* 
* @version	1
*
*	@package wikindx4\core\usergroups
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*
*/
class USERTAGS
{
/** object */
private $db;
/** object */
private $html;
/** object */
private $session;
/** object */
private $commonBib;

/**
*	USERTAGS
*/
	public function __construct()
	{
		$this->db = FACTORY_DB::getInstance();
		$this->html = FACTORY_HTML::getInstance();
		$this->session = FACTORY_SESSION::getInstance();
		$this->commonBib = FACTORY_BIBLIOGRAPHYCOMMON::getInstance();
	}
/**
* grab all current usertags for this user as assoc. array
*
* @param boolean $userBib Default is FALSE
* @param array $typeArray Default is FALSE
* @param boolean $populated Default is FALSE
* @return array
*/
	public function grabAll($userBib = FALSE, $typeArray = FALSE, $populated = FALSE)
	{
		$this->db->leftJoin('resource_user_tags', 'resourceusertagsTagId', 'usertagsId');
		if($userBib)
			$this->commonBib->userBibCondition('resourceusertagsResourceId');
		if(is_array($typeArray) && !empty($typeArray))
		{
			$this->db->leftJoin('resource', 'resourceId', 'resourceusertagsResourceId');
			foreach($typeArray as $type)
				$conditions[] = $type;
			$this->db->formatConditionsOneField($conditions, 'resourceType');
		}
		$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
		if($populated)
		{
			$this->db->groupBy(array('usertagsId', 'usertagsTag', 'resourceusertagsTagId'), TRUE, $this->db->count('resourceusertagsTagId') . '>0');
			$resultset = $this->db->selectCounts('user_tags', 'resourceusertagsTagId', array('usertagsId', 'usertagsTag'), FALSE, FALSE);
		}
		else
			$resultset = $this->db->select('user_tags', array('usertagsId', 'usertagsTag'));
		$array = array();
		while($row = $this->db->fetchRow($resultset))
			$array[$row['usertagsId']] = $this->html->dbToFormTidy($row['usertagsTag']);
		return $array;
	}
/**
* Does this usertag already exist in the database?
*
* @param string $usertag
* @return mixed FALSE|usertag ID
*/
	public function checkExists($usertag)
	{
		$this->db->formatConditions(array('usertagsUserId' => $this->session->getVar('setup_UserId')));
		$this->db->formatConditions($this->db->formatFields('usertagsTag') . $this->db->like(FALSE, $usertag, FALSE));
		$resultset = $this->db->select('user_tags', 'usertagsId');
		if($this->db->numRows($resultset))
			return $this->db->fetchOne($resultset);
		return FALSE; // not found
	}
/**
* Make the usertag select transfer arrows to transfer usertags between select boxes with onclick
*
* @return array (toRightImage, toLeftImage)
*/
	public function transferArrows()
	{
		$ajax = FACTORY_AJAX::getInstance();
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'selectUserTag', 
			);
		$toRightImage = $ajax->jActionIcon('toRight', 'onclick', $jsonArray);
		$jsonArray = array();
		$jsonArray[] = array(
			'startFunction' => 'discardUserTag', 
			);
		$toLeftImage = $ajax->jActionIcon('toLeft', 'onclick', $jsonArray);
		return array($toRightImage, $toLeftImage);
	}
}
?>
