<?php
/**
* WIKINDX: Bibliographic Management system. Copyright (C)

* This program is free software; you can redistribute it and/or modify it under the terms
* of the GNU General Public License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along with this program;
* if not, write to the
* Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* The WIKINDX Team 2013
* sirfragalot@users.sourceforge.net.
*
*	index.php
*
*	Gateway to the system.  Here, the form variable 'action' is parsed to
*	determine the course of action.
*	All actions and requests pass via this gateway.
*
*	@package wikindx
*	@author Mark Grimshaw <sirfragalot@users.sourceforge.net>
*/
/**
* Default error reporting -- other settings in core/startup/LOADCONFIG.php which is first called in AUTHORIZE.  
* To debug from here to AUTHORIZE, change the error_reporting() here to E_ALL
*/
	if(version_compare(PHP_VERSION, '5.3.0', '<')) 
		error_reporting(~E_STRICT);
	else
		error_reporting(~E_DEPRECATED & ~E_STRICT);
// Check for presence of config.php
	if(!is_file('config.php'))
	{
		$message = "<p>config.php is missing. If this is a new installation, copy config.php.dist to config.php and edit 
		that file to ensure the MySQL access protocols match those you have specified for the WIKINDX database.</p>";
		$message .= "<p>Ensure also that the 'files/', 'attachments/', 'templates/compiled_templates', 'styles/cache' and 
		'styles/bibliography/' folders and the XML files within the 
		'styles/bibliography/xxx/' folders are writable by the web server user (usually 'nobody').</p>";
		die($message);
	}
/**
* Import CONSTANTS
*/
	include_once("core/startup/CONSTANTS.php");
/**
* Set up the FACTORY objects of commonly used classes and start the timer.
*/
	include_once("core/startup/FACTORY.php");
	FACTORY_TIMER::getInstance();
/**
* Set and store some global variables in preference to using the PHP $GLOBALS array
*/
	include_once("core/startup/GLOBALS.php");
// Load core javascript functions
	GLOBALS::buildOutputScript("<script type=\"text/javascript\" src=\"core/coreJavascript.js\"></script>");
/**
*	Initialize the system
*	As ENVIRONMENT starts the session, we must check for 'remember me' cookie requests from MYWIKINDX.php prior to 
*	setting the session.
*/
	include_once("core/startup/ENVIRONMENT.php");
	$env = new ENVIRONMENT();
	$vars = GLOBALS::getVars();
	if(!empty($vars) && array_key_exists('cookie', $vars) && $vars['cookie'] && 
		array_key_exists('uname', $vars) && trim($vars['uname']))
	{
// set the cookie if requested
		$cookie = FACTORY_COOKIE::getInstance();
		$cookie->storeCookie($vars['uname']);
		unset($cookie);
	}
// start the session
	$env->startSession();
	unset($env);
// Login
	$authorize = FACTORY_AUTHORIZE::getInstance();
// Are we upgrading the database?
	if(array_key_exists('action', $vars) && ($vars['action'] == 'upgradeDBLogon'))
		$authorize->gateKeep(); // successfully logged in so session variable superadmin is set to TRUE which is then picked up in UPDATEDATABASE
// Do database upgrade check
	include_once("core/startup/UPDATEDATABASE.php");
	$update = new UPDATEDATABASE(); // __construct() runs on autopilot
	unset($update);
	if(array_key_exists('action', $vars) && ($vars['action'] == 'continueExecution'))
		unset($vars['action']);
// Only superadmin can upgrade the database so set up their environment
	if(array_key_exists('action', $vars) && ($vars['action'] == 'upgradeDB'))
	{
		if($authorize->gateKeep() === FALSE)
		$vars['action'] = 'front';
	}
/**
*	First pass through authentication.
*/
// return from gatekeep is either TRUE (meaning proceed without change) or FALSE (meaning set $vars['action'] = "front') 
// which will print the front page of WIKINDX.
// gatekeep() can itself exit the script (usually to the initial logon prompt when forbidding access)
	if($authorize->gateKeep() === FALSE)
		$vars['action'] = 'front';
// RSS feed access granted
	if(array_key_exists('method', $vars) && ($vars['method'] == 'RSS'))
		unset($vars['method']);
// User bookmarks can only be added when looking at a single resource's details or resource lists.  
// Set default behaviour here to remove 'add bookmark' link from menu
	$session = FACTORY_SESSION::getInstance();
	$session->setVar("bookmark_DisplayAdd", FALSE);
	$session->delVar('bookmark_MultiView');
// Set the current working directory -- useful for ensuring TinyMCE plug-ins can find the wikindx4 base path for include() commands.
// Not all OSs allow getcwd() or sometimes the wikindx installation is in a directory that is not searchable.
	if(!$_SESSION['wikindxBasePath'] = getcwd())
	{
		$config = FACTORY_CONFIG::getInstance();
		if(!isset($config->WIKINDX_WIKINDX_PATH) || !$config->WIKINDX_WIKINDX_PATH)
			die("WIKINDX is unable to set the installation path automatically.  You should set \$WIKINDX_WIKINDX_PATH in config.php");
		$path = $config->WIKINDX_WIKINDX_PATH;
// test path is correct
		if(!is_file($path . '/core/startup/LOADCONFIG.php'))
			die("\$WIKINDX_WIKINDX_PATH in config.php is set incorrectly");
		$_SESSION['wikindxBasePath'] = $path;
	}
/**
* If we get here, we're cleared to proceed.  If there is no $vars['action'], send users to the front of wikindx, 
* otherwise load the requested module.
* WIKINDX has a modular structure.  If $vars['action'] has 'xxx_XXXXXX_CORE in it, it actions a core/modules/ module, 
* otherwise, we look in the plugins/ folder for a third-party plug-in.
*/
//print_r($vars); print "<P>";
//print_r($session->getArray('mywikindx'));
	$actionFound = FALSE;
	if(!array_key_exists('action', $vars) || ($vars['action'] == 'front')) // display front page
	{
// Check if statistics need compiling and emailing out to registered users.
		$stats = FACTORY_STATISTICS::getInstance();
		$stats->compile();
		include_once("core/display/FRONT.php");
		$front = new FRONT(); // __construct() runs on autopilot
		unset($front);
		$actionFound = TRUE;
	}
	else if(array_key_exists('action', $vars) && ($vars['action'] == 'noMenu')) // display menu links as hyperlinks
	{
		include_once('core/navigation/MENU.php');
		$menu = new MENU();
		$menu->noMenu();
		$actionFound = TRUE;
	}
	else if(array_key_exists('action', $vars) && ($vars['action'] == 'noSubMenu')) // display submenu links as hyperlinks
	{
		include_once('core/navigation/MENU.php');
		$menu = new MENU();
		$menu->noSubMenu();
		$actionFound = TRUE;
	}
/**
* Core wikindx modules have the querystring 'action' as <directory>_<FILE>_CORE the assumption being that <directory>
* is under 'core/modules/'.
* <FILE> could be of the form <FILE::CLASS> to allow for multiple classes (e.g. inheritance) in the one file.
* The optional querystring 'method' defines the class method to use -- if not present, the init() class method is the default
*/
	else
	{
		$split = explode("_", $vars["action"]);
		if((sizeof($split) == 3) && ($split[2] == 'CORE')) // this is a core module
		{
			if(sizeof($fileSplit = explode('::', $split[1])) == 2) // file and named class in that file
			{
				$file = "core/modules/" . $split[0] . '/' . $fileSplit[0] . '.php';
				$class = $fileSplit[1];
			}
			else
			{
				$file = "core/modules/" . $split[0] . '/' . $split[1] . '.php';
				$class = $split[1];
			}
			if(file_exists($file))
			{
				include_once($file);
				$obj = new $class();
				$method = array_key_exists('method', $vars) ? $vars['method'] : 'init';
				if(method_exists($obj, $method))
				{
					$obj->{$method}();
					$actionFound = TRUE;
				}
			}
		}
	}
	if(!$actionFound) // third-party plug-in?
	{
// To guarantee that the action label is not the same as a core one, the module folder name is prepended to the 
// given action (e.g. folderName_actionName). We must remove the prepend.
		$split = explode("_", $vars["action"]);
		if(sizeof($split) == 2)
		{
			$index = "plugins/" . $split[0] . '/index.php';
			if(file_exists($index))
			{
				include_once($index);
// class name must be in the form $dirName . _MODULE
				$class = $split[0] . "_MODULE";
				$obj = new $class(FALSE);
				$method = $split[1];
				if(method_exists($obj, $method))
				{
					$obj->{$method}();
					$actionFound = TRUE;
				}
			}
		}
		unset($loadmodules);
	}
	if(!$actionFound)
	{
// We've finally come to a dead end.....
		$errors = FACTORY_ERRORS::getInstance();
		GLOBALS::buildOutputString($errors->text("inputError", "invalid"));
	}

// Store up to 10 of the last REQUEST_URIs
	if($_SERVER['REQUEST_URI'])
	{
		$qs = $session->getArray('QueryStrings');
		if(!is_array($qs))
			$qs = array();
		if(!empty($qs) && $_SERVER['REQUEST_URI'] != $qs[0])
		{
			if(sizeof($qs) == 10)
				array_pop($qs);
		}
		array_unshift($qs, $_SERVER['REQUEST_URI']);
		$session->writeArray($qs, 'QueryStrings');
		$session->saveState('QueryStrings');
	}


// Debugging - print all SESSION variables.
// COMMENT OUT IN PRODUCTION!!!!!!!!!!!!!!!!!!!!!
//	global $_SESSION ;	
//	print "<hr><strong>[_SESSION]</strong><hr>";
//	foreach($_SESSION as $key => $value)
//		print "$key: $value<br>";
	//print_r($_SESSION);
	//print "<hr><strong>[session->getArray('setup')] </strong>";
	//print_r($session->getArray('setup'));
	
// SERVER variables
//	foreach($_SERVER as $key => $value)
//		print "$key: $value<br>";

	/* Memory checking */
	//print "<hr><strong>[MEMORY USED]</strong> " . memory_get_peak_usage() / 1000000 . " MB";
	//print "<hr><strong>[PHP INI MEMORY SET]</strong> " . ini_get('memory_limit');
// COMMENT OUT IN PRODUCTION!!!!!!!!!!!!!!!!!!!!!
// Debugging - print all SESSION variables.

/**
*	Close the HTML code by calling the constructor of CLOSE which also 
*	prints the HTTP header, body, menus, footer, flushes the print buffer, closes the database and exits the script.
*/
	FACTORY_CLOSE::getInstance();
?>